AgentAI_RaceDriver = class(function(self, Car)
	self.Car = Car
	self.Agent = LuaAgent()
	
	self.MinAISpeed = 75 / 0.36 --500
	self.MaxAISpeed = 300 / 0.36 --2000
	self.MaxAcceleration = 1
	self.SteerAttack = 1
	self.DistanceCheck = 1
	
	self.Destination = nil
	
	AddKeyListeners("DCK_F", "Down", function(Listeners)
		self.Destination = self.Agent:GetPathDestination()
	end)
end)

function AgentAI_RaceDriver:Update(ElapsedTime)
	local CarMatrix = self.Car:GetMatrix()
	local CarVelocity = self.Car:GetVelocity()
	self.Agent:UpdateData(CarMatrix, CarVelocity)
	
	self.Destination = self.Agent:GetPathDestination()
	
	if (self.Destination ~= nil) then
		
		--print("Destination; " .. tostring(self.Destination))
		local Dir = (CarMatrix:GetTranslation() - self.Destination):Normalize()
		local Dist = CarMatrix:GetTranslation():DistanceTo(self.Destination)
		
		local SteerValue = self.Car:GetSteerDir(Dir)
		local ClampedSteer = ClampMinMax(SteerValue, -1.0, 1.0)
		local Steer = math.abs(SteerValue)
		self.Car:Steer(ClampedSteer)
		
		--print(Dist)
		
		if (CarVelocity:Magnitude() < self.MaxAISpeed and Dist > self.DistanceCheck) then
			if (CarVelocity:Magnitude() > self.MinAISpeed) then
				local Accel = 1 * ClampMinMax(1.0 - (ClampedSteer / 2), 0.0, 1.0)
				self.Car:Accelerate(ClampMax(Accel, self.MaxAcceleration))
				
				local Brake = ClampMinMax((ClampedSteer - 0.25) * self.SteerAttack, 0.0, 1.0)
				self.Car:Brake(Brake)
			else
				self.Car:Accelerate(ClampMax(1, self.MaxAcceleration))
				self.Car:Brake(0)
			end
		elseif (Dist <= self.DistanceCheck) then
			self.Car:Steer(0)
			self.Car:Accelerate(0)
			self.Car:Brake(1)
		else
			self.Car:Accelerate(0)
			local Brake = ClampMinMax((Steer * 0.5), 0.0, 1.0)
			self.Car:Brake(Brake)
		end
	
		self.Car:Handbrake(0)
		self.Car:ChangeGear(self.Car:GetBestGear())
	else
		--print("No Destination...")
	
		self.Car:Steer(0)
		self.Car:Accelerate(0)
		self.Car:Brake(1)
		self.Car:Handbrake(0)
		self.Car:ChangeGear(self.Car:GetBestGear())
	end
end

-- LuaDriver functions

function AgentAI_RaceDriver:Init()
	print("AgentAI_RaceDriver:Init TODO")
end

function AgentAI_RaceDriver:Enable()
	print("AgentAI_RaceDriver:Enable TODO")
end

function AgentAI_RaceDriver:Disable()
	print("AgentAI_RaceDriver:Disable TODO")
end

function AgentAI_RaceDriver:EnableBoost()
	print("AgentAI_RaceDriver:EnableBoost TODO")
end

function AgentAI_RaceDriver:DisableBoost()
	print("AgentAI_RaceDriver:DisableBoost TODO")
end

function AgentAI_RaceDriver:SetSpeedMult()
	print("AgentAI_RaceDriver:SetSpeedMult TODO")
end

-- 

function AgentAI_RaceDriver:Checkpoint()
	print("AgentAI_RaceDriver:Checkpoint")
end

function AgentAI_RaceDriver:NextCheckpoint(Checkpoint)
	print("AgentAI_RaceDriver:NextCheckpoint")
	
	local CheckpointPos = Checkpoint:GetMatrix():GetTranslation()
	print(CheckpointPos)
	
	self.Agent:FindPath(CheckpointPos.x, CheckpointPos.y, CheckpointPos.z)
end
