-- Game modes
CarClass = class(function(self, Name)
	self.Name = Name;
	
	if (Name ~= "MonstertruckCar") then
		self.Title = "Class_" .. Name
	else
		self.Title = "Class_" .. "Monstertruck"
	end

	self.Cars = {}
	self.UnrestrictedCars = {}
	
	self.Wheels = {}
	
	self.CarsCopy = self.Cars
end)

function CarClass:AddCar(Car)
	if (Car == nil or TableContains(self.Cars, Car)) then
		return
	end
	
	--table.insert(UnlockPopups, string.format(Translate("NEW_CAR_UNLOCKED"), Translate(Car.Title)))
	--local Popup = {UnlockMenu, Car}
	--table.insert(UnlockPopups, Popup)

	Car.CarClass = self
	
	table.insert(self.Cars, Car);
	return Car;
end

function CarClass:AddUnrestrictedCar(Car)
	if (Car == nil or TableContains(self.UnrestrictedCars, Car)) then
		return
	end
	
	Car.CarClass = self
	
	table.insert(self.UnrestrictedCars, Car);
	return Car;
end

function CarClass:AddWheel(Wheel)
	if (Wheel == nil or TableContains(self.Wheels, Wheel)) then
		return
	end
	
	--table.insert(UnlockPopups, string.format(Translate("NEW_CAR_UNLOCKED"), Translate(Car.Title)))
	--local Popup = {UnlockMenu, Car}
	--table.insert(UnlockPopups, Popup)

	Wheel.CarClass = self
	
	table.insert(self.Wheels, Wheel);
	return Wheel;
end

function CarClass:GetCar(Index)
	return self.Cars[Index + 1]
end

-- Slow, but seeing as how the CarClass and the corresponding menus were designed
-- to use numeric indices rather than string keys (like a hashmap structure), this
-- is the only way to find a car from outside apart from making an enumeration that
-- has the numeric keys hardcoded into itself (very inflexible).
function CarClass:GetCarByName(Name)
	for i = 1, #self.Cars do
		local Car = self.Cars[i]
		if (Car.Name == Name) then
			return Car
		end
	end
	
	return -- Return nil if no car was found
end

function CarClass:LoadTextures()
	for i, Car in ipairs(self.Cars) do
		Car:LoadTexture()
	end
end

function CarClass:UnloadTextures()
	for i, Car in ipairs(self.Cars) do
		Car:UnloadTexture()
	end
end

function CarClass:Read(File, Version)
	local CarCount = File:ReadInt()
	for i = 1, CarCount do
		self.Cars[i] = _G[File:ReadString()]
		--self.Cars[i].DriveCount = File:ReadInt()
		self.Cars[i]:Read(File, Version)
	end
	
	local WheelCount = File:ReadInt()
	for i = 1, WheelCount do
		self.Wheels[i] = _G[File:ReadString()]
		self.Wheels[i]:Read(File)
	end
end

function CarClass:Write(File)
	File:WriteInt(#self.Cars);
	
	for i = 1, #self.Cars do
		File:WriteString(self.Cars[i].Name)
		--File:WriteInt(self.Cars[i].DriveCount)
		self.Cars[i]:Write(File)
	end
	
	
	File:WriteInt(#self.Wheels);
	for i = 1, #self.Wheels do
		File:WriteString(self.Wheels[i].Name)
		self.Wheels[i]:Write(File)
	end
end

Car = class(function(self, Name)
	self.Name = Name;
	self.Title = Name .. "_Name";
	self.Description = Name .. "_Description";
	self.Compliment = Name .. "_Compliment";
	self.Image = nil
	self.HudImage = nil
	self.LogoImage = nil
	
	self.Handling = 1
	self.Acceleration = 1
	self.Speed = 1
	self.Strength = 1
	
	self.MechanicActiveRaces = 0
	
	--Stats data
	self.DriveCount = 0
	self.BoringCount = 0
	
	--Car data
	self.Tunings = {}
	
	--Color config
	self.ColorConfig = {}
	self.ColorConfig.DiffuseColor = color(64, 0, 0, 255)
	self.ColorConfig.SpecularColor = color(255, 0, 0, 255)
	self.ColorConfig.SpecularIntensity = 10000000
	self.ColorConfig.SpecularPower = 40.0 / 255.0
	
	--Wheel config
	self.WheelConfig = {}
	self.WheelConfig.Model = "Wheel_1"
	self.WheelConfig.Scale = 1
	
	--Shop data
	self.Bought = false
	--self.ShopDefaultColor = nil
	self.ShopPriceCoins = 1000
	self.ShopPriceSpecialCoins = 1000
	self.ShopDescription = translate(Name .. "_ShopDescription");
	
	self.Available = false
	self.DeliveryTime = 0
	self.ShowPopup = true
	
	self.DiscountOffered = false
end)

function Car:Buy()
	self.Bought = true
end

function Car:AddTuning(Tuning)
	self.Tunings[#self.Tunings + 1] = Tuning;
	return Tuning;
end

function Car:GetTuning(Index)
	return self.Tunings[Index + 1]
end

function Car:LoadTexture()
	self.LogoImage = texture("MonsterTruckLogos\\" .. self.Name .. "_NameImage.DCT")
	
	local TruckImageFilename = "Menu\\Images\\MonsterTrucks\\" .. self.Name .. ".DCT"
	if (GetFS():Exists(TruckImageFilename)) then
		self.TruckImage = texture(TruckImageFilename)
	else
		self.TruckImage = nil
	end
end

function Car:UnloadTexture()
	self.LogoImage = nil
	self.TruckImage = nil
end

function Car:Read(File, Version)
	self.DriveCount = File:ReadInt()
	self.BoringCount = File:ReadInt()
	self.Bought = File:ReadBoolean()
	self.Available = File:ReadBoolean()
	self.DeliveryTime = File:ReadInt()
	
	self.DiscountOffered = File:ReadBoolean()
	
	if (self.Upgrades == nil) then 
		self.Upgrades = {} 
    end 
    local TotalUpgradeItems = File:ReadInt()

    for i = 1, TotalUpgradeItems do 
        local UpgradeItemName = File:ReadString() 
        local CurrentLevel = File:ReadInt() 
        local BoughtLevel = File:ReadInt() 
		local Level = File:ReadInt()
		local TimeStamp = File:ReadInt()
           
        if (self.Upgrades[UpgradeItemName] == nil) then 
            self.Upgrades[UpgradeItemName] = {} 
        end 
           
        self.Upgrades[UpgradeItemName].CurrentLevel = Max(self.Upgrades[UpgradeItemName].CurrentLevel, CurrentLevel) 
        self.Upgrades[UpgradeItemName].BoughtLevel = Max(self.Upgrades[UpgradeItemName].BoughtLevel, BoughtLevel) 
		
		if (TimeStamp ~= 0 and Level ~= 0) then
			self.Upgrades[UpgradeItemName].DeliveryLevel = Level
			self.Upgrades[UpgradeItemName].DeliveryTime = TimeStamp
		end
    end
	
	if (Version > 7) then
		self.MechanicActiveRaces = File:ReadInt()
	end
	
	--[[self.WheelConfig.Model = File:ReadString()
	
	local DiffuseColor_R = File:ReadInt()
	local DiffuseColor_G = File:ReadInt()
	local DiffuseColor_B = File:ReadInt()
	self.ColorConfig.DiffuseColor = color(DiffuseColor_R, DiffuseColor_G, DiffuseColor_B, 255)
	
	local SpecularColor_R = File:ReadInt()
	local SpecularColor_G = File:ReadInt()
	local SpecularColor_B = File:ReadInt()
	self.ColorConfig.SpecularColor = color(SpecularColor_R, SpecularColor_G, SpecularColor_B, 255)]]
end

function Car:Write(File)
	File:WriteInt(self.DriveCount)
	File:WriteInt(self.BoringCount)
	File:WriteBoolean(self.Bought)
	File:WriteBoolean(self.Available)
	File:WriteInt(self.DeliveryTime)
	
	File:WriteBoolean(self.DiscountOffered)
	
	File:WriteInt(TableLength(self.Upgrades))
	
    for Key, Value in pairs(self.Upgrades) do 
        File:WriteString(Key) 
        File:WriteInt(Value.CurrentLevel) 
        File:WriteInt(Value.BoughtLevel)
		File:WriteInt(Value.DeliveryLevel)
		File:WriteInt(Value.DeliveryTime)
    end
	
	File:WriteInt(self.MechanicActiveRaces)
	
	--[[File:WriteString(self.WheelConfig.Model)
	
	if (self.Name == "MonsterTruckCar01") then
		print(self.WheelConfig.Model)
	end
	
	File:WriteInt(self.ColorConfig.DiffuseColor:getr())
	File:WriteInt(self.ColorConfig.DiffuseColor:getg())
	File:WriteInt(self.ColorConfig.DiffuseColor:getb())
	
	File:WriteInt(self.ColorConfig.SpecularColor:getr())
	File:WriteInt(self.ColorConfig.SpecularColor:getg())
	File:WriteInt(self.ColorConfig.SpecularColor:getb())]]
end

Tuning = class(function(self, Title, Name)
	self.Title = Title
	self.Name = Name
end)

Wheel = class(function(self, Name)
	self.Name = Name;
	self.Title = Name .. "_Name";
	self.Description = Name .. "_Description";
	
	--self.Model = "Models\\Wheels\\" .. Name .. ".DCM"
	self.Model = Name
	
	--Shop data
	self.Bought = false
	self.ShopPriceCoins = 1000
	self.ShopDescription = translate(Name .. "_ShopDescription");
	
	self.Available = false
	self.DeliveryTime = 0
	self.ShowPopup = true
	
	self.DiscountOffered = false
end)

function Wheel:Read(File)
	self.Bought = File:ReadBoolean()
	self.Available = File:ReadBoolean()
	self.DeliveryTime = File:ReadInt()
	self.ShowPopup = File:ReadBoolean()
	
	self.DiscountOffered = File:ReadBoolean()
end

function Wheel:Write(File)
	File:WriteBoolean(self.Bought)
	File:WriteBoolean(self.Available)
	File:WriteInt(self.DeliveryTime)
	File:WriteBoolean(self.ShowPopup)
	
	File:WriteBoolean(self.DiscountOffered)
end


---

function Cars_Sort(x, y)
	if (x.Bought == y.Bought) then
		if (x.OrderIndex ~= nil and y.OrderIndex ~= nil) then
			return x.OrderIndex < y.OrderIndex
		else
			return x.Name < y.Name
		end
	elseif (x.Bought and not y.Bought) then
		return true
	elseif (not x.Bought and y.Bought) then
		return false
	end
end