Chain = class(function(self, Segments, StartMatrix, StartDir, Target1, Target2, Target1Offset, Target2Offset)	 -- Target1/Target2 = {Type = "Car/Body", Object = "UserData"}
	print("^3INIT Chain")
	
	self.Segments = Segments
	self.StartMatrix = StartMatrix
	self.StartDir = StartDir
	
	self.Target1IsCar = false
	self.Target2IsCar = false
	
	self.StressMeasured = 0
	self.IndexToSnap = 10
	self.CanSnap = true
	self.SnapForce = 1.5
	
	if (Target1.Type == "Car") then
		if (Target1.Object ~= nil) then
			self.Target1IsCar = true
		else
			print("^1Error; ^7Target1.Object is nil!")
			return
		end
	elseif (Target1.Type ~= "Body") then
		print("^1Error; ^7Target1.Type is not a '^2Car^7' or '^2Body^7'!")
		return
	end
	
	self.Target1 = Target1.Object
	
	if (Target2.Type == "Car") then
		if (Target2.Object ~= nil) then
			self.Target2IsCar = true
		else
			print("^1Error; ^7Target2.Object is nil!")
			return
		end
	elseif (Target2.Type ~= "Body") then
		print("^1Error; ^7Target2.Type is not a '^2Car^7' or '^2Body^7'!")
		return
	end
	
	self.Target2 = Target2.Object

	self.Target1Pos = self.Target1:GetMatrix():GetTranslation()
	self.Target1Pos = self.Target1Pos + Target1Offset
	
	self.Target2Pos = self.Target2:GetMatrix():GetTranslation()
	self.Target2Pos = self.Target2Pos + Target2Offset
	
	self.InitialDistance = self.Target1Pos:DistanceTo(self.Target2Pos)
	
	self.AllLinks = {}
	
	JointColor = color(1,0,0,1)
	

	self.StartLink = ChainLink(self, 1, self.Target1, "Chain_Hook", self.Target1Pos, self.Target1IsCar)
	table.insert(self.AllLinks, self.StartLink)
	
	self.EndLink = ChainLink(self, self.Segments, self.Target2, "Chain_Attachment", self.Target2Pos, self.Target2IsCar)
	
	for i = 1, self.Segments - 1 do
		local PrevBody = self.AllLinks[i].NewBody
		
		if (PrevBody ~= nil) then
			local Link = ChainLink(self, i, PrevBody, "Chain_Section")
			table.insert(self.AllLinks, Link)
		end
	end
	
	table.insert(self.AllLinks, self.EndLink)
	
	
	
	-- Attach the chords
	local Link = self.AllLinks[#self.AllLinks - 1]
	
	self.ConnectionJoint = BallSocketJoint(GetPhysicsScene())
	
	self.ConnectionJoint:Attach(Link.NewBody, self.EndLink.NewBody)
	self.ConnectionJoint:Enable()
	self.ConnectionJoint:SetPosition(Link.Pos)
	

	
	self.UpdateThread = PhysicsThread:Spawn(function()		
		-- Updateloop
		while (true) do
			--DrawLineASync(Link.NewBody:GetMatrix():GetTranslation(), self.EndLink.NewBody:GetMatrix():GetTranslation(), JointColor)	

			self.StressMeasured = 1
			
			for i, Link in ipairs(self.AllLinks) do
				Link:Update()
			end
			PhysicsThread:Wait(1)
		end
	end)
	
	--print("^3END Chain")
end)

function Chain:Destroy()
	if (self.UpdateThread ~= nil) then
		PhysicsThread:Stop(self.UpdateThread)
		self.UpdateThread = nil
	end
	
	if (self.ConnectionJoint ~= nil) then
		self.ConnectionJoint:Destroy()
		self.ConnectionJoint  = nil
	end
end

ChainLink = class(function(self, Chain, Index, PrevBody, Object, CustomStartPos, AttachToCar)
	self.Index = Index
	self.Chain = Chain
	self.Object = Object
	
	self.MayScale = true
	
	--print("Make ChainLink; " .. self.Index .. "-" .. self.Index + 1)
	
	if (CustomStartPos ~= nil) then
		self.CustomStartPos = CustomStartPos
	end
	
	self.PrevBody = PrevBody
	
	local Perc = self.Index * (self.Chain.InitialDistance / self.Chain.Segments) / self.Chain.InitialDistance
	
	self.Pos = Lerp(self.Chain.Target1Pos, self.Chain.Target2Pos, Perc)
	
	if (self.CustomStartPos ~= nil) then
		self.Pos = self.CustomStartPos
	end
	
	local PhysicsMesh = CreatePhysicsMesh(self.Object, self.Pos)
	
	self.Mesh = PhysicsMesh.Mesh
	self.NewBody = PhysicsMesh.Body
	self.Geometry = PhysicsMesh.Geometry
	
	self.Joint = BallSocketJoint(GetPhysicsScene())
	
	self.InitialDistance = self.NewBody:GetMatrix():GetTranslation():DistanceTo(self.PrevBody:GetMatrix():GetTranslation())
	
	self.Contact = {}
	
	if (self.PrevBody ~= nil and self.NewBody ~= nil) then
		if (AttachToCar) then
			self.PrevBody:AttachToJoint(self.Joint, self.NewBody)
		else
			self.Joint:Attach(self.PrevBody, self.NewBody)
		end
		
		self.Joint:Enable()
		self.Joint:SetPosition(self.Pos)
	end
	
	self.Mesh:SetDestroyCallback(function(...)
		self:Destroy(...)
	end)
	
	self.Mesh:SetRenderCallback(function(...)
		self:Render(...)
	end)
	
	self.Geometry:SetContactCallback(function(...)
		return self:OnContact(...)
	end)
	
	self.Geometry:SetCollisionCallback(function(...)
		return self:OnCollision(...)
	end)
end)

function ChainLink:Update()	
	local Matrix = self.NewBody:GetMatrix()
	local Vel = self.NewBody:GetVelocity()
	local AngVel = self.NewBody:GetAngularVelocity()
	
	local StartPos = Matrix:GetTranslation()
	local EndPos = self.PrevBody:GetMatrix():GetTranslation()
	
	Matrix = Matrix:LookAt(StartPos, EndPos, 0)
	Matrix = Matrix:RotateLocal(Vector(0,1,0), math.rad(-90))
	
	local Distance = 1 
	if (self.MayScale) then
		-- Calculate link stress
		Distance = self.NewBody:GetMatrix():GetTranslation():DistanceTo(self.PrevBody:GetMatrix():GetTranslation()) / self.InitialDistance
	end
	
	self.Chain.StressMeasured = Max(self.Chain.StressMeasured, Distance)
	
	if (self.Chain.StressMeasured > self.Chain.SnapForce and self.Chain.CanSnap) then
		self.Chain:BreakChain()
	end

	local CurrentMatrix = self.Mesh:GetMatrix():Lerp(Matrix, 0.5)
	CurrentMatrix = CurrentMatrix:ScaleLocal(Vector(1 * Distance,1,1))
	
	if (self.Index == self.Chain.IndexToSnap - 1 and not self.Chain.CanSnap) then
		CurrentMatrix = MatrixIdentity
	end
	
	self.Mesh:Update(CurrentMatrix, Vel, AngVel)
end

function Chain:BreakChain(DisableInput)
	if (not self.CanSnap) then
		return
	end
	
	if (DisableInput == nil) then
		DisableInput = true
	end
	
	self.CanSnap = false

	local Link = self.AllLinks[self.IndexToSnap]
	
	Link:BreakChain(Link)
	
	local TotalForce = 200000000
	local Length = 1
	
	local Player = ActiveMission.HumanPlayers[1]
	
	if (DisableInput) then
		Player.InputManager:Disable()
		Player.InputManager:Reset()
	end
		
	PhysicsThread:Spawn(function()
		if (Player ~= nil) then
			for i = 1, Length do
				Player.Car:AddForce(Player.Car:GetMatrix():GetDirection() * (-TotalForce / Length))
				PhysicsThread:Wait(1)
			end
		end
	end)
end

function ChainLink:BreakChain(Link)
	self.Chain.CanSnap = false
	self.MayScale = false

	Link.Joint:Destroy()
	Link.Joint = nil
	
	playsound("MJ_ChainBreak.wav")
	
	ActiveMission:EndGame()
end

function ChainLink:OnCollision(Object)
	return true
end

function ChainLink:OnContact(Entity, Pos, Normal, Depth)
	local Contact = self.Contact
	Contact.Friction1 = 10000
	Contact.Friction2 = 10000

	return Contact
end

function ChainLink:Render()
	if (self.PrevBody ~= nil and self.NewBody ~= nil) then	 
		local RenderJoint = false
		local RenderGeometry = false
		
		if (RenderJoint) then
			self.Joint:Render(JointColor)
		end
		if (RenderGeometry) then
			self.Geometry:Render(color(1, 1, 1, 1), self.Geometry:GetMatrix():GetTranslation())
		end
	end
end

function ChainLink:Destroy()
	self.Chain:Destroy()

	if (self.Joint ~= nil) then
		self.Joint:Destroy()
		self.Joint = nil
	end

	if (self.Geometry ~= nil) then
		self.Geometry:Destroy()
		self.Geometry = nil
	end

	if (self.NewBody ~= nil) then
		self.NewBody:Destroy()
		self.NewBody = nil
	end
end

function CreatePhysicsMesh(Object, Pos)
	local Mesh = {}
	
	Mesh.Mesh = CreateAdvancedMesh(nil, "Models\\" .. Object .. ".DCM", "")
		GetEntityScene():AddEntity(Mesh.Mesh)
		local StartMatrix = Matrix34():Translate(Pos)
		Mesh.Mesh:Update(StartMatrix, VectorZero, VectorZero)
		
	local ExtendsMult = 1
	
	GetPhysicsScene():SetCFM(0.00000000000000001)
	GetPhysicsScene():SetERP(0.999999999999999999999)
		
	local UseSphere = true
	local Size = 5
	
	Mesh.Body = Body(GetPhysicsScene())
		Mesh.Body:SetMass(100)
		Mesh.Body:SetInertiaTensor(Vector(Size,Size,Size))
		Mesh.Body:SetMatrix(Matrix34():Translate(Pos))
	
	if (not UseSphere) then
		Mesh.Geometry = BoxGeometry()
			Mesh.Geometry:SetEntity(Mesh.Mesh)
			Mesh.Geometry:SetOffsetMatrix(MatrixIdentity:Translate(VectorZero))
			Mesh.Geometry:SetBody(Mesh.Body)
			Mesh.Geometry:SetExtents(Vector(Size,Size,Size)) 
			Mesh.Geometry:SetType(bit32.bor(0))
			Mesh.Geometry:SetCollision(bit32.bor(DCGF_WORLD))
			
			Mesh.Geometry:Enable()
			Mesh.Geometry:SetMatrix(Matrix34():Translate(Pos))

			GetPhysicsScene():AddGeometry(Mesh.Geometry)
	else
		Mesh.Geometry = SphereGeometry()
			Mesh.Geometry:SetEntity(Mesh.Mesh)
			Mesh.Geometry:SetOffsetMatrix(MatrixIdentity:Translate(VectorZero))
			Mesh.Geometry:SetBody(Mesh.Body)
			Mesh.Geometry:SetRadius(Size)
			Mesh.Geometry:SetType(bit32.bor(0))
			Mesh.Geometry:SetCollision(bit32.bor(DCGF_WORLD))
			
			Mesh.Geometry:Enable()
			Mesh.Geometry:SetMatrix(Matrix34():Translate(Pos))

			GetPhysicsScene():AddGeometry(Mesh.Geometry)
	end
	
	return Mesh
end