DailyMissionsManager = class(function(self)
	self.DailyMissions = {}
	self.CurrentMission = 1
	self.CurrentThread = nil
	self.IDs = {}
	self.Achieves = {}
	self.Times = {}
	self.MissionValue = nil
	self.UsedCars = {}
	self.UseServerTime = false
	--GetLiveData()
	
	self:LoadTextures()
end)

function DailyMissionsManager:Reset()
	
end
function DailyMissionsManager:LoadTextures()
	local Texture1 = texture("Daily_Moneh1.dct")
	local Texture2 = texture("Daily_Moneh2.dct")
	local Texture3 = texture("Daily_Moneh3.dct")
	local Texture4 = texture("Daily_Moneh4.dct")
	local Texture5 = texture("Daily_Moneh5.dct")
	
	self.Textures = {Texture1, Texture2, Texture3, Texture4, Texture5}
end

function DailyMissionsManager:StartChecking(Mission, ChallengeName)
	
	if (self.CurrentThread ~= nil) then
		PhysicsThread:Stop(self.CurrentThread)
		self.CurrentThread = nil
	end
	
	if (self.DailyMissions[self.CurrentMission].Achieved) then
		return
	end
	
	if(self.DailyMissions[self.CurrentMission].Name == ChallengeName) then
		self.MissionValue = Mission
		self.DailyMissions[self.CurrentMission]:CheckFunction()
	end
end

function DailyMissionsManager:MissionAchieved()
	self.DailyMissions[self.CurrentMission]:Achieve()

	if(ActiveMission.HumanPlayers ~= nil) then
		ActiveMission.HumanPlayers[1].HUD.ScorePopup:ShowPopup("Mission Achieved!", Translate(self.DailyMissions[self.CurrentMission].Name), 0)	
	end
	MissionValue = nil
end

function DailyMissionsManager:AddMission(Mission)
	table.insert(self.DailyMissions, Mission)
end

function DailyMissionsManager:GetDailyChallengeTime()
	local Time = os.date("!%X",(StartTime - os.time()) + 86400)
	
	--if (os.time() - StartTime >= 86400) then
	if (UserServerTime) then
		GetServerTime("FullDate", function(Data)
			if (self.DailyMissions[self.CurrentMission].ActiveTime <= Data and self.DailyMissions[self.CurrentMission].ActiveTime > 0) then
				self.CurrentMission = Inc(self.CurrentMission)
				DailyMissionsManager:ValidateMissions()
				StartTime = os.time()
			end
		end)
	else
		if (self.DailyMissions[self.CurrentMission].ActiveTime <= os.time() and self.DailyMissions[self.CurrentMission].ActiveTime > 0) then
			self.CurrentMission = Inc(self.CurrentMission)
			DailyMissionsManager:ValidateMissions()
			StartTime = os.time()
		end
	end
	
	return Time
end

function DailyMissionsManager:ValidateMissions()

	if (not self.DailyMissions[self.CurrentMission - 1].Achieved or self.CurrentMission > 5) then
		print("Previous mission has not been achieved, resetting all!")
		DailyMissionsManager:ResetMissions()
	end
	
	if (ActiveMenu ~= nil and ActiveMenu:is_a(DailyMissionMenu)) then
		ActiveMenu:ValidateMenuLabels()
	end
end

function DailyMissionsManager:ResetMissions()
	for i, Mission in ipairs(self.DailyMissions) do
		Mission.Achieved = false
	end
	
	self.DailyMissions = Shuffle(self.DailyMissions)
	
	for i, Mission in ipairs(self.DailyMissions) do
		Mission.Day = i
		if (self.UseServerTime) then
			GetServerTime("FullDate", function(Data)
				Mission.ActiveTime = Data + (Mission.Day * 86400)
			end)
		else
			Mission.ActiveTime = os.time() + (Mission.Day * 86400)
		end
	end
	
	self.CurrentMission = 1
	WriteGameSettings()
end

DailyMission = class(function(self, ID, Name, CheckFunction)
	self.ID = ID
	self.Name = Name
	self.Day = #DailyMissionsManager.DailyMissions + 1
	self.CheckFunction = CheckFunction
	self.Achieved = false
	self.ActiveTime = 0
	if (DailyMissionsManager.UseServerTime) then
		GetServerTime("FullDate", function(Data)
			self.ActiveTime = Data + (self.Day * 86400)
		end)
	else
		self.ActiveTime = os.time() + (self.Day * 86400)
	end
	DailyMissionsManager:AddMission(self)
end)

function DailyMission:Achieve()
	if(not self.Achieved) then
		self.Achieved = true
		WriteGameSettings()
		local CoinsGained = GetDailyMissionReward(self.Day)
		VirtualCoins:AddCoinCount(CoinsGained)
		VirtualCoins:SendCollectedCoinsStats("DailyChallenge", CoinsGained, 0)
		
		local name = "ID: " .. self.ID .. " Name: " .. Translate(self.Name)
		AnalyticsSendEvent("Dailymission", "Achieved", name, "1")
	end
end

function DailyMission:Read(File, Version)
	table.insert(DailyMissionsManager.Achieves, File:ReadBoolean())
	table.insert(DailyMissionsManager.IDs, File:ReadInt())
	table.insert(DailyMissionsManager.Times, File:ReadInt())
end

function DailyMission:Write(File)
	File:WriteBoolean(self.Achieved)
	File:WriteInt(self.ID)
	File:WriteInt(self.ActiveTime)
end



DailyMissionsManager = DailyMissionsManager()

DailyMission_1 = DailyMission(001, "Challenge_1", function()
	if (DailyMissionsManager.MissionValue.Bounces >= 2) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_2 = DailyMission(002, "Challenge_2", function()
	if (DailyMissionsManager.MissionValue.Duration >= 1) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_3 = DailyMission(003, "Challenge_3", function()
	if (DailyMissionsManager.MissionValue.Duration >= 1) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_4 = DailyMission(004, "Challenge_4", function()
	if (DailyMissionsManager.MissionValue.Duration >= 2) then
		DailyMissionsManager:MissionAchieved()
	end
end)

--[[DailyMission_5 = DailyMission(005, "Challenge_5", function()
end)

DailyMission_6 = DailyMission(006, "Challenge_6", function()
end)

DailyMission_7 = DailyMission(007, "Challenge_7", function()
end)

DailyMission_8 = DailyMission(008, "Challenge_8", function()
end)--]]

DailyMission_5 = DailyMission(005, "Challenge_9", function()
	if (DailyMissionsManager.MissionValue.RaceTimer <= 60) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_6 = DailyMission(006, "Challenge_10", function()
	if (DailyMissionsManager.MissionValue.RaceTimer <= 55) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_7 = DailyMission(007, "Challenge_11", function()
	if (DailyMissionsManager.MissionValue.DistanceTowed >= 200) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_9 = DailyMission(009, "Challenge_13", function()
	local HumanPlayer = nil
	local AIPlayer = nil
	for i, Player in ipairs(DailyMissionsManager.MissionValue.Players) do
		if (Player.Human) then
			HumanPlayer = Player
		else
			AIPlayer = Player
		end
	end
	
	if (AIPlayer.TimeOffset >= 1) then
		DailyMissionsManager:MissionAchieved()
	end
end)

DailyMission_10 = DailyMission(010, "Challenge_14", function()
	local HumanPlayer = nil
	local AIPlayer = nil
	for i, Player in ipairs(DailyMissionsManager.MissionValue.Players) do
		if (Player.Human) then
			HumanPlayer = Player
		else
			AIPlayer = Player
		end
	end
	
	if (AIPlayer.TimeOffset >= 2) then
		DailyMissionsManager:MissionAchieved()
	end
end)

function DailyMissions_Read(File, Version)
	local DailyMissionsCount = File:ReadInt()
	
	DailyMissionsManager.Achieves = {}
	DailyMissionsManager.IDs = {}
	DailyMissionsManager.Times = {}
	
	local TestArr = {}
	local i = 1
	local Day = 1
	
	for i = 1, DailyMissionsCount do
		DailyMissionsManager.DailyMissions[i]:Read(File, Version)
	end
	
	while (#TestArr ~= #DailyMissionsManager.DailyMissions)do
		--print("DailyMissions ID", DailyMissionsManager.DailyMissions[i].ID, "DailyMission Day ID", DailyMissionsManager.IDs[Day])
		if(DailyMissionsManager.DailyMissions[i].ID == DailyMissionsManager.IDs[Day])then
			DailyMissionsManager.DailyMissions[i].Day = Day
			DailyMissionsManager.DailyMissions[i].Achieved = DailyMissionsManager.Achieves[Day]
			DailyMissionsManager.DailyMissions[i].ActiveTime = DailyMissionsManager.Times[Day]
			
			table.insert(TestArr, DailyMissionsManager.DailyMissions[i])
			Day = Inc(Day)
			i = 0
		end
		
		i = Inc(i)
		
		if(i > #DailyMissionsManager.DailyMissions) then
			print("Couldn't order all missions, breaking @ ", i, " Length of arr test: ", #TestArr, " Length of MainArr: ", #DailyMissionsManager.DailyMissions)
			break
		end
	end
	
	DailyMissionsManager.DailyMissions = TestArr
	
	
end

function DailyMissions_Write(File)
	File:WriteInt(#DailyMissionsManager.DailyMissions)
	for i = 1, #DailyMissionsManager.DailyMissions do
		DailyMissionsManager.DailyMissions[i]:Write(File, Version)
	end
end