CameraHelper = class(function(self, CameraIndex, Rotation, Model)
	if (CameraIndex == nil) then
		CameraIndex = 0
	end

	self.Camera = EntityCamera(CameraIndex)

	-- Config
	self.MinHeight = 64
	self.MaxHeight = 1280
	self.MinRotation = nil
	self.MaxRotation = nil
	
	-- Data
	self.CameraTarget = Vector(0,128,0)
	self.CameraPos = Vector(0,500,0)
	self.CameraHeight = 500
	self.CameraRotation = 0
	
	self.CamMatrix = MatrixIdentity
	self.Dir = Vector(0,0,1)
	self.Right = Vector(1,0,0)
	
	self.CamSpeed = 15
	
	self.Camera:SetFOV(math.rad(46))
	
	self.InputEnabled = true
	self.TargetFunction = nil
end)

function CameraHelper:Destroy()
	self.TargetFunction = nil
	GameThread:Stop(self.Thread)
end

function CameraHelper:MoveForward()
	self.TargetFunction = nil	--Clear TargetFunction
	self.CameraTarget = self.CameraTarget + (self.Dir * -self.CamSpeed)
end

function CameraHelper:MoveBack()
	self.TargetFunction = nil	--Clear TargetFunction
	self.CameraTarget = self.CameraTarget + (self.Dir * self.CamSpeed)
end

function CameraHelper:MoveLeft()
	self.TargetFunction = nil	--Clear TargetFunction
	self.CameraTarget = self.CameraTarget + (self.Right * self.CamSpeed)
end

function CameraHelper:MoveRight()
	self.TargetFunction = nil	--Clear TargetFunction
	self.CameraTarget = self.CameraTarget + (self.Right * -self.CamSpeed)
end

function CameraHelper:MoveUp(Multiplier)
	if (Multiplier == nil) then
		Multiplier = 1
	end
	--self.CameraHeight = ClampMinMax(self.CameraHeight + (self.CamSpeed * Multiplier), self.MinHeight, self.MaxHeight)
	self:Lift(self.CamSpeed * Multiplier)
end

function CameraHelper:MoveDown(Multiplier)
	if (Multiplier == nil) then
		Multiplier = 1
	end
	--self.CameraHeight = ClampMinMax(self.CameraHeight - (self.CamSpeed * Multiplier), self.MinHeight, self.MaxHeight)
	self:Lift(-(self.CamSpeed * Multiplier))
end

function CameraHelper:Lift(Value)
	self.CameraHeight = ClampMinMax(self.CameraHeight + Value, self.MinHeight, self.MaxHeight)
end

function CameraHelper:RotateLeft()
	--self.CameraRotation = Mod(self.CameraRotation + (self.CamSpeed / 10), 360)
	self:Rotate(self.CamSpeed / 10)
end

function CameraHelper:RotateRight()
	--self.CameraRotation = Mod(self.CameraRotation - (self.CamSpeed / 10), 360)
	self:Rotate(-(self.CamSpeed / 10))
end

function CameraHelper:Rotate(Value)
	if (self.MinRotation ~= nil and self.MaxRotation ~= nil) then
		--self.CameraRotation = ClampMinMax(self.CameraRotation + Value, self.MinRotation, self.MaxRotation)
		
		local NewRotation = self.CameraRotation + Value
		if (Value > 0) then
			local Dist = self.MaxRotation - NewRotation
			if (Dist < 25) then
				local Mult = ((Dist / 25))
				Value = Value * Mult
			end
		elseif (Value < 0) then
			local Dist = NewRotation - self.MinRotation
			if (Dist < 25) then
				local Mult = ((Dist / 25))
				Value = Value * Mult
			end
		end
		
		self.CameraRotation = self.CameraRotation + Value
	else
		self.CameraRotation = Mod(self.CameraRotation + Value, 360)
	end
end

function CameraHelper:Update()
	self.CamSpeed = 10
	if (KeyPressed(GetKeyCode("DCK_LSHIFT"))) then
		self.CamSpeed = self.CamSpeed * 3
	end
	
	self.Right = self.Camera:GetMatrix():GetRotation():Inverse():GetRight()
	self.Right.y = 0
	self.Right = self.Right:Normalize()
	
	self.Dir = self.Camera:GetMatrix():GetRotation():Inverse():GetDirection()
	self.Dir.y = 0
	self.Dir = self.Dir:Normalize()
	
	if (self.TargetFunction ~= nil) then
		self:TargetFunction()
	end
	
	if (self.InputEnabled) then
		--[[ if (KeyPressed(GetKeyCode("DCK_A"))) then
			self:MoveLeft()
		end
		
		if (KeyPressed(GetKeyCode("DCK_D"))) then
			self:MoveRight()
		end
		
		if (KeyPressed(GetKeyCode("DCK_W"))) then
			self:MoveForward()
		end
		
		if (KeyPressed(GetKeyCode("DCK_S"))) then
			self:MoveBack()
		end ]]
		
		if (KeyPressed(GetKeyCode("DCK_PAGEUP"))) then
			self:MoveUp()
		end
		
		if (KeyPressed(GetKeyCode("DCK_PAGEDOWN"))) then
			self:MoveDown()
		end
		
		if (KeyPressed(GetKeyCode("DCK_HOME"))) then
			self:RotateLeft()
		end
		
		if (KeyPressed(GetKeyCode("DCK_END"))) then
			self:RotateRight()
		end
	end
	
	local MinDistance = 85 --70
	local MaxDistance = 100 --85
	
	local CameraDistance = Lerp(MaxDistance, MinDistance, ClampMax(self.CameraHeight - self.MinHeight, 1000) / 1000)
	local rads = math.rad(self.CameraRotation)
	local posX = self.CameraTarget.x + math.cos(rads) * CameraDistance;
	local posZ = self.CameraTarget.z + math.sin(rads) * CameraDistance;
	--self.CameraPos = Vector(posX, self.CameraHeight, posZ)
	self.CameraPos = Vector(posX, self.CameraTarget.y + self.CameraHeight, posZ)
	
	--local WorldPosMatrix = MatrixIdentity:Translate(self.CameraTarget + Vector(0,200,0))
	--WorldPosMatrix = WorldPosMatrix:RotateLocal(VectorY, self.Rotation)
	--self.Mesh:Update(WorldPosMatrix)
	
	local CamDir = (self.CameraPos - self.CameraTarget):Normalize() * -1
	CamDir.x = CamDir.x * -1
	
	self.CamMatrix = MatrixIdentity:LookAt(Vector(0,0,0), CamDir, 0)
	
	self.CamMatrix = self.CamMatrix:Rotate(Vector(1,0,0), math.rad(180)):Inverse()
	self.CamMatrix = self.CamMatrix:Rotate(Vector(0,0,1), math.rad(180))
	
	self.CamMatrix = self.CamMatrix:Translate(self.CameraPos)
	
	local NewMatrix = self.Camera:GetMatrix():Lerp(self.CamMatrix, 1)
	--self.Camera:SetMatrix(self.CamMatrix)
	self.Camera:SetMatrix(NewMatrix)
end