CarManager = class(function(self)
	self.Car = nil
	self.SuspensionManager = SuspensionManager(ActiveMission)
	self.SelectedWheel = nil
end)

function CarManager:Destroy()
	if (self.Car ~= nil) then
		-- Disable suspension for the car
		self.SuspensionManager:Disable()
		
		-- Remove the car from the EntityScene
		GetEntityScene():DestroyEntity(self.Car.Car);	-- Dit gebeurt vanzelf wel
		
		-- Remove the car reference
		self.Car = nil
	end
end

-- ==========

function CarManager:ShowCar(Car)
	-- Zinloze check
	if (Car == nil) then
		print("Waarom stuur je een ShowCar(nil) door?")
		return
	end
	
	-- Check of we toevallig deze auto al geladen hadden
	if (self.Car == Car) then
		return
	end
	
	-- Gooi de huidige auto weg
	if (self.Car ~= nil) then
		self:Destroy()
	end
	
	-- Assign de nieuwe auto
	self.Car = Car
	self.CarLoaded = false
	
	-- Maak de nieuwe auto
	local CarBuilder = LuaCarBuilder(Car.Name, 0)
	
	self.CarBuilder = CarBuilder
	
	PhysicsThread:Spawn(function()
		PhysicsThread:Wait(10)
		CarBuilder:BuildAsync(function(Car)
			-- Check of we nog wel gewenst zijn
			if (self.CarBuilder ~= CarBuilder) then
				--self:Destroy()
				GetEntityScene():DestroyEntity(Car)
				return
			end
			
			self.CarLoaded = true
			self.Car.Car = Car
			GetEntityScene():AddEntity(self.Car.Car);
		
			self.Car.Car:SetMatrix(GetScriptMatrix("Car"))
		
			if (ActiveMission.CarRender ~= nil) then
				--ActiveMission.CarRender:LookAtTarget(self.Car)
				ActiveMission.CarRender:LookAt(GetScriptMatrix("Car"):GetTranslation() + Vector(0, 15, 0))
			end
		
			if (ActiveMission.CarRender ~= nil) then
				ActiveMission.CarRender.CamHelper:Update()
				ActiveMission.CarRender.CamHelper:Update()
			end
		
			-- Create a fake player for suspension purposes (could be replaced with an actual Player() object)
			local Player = {}
			Player.Car = self.Car.Car
		
			-- Set the player as the suspension target (this enabled "UseSingleTarget" in the SuspensionManager)
			self.SuspensionManager:SetPlayer(Player)
			self.SuspensionManager:Enable()
		end);
	end)
end

function CarManager:ChangeWheel(Wheel)
	GameThread:Spawn(function()
		if (Wheel ~= nil and Wheel.Name ~= nil) then
			if (self.Car ~= nil) then
				self.SelectedWheel = Wheel
				self.Car:ChangeWheels(Wheel.Model)
				
				self.Car:Update()
				
				--Check Performance
				local LowPerformance = false
				if (IsSlowDevice ~= nil and IsSlowDevice()) then
					LowPerformance = true
				end
				
				if (LowPerformance) then
					self.Car:Stop()
				end
			end
		end
	end)
end

function CarManager:SetCarColorConfig(ColorConfig)
	if (self.Car ~= nil and ColorConfig ~= nil) then
		for Key, Value in pairs(ColorConfig) do
			if (Key == "DiffuseColor" or Key == "SpecularColor") then
				self.Car:GetMesh():SetVar("Car" .. Key, Value / 511);
			else
				self.Car:GetMesh():SetVar("Car" .. Key, Value);
			end
		end
	end
end





function CarManager:ResetCarSelection()
	--local ClassSelection = ActiveGameMode:GetClassSelection()
	--self:ShowCar(ClassSelection.Car)
	self:ShowCar(ActiveGameMode:GetSelectedCar())
end

function CarManager:SaveCarSelection(Car)
	for i, GameMode in ipairs(GameModes) do
		GameMode:SetSelectedCar(Car)
	end
	--WriteGameSettings()
end

function CarManager:GetCarSelection()
	if (self.Car ~= nil) then
		return self.Car
	--elseif (ActiveGameMode:GetSelectedCar() ~= nil) then
		--return ActiveGameMode:GetSelectedCar()
	else
		print("ERROR: There is no Car selection saved!!!")
	end
end

function CarManager:ResetWheelSelection()
	--local ClassSelection = ActiveGameMode:GetClassSelection()
	--self:ShowCar(ClassSelection.Car)
	--self:ShowCar(ActiveGameMode:GetSelectedCar())
	
	self:ChangeWheel(ActiveGameMode:GetSelectedWheel(ActiveGameMode:GetSelectedCar()))
end

function CarManager:SaveWheelSelection()
	ActiveGameMode:SetSelectedWheel(self.SelectedWheel)
	WriteGameSettings()
end

function CarManager:GetWheelSelection()
	return self.SelectedWheel
end







function CarManager:ChangeRandomWheelConfig()
	GameThread:Spawn(function()
		WheelConfig_GarageShow1 = {}
		WheelConfig_GarageShow1.Name = "GarageShow1"
		WheelConfig_GarageShow1.Limits = {}
		WheelConfig_GarageShow1.AngularFriction = Vector(0,0,0)
		WheelConfig_GarageShow1.MotorForce = {1000000 * 0.6, 1000000 * 0.6}
		WheelConfig_GarageShow1.InertiaScale = VectorOne
		
		local rand1 = math.random(1, 9)
		local rand2 = math.random(1, 9)
		local rand3 = math.random(1, 9)
		local rand4 = math.random(1, 9)
		
		WheelConfig_GarageShow1.Limits[1] = {Min(rand1, rand2), Max(rand1, rand2)}
		WheelConfig_GarageShow1.Limits[2] = {Min(rand3, rand4), Max(rand3, rand4)}
		WheelConfig_GarageShow1.Limits[3] = {Min(rand3, rand4), Max(rand3, rand4)}
		
		self.Car:ChangeWheelConfig(WheelConfig_GarageShow1)
	end)
end