CarRender = class(function(self, Width, Height)
	self.Enabled = true
	
	--Camera
	local CameraIndex = 0
	self.Camera = EntityCamera(CameraIndex)
	
	--Renderer
	local TextureFormat = 0
	self.Width = Width
	self.Height = Height
	local MenuOverlay = "New_Menu.dcMenu"
	self.WorldRender = LuaWorldRender(TextureFormat, self.Width, self.Height, MenuOverlay, self.Camera)
	
	self.MenuImageButton = nil
	self.ImageReady = false
	
	RenderGame(function(...)
		if (self.WorldRender ~= nil) then
			if (ActiveMenu ~= nil and (ActiveMenu:is_a(Garage) or ActiveMenu:is_a(Garage_Viewer))) then
				self.WorldRender:Render()
			end
			
			if (self.MenuImageButton ~= nil and self.MenuImageButton.SetImage ~= nil) then
				if (not self.ImageReady) then
					self.MenuImageButton:SetImage(self.WorldRender:GetTexture(), GetPlatform() == "Android" or GetPlatform() == "IOS" or WIN32_OGL)
					self.ImageReady = true
				end
			end
		end
	end)
	
	--Check Performance
	local LowPerformance = false
	--if (IsSlowDevice ~= nil and IsSlowDevice()) then
		--LowPerformance = true
	--end
	
	--CameraHelper
	self.CamHelper = CameraHelper(CameraIndex)
	if (not LowPerformance) then
		self.CamHelper.Thread = GameThread:Spawn(function()
			while (true) do
				if (self.Enabled) then
					self.CamHelper:Update()
				end
				GameThread:Wait(1 / 120)
			end
		end)
	end
	
	local Matrix = MatrixIdentity:Translate(Vector(0, 10, 0))
	local CameraTarget = Matrix:GetTranslation()
	self.CamHelper.CameraTarget = CameraTarget
	
	self.CamHelper.MinHeight = -5
	self.CamHelper.MaxHeight = 50
	self.CamHelper.MinRotation = 230 + 65
	self.CamHelper.MaxRotation = 340 + 65
	self.CamHelper.CameraHeight = 32 + 8
	self.CamHelper.CameraRotation = 240 + 65
	
	--[[ self.CamHelper.TargetFunction = function()
		local Matrix = MatrixIdentity:Translate(VectorZero) --self.Player.WorldCharacter:GetMatrix()
		local CameraTarget = Matrix:GetTranslation() + VectorZero
		self.CamHelper.CameraTarget = CameraTarget
	end ]]
	
	--Rotation func
	self.RotationSpeed = 0
	self.LiftSpeed = 0
	
	--self.CarRotationThread = PhysicsThread:Spawn(function()
	if (not LowPerformance) then
		self.CarRotationThread = GameThread:Spawn(function()
			while (true) do
				self.RotationSpeed = Lerp(self.RotationSpeed, 0, 0.1)
				self.LiftSpeed = Lerp(self.LiftSpeed, 0, 0.1)
				if (math.abs(self.RotationSpeed) > 0.1) then
					if (ActiveMission.CarRender ~= nil) then
						ActiveMission.CarRender:RotateCamera(self.RotationSpeed)
					end
				end
				
				if (math.abs(self.LiftSpeed) > 0.1) then
					if (ActiveMission.CarRender ~= nil) then
						ActiveMission.CarRender:LiftCamera(self.LiftSpeed)
					end
				end
				GameThread:Wait(1 / 120)
			end
		end)
	end
end)

function CarRender:Enable()
	if (not self.Enabled) then
		self.Enabled = true
	end
end

function CarRender:Disable()
	if (self.Enabled) then
		self.Enabled = false
	end
end

function CarRender:Destroy()
	--SetGameRenderCallback(function(...)end)
	SetGameRenderCallback(nil)
	
	--PhysicsThread:Stop(self.CarRotationThread)
	if (self.CarRotationThread ~= nil) then
		GameThread:Stop(self.CarRotationThread)
		self.CarRotationThread = nil
	end
	
	if (self.CamHelper ~= nil) then
		self.CamHelper:Destroy()
	end
	
	self.Camera = nil
	
	if (self.WorldRender ~= nil) then
		self.WorldRender:Destroy()
		self.WorldRender = nil
	end
	
	self.MenuImageButton = nil
	self.ImageReady = false
end

function CarRender:SetMenuImage(Image)
	self.MenuImageButton = Image
end

function CarRender:LookAtTarget(Target)
	self.Target = Target
	
	if (self.Target.GetMatrix ~= nil) then
		if (self.CamHelper ~= nil) then
			self.CamHelper.TargetFunction = function()
				local Matrix = self.Target:GetMatrix()
				local CameraTarget = Matrix:GetTranslation()
				self.CamHelper.CameraTarget = CameraTarget
			end
		end
	else
		print("^1No GetMatrix() found on camera-target")
	end
end

function CarRender:LookAt(Pos)
	self.Target = nil
	
	if (Pos ~= nil) then
		if (self.CamHelper ~= nil) then
			self.CamHelper.TargetFunction = function()
				local CameraTarget = Pos
				self.CamHelper.CameraTarget = CameraTarget
			end
		end
	else
		print("^1No pos")
	end
end

function CarRender:RotateCamera(Value)
	if (self.CamHelper ~= nil) then
		self.CamHelper:Rotate(Value)
	end
end

function CarRender:LiftCamera(Value)
	if (self.CamHelper ~= nil) then
		self.CamHelper:Lift(Value)
	end
end