-- Game modes
GameMode = class(function(self, Name)
	self.Name = Name;
	self.Image = nil
	self.Title = "menu_gametype_title_" .. Name
	self.Description = "menu_gametype_description_" .. Name
	self.LoadingText = "menu_gametype_loading_" .. Name
	
	--CarClasses
	self.CarClasses = {}
	self.EnabledClasses = {}
	self.DefaultEnabledClasses = nil
	self.RestrictedCars = false
	
	--CarClassSelection
	self.CarClassSelection = {}
	self.SelectedClass = nil
	
	--Chapters
	self.Chapters = {}
	self.ActiveChapter = nil
	
	--Progress data
	self.PlayCount = 0
	self.Locked = false
	
	--Config options
	--TODO: netjes maken
	self.AIDifficultyValues = {0.75, 1.0, 1.25}
	
	self.BoostMultiplier = {"0x", "1x", "2x", "3x"}
	self.ActiveBoostMultiplier = 1;
	
	self.PhysicsSpeedValues = {"0.84", "0.7", "0.6"}
	self.PhysicsSpeed = {"value_Chaos", "value_Compromise", "value_Classic" }
	self.ActivePhysicsSpeed = 0
	
	self.BoostHealingValues = {true, false}
	self.BoostHealing = {"value_On", "value_Off" }
	self.ActiveBoostHealing = 1
	
	-- self.DamageMultiplierValues = {0.0625, 0.125, 0.1875, 0.25, 0.375, 0.5 }
	-- self.DamageMultiplier = {"0.25x", "0.50x", "0.75x", "1.00x", "1.50x", "2.00x" }
	-- self.ActiveDamageMultiplier = 3
	
	self.DamageMult = 2.5
	
	self.TimeLimit =  {"last_man", "5_minutes", "6_minutes", "7_minutes", "8_minutes", "9_minutes", "10_minutes", "11_minutes", "12_minutes", "13_minutes", "14_minutes", "15_Minutes" }
	self.ActiveTimeLimit = 0
end)

function GameMode:Read(File, Version)
	--Progress data
	self.PlayCount = File:ReadInt()
	
	--Config options
	--self.ActiveAICount = File:ReadInt()
	--self.ActiveLapCount = File:ReadInt()
	--self.ActiveAIDifficulty = File:ReadInt()
	--self.ActivePhysicsSpeed = File:ReadInt()
	--self.ActiveBoostHealing = File:ReadInt()
	--self.ActiveDamageMultiplier = File:ReadInt()
	--self.ActiveTimeLimit = File:ReadInt()
	
	--CarClasses
	self.CarClasses = {}
	self.EnabledClasses = {}
	
	--CarClassSelection
	self.CarClassSelection = {}
	self.SelectedClass = nil
	
	local CarClassCount = File:ReadInt()
	for i = 1, CarClassCount do
		local Class = CarClasses[File:ReadInt() + 1];
		--print(Class.Name)
		self:AddCarClass(Class)
		
		--Car
		self.CarClassSelection[i].Car = Class.Cars[File:ReadInt() + 1]
		
		--Skin
		--[[ local CarSkinCount = File:ReadInt();
		--print(CarSkinCount)
		for j = 1, CarSkinCount do
			local Name = File:ReadString()
			--print(Name)
			self.CarClassSelection[i].Skin[Name] = File:ReadInt()
		end ]]
		
		--Tuning
		-- self.CarClassSelection[i].Tuning = File:ReadInt()
		
		--Wheel
		local CarWheelCount = File:ReadInt();
		--print(CarWheelCount)
		for j = 1, CarWheelCount do
			local Name = File:ReadString()
			--print(Name)
			self.CarClassSelection[i].Wheel[Name] = File:ReadInt()
		end
		
		--ColorConfig
		--[[ local CarColorConfigCount = File:ReadInt();
		--print(CarColorConfigCount)
		for j = 1, CarColorConfigCount do
			local Name = File:ReadString()
			--print(Name)
			--self.CarClassSelection[i].ColorConfig[Name] = File:ReadInt()
		end ]]
	end
	
	--CarClassSelection
	self.SelectedClass = self.CarClasses[File:ReadInt() + 1]
	
	--Chapters
	--local ChapterCount = File:ReadInt()
	for i, Chapter in ipairs(self.Chapters) do
		Chapter:Read(File, Version)
	end
end

function GameMode:Write(File)
	--Progress data
	File:WriteInt(self.PlayCount)
	
	--Config options
	--File:WriteInt(self.ActiveAICount)
	--File:WriteInt(self.ActiveLapCount)
	--File:WriteInt(self.ActiveAIDifficulty)
	--File:WriteInt(self.ActivePhysicsSpeed)
	--File:WriteInt(self.ActiveBoostHealing)
	--File:WriteInt(self.ActiveDamageMultiplier)
	--File:WriteInt(self.ActiveTimeLimit)
	
	--CarClasses
	File:WriteInt(#self.CarClasses)
	for i, Class in ipairs(self.CarClasses) do
		local ClassIndex = IndexInTable(CarClasses, Class)
		File:WriteInt(ClassIndex)
		
		--Car
		local CarIndex = IndexInTable(Class.Cars, self.CarClassSelection[i].Car);
		File:WriteInt(CarIndex)
		
		--Skin
		--print("Count = " .. TableLength(self.CarClassSelection[i].Skin))
		--[[ File:WriteInt(TableLength(self.CarClassSelection[i].Skin))
		for Key in pairs(self.CarClassSelection[i].Skin) do
			--print(Key)
			File:WriteString(Key)
			File:WriteInt(self.CarClassSelection[i].Skin[Key])
		end ]]
		
		--Tuning
		-- File:WriteInt(self.CarClassSelection[i].Tuning)
		
		--Wheel
		--print("Count = " .. TableLength(self.CarClassSelection[i].Wheel))
		File:WriteInt(TableLength(self.CarClassSelection[i].Wheel))
		for Key in pairs(self.CarClassSelection[i].Wheel) do
			--print(Key)
			File:WriteString(Key)
			File:WriteInt(self.CarClassSelection[i].Wheel[Key])
		end
		
		--ColorConfig
		--print("Count = " .. TableLength(self.CarClassSelection[i].ColorConfig))
		--[[ File:WriteInt(TableLength(self.CarClassSelection[i].ColorConfig))
		for Key in pairs(self.CarClassSelection[i].ColorConfig) do
			--print(Key)
			File:WriteString(Key)
			--File:WriteInt(self.CarClassSelection[i].ColorConfig[Key])
		end ]]
	end
	
	File:WriteInt(IndexInTable(self.CarClasses, self.SelectedClass))
	
	
	--Chapters
	--File:WriteInt(#self.Chapters)
	for i, Chapter in ipairs(self.Chapters) do
		Chapter:Write(File)
	end
end

function GameMode:AddCarClass(CarClass)
	if (CarClass == nil) then
		print("CarClass == nil")
		return
	end
	
	if (TableContains(self.CarClasses, CarClass)) then
		return
	end
	
	if (not GameMode.MP and self.Name ~= "MPFake") then
		--table.insert(UnlockPopups, string.format(Translate("Class_unlock_4all"), Translate(CarClass.Title), Translate(self.Title)))
		if (false) then	--ff tijdelijk uitzetten
			local Popup = {UnlockMenu, CarClass}
			table.insert(UnlockPopups, Popup)
		end
	end
	
	table.insert(self.CarClasses, CarClass)
	
	-- Select car 0 with default tuning for the class
	local Selection = {}
	Selection.Car = CarClass:GetCar(0)
	if (Selection.Car == nil) then
		Selection.Car = CarClass.UnrestrictedCars[1]
	end
	Selection.Skin = {}
	Selection.Tuning = 0	-- Dit is een index, omdat tuning gedeeld wordt binnen een class
	Selection.Wheel = {}
	Selection.ColorConfig = {}
	
	table.insert(self.CarClassSelection, Selection)
	
	-- Make sure we select a class
	if (self.SelectedClass == nil) then
		self.SelectedClass = CarClass;
	end
	
	if self.DefaultEnabledClasses ~= nil then
		for i, EnabledCarClass in ipairs(self.DefaultEnabledClasses) do
			if CarClass.Name == EnabledCarClass.Name then
				table.insert(self.EnabledClasses, CarClass)
			end
		end
	else
		table.insert(self.EnabledClasses, CarClass)
	end
end

function GameMode:PrintCarClasses()
	for i, CarClass in ipairs(self.CarClasses) do
		print(CarClass.Name)
		print(CarClass.Title)
	end
end

function GameMode:PrintCars()
	for i, CarClass in ipairs(self.CarClasses) do
		print(CarClass.Name)
		--print(Translate(CarClass.Title))
		print("******")
	
		for j, Car in ipairs(CarClass.Cars) do
			print(Car.Name)
			--print(Translate(Car.Title))
		end
		
		print("******")
	end
end

--Class
function GameMode:GetClassSelection()
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)	-- Lelijke oplossing
	
	return self.CarClassSelection[Index + 1]
end

--Car
function GameMode:GetSelectedCar()
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)	-- Lelijke oplossing
	return self.CarClassSelection[Index + 1].Car
end

function GameMode:SetSelectedCar(Car)
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)	-- Lelijke oplossing
	
	if (self.CarClassSelection[Index + 1] ~= nil) then
		self.CarClassSelection[Index + 1].Car = Car
	end
end

--Skin
function GameMode:GetSelectedSkin()
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)	-- Lelijke oplossing
	
	local Car = self.CarClassSelection[Index + 1].Car
	
	if (self.CarClassSelection[Index + 1].Skin[Car.Name] == nil) then
		self.CarClassSelection[Index + 1].Skin[Car.Name] = 0
	end
	
	return self.CarClassSelection[Index + 1].Skin[Car.Name]
end

--Tuning
function GameMode:GetSelectedTuning()
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)	-- Lelijke oplossing
	
	local Car = self.CarClassSelection[Index + 1].Car
	
	return Car.Tunings[Mod(self.CarClassSelection[Index + 1].Tuning, #Car.Tunings) + 1]
end

--Wheel
function GameMode:GetSelectedWheel(Car)
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)
	
	if (Car == nil) then
		Car = self.CarClassSelection[Index + 1].Car
	end
	
	if (self.CarClassSelection[Index + 1].Wheel[Car.Name] == nil) then
		self.CarClassSelection[Index + 1].Wheel[Car.Name] = 0
		
		if (Car.WheelConfig ~= nil and Car.WheelConfig.Model ~= nil) then
			for i, Wheel in ipairs(self.SelectedClass.Wheels) do
				if (Wheel.Model == Car.WheelConfig.Model) then
					self.CarClassSelection[Index + 1].Wheel[Car.Name] = i - 1
				end
			end
		end
	end
	
	return self.SelectedClass.Wheels[Mod(self.CarClassSelection[Index + 1].Wheel[Car.Name], #self.SelectedClass.Wheels) + 1]
end

function GameMode:SetSelectedWheel(Wheel, Car)
	
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)
	
	if (Car == nil) then
		Car = self.CarClassSelection[Index + 1].Car
	end
	
	local WheelIndex = IndexInTable(self.SelectedClass.Wheels, Wheel)
	if (self.CarClassSelection[Index + 1].Wheel[Car.Name] ~= nil) then
		self.CarClassSelection[Index + 1].Wheel[Car.Name] = WheelIndex
	end
end

--ColorConfig
function GameMode:GetSelectedColorConfig(Car)
	local Index = IndexInTable(self.CarClasses, self.SelectedClass)
	
	if (Car == nil) then
		Car = self.CarClassSelection[Index + 1].Car
	end
	
	if (self.CarClassSelection[Index + 1].ColorConfig[Car.Name] == nil) then
		self.CarClassSelection[Index + 1].ColorConfig[Car.Name] = deepcopy(Car.ColorConfig)
	end
	
	return self.CarClassSelection[Index + 1].ColorConfig[Car.Name]
end

--function GameMode:SetSelectedColorConfig(ColorConfig, Car)
	
	--local Index = IndexInTable(self.CarClasses, self.SelectedClass)
	
	--if (Car == nil) then
		--Car = self.CarClassSelection[Index + 1].Car
	--end
	
	--local WheelIndex = IndexInTable(self.SelectedClass.Wheels, Wheel)
	--if (self.CarClassSelection[Index + 1].Wheel[Car.Name] ~= nil) then
		--self.CarClassSelection[Index + 1].Wheel[Car.Name] = WheelIndex
	--end
--end

function GameMode:AddChapter(Chapter)
	self.OrigCarClasses = {}
	
	for i, CarClass in ipairs(self.CarClasses) do
		table.insert(self.OrigCarClasses, CarClass)
	end
	
	table.insert(self.Chapters, Chapter)
	if (self.ActiveChapter == nil) then
		self.ActiveChapter = Chapter
	end
	
	return Chapter
end

function GameMode:GetChapter(Index)
	return self.Chapters[Index + 1]
end

function GameMode:FindChapter(Name)
	for i, Chapter in ipairs(self.Chapters) do
		if (Chapter.Name == Name) then
			return Chapter;
		end
	end
	
	return self.Chapters[1]
end

function GameMode:LoadTextures()
	self.Image = texture("GameMode\\Mode_" .. self.Name .. ".DCT")
	
	for i, Chapter in ipairs(self.Chapters) do
		Chapter:LoadTextures()
	end
end

function GameMode:UnloadTextures()
	self.Image = nil

	for i, Chapter in ipairs(self.Chapters) do
		Chapter:UnloadTextures()
	end
end

function GameMode:UnlockRace(Name)
	local Success = false
	for i, Chapter in ipairs(self.Chapters) do
		Success = Success or Chapter:UnlockRace(self, Name)
	end
	
	if (not Success) then
		print(self.Name .. ": trying to unlock non-existing race " .. Name)
	end
end

function GameMode:Win(Place, Score, TestOnly)
	print(TestOnly)
	if (not self.ActiveChapter.ActiveRace:Won(Place, Score)) then
		if (not TestOnly) then
			print("IncreaseLose")
			self.ActiveChapter.ActiveRace.LoseCount = Inc(self.ActiveChapter.ActiveRace.LoseCount)
		end
		
		if Place < self.ActiveChapter.ActiveRace.HighestPos then
			self.ActiveChapter.ActiveRace.HighestPos = Place
		end
		if Score > self.ActiveChapter.ActiveRace.HighestScore then
			self.ActiveChapter.ActiveRace.HighestScore = Score
		end
		
		if (not TestOnly) then
			--Steam_IncStat('races_lost', 1)
		end
		
		if (self.ActiveChapter.ActiveRace.Lose ~= nil) then
			self.ActiveChapter.ActiveRace:Lose()
		end
	
		return false
	end

	if (not TestOnly) then
		print("IncreaseWin")
		self.ActiveChapter.ActiveRace.WinCount = Inc(self.ActiveChapter.ActiveRace.WinCount)
	end
	
	if Place < self.ActiveChapter.ActiveRace.HighestPos then
		self.ActiveChapter.ActiveRace.HighestPos = Place
	end
	if Score > self.ActiveChapter.ActiveRace.HighestScore then
		self.ActiveChapter.ActiveRace.HighestScore = Score
	end
	
	if (not TestOnly) then
		--Steam_IncStat('races_won', 1)
	end
	
	if (self.ActiveChapter.ActiveRace.Unlock ~= nil) then
		if (not Demo) then
			self.ActiveChapter.ActiveRace:Unlock()
		end
	else
		print("Race won: No unlock defined!")
	end
	
	for i, Chapter in ipairs(self.Chapters) do
		if (not Chapter:Won()) then
			WriteGameSettings()
			return true
		end
	end
	
	if (self.Won ~= nil) then
		self:Won()
	else
		print("GameMode won: No unlock defined!")
	end
	
	WriteGameSettings()
	
	return true
end

function GameMode:GetLapCount()
	return self.LapCount[self.ActiveLapCount + 1];
end

function GameMode:GetUniqueWinCount()
	local Count = 0
	
	for i, Chapter in ipairs(self.Chapters) do
		Count = Count + Chapter:GetUniqueWinCount()
	end
	
	return Count
end

function GameMode:GetWinCount()
	local Count = 0
	
	for i, Chapter in ipairs(self.Chapters) do
		Count = Count + Chapter:GetWinCount()
	end
	
	return Count
end

function GameMode:GetLoseCount()
	local Count = 0
	
	for i, Chapter in ipairs(self.Chapters) do
		Count = Count + Chapter:GetLoseCount()
	end
	
	return Count
end	

Chapter = class(function(self, Name)
	self.Name = Name;
	self.Title = Name .. "_Chapter_Title";
	self.Description = Name .. "_Chapter_Description";
	
	self.Corners = nil
	self.Bumpiness = nil
	self.Speed = nil
	self.TrackWidth = nil
	
	self.Races = {}
end)

function Chapter:AddRace(Race)
	table.insert(self.Races, Race)
	
	if (self.ActiveRace == nil) then
		self.ActiveRace = Race
	end
	
	return Race
end

function Chapter:GetRace(Index)
	return self.Races[Index + 1]
end

function Chapter:FindRace(Name)
	for i, Race in ipairs(self.Races) do
		if (Race.Name == Name) then
			return Race;
		end
	end
	
	return self.Races[1]
end

function Chapter:LoadTextures()
	for i, Race in ipairs(self.Races) do
		Race:LoadTexture()
	end
end

function Chapter:UnloadTextures()
	for i, Race in ipairs(self.Races) do
		Race:UnloadTexture();
	end
end

function Chapter:GetTexture(Index)
	Index = Mod(Index, #self.Races)
	
	return self.Races[Index + 1].Texture;
end

function Chapter:GetProgress()
	local Progress = 0
	for i, Race in ipairs(self.Races) do
		if (Race.WinCount > 0) then
			Progress = Inc(Progress)
		end
	end
	
	return Progress / #self.Races
end

function Chapter:Available(Name)
	for i, Race in ipairs(self.Races) do
		if (Race.Available) then
			return true
		end
	end
	
	return false
end

function Chapter:UnlockRace(GameMode, Name)
	local Success = false
	
	for i, Race in ipairs(self.Races) do
		if (Race.Name == Name) then
			if (not Race.Available) then
				--table.insert(UnlockPopups, string.format(Translate(GameMode.RaceUnlockText), Translate(Race.Title)))
				local Popup = {UnlockMenu, Race}
				table.insert(UnlockPopups, Popup)
			end
			
			Race.Available = true
			
			Success = true
		end
	end
	
	return Success
end

function Chapter:Won()
	for i, Race in ipairs(self.Races) do
		if (not (Race.WinCount > 0)) then
			return false
		end
	end
	
	return true
end

function Chapter:GetUniqueWinCount()
	local Count = 0
	
	for i, Race in ipairs(self.Races) do
		if (Race.WinCount > 0) then
			Count = Count + 1
		end
	end
	
	return Count
end	

function Chapter:GetWinCount()
	local Count = 0
	
	for i, Race in ipairs(self.Races) do
		 Count = Count + Race.WinCount
	end
	
	return Count
end	

function Chapter:GetLoseCount()
	local Count = 0
	
	for i, Race in ipairs(self.Races) do
		Count = Count + Race.LoseCount
	end
	
	return Count
end	

function Chapter:Read(File, Version)
	local RaceCount = File:ReadInt()
	for i = 1, RaceCount do
		self.Races[i]:Read(File, Version)
	end
end

function Chapter:Write(File)
	File:WriteInt(#self.Races)
	for i, Race in ipairs(self.Races) do
		Race:Write(File)
	end
end

TrackScore = class(function(self, LapCount)
	self.LapCount = LapCount;
	self.Scores = {}
end)

Mission = class(function(self, Name)
	self.Name = Name
	self.Title = Translate(Name .. "_Title")
	self.Description = Translate(Name .. "_Description")
	self.Welcome = Translate(Name .. "_Welcome")
	
	self.Country = "None"
	
	self.Texture = nil
	
	self.LapScores = {}
	self.TrackScores = {}
	self.HighScores = {}
	
	self.WinCount = 0
	self.LoseCount = 0;
	self.HighestPos = 99;
	self.HighestScore = 0;
	self.MaximumScore = 300000
	self.TotalPlaytime = 0
	
	self.AISettings = {}
	self.AISettings.MinAISpeed = 50
	self.AISettings.MaxAISpeed = 200
	self.AISettings.MaxAcceleration = 0.1
	
	self.InstructionText = {}
	table.insert(self.InstructionText, "")
	
	--Shop data
	self.Bought = false
	self.ShopPriceCoins = 10
	self.ShopPriceGoloCoins = 0
	
	self.Available = false
	self.DeliveryTime = 0
	self.ShowPopup = true
	
	self.DiscountOffered = false
end)

function Mission:GetTrackScores(LapCount)
	--return het juste item
	for i, TrackScore in ipairs(self.TrackScores) do
		if TrackScore.LapCount == LapCount then
			return TrackScore
		end
	end
	
	print("No Trackscore list for " .. LapCount .. " in race " .. self.Name)
	return self.TrackScores[1]
end

function Mission:GetHighScores(ID)
	return self.HighScores[ID]
end

function Mission:GetCurrentTrackScore(LapCount)
	local TrackScores = self:GetTrackScores(LapCount)

	if (TrackScores ~= nil and TrackScores.Scores ~= nil) then
		local LapScore = TrackScores.Scores[1]
		
		if (LapScore ~= nil) then
			return LapScore.Score
		end
	end
	
	return nil
end

function Mission:AddLapScore(ID, Name, Car, Score, VisualScore)
	local Entry = HighScore(Name, ID)
	Entry.Car = Car;
	Entry.Score = Score;
	Entry.VisualScore = VisualScore;

	table.insert(self.LapScores, Entry)
	
	local function ScoreSort(x, y)
		return x.Score > y.Score
	end
	
	table.sort(self.LapScores, ScoreSort)

	return self.LapScores[#self.LapScores] == Entry;
end

function Mission:AddTrackScore(ID, Name, Car, Score, VisualScore, LapCount)
	local Entry = HighScore(Name, ID)
	Entry.Car = Car;
	Entry.Score = Score;
	Entry.VisualScore = VisualScore;

	local TrackScore = self:GetTrackScores(LapCount)
	table.insert(TrackScore.Scores, Entry)
	
	local function ScoreSort(x, y)
		return x.Score < y.Score
	end
	
	table.sort(TrackScore.Scores, ScoreSort)	
	
	return TrackScore.Scores[1] == Entry;
end

function Mission:AddHighScore(ID, Name, Car, Score, StarScore)
	local Entry = HighScore(Name, ID)
	Entry.Car = Car
	Entry.Score = Score
	Entry.StarScore = StarScore
	
	table.insert(self.HighScores, Entry)
	
	local function ScoreSort(x, y)
		return x.Score > y.Score
	end
	
	for i, scores in ipairs(self.HighScores) do -- ERIK: ENG DIT :| Kijk even of dit "goed" gaat (PRINTEN!) en anders Maik of Vincent vragen.
		table.sort(self.HighScores, function(a,b) return a.Score > b.Score end)
	end
	
	for i = 11, #self.HighScores do
		table.remove(self.HighScores, i)
	end
	
	return ID
end

function Mission:LoadTexture()
	--return self.Texture
end

function Mission:UnloadTexture()
	--self.Texture = nil	-- This actually causes the garbage collector to clean it later
end

function Mission:Unlock()
	self.Available = true
end

function Mission:Read(File, Version)
	self.Bought = File:ReadBoolean()
	self.Available = File:ReadBoolean()
	self.DeliveryTime = File:ReadInt()
	
	self.DiscountOffered = File:ReadBoolean()
	
	self.WinCount = File:ReadInt()
	self.LoseCount = File:ReadInt()
	self.HighestPos = File:ReadInt()
	self.HighestScore = File:ReadFloat()
	self.TotalPlaytime = File:ReadFloat()
	
	local LapScoresCount = File:ReadInt()
	for i = 1,LapScoresCount do
		local Name = File:ReadString()
		local ID = File:ReadInt()
		local Car = File:ReadString()
		local Score = File:ReadFloat()
		local VisualScore = File:ReadString()
		self:AddLapScore(ID, Name, Car, Score, VisualScore)
	end
	
	local TrackScoresCount = File:ReadInt()
	for i = 1,TrackScoresCount do
		local LapCount = File:ReadString()
		local ScoresCount = File:ReadInt()
		
		for j = 1,ScoresCount do
			local Name = File:ReadString()
			local ID = File:ReadInt()
			local Car = File:ReadString()
			local Score = File:ReadInt()
			local VisualScore = File:ReadString()
			self:AddTrackScore(ID, Name, Car, Score, VisualScore, LapCount)
		end
	end
	
	local HighScoresCount = File:ReadInt()
	for i = 1, HighScoresCount do
		local ID = File:ReadInt()
		local Name = File:ReadString()
		local Car = File:ReadString()
		local Score = File:ReadFloat()
		local StarScore = File:ReadInt()
		self:AddHighScore(ID, Name, Car, Score, StarScore)
	end
end

function Mission:Write(File)
	File:WriteBoolean(self.Bought)
	File:WriteBoolean(self.Available)
	File:WriteInt(self.DeliveryTime)
	
	File:WriteBoolean(self.DiscountOffered)
	
	File:WriteInt(self.WinCount)
	File:WriteInt(self.LoseCount)
	File:WriteInt(self.HighestPos)
	File:WriteFloat(self.HighestScore)
	File:WriteFloat(self.TotalPlaytime)
	
	File:WriteInt(#self.LapScores)
	for i, LapScore in ipairs(self.LapScores) do
		--File:WriteString(LapScore.Name)
		File:WriteString('')
		File:WriteInt(LapScore.ID)
		File:WriteString(LapScore.Car)
		File:WriteFloat(LapScore.Score)
		File:WriteString(LapScore.VisualScore)
	end
	
	File:WriteInt(#self.TrackScores)
	for i, TrackScore in ipairs(self.TrackScores) do
		File:WriteString(TrackScore.LapCount)
		File:WriteInt(#TrackScore.Scores)
		
		for j, Score in ipairs(TrackScore.Scores) do
			--File:WriteString(Score.Name)
			File:WriteString('')
			File:WriteInt(Score.ID)
			File:WriteString(Score.Car)
			File:WriteInt(Score.Score)
			File:WriteString(Score.VisualScore)
		end
	end
	
	File:WriteInt(#self.HighScores)
	for i, HighScore in ipairs(self.HighScores) do
		File:WriteInt(HighScore.ID)
		File:WriteString(HighScore.Name)
		File:WriteString(HighScore.Car)
		File:WriteFloat(HighScore.Score)
		File:WriteInt(HighScore.StarScore)
	end
end

function Mission:Won(Place, Score)
	return Place == 0
end

HighScore = class(function(self, Name, ID)
	-- Voor het geval de speler een steamID heeft, dan vragen we zijn meest actuele naam op}
	self.ID = ID
	
	if (ID ~= nil and steam_getname ~= nil) then
		self.Name = steam_getname(ID)
	else
		self.Name = Name
	end
end)

function HighScore:GetName()
	if (self.ID ~= nil) then
		return steam_getname(ID)
	else
		return self.Name
	end
end