SingleRace = GameMode("SingleRace")

function SingleRace:HUD(...)
	return HUD_Race(...)
end

SingleRace.Configuration = {}

SingleRace.Configuration[1] = {}
SingleRace.Configuration[1].Name = "Difficulty"
SingleRace.Configuration[1].Options = {"Novice", "Easy", "Normal", "Hard", "Expert"}
SingleRace.Configuration[1].Selection = 2

SingleRace.Configuration[2] = {}
SingleRace.Configuration[2].Name = "Lapcount"
SingleRace.Configuration[2].Options = {"1", "2", "4", "6", "8"}
SingleRace.Configuration[2].Selection = 1

SingleRace.Configuration[3] = {}
SingleRace.Configuration[3].Name = "Opponents"
SingleRace.Configuration[3].Options = {"1", "3", "5", "7", "9"}
SingleRace.Configuration[3].Selection = 1

SingleRace.Configuration[4] = {}
SingleRace.Configuration[4].Name = "FallDamage"
SingleRace.Configuration[4].Options = {"No", "Yes"}
SingleRace.Configuration[4].Selection = 1

SingleRace:AddCarClass(MonsterTruckCars)

SingleRace.RandomizeAIData = false

--Add TrackScores
local TrackScores = {}
for i = 1, #SingleRace.Configuration[2].Options do
	local LapCount = SingleRace.Configuration[2].Options[i]
	table.insert(TrackScores, TrackScore(tostring(LapCount)))
end

SingleRace.RaceUnlockText = "NEW_STUNT_UNLOCKED"

SingleRace.PhysicsSpeedValues = {"0.84"}
SingleRace.ActivePhysicsSpeed = 0

function SingleRace:PrepareScript(RaceData)
	if (self.RandomizeAIData) then
		RaceData = self:GetNewAIData(RaceData)
		self.RandomizeAIData = false
	end

	ActiveMission = RaceMission(RaceData)
end

SingleRace.Won = function(self)
	--
end



-- Add chapters
local Chapter1 = SingleRace:AddChapter(Chapter("Race_Chapter1")) 

-- Add chapter 1 races
local PrefMission = nil
if (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME8) then
	PrefMission = Mission("MJ_THEME8_RACE")
elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME7) then
	PrefMission = Mission("MJ_THEME7_RACE")
elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME6) then
	PrefMission = Mission("MJ_THEME6_RACE")
elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME5) then
	PrefMission = Mission("MJ_THEME5_RACE")
elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME4) then
	PrefMission = Mission("MJ_Theme4_Race")
elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME3) then
	PrefMission = Mission("MJ_Theme3_Race")
elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME2) then
	PrefMission = Mission("MJ_Theme2_Race")
else
	PrefMission = Mission("MJ_Theme1_Race")
end
local NewRace = Chapter1:AddRace(PrefMission)
NewRace.TrackScores = deepcopy(TrackScores)
NewRace.Available = true
NewRace.Unlock = function()
	-- Doe iets hier
end
NewRace.Won = function(self, Place, Score)
	return Place <= 3
end



function SingleRace:StartRace(Race, Car, Character, Career)
	local RaceData = {}
	
	if (LapCount == nil) then
		LapCount = 1
	end
	
	if (Career == nil) then
		RaceData.Career = false
	else
		RaceData.Career = Career
	end

	-- De speler
	RaceData.Name = GetPlayername()
	RaceData.Car = Car
	RaceData.Skin = 0
	RaceData.LapCount = LapCount
	RaceData.Tuning = "Default"
	RaceData.Character = Character
	RaceData.CharacterSkin = 0
	RaceData.CarType = _G[Car]
	
	-- AI data
	RaceData = self:GetNewAIData(RaceData)

	-- Dit moet om de mission te maken
	PrepareScript = function()
		self:PrepareScript(RaceData)
	end

	-- Laad
	loadmap(Race)
end

function SingleRace:GetNewAIData(RaceData)
	RaceData.AI = {}
	
	if (AIDifficulty == nil) then
		AIDifficulty = 1
	end
	
	local AIPool = self:FillAIPool(AIDifficulty)
	
	if (AICount == nil) then
		AICount = 1
	end
	
	for i = 1, AICount do
		if (#AIPool == 0) then
			AIPool = self:FillAIPool(AIDifficulty)
		end
		
		local Index = math.random(#AIPool)
		local AIPlayer = AIPool[Index]
		
		--Tijdelijk uit, voorkomt infinite loop bij dezelfde auto keuze
		while (AIPlayer ~= nil and AIPlayer.Car == RaceData.Car) do
			Index = math.random(#AIPool)			
			AIPlayer = AIPool[Index]
		end
		
		AIPlayer.Index = i + 1
		RaceData.AI[i] = AIPlayer
					
		table.remove(AIPool, Index)
	end
	
	return RaceData
end

function SingleRace:FillAIPool(AIDifficulty)
	local AIPool = {}
	
	for i, AIPlayer in ipairs(AIPlayers) do
		local AI = {}
		AI.Name = Translate(AIPlayer.Name)
		--AI.Character = AIPlayer.Character.Name
		AI.CharacterSkin = AIPlayer.CharacterSkin
		AI.Car, AI.Skin = AIPlayer:GetCar({MonsterTruckCar})
		AI.Difficulty = AIDifficulty * AIPlayer.Skill
		AI.ID = i

		table.insert(AIPool, AI)
	end
	
	return AIPool
end

function SingleRace:StartRaceCareer(Race, Car, Character, LapCount, Players)
	local RaceData = {}
	
	if (LapCount == nil) then
		LapCount = self.Configuration[2].Options[self.Configuration[2].Selection + 1]
	end
	RaceData.LapCount = LapCount
	
	RaceData.Career = true

	-- De speler
	RaceData.Name = GetPlayername()
	RaceData.Car = Car
	RaceData.Skin = 0
	RaceData.Tuning = "Default"
	RaceData.Character = Character
	RaceData.CharacterSkin = 0
	RaceData.CarType = _G[Car]
	RaceData.PlayerID = #Players
	
	-- AI data
	RaceData.AI = {}
	
	for i, Player in ipairs(Players) do
		if (not Player.Human) then
			table.insert(RaceData.AI, Player)
		end
	end

	-- Dit moet om de mission te maken
	PrepareScript = function()
		self:PrepareScript(RaceData)
	end

	-- Laad
	loadscript(Race)
end