HUD_E_StuntList = class(HUD_E_Base, function(self, HUD, Panel, Element, LocationData, ScreenButtonKey, AnalogueStickKey, ToggleButton, Callback)
	self.ExpandingIcon = self.Item:GetChild("ExpandingIcon")
	self.ParentButton = ToggleButton
	self.CloseStuntPanel = self.Item:GetChild("CloseStuntPanel")
	self.StuntPanel = self.Item:GetChild("StuntPanel")
	self.StuntPanelItems = self.Item:GetChild("StuntPanelItems")
	self.Header = self.Item:GetChild("Header")
	self.StuntDescription = self.Item:GetChild("StuntDescription")
	self.Callback = Callback
	self.Header:SetText(Translate("Tap_Stunt_For_Info"))
	self.StuntDescription:SetColor(Invisible)
	
	self.Item:SetColor(Invisible)
	self.IsToggled = false
	
	self.CloseStuntPanel:SetReleasedFunc(function()
		self:ToggleWindow()
	end)
	
	self.StuntList = NewItemList(self, nil, nil)

	self:PresetScoreList()
	
	
	local YOffset = 64
	
	self.StuntList:Format(ALIGN_NS, 18, YOffset, 0, 15, 0, 0, 0, 0, 0, 6)
	self.StuntList:Format(ALIGN_NS, 458, YOffset, 0, 15, 0, 0, 0, 0, 6, 12)
end)

function HUD_E_StuntList:Show(Force)
	self.StuntPanel:SetColorImmediate(Invisible)
end

function HUD_E_StuntList:ToggleDescription(StuntName, Show)
	self:ToggleStunts(not Show)
	self.CloseStuntPanel:SetImage(BackButtonTexture)
end

function HUD_E_StuntList:ToggleStunts(Show)
	if (not Show) then
		for i, Stunt in ipairs(ActiveMission.StuntManager.Stunts) do
			self.StuntList.Items[i].Item:SetColor(Invisible)
		end
		
		self.CloseStuntPanel:SetReleasedFunc(function()
			self:ToggleStunts(true)
			self.Header:SetText(Translate("Tap_Stunt_For_Info"))
			self.StuntDescription:SetColor(Invisible)
		end)
		
	else
		for i, Stunt in ipairs(ActiveMission.StuntManager.Stunts) do
			self.StuntList.Items[i].Item:SetColor(White)
		end
		
		self.CloseStuntPanel:SetReleasedFunc(function()
			self:ToggleWindow()
		end)
		
		self.CloseStuntPanel:SetImage(CloseButtonTexture)
	end
end

function HUD_E_StuntList:SetStuntScore(ID, Score)
	if(Score <= 0) then
		self.StuntList.Items[ID].Item:GetChild("Name"):SetColor(color(1, 1, 1, 0.6))
	else
		self.StuntList.Items[ID].Item:GetChild("Name"):SetColor(White)
	end
	
	self.StuntList.Items[ID]:SetScore(Score)
end

function HUD_E_StuntList:ToggleWindow()
	if (not self.IsToggled) then
		var("IntPause", 1)
		self.StuntPanel:SetColor(White)
	else
		var("IntPause", 0)
		self.StuntPanel:SetColor(Invisible)
	end
	self:ToggleButton()
	self.IsToggled = not self.IsToggled
	
	if (self.Callback) then
		self.Callback()
	end
end

function HUD_E_StuntList:ToggleButton()
	if (self.ParentButton == nil) then
		return
	end
	
	if (not self.IsToggled) then
		self.ParentButton.Item:SetColor(Invisible)
	else
		self.ParentButton.Item:SetColor(White)
	end
end

function HUD_E_StuntList:PresetScoreList()
	for i, Stunt in ipairs(ActiveMission.StuntManager.Stunts) do
		local StuntName = Stunt.ScreenName
		--print(i..". "..Translate(StuntName))
		
		self.StuntsPanel = self.StuntList:AddItem(self.StuntPanelItems, MenuElements.Stunt(HUD, self.StuntList, "hud\\elements\\hud_Stunt.dcMenu", Translate(StuntName)))
		self.StuntList.Items[i].Item:GetChild("Name"):SetReleasedFunc(function()
			self:ToggleDescription(Stunt, true)
			self.Header:SetText(Translate(StuntName))
			self.StuntDescription:SetColor(White)
			self.StuntDescription:SetText(Translate("Description_" .. StuntName))
		end)
		self:SetStuntScore(i, 0)
	end
end

function HUD_E_StuntList:UpdateScores()
	for i, Score in ipairs(ActiveMission.HumanPlayers[1].Scores) do
		local ConvertedScore = 0
		
		if (Score > 0) then
			ConvertedScore = ActiveMission.HumanPlayers[1]:ConvertToStarScore(Score)
		end

		self:SetStuntScore(i, ConvertedScore)
	end
end