function LoadHuds()
	dofile("Lua\\Huds\\Elements\\HUD_E_Base.lua") 

	dofile("Lua\\Huds\\HUD_DeathMatch.lua")
	dofile("Lua\\Huds\\HUD_FreestyleFest.lua")
	dofile("Lua\\Huds\\HUD_HillClimb.lua")
	dofile("Lua\\Huds\\HUD_HillJump.lua")
	dofile("Lua\\Huds\\HUD_Race.lua")
	dofile("Lua\\Huds\\HUD_Test.lua")
	dofile("Lua\\Huds\\HUD_TowTest.lua")
	dofile("Lua\\Huds\\HUD_Destroyer.lua")
	dofile("Lua\\Huds\\HUD_FrameTwister.lua")
	dofile("Lua\\Huds\\HUD_TankTrap.lua") 
end

function LoadHUDTextures()
	CountdownTexture1 = texture("Hud\\1.DCT")
	CountdownTexture2 = texture("Hud\\2.DCT")
	CountdownTexture3 = texture("Hud\\3.DCT")
	CountdownTextureGo = texture("Hud\\go!.DCT")
	
	BackButtonTexture = texture("Menu\\Images\\hud\\Hud_BackButton.DCT")
	CloseButtonTexture = texture("Menu\\Images\\hud\\Hud_CloseButton.DCT")
end

function UnloadHUDTextures()
	CountdownTexture1 = nil
	CountdownTexture2 = nil
	CountdownTexture3 = nil
	CountdownTextureGo = nil
	
	BackButtonTexture = nil
	CloseButtonTexture = nil
end

HUD_Base = class(function(self, Mission, Player)
	LoadHUDTextures()
	ClearAllScreenButtons()
	
	self.Player = Player 
	self.Mission = Mission

	self.Stack = createhud("HUD\\hud.dcMenu")   --hier offset aangeven en de scale voor splitscreen
	
	self.HUDElements = {}
	
	-- gooi menu er op
	self.Menu = self.Stack:GetRootPanel()
	self.SafeFrame = self.Menu:GetChild("SafeFrame")
	self.SafeFrame:SetSize(SafeFrameWidth, SafeFrameHeight) 
	
	-- Fade in items
	self.Menu:SetColorFade(1, 10)
	self.Menu:SetColorImmediate(White) 

	-- Hier staan alle hud elementen in
	self.Hud = self.Menu:GetChild("Hud")
	self.Hud:SetColorImmediate(White)
	
	self.Tutorial_Accelerometer = self.Hud:GetChild("Tutorial_Accelerometer")
	
	local BaseTextColorValue = 0.8
	local BaseTextColorValue_Flash = 1 
	
	self.BaseTextColor = color(BaseTextColorValue, BaseTextColorValue, BaseTextColorValue, 1)
	self.BaseTextColor_Flash = color(BaseTextColorValue_Flash, BaseTextColorValue_Flash, BaseTextColorValue_Flash, 1)
	
	-- Debug Text
	local LocationData = {0,-100,0.5,1,0.5,1}
	self.DebugText = HUD_E_DebugText(self, self.SafeFrame, "hud_DebugText", LocationData)
	table.insert(self.HUDElements, self.DebugText) 
	
	-- Ingame Button
	local LocationData = {0,0,0,0,0,0}
	self.IngameButton = HUD_E_SimpleScreenButton(self, self.SafeFrame, "hud_SimpleScreenButton", LocationData, "DCK_SCREENBUTTON_1")
	table.insert(self.HUDElements, self.IngameButton)
	
	-- Countdown Element
	local LocationData = {0,0,0.5,0.5,0.5,0.25}
	self.CountDown = HUD_E_Countdown(self, self.SafeFrame, "hud_Countdown", LocationData)
	table.insert(self.HUDElements, self.CountDown)

	local UseAccelerometer = tonumber(getvar("UseAccelerometer"))
	if (UseAccelerometer == nil or UseAccelerometer > 0) then
		UseAccelerometer = true
	else
		UseAccelerometer = false
	end
	
	local UseAccelerateSlider = (UseAccelerometer or ActiveGameMode.UseAccelerateSlider)	-- use during special missions or when accelerometer is enabled
	
	-- Bonus buttons!
	local OffsetX = -32
	local OffsetY = 0
	
	local LocationData = nil
	if (not UseAccelerateSlider) then
		LocationData = {-64 + OffsetX, 128 + OffsetY, 1,0,1,0}
	else
		LocationData = {-64 + OffsetX, -64 + OffsetY, 1,1,1,1}
	end
	self.RepairButton = HUD_E_PowerUpButton(self, self.SafeFrame, "hud_RepairButton", LocationData, "DCK_SCREENBUTTON_2")
	table.insert(self.HUDElements, self.RepairButton)
	
	if (not UseAccelerateSlider) then
		LocationData = {-64 + OffsetX, 256 + OffsetY, 1,0,1,0}
	else
		LocationData = {-64 + OffsetX, -192 + OffsetY, 1,1,1,1}
	end
	self.JumpButton = HUD_E_PowerUpButton(self, self.SafeFrame, "hud_JumpButton", LocationData, "DCK_SCREENBUTTON_3")
	table.insert(self.HUDElements, self.JumpButton)
	
	if (not UseAccelerateSlider) then
		LocationData = {-64 + OffsetX, 384 + OffsetY, 1,0,1,0}
	else
		LocationData = {-64 + OffsetX, -320 + OffsetY, 1,1,1,1}
	end
	self.FireWorksButton = HUD_E_PowerUpButton(self, self.SafeFrame, "hud_FireWorksButton", LocationData, "DCK_SCREENBUTTON_4")
	table.insert(self.HUDElements, self.FireWorksButton)
	
	if (UseAccelerateSlider) then
		-- Accelerate Slider
		local LocationData = {0,-64,0,1,0,1}
		self.AccelerateSlider = HUD_E_AccelerateSlider(self, self.SafeFrame, "hud_AcceleratePanel", LocationData, nil, "DCK_SCREENANALOGSTICK_0")
		table.insert(self.HUDElements, self.AccelerateSlider)
	else
		-- Steering slider
		--local LocationData = {-16, 0, 1, 1, 1, 1}
		--self.SteerSlider = HUD_E_SteerSlider(self, self.SafeFrame, "hud_SteerPanel", LocationData, nil, "DCK_SCREENANALOGSTICK_1")
		--table.insert(self.HUDElements, self.SteerSlider)
		
		-- Joystick
		local LocationData = {5, -64, 0, 1, 0, 1}
		self.JoyStick = HUD_E_JoyStick(self, self.SafeFrame, "hud_JoystickPanel", LocationData, nil, "DCK_SCREENANALOGSTICK_0")
		table.insert(self.HUDElements, self.JoyStick)
		
		-- Boost Button
		local LocationData = {-64, -64, 1, 1, 1, 1}
		self.BoostButton = HUD_E_BoostButton(self, self.SafeFrame, "hud_BoostButton", LocationData, "DCK_SCREENBUTTON_6")
		table.insert(self.HUDElements, self.BoostButton)
	end
  
	-- Special score popup
	local LocationData = {0, 0, 1, 0.5, 1, 0.3}
	self.ScorePopup = HUD_E_SpecialScorePopup(self, self.SafeFrame, "hud_SpecialScorePopup", LocationData)
	table.insert(self.HUDElements, self.ScorePopup)
	
	-- ConfirmMenu
	local LocationData = {0,0,0.5,0.5,0.5,0.50}
	self.ConfirmMenu = HUD_E_ConfirmMenu(self, self.SafeFrame, "hud_ConfirmMenu", LocationData)
	table.insert(self.HUDElements, self.ConfirmMenu)
end)

--=== PRIMARY HUD FUNCTIONS ===--
function HUD_Base:Destroy() 
	UnloadHUDTextures()

	for i, Element in ipairs(self.HUDElements) do
		Element:Destroy() 
	end 

	if (self.Stack ~= nil) then
		self.Stack:Destroy()
		self.Stack = nil 
	end
end

function HUD_Base:Show(Force)	
	if (Force) then
		self.Hud:SetColorImmediate(White) 
		self.SafeFrame:SetColorImmediate(White) 
	else
		self.Hud:SetColorData(White, 2, Sine, false) 
		self.SafeFrame:SetColorData(White, 2, Sine, false) 
	end
	
	for i, Element in ipairs(self.HUDElements) do 
		Element:Show(Force) 
	end
end

function HUD_Base:Hide(Force, OnlyBackground)
	if (OnlyBackground) then
		self.SafeFrame:SetColor(Invisible)
		self.SafeFrame:SetColorFade(1,3)
		return
	end

	if (Force) then
		self.Hud:SetColorImmediate(Invisible)
	else
		self.Hud:SetColor(Invisible)
	end
	
	for i, Element in ipairs(self.HUDElements) do
		Element:Hide(Force)
	end
end

function HUD_Base:Active()
	return true
end

function HUD_Base:Refresh(Player)
	self:Destroy()
	
	LoadHuds()
	
	return ActiveGameMode:HUD(ActiveMission, Player)
end

function HUD_Base:UpdatePhysics(ElapsedTime)
	for i, Element in ipairs(self.HUDElements) do
		Element:UpdatePhysics(ElapsedTime) 
	end 
end

function HUD_Base:Update(ElapsedTime)
	for i, Element in ipairs(self.HUDElements) do
		Element:Update(ElapsedTime)
	end
end

--=== SECONDARY HUD FUNCTIONS ===-- 
function HUD_Base:ShowCheckpoint(Checkpoint)
	--
end

function HUD_Base:ShowLapCount()
	--
end

function HUD_Base:ShowCountdown(Count)
	if (self.CountDown ~= nil) then
		self.CountDown:SetText(Count)
		self.CountDown:Flash(1)
	end
	
	if (Count == "2") then
		self:SetCountdownFont("ArialBlack70")
	elseif (Count == "1") then
		self:SetCountdownFont("ArialBlack80")
	elseif (Count == "GO!") then
		self:SetCountdownFont("ArialBlack90")
	else
		self:SetCountdownFont("ArialBlack70")
	end
end

function HUD_Base:SetLapCount(Count) 
	--
end

function HUD_Base:SetLapRecord(Count)
	--
end

function HUD_Base:ShowRank(Count)
	--
end

function HUD_Base:ShowPhotoMessage()
	--
end

function HUD_Base:ActivateScore()
	--
end

function HUD_Base:UpdateScore() 
	--
end 

function HUD_Base:SetScore(Score, LerpBool, FormatTime)
	if (self.ScoreElement ~= nil) then
		self.ScoreElement:SetScore(Score, LerpBool, FormatTime)
	end
end

function HUD_Base:SetCountdownFont(Font)
	if (self.CountDown ~= nil) then 
		self.CountDown:SetFont(Font) 
	end
end

function HUD_Base:SetScoreFont(Font, Flash)
	if (self.ScoreElement ~= nil) then 
		self.ScoreElement:SetFont(Font, Flash) 
	end
end

function HUD_Base:HideDebugText(Force)
	self.DebugText:Hide(Force)
end

function HUD_Base:SetDebugText(Text1, Text2, Text3)
	self.DebugText:ShowDebug(true)
	
	if (Text1 == nil) then
		Text1 = ""
	end
	if (Text2 == nil) then
		Text2 = ""
	end
	if (Text3 == nil) then
		Text3 = ""
	end
	
	self.DebugText:SetText(Text1, Text2, Text3)
end

function HUD_Base:StartWarning(Duration)
	if (self.WarningPopup ~= nil) then
		self.WarningPopup:Start(Duration)
	end
end

function HUD_Base:EndWarning()
	if (self.WarningPopup ~= nil) then
		self.WarningPopup:End(Invisible)
	end
end

function HUD_Base:DisplayBonusTime()
	if (self.BonusTimePopup ~= nil) then
		self.BonusTimePopup:Start()
	end
end

function HUD_Base:HidewBuyPowerupScreen(Powerup)
	var("intpause", 0)
end

function HUD_Base:ShowSteerTutorial(CallbackFunction)
	if (self.TutorialThread ~= nil) then
		PhysicsThread:Spawn(function()
			PhysicsThread:Stop(self.TutorialThread)
			self.TutorialThread = nil
		end)
	end

	if (self.Menu ~= nil) then
		self.Menu:SetClickedFunc(function()		
			if (self.TutorialThread ~= nil) then
				PhysicsThread:Spawn(function()
					PhysicsThread:Stop(self.TutorialThread)
					self.TutorialThread = nil
				end)
			end
			
			self:EndSteerTutorial(CallbackFunction)
		end)
	end
	
	--
	
	self.TutorialThread = PhysicsThread:Spawn(function()
		PhysicsThread:Wait(60)
		
		self.Tutorial_Accelerometer:SetColorFade(1, 10)
		self.Tutorial_Accelerometer:SetColorImmediate(Invisible)
		self.Tutorial_Accelerometer:SetColor(White)
		
		if (self.TutorialText ~= nil) then
			self.TutorialText:SetText(Translate("tutorial_Steer1"))
		else
			print("No TutorialText element found")
		end
		
		local Width, Height = self.Tutorial_Accelerometer:GetSize()
		self.Tutorial_Accelerometer:SetOffset(Width / 2, Height / 2)
		
		local Rot = 180
		
		for i = 0, 30 do
			Rot = Rot + 0.9
			self.Tutorial_Accelerometer:SetRot(Rot)
			PhysicsThread:Wait(1)
		end
		
		for i = 0, 60 do
			Rot = Rot - 0.9
			self.Tutorial_Accelerometer:SetRot(Rot)
			PhysicsThread:Wait(1)
		end
		
		for i = 0, 60 do
			Rot = Rot + 0.9
			self.Tutorial_Accelerometer:SetRot(Rot)
			PhysicsThread:Wait(1)
		end
		
		for i = 0, 30 do
			Rot = Rot - 0.9
			self.Tutorial_Accelerometer:SetRot(Rot)
			PhysicsThread:Wait(1)
		end
		
		PhysicsThread:Wait(60 * 3)
		
		self:EndSteerTutorial(CallbackFunction)
	end)
end

function HUD_Base:EndSteerTutorial(CallbackFunction)
	if (self.TutorialText ~= nil) then
		self.TutorialText:SetText("")
	end
	
	if (self.Tutorial_Accelerometer ~= nil) then
		self.Tutorial_Accelerometer:SetColor(Invisible)
	end
	
	if (self.Menu ~= nil) then
		self.Menu:SetClickedFunc(nil)
	end
	
	if (CallbackFunction ~= nil) then
		CallbackFunction()
	end
end

function HUD_Base:EnablePowerUps()
	print("HUD_Base:EnablePowerUps")
	local Player = ActiveMission.HumanPlayers[1]
	
	if (Player.PowerupManager.RepairPowerup.Enabled) then
		self.RepairButton:TurnOn()
	end
	
	if (Player.PowerupManager.JumpPowerup.Enabled) then
		self.JumpButton:TurnOn()
	end
	
	if (Player.PowerupManager.FirworksPowerup.Enabled) then
		self.FireWorksButton:TurnOn()
	end
end

function HUD_Base:CarFlipped(SecondsBeforeEnd)
	local Player = ActiveMission.HumanPlayers[1]
	
	if(not self.ConfirmMenu.IsActive) then
		local FlippedTimer = SecondsBeforeEnd + 1
		self.CarFlippedThread = GameThread:Spawn(function(Thread)
			GameThread:Wait(1)
		
			Player.HUD.RepairButton:TurnOff()
			self.ConfirmMenu:ShowConfirmMenu(Translate("confirm_CarFlipped_title"), Translate("confirm_CarFlipped_text"), function()
				Player.PowerupManager:Repair()
				--Player.Car:Respawn()
				--teleport()
				--Player.Finished = false
				Player.Flipped = false
				
				GameThread:Stop(self.CarFlippedThread)
				self.CarFlippedThread = nil
			end, function()
				GameThread:Stop(self.CarFlippedThread)
				self.CarFlippedThread = nil
				Player.Finished = false
				Player.LastLapTime = ActiveMission.RaceTimer
				Player.Failed = true
				ActiveMission:EndGame()
			end)
			self.ConfirmMenu.YesButton:Flash(1, true, true, 0.3)
			
			while (FlippedTimer > 0) do
				FlippedTimer = FlippedTimer - 1
				self.ConfirmMenu.YesButton:SetText(string.format(Translate("confirm_CarFlipped_YesButton"), FlippedTimer))
				
				local Matrix = Player.Car:GetMatrix()
				local Up = Matrix:GetUp();
				if (Up.y > 0) then
					self.ConfirmMenu:HideMenu()
					Player.HUD.RepairButton:TurnOn()
					Player.Flipped = false
					GameThread:Stop(self.CarFlippedThread)
					self.CarFlippedThread = nil
				end
				
				GameThread:Wait(1)
			end
			
			self.ConfirmMenu:CancelFunction()
		end)
	end
end

LoadHuds()