InputManager = class(function(self, Player, Mission)
	self.Player = Player
	self.Mission = Mission
	self.Enabled = false
	self.SteeringEnabled = true
	
	-- Input variables
	self:Reset()
	
	-- Store all buttons in a local variable
	local Buttons = {}
	
	-- Initialize standard buttons
	self.AccelerateButtons = {"DCK_UP"}
	self.BrakeButtons = {"DCK_DOWN"}
	self.BoostButtons = {"DCK_LCONTROL", "DCK_RCONTROL", "DCK_SCREENBUTTON_6"}
	self.LeftButtons = {"DCK_LEFT"}
	self.RightButtons = {"DCK_RIGHT"}
	
	self.UseAccelerometer = tonumber(getvar("UseAccelerometer"))
	if (self.UseAccelerometer == nil or self.UseAccelerometer > 0) then
		self.UseAccelerometer = true
	else
		self.UseAccelerometer = false
	end
	
	if (self.UseAccelerometer) then
		table.insert(self.LeftButtons, "DCK_ACCELEROMETER_LEFT")
		table.insert(self.RightButtons, "DCK_ACCELEROMETER_RIGHT")
	else
		table.insert(self.LeftButtons, "DCK_SCREENANALOGSTICK_0_LEFT")
		table.insert(self.RightButtons, "DCK_SCREENANALOGSTICK_0_RIGHT")
		
		--self.LeftButtons = {"DCK_LEFT", "DCK_SCREENANALOGSTICK_1_LEFT"}
		--self.RightButtons = {"DCK_RIGHT", "DCK_SCREENANALOGSTICK_1_RIGHT"}
	end
	
	table.insert(self.AccelerateButtons, "DCK_SCREENANALOGSTICK_0_UP")
	table.insert(self.BrakeButtons, "DCK_SCREENANALOGSTICK_0_DOWN")
	
	-- Initialize specific buttons, which call functions back to the mission
	self.GearShiftButtons = {}
	self.NitroActivatedButtons = {}
	
	-- Pre-setting the KeyCodes, instead of reading the keycodes realtime, which is slow
	table.insert(Buttons, self.AccelerateButtons)
	table.insert(Buttons, self.BrakeButtons)
	table.insert(Buttons, self.BoostButtons)
	table.insert(Buttons, self.LeftButtons)
	table.insert(Buttons, self.RightButtons)
	table.insert(Buttons, self.GearShiftButtons)
	table.insert(Buttons, self.NitroActivatedButtons)
	
	local PrintKeyCodes = false
	
	for i, ButtonsInternal in ipairs(Buttons) do
		if (PrintKeyCodes) then
			print("--- New Button ---")
		end
		
		for j, InternalButton in ipairs(ButtonsInternal) do
			if (PrintKeyCodes) then
				print("Button Print; " .. tostring(InternalButton))
			end
			
			Buttons[i][j] = GetKeyCode(InternalButton)
			
			if (PrintKeyCodes) then
				print("Button Override ^2; " .. tostring(Buttons[i][j]))
			end
		end
	end
	
	-- Initialize input (floats), directly from the mission, which can be used in the input
	--self.GasPedalSliderThrottleKey_Up = GetKeyCode("DCK_SCREENANALOGSTICK_0_UP")
	--self.GasPedalSliderThrottleKey_Down = GetKeyCode("DCK_SCREENANALOGSTICK_0_DOWN")
	--self.GasPedalSliderThrottleKey_Left = GetKeyCode("DCK_SCREENANALOGSTICK_0_LEFT")
	--self.GasPedalSliderThrottleKey_Right = GetKeyCode("DCK_SCREENANALOGSTICK_0_RIGHT")
	
	--self.SteerSliderThrottleKey_Left = GetKeyCode("DCK_SCREENANALOGSTICK_1_LEFT")
	--self.SteerSliderThrottleKey_Right = GetKeyCode("DCK_SCREENANALOGSTICK_1_RIGHT")
	
	--self.BrakeLimit = 0.15
	--self.GasLimit = 0.85
	--self.NitroLimit = 1
	self.BrakeLimit = 0.5
	self.GasLimit = 1.0
	self.NitroLimit = 10000
	self.BrakeAdditiveStrength = 0.6
	
	self.InputSpeedY = self.BrakeLimit
end)

function InputManager:Reset()
	self.AccelSpeed = 0
	
	self.BrakeValue = 1
	self.BrakeStrength = 0.5
	
	self.SteerValue_Raw = 0
	self.SteerValue = 0
	self.SteerStrength = 2
	
	self.MaxSteer = 1.5
	self.MaxBrake = 1
	
	self.Steering = false
	self.Boosting = false
	self.Braking = false
	self.Accelerating = false
	
	self.Player.Car:ChangeGear(self.Player.Car:GetBestGear())
	self.Player.Car:Accelerate(self.AccelSpeed) 
	
	self.Player.Car:Brake(self.BrakeValue)
	self.Player.Car:Steer(self.SteerValue)
end

function InputManager:Enable()
	if (not self.Enabled) then
		self.Enabled = true
	end
end

function InputManager:Disable()
	if (self.Enabled) then
		self.Enabled = false
	end
end

function InputManager:EnableSteering()
	if (not self.SteeringEnabled) then
		self.SteeringEnabled = true
	end
end

function InputManager:DisableSteering()
	if (self.SteeringEnabled) then
		self.SteeringEnabled = false
	end
end

function InputManager:Update(ElapsedTime)
	if (not self.Enabled) then
		return
	end
	
	local Input = {}
	Input.Accel = 0
	Input.RawAccel = 0
	Input.Brake = 0
	Input.Gear = 0
	Input.Boost = false
	Input.Handbrake = false
	Input.Steer = 0
	
	-- ===============================
	-- Fetch KeyInput
	local ForwardInput = 0
	local ReverseInput = 0
	for i, Button in ipairs(self.AccelerateButtons) do
		--if (KeyPressed(Button)) then
		if (KeyPressure(Button) > 0) then
			ForwardInput = Max(ForwardInput, KeyPressure(Button))
		end
	end
	
	for i, Button in ipairs(self.BrakeButtons) do
		--if (KeyPressed(Button)) then
		if (KeyPressure(Button) > 0) then
			ReverseInput = Max(ReverseInput, KeyPressure(Button))
		end
	end
	
	local BoostButtonPressed = false
	for i, Button in ipairs(self.BoostButtons) do
		--if (KeyPressed(Button)) then
		if (KeyPressure(Button) > 0) then
			BoostButtonPressed = true
		end
	end
	-- ===============================
	
	Input.RawAccel = -ForwardInput + ReverseInput
	
	-- ===============================
	-- Dit is speciaal voor de AccelSlider (ugly stuff)
	local BrakeLimit = 0.15
	local BoostLimit = 0.7
	local Tmp = BrakeLimit
	if (not self.SteeringEnabled or self.UseAccelerometer) then
		if (ForwardInput > 0 or ReverseInput > 0) then
			Tmp = (-Input.RawAccel + 1) / 2
		end
		--print("Tmp(" .. tostring(Tmp) .. ")")
		if (Tmp > BrakeLimit) then
			ForwardInput = (Tmp * 1.0) - (BrakeLimit)
			ForwardInput = (ForwardInput / BoostLimit)
			ReverseInput = 0
		elseif (Tmp < BrakeLimit) then
			ForwardInput = 0
			ReverseInput = -((Tmp * 1.0) + (1 - BrakeLimit)) + 1
			ReverseInput = (ReverseInput / BrakeLimit)
		else
			Input.RawAccel = BoostLimit
		end
	end
	-- ===============================
	
	local ForwardVel = -self.Player.Car:GetLocalSpeed()
	
	if (ForwardInput > 0) then
		if (ForwardVel > -5) then
			Input.Accel = ForwardInput
			Input.Gear = self.Player.Car:GetBestGear()
		else
			Input.Brake = 1
		end
	end
	
	if (ReverseInput > 0) then
		if (ForwardVel < 5) then
			Input.Accel = ReverseInput
			Input.Gear = -1
		else
			-- ================
			-- This is needed to make a stoppie
			Input.Accel = ClampMax(ReverseInput, 0.001)
			Input.Gear = -1
			-- ================
			
			Input.Brake = 1
		end
	end
	
	if (ForwardInput > 1 or BoostButtonPressed) then
		Input.Accel = 1
		Input.Gear = self.Player.Car:GetBestGear()
		Input.Boost = true
		ForwardInput = ClampMax(ForwardInput, 1)
	end
	
	local LeftSteer = 0
	local RightSteer = 0
	
	if (self.SteeringEnabled) then
		for i, Button in ipairs(self.LeftButtons) do
			LeftSteer = Max(LeftSteer, KeyPressure(Button))
		end
		
		for i, Button in ipairs(self.RightButtons) do
			RightSteer = Max(RightSteer, KeyPressure(Button))
		end
	end
	
	Input.Steer = -LeftSteer + RightSteer
	
	-- ================
	-- DeadZone for Accelerometer
	if (self.UseAccelerometer) then
		if (math.abs(Input.Steer) < 0.07) then
			Input.Steer = 0
		end
	end
	-- ================
	
	self.Input = Input
	
	if (false) then
		print("Input.Accel(" .. tostring(Input.Accel) .. ")")
		print("Input.RawAccel(" .. tostring(Input.RawAccel) .. ")")
		print("Input.Brake(" .. tostring(Input.Brake) .. ")")
		print("Input.Gear(" .. tostring(Input.Gear) .. ")")
		print("Input.Boost(" .. tostring(Input.Boost) .. ")")
		print("Input.Steer(" .. tostring(Input.Steer) .. ")")
		cls()
	end
	
	self.Player.Car:Accelerate(Input.Accel)
	self.Player.Car:Brake(Input.Brake)
	self.Player.Car:ChangeGear(Input.Gear)
	self.Player.Car:Steer(Input.Steer)
	
	if (Input.Boost) then
		if (self.Player.PowerupManager.BoostPowerup.Enabled) then
			self.Player.Car:StartTimedBoost(1 / 10)
		
			local Dir = self.Player.Car:GetMatrix():GetDirection()
			self.Player.Car:AddForce(-Dir * 800000)
			
			-- Ubervlam
			self.Player.Car:GetMesh():SetProperty("BoostLightExtra", 1)
			self.Player.Car:GetMesh():SetProperty("BoostLight", 0)
		else
			self.Player.Car:StartTimedBoost(1 / 10)
			
			-- Normale vlam
			self.Player.Car:GetMesh():SetProperty("BoostLightExtra", 0)
			self.Player.Car:GetMesh():SetProperty("BoostLight", 1)
		end
	else
		self.Player.Car:GetMesh():SetProperty("BoostLightExtra", 0)
		self.Player.Car:GetMesh():SetProperty("BoostLight", 0)
	end
	
	-- ===========================================
end