-- Just in case
dofile("Lua\\Menu\\ConfirmMenu.lua")

function ShowConfirm_Exit()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_exit_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_exit_text"), FreeCoins))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("button_yes"))
	ActiveMenu:SetCancelText(Translate("button_no"))
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			exit()
		end)
	end
	
	ActiveMenu.Back = function()
		if (ActiveMenu.NoButton.OnReleased ~= nil) then
			ActiveMenu.NoButton:OnReleased()
		end
	end
end

function ShowConfirm_Intro(FreeCoins)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_intro_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_intro_text"), FreeCoins))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetCancelText(Translate("menu_okcool"))
	ActiveMenu.YesButtonPanel:Clear()
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			VirtualCoins:AddCoinCount(FreeCoins)
			VirtualCoins:SendCollectedCoinsStats("Other", FreeCoins, 0)
			WriteGameSettings()
		end)
	end
end

function ShowConfirm_InfoButton(Name, Discription)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate(Name))
	ActiveMenu:SetText(Translate(Discription))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetCancelText(Translate("menu_ok"))
	ActiveMenu.YesButtonPanel:Clear()
	
	ActiveMenu.Back = function()
		if (ActiveMenu.NoButton.OnReleased ~= nil) then
			ActiveMenu.NoButton:OnReleased()
		end
	end
end

function ShowConfirm_ScoreButton(Race)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu, false, "Confirm_ScoreOverview")
	ActiveMenu:SetTitle("")
	ActiveMenu:SetText("")
	ActiveMenu.YesButtonPanel:Clear()
	ActiveMenu.NoButtonPanel:Clear()
	
	ActiveMenu.CurrentRace = Race
	
	ActiveMenu.ButtonItems = NewItemList(ActiveMenu, nil, nil)
	
	ActiveMenu.HighscoreItems = ActiveMenu.Menu:GetChild("HighscoreItems")
	
	ActiveMenu.CloseButton = ActiveMenu.Menu:GetChild("CloseButton")
	ActiveMenu.WorldButtonPanel = ActiveMenu.Menu:GetChild("WorldButton")
	ActiveMenu.LocalButtonPanel = ActiveMenu.Menu:GetChild("LocalButton")
	ActiveMenu.FriendsButtonPanel = ActiveMenu.Menu:GetChild("FriendsButton")
	
	ActiveMenu.LoadingImage = ActiveMenu.Menu:GetChild("LoadingImage")
	ActiveMenu.LoadingImage:SetAlpha(0)
	ActiveMenu.LoadingThread = GameThread:Spawn(function(Thread)
		local Counter = 1
		while(true) do
			GameThread:Sleep(0.05)
			
			if (ActiveMenu.LoadingImage ~= nil) then
				ActiveMenu.LoadingImage:SetImage(LoadingImages[Counter])
				Counter = Inc(Counter)
				if (Counter == 11) then
					Counter = 1
				end
			end
		end
	end)
	
	ActiveMenu.WorldButtonPanel:Clear()
	ActiveMenu.LocalButtonPanel:Clear()
	ActiveMenu.FriendsButtonPanel:Clear()
	
	ActiveMenu.CloseButton:SetReleasedFunc(function()
		if (ActiveMenu.NoButton.OnReleased ~= nil) then
			ActiveMenu.NoButton:OnReleased()
		end
	end)
	
	ActiveMenu.Back = function()
		if (ActiveMenu.NoButton.OnReleased ~= nil) then
			ActiveMenu.NoButton:OnReleased()
		end
	end
	

	local buttons = {}
	-- ================================
	-- World
	ActiveMenu.WorldButton = ActiveMenu.ButtonItems:AddItem(ActiveMenu.WorldButtonPanel, MenuElements.Label(ActiveMenu.ButtonItems, "Elements\\HighScores_MenuButton.dcMenu", "button_world"))
	ActiveMenu.WorldButton.Item:SetActivateFunc(function() end)
	ActiveMenu.WorldButton.Item:SetClickedFunc(function() end)
	table.insert(buttons, ActiveMenu.WorldButton)
	-- ================================
	-- Local
	ActiveMenu.LocalButton = ActiveMenu.ButtonItems:AddItem(ActiveMenu.LocalButtonPanel, MenuElements.Label(ActiveMenu.ButtonItems, "Elements\\HighScores_MenuButton.dcMenu", "button_local"))
	ActiveMenu.LocalButton.Item:SetActivateFunc(function() end)
	ActiveMenu.LocalButton.Item:SetClickedFunc(function() end)
	table.insert(buttons, ActiveMenu.LocalButton)
	
	
	-- ================================
	-- Friends
	if (FBManager:IsLoginValid()) then
		ActiveMenu.FriendsButton = ActiveMenu.ButtonItems:AddItem(ActiveMenu.FriendsButtonPanel, MenuElements.Label(ActiveMenu.ButtonItems, "Elements\\HighScores_MenuButton.dcMenu", "button_friends"))
		ActiveMenu.FriendsButton.Item:SetActivateFunc(function() end)
		ActiveMenu.FriendsButton.Item:SetClickedFunc(function() end)
		table.insert(buttons, ActiveMenu.FriendsButton)
		ActiveMenu.FriendsButton.OnReleased = function()
			ActiveMenu.ButtonItems.Clicked = false
			ActiveMenu:ShowFriendsList()
			for i,Button in ipairs(buttons) do
				Button:Disable()
			end
		end
	end
	
	ActiveMenu.WorldButton.OnReleased = function()
		ActiveMenu.ButtonItems.Clicked = false
		ActiveMenu:ShowWorldList()
		for i,Button in ipairs(buttons) do
			Button:Disable()
		end
	end
	
	ActiveMenu.LocalButton.OnReleased = function()
		ActiveMenu.ButtonItems.Clicked = false
		ActiveMenu:ShowLocalList()
	end
	
	-- ================================
	-- ShowList - functions
	ActiveMenu.ShowLocalList = function()
		ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
		ActiveMenu.HighscoreItems:Clear()
		ActiveMenu:SetText("")
		
		if (ActiveMenu.CurrentRace.HighScores ~= nil and #ActiveMenu.CurrentRace.HighScores > 0) then
			for Key, HighScore in pairs(ActiveMenu.CurrentRace.HighScores) do
				if (Key <= 15) then
					ActiveMenu.HighscoreItemList:AddItem(ActiveMenu.HighscoreItems, MenuElements.HighScores(ActiveMenu.HighscoreItemList, "elements\\Highscore_Template.dcMenu", Key, HighScore.Name, HighScore.Car, HighScore.Score, false))
				end
			end
		else
			ActiveMenu:SetText(Translate("menu_no_highscores_text"))
		end
		
		ActiveMenu.HighscoreItemList:Format(ALIGN_NS, 0, 0, 0, 0, 0, 0, 0, 0, 0, #ActiveMenu.HighscoreItemList.Items)
	end
	
	ActiveMenu.ShowWorldList = function()
		ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
		ActiveMenu.HighscoreItems:Clear()
		ActiveMenu:SetText("")
		
		local FailureFunction = function(Data)
			print("Error getting score")
			
			ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
			ActiveMenu.HighscoreItems:Clear()
			
			ActiveMenu:SetText(Translate("menu_no_highscores_text"))
			
			ActiveMenu.LoadingImage:SetAlpha(0)
			for i,Button in ipairs(buttons) do
				Button:Enable(false)
			end
			ActiveMenu.ButtonItems:SetSelection(ActiveMenu.WorldButton)
		end
		
		local SuccessFunction = function(Data)
			print('OnGetScore')
			
			if (Data ~= nil and Data['Scores'] ~= nil) then
				ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
				ActiveMenu.HighscoreItems:Clear()
				
				ActiveMission.Results = Data
				local Entries = ActiveMission.Results['Scores'];
				--local Entries = ActiveMission.Results['FriendsScores'];
				
				table.sort(Entries, function(a, b)
					if (b.Score < a.Score) then
						return true;
					elseif (b.Score == a.Score and (a.Place ~= nil and b.Place ~= nil)) then
						return a.Place < b.Place;
					else
						return false
					end
				end)
				
				for i, Entry in ipairs(Entries) do
					local UserTable = {}
					UserTable['id'] = Entry.FacebookID
					UserTable['name'] = ''
					UserTable['score'] = math.floor(Entry.Score)
					UserTable['rank'] = 1
					
					if (Entry.FacebookID ~= 0) then
						if (FBManager.UserFriendsTable ~= nil) then
							FacebookUser = FBManager.UserFriendsTable[Entry.FacebookID]
							if (FacebookUser ~= nil) then
								UserTable['picture'] = FacebookUser['picture']
							end
						elseif (FBManager.UserinfoTable ~= nil) then
							if (Entry.FacebookID == FBManager.UserinfoTable['id']) then
								UserTable['picture'] = FBManager.UserinfoTable['picture']
							end
						end
					end
					
					local Place = Entry.Place
					--local Place = i
					
					local CarName = Translate(MonsterTruckCars.Cars[Entry.CarID].Title)
					ActiveMenu.HighscoreItemList:AddItem(ActiveMenu.HighscoreItems, MenuElements.HighScores(ActiveMenu.HighscoreItemList, "elements\\Highscore_Template.dcMenu", Place, Entry.Name, CarName, Entry.Score, Entry.LocalUser))
				end
				
				if (#ActiveMenu.HighscoreItemList.Items == 0) then
					ActiveMenu:SetText(Translate("menu_no_highscores_text"))
				end
				
				ActiveMenu.HighscoreItemList:Format(ALIGN_NS, 0, 0, 0, 0, 0, 0, 0, 0, 0, #ActiveMenu.HighscoreItemList.Items)
				ActiveMenu.LoadingImage:SetAlpha(0)

				for i,Button in ipairs(buttons) do
					Button:Enable(false)
				end
				ActiveMenu.ButtonItems:SetSelection(ActiveMenu.WorldButton)
			else
				FailureFunction()
			end
		end
		
		ActiveMenu.LoadingImage:SetAlpha(1)
		
		local UserID = 0
		if (FBManager:IsLoginValid()) then
			UserID = FBManager.UserinfoTable['HighscoresID']
		else
			UserID = Scores.UserID
		end
		
		local Script = ActiveMenu.CurrentRace.Name
		ScoresUser:AddScore(UserID, Script, 0, 0, SuccessFunction, FailureFunction)
	end
	
	ActiveMenu.ShowFriendsList = function()
		ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
		ActiveMenu.HighscoreItems:Clear()
		ActiveMenu:SetText("")
		
		local FailureFunction = function(Data)
			print("Error getting score")
			
			ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
			ActiveMenu.HighscoreItems:Clear()
			
			ActiveMenu:SetText(Translate("menu_no_highscores_text"))
			
			ActiveMenu.LoadingImage:SetAlpha(0)

			for i,Button in ipairs(buttons) do
				Button:Enable(false)
			end
			ActiveMenu.ButtonItems:SetSelection(ActiveMenu.FriendsButton)
		end
		
		local SuccessFunction = function(Data)
			print('OnGetScore')
			
			if (Data ~= nil and Data['Scores'] ~= nil) then
				ActiveMenu.HighscoreItemList = NewItemList(ActiveMenu, nil, nil)
				ActiveMenu.HighscoreItems:Clear()
				
				ActiveMission.Results = Data
				--local Entries = ActiveMission.Results['Scores'];
				local Entries = ActiveMission.Results['FriendsScores'];
				
				table.sort(Entries, function(a, b)
					if (b.Score < a.Score) then
						return true;
					elseif (b.Score == a.Score and (a.Place ~= nil and b.Place ~= nil)) then
						return a.Place < b.Place;
					else
						return false
					end
				end)
				
				for i, Entry in ipairs(Entries) do
					local UserTable = {}
					UserTable['id'] = Entry.FacebookID
					UserTable['name'] = ''
					UserTable['score'] = math.floor(Entry.Score)
					UserTable['rank'] = 1
					
					if (Entry.FacebookID ~= 0) then
						if (FBManager.UserFriendsTable ~= nil) then
							FacebookUser = FBManager.UserFriendsTable[Entry.FacebookID]
							if (FacebookUser ~= nil) then
								UserTable['picture'] = FacebookUser['picture']
							end
						elseif (FBManager.UserinfoTable ~= nil) then
							if (Entry.FacebookID == FBManager.UserinfoTable['id']) then
								UserTable['picture'] = FBManager.UserinfoTable['picture']
							end
						end
					end
					
					local Place = i
					
					local CarName = Translate(MonsterTruckCars.Cars[Entry.CarID].Title)
					ActiveMenu.HighscoreItemList:AddItem(ActiveMenu.HighscoreItems, MenuElements.HighScores(ActiveMenu.HighscoreItemList, "elements\\Highscore_Template.dcMenu", Place, Entry.Name, CarName, Entry.Score, Entry.LocalUser))
				end
				
				if (#ActiveMenu.HighscoreItemList.Items == 0) then
					ActiveMenu:SetText(Translate("menu_no_highscores_text"))
				end
				
				ActiveMenu.HighscoreItemList:Format(ALIGN_NS, 0, 0, 0, 0, 0, 0, 0, 0, 0, #ActiveMenu.HighscoreItemList.Items)
				ActiveMenu.LoadingImage:SetAlpha(0)

				for i,Button in ipairs(buttons) do
					Button:Enable(false)
				end
				ActiveMenu.ButtonItems:SetSelection(ActiveMenu.FriendsButton)
			else
				FailureFunction()
			end
		end
		
		ActiveMenu.LoadingImage:SetAlpha(1)
		
		local UserID = 0
		if (FBManager:IsLoginValid()) then
			UserID = FBManager.UserinfoTable['HighscoresID']
		else
			UserID = Scores.UserID
		end
		
		local Script = ActiveMenu.CurrentRace.Name
		ScoresUser:AddScore(UserID, Script, 0, 0, SuccessFunction, FailureFunction)
	end
	-- ================================
	
	-- Init
	for i,Button in ipairs(buttons) do
		Button:Enable(false)
	end
	ActiveMenu.ButtonItems:SetSelection(ActiveMenu.LocalButton)
	ActiveMenu:ShowLocalList()

end

function ShowConfirm_NoPower()
	local TimeNeeded = FormatTimeShort(FuelEnergy:GetWaitTime())
	--local TotalRecharge_GoldPrice = 100
	local TotalRecharge_GoloPrice = 1
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_nopower_title"))
	--ActiveMenu:SetText(string.format(Translate("confirm_nopower_text"), TimeNeeded, TotalRecharge_GoldPrice))
	ActiveMenu:SetText(string.format(Translate("confirm_nopower_text"), TimeNeeded, TotalRecharge_GoloPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_recharge"))
	ActiveMenu:SetCancelText(Translate("menu_wait"))
	
	local ClockThread = GameThread:Spawn(function()
		while(true)do
			local TimeNeeded = FormatTimeShort(FuelEnergy:GetWaitTime())
			
			if (FuelEnergy:IsPowerAvailable() or FuelEnergy:GetWaitTime() == 0) then
				ActiveMenu.YesButtonPanel:Clear()
				
				local Text =  ""
				if (FuelEnergy:GetTotalCapacity() > 1) then
					Text = tostring(FuelEnergy:GetTotalCapacity()) .. " " .. Translate("word_shows")
				else
					Text = "1 " .. Translate("word_show")
				end
				
				ActiveMenu:SetTitle(Translate("confirm_gotpower_title"))
				ActiveMenu:SetText(string.format(Translate("confirm_gotpower_text"), Text))
				ActiveMenu:SetCancelText(Translate("button_proceed"))
				
				GameThread:Stop(ClockThread)
				ClockThread = nil
			else
				ActiveMenu:SetText(string.format(Translate("confirm_nopower_text"), TimeNeeded, TotalRecharge_GoloPrice))
			end
			
			GameThread:Wait(1)
		end
	end)
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
			
			BuyPower(TotalRecharge_GoloPrice)
		end)
	end
end

function ShowConfirm_BuyPower()
	--local TotalRecharge_GoldPrice = 100
	local TotalRecharge_GoloPrice = 1
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_buypower_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_buypower_text"), TotalRecharge_GoloPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_recharge"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			BuyPower(TotalRecharge_GoloPrice)
		end)
	end
	
end

function ShowConfirm_GameModeInfo()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(string.format(Translate("confirm_gamemodeinfo_title"), ActiveGameMode.Name))
	ActiveMenu:SetText(Translate("menu_gametype_" .. ActiveGameMode.Name .. "_discription"))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_okcool"))
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			BuyPower(TotalRecharge_GoloPrice)
		end)
	end
end

function ShowConfirm_Refuel()
	local TimeNeeded = FormatTimeShort(FuelEnergy:GetWaitTime())
	local FuelPrice = 1
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_nofuel_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_nofuel_text"), TimeNeeded, FuelPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_refill"))
	ActiveMenu:SetCancelText(Translate("menu_wait"))
	
	local ClockThread = GameThread:Spawn(function()
		while(true)do
			local TimeNeeded = FormatTimeShort(FuelEnergy:GetWaitTime())
			
			if (FuelEnergy:IsPowerAvailable() or FuelEnergy:GetWaitTime() <= 0) then
				ActiveMenu.YesButtonPanel:Clear()
				
				local Text =  ""
				if (FuelEnergy:GetCurrentCapacity() > 1) then
					Text = tostring(FuelEnergy:GetCurrentCapacity()) .. " " .. Translate("word_races")
				else
					Text = "1 " .. Translate("word_race")
				end
				
				ActiveMenu:SetTitle(Translate("confirm_gotfuel_title"))
				if (FuelEnergy:GetCurrentCapacity() >= FuelEnergy:GetTotalCapacity()) then
					ActiveMenu:SetText(string.format(Translate("confirm_gotfuel_text"), Text, TimeNeeded))
				else
					ActiveMenu:SetText(string.format(Translate("confirm_gotfuel_refill_text"), Text, TimeNeeded))
				end
				ActiveMenu:SetCancelText(Translate("button_proceed"))
				
				GameThread:Stop(ClockThread)
				ClockThread = nil
			else
				ActiveMenu:SetText(string.format(Translate("confirm_nofuel_text"), TimeNeeded, FuelPrice))
			end
			
			GameThread:Wait(1)
		end
	end)
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
			
			BuyFuel(FuelPrice, PreviousMenu._Class)
		end)
	end
	
end

function ShowConfirm_NoGold(CreditsNeeded, PrevMenu, CancelAction)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_nogold_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_nogold_text"), CreditsNeeded))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_getsome"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	if (CancelAction ~= nil) then
		ActiveMenu.CancelAction = function()
			CancelAction()
		end
	end
	
	ActiveMenu.ProceedAction = function()
		ActiveMenu.PreviousMenu = nil
		
		GameThread:Spawn(function()
			PreviousMenu:HideMenu(true)
			if (PreviousMenu.DeleteButtons ~= nil) then
				PreviousMenu:DeleteButtons()
			end
			
			--ActiveMenu = CoinShop(Garage, PreviousMenu)
			ActiveMenu = CoinShop(PrevMenu, PrevMenu.PreviousMenu)
		end)
	end
end

function ShowConfirm_NoGolo(CreditsNeeded, PrevMenu)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_nogolo_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_nogolo_text"), CreditsNeeded))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_getsome"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function()
		ActiveMenu.PreviousMenu = nil
		
		GameThread:Spawn(function()
			PreviousMenu:HideMenu(true)
			if (PreviousMenu.DeleteButtons ~= nil) then
				PreviousMenu:DeleteButtons()
			end
			
			print("^3" .. tostring(PrevMenu))
			
			--ActiveMenu = CoinShop(Garage, PreviousMenu)
			ActiveMenu = CoinShop_Golo(PrevMenu, PrevMenu.PreviousMenu)
			
			if (PrevMenu == GameModeSelect) then
				ReDirected = true
			end
		end)
	end
end

function ShowConfirm_TrackNotAvailable(Race)
	local CurrentTime = os.time()
	local TimeNeeded = Race.DeliveryTime - CurrentTime
	local HurryUp_GoloPrice = ClampMax(math.ceil(Race.ShopPriceCoins / 1000), 10)
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_TrackNotAvailable_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_TrackNotAvailable_text"), os.date("%M:%S", TimeNeeded), HurryUp_GoloPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_impatient"))
	ActiveMenu:SetCancelText(Translate("menu_wait"))
	
	local ClockThread = GameThread:Spawn(function()
		while(true)do
			local CurrentTime = os.time()
			local TimeNeeded = Race.DeliveryTime - CurrentTime
			
			if (TimeNeeded == nil or TimeNeeded <= 0) then
				Race.ShowPopup = false
				
				ActiveMenu.YesButtonPanel:Clear()
				local Text = "^3" .. Translate(Race.Title)
				ActiveMenu:SetTitle(Translate("confirm_delivery_track_title"))
				ActiveMenu:SetText(string.format(Translate("confirm_delivery_track_shorttext"), Text))
				ActiveMenu:SetCancelText(Translate("button_proceed"))
				
				GameThread:Stop(ClockThread)
				ClockThread = nil
			else
				ActiveMenu:SetText(string.format(Translate("confirm_TrackNotAvailable_text"), os.date("%M:%S", TimeNeeded), HurryUp_GoloPrice))
			end
			
			GameThread:Wait(1)
		end
	end)
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
			
			FastDeliverRace(Race, HurryUp_GoloPrice, true)
			
			if (PreviousMenu.UpdateRaces ~= nil) then
				PreviousMenu:UpdateRaces(true)
			end
		end)
	end
end

function ShowConfirm_TrackLocked(Race, Price, UseGolo)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_tracklocked_title"))
	
	local StandardText = Translate("confirm_tracklocked_text")
	if (UseGolo) then
		ActiveMenu:SetText(StandardText .. string.format(Translate("confirm_tracklocked_text_golo"), Price))
	else
		ActiveMenu:SetText(StandardText .. string.format(Translate("confirm_tracklocked_text_gold"), Price))
	end
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("menu_UNLOCK"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function()
		BuyRace(Race, UseGolo)
		
		if (PreviousMenu.UpdateRaces ~= nil) then
			PreviousMenu:UpdateRaces(true)
		end
	end
end

function ShowConfirm_CarNotAvailable(Car)
	local CurrentTime = os.time()
	local TimeNeeded = Car.DeliveryTime - CurrentTime
	local HurryUp_GoloPrice = 1 --ClampMax(math.ceil(Car.ShopPriceCoins / GetPropertyData("GearPriceMultiplier")), 10)
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_CarNotAvailable_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_CarNotAvailable_text"), os.date("%M:%S", TimeNeeded), HurryUp_GoloPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_impatient"))
	ActiveMenu:SetCancelText(Translate("menu_wait"))
	
	local ClockThread = GameThread:Spawn(function()
		while(true)do
			local CurrentTime = os.time()
			local TimeNeeded = Car.DeliveryTime - CurrentTime
			
			if (TimeNeeded == nil or TimeNeeded <= 0) then
				Car.ShowPopup = false
				
				ActiveMenu.YesButtonPanel:Clear()
				local Text = "^3" .. Translate(Car.Title)
				ActiveMenu:SetTitle(Translate("confirm_delivery_car_title"))
				ActiveMenu:SetText(string.format(Translate("confirm_delivery_car_shorttext"), Text))
				ActiveMenu:SetCancelText(Translate("button_proceed"))
				
				GameThread:Stop(ClockThread)
				ClockThread = nil
			else
				ActiveMenu:SetText(string.format(Translate("confirm_CarNotAvailable_text"), os.date("%M:%S", TimeNeeded), HurryUp_GoloPrice))
			end
			
			GameThread:Wait(1)
		end
	end)
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
			
			--instant delivery code here
			FastDeliverCar(Car, HurryUp_GoloPrice, true)
			
			if (PreviousMenu.UpdateCarButtons ~= nil) then
				PreviousMenu:UpdateCarButtons()
			end
			
			if (PreviousMenu.UpdateWheelButtons ~= nil) then
				PreviousMenu:UpdateWheelButtons()
			end
			
			if (PreviousMenu.UpdateCategoryButtons ~= nil) then
				PreviousMenu:UpdateCategoryButtons()
			end
		end)
	end
end

function ShowConfirm_BuyCar(Car, Price, Gears)
	print("^21")
	local Name = Translate(Car.Title)
	local Compliment = Translate(Car.Compliment)
	local HurryUp_GoldPrice = 100

	local EnableFastDelivery = false
	if (VirtualCoins:GetCoinCount() >= (Price + HurryUp_GoldPrice)) then
		--EnableFastDelivery = true
	end
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu, EnableFastDelivery)
	ActiveMenu:SetTitle(Translate("confirm_buycar_title"))
	if (not Gears) then
		ActiveMenu:SetText(string.format(Translate("confirm_buycar_text_gold"), Compliment, Name, Price))
	else
		ActiveMenu:SetText(string.format(Translate("confirm_buycar_text_gears"), Compliment, Name, Price))
	end
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	ActiveMenu:SetCheckBoxText(Translate("button_fastDelivery") .. " ^3(+ " .. tostring(HurryUp_GoldPrice) .. " " .. Translate("menu_gold") .. ")")
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
	
		ActiveMenu.ProceedAction = nil
		AnalyticsSendEvent("Car", "Bought", Name, "1")
		
		local TotalPrice = Price
		if (IsChecked) then
			TotalPrice = Price + HurryUp_GoldPrice
		end
		if (Gears) then
			IsChecked = true
		end
		
		BuyCar(Car, TotalPrice, IsChecked, Gears)
		
		if (not FirstCarBought) then
			FirstCarBought = true
		end
		
		--if (PreviousMenu.UpdateCategoryButtons ~= nil) then
			--PreviousMenu:UpdateCategoryButtons()
		--end
		
		if (PreviousMenu.ShowCategoryButtons ~= nil) then
			PreviousMenu:ShowCategoryButtons()
		end
		--ShowCategoryButtons
		
		--if (PreviousMenu.HideCarButtons ~= nil and PreviousMenu.ShowCategoryButtons ~= nil) then
			--PreviousMenu:HideCarButtons()
			--PreviousMenu:ShowCategoryButtons()
		--end
		
		--if (PreviousMenu.Items_Cars ~= nil) then
			--PreviousMenu.Items_Cars = nil
		--end
	end
end

function ShowConfirm_BuyCar_Choice(Car)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu, false)
	ActiveMenu:SetTitle(Translate("confirm_buycar_title"))
	
	local CoinsPrice = Car.ShopPriceCoins
	local GearsPrice = Car.ShopPriceSpecialCoins
	
	if (CoinsPrice > 0 and GearsPrice > 0) then
		ActiveMenu:SetText("Would you like to use coins or gears?")
	else
		ActiveMenu:SetText(string.format(Translate("Would you like to buy %s?"), Translate(Car.Name.."_Name")))
	end
	
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	ActiveMenu.YesButtonPanel:Clear()
	
	local InternalItemsPanel = ActiveMenu.Menu:GetChild("Items")
	InternalItemsPanel:Clear()
	
	
	if (CoinsPrice > 0) then
		ActiveMenu.ProceedButtonGold = ActiveMenu.Items:AddItem(InternalItemsPanel, MenuElements.LabelGarageUnlock(ActiveMenu.Items, "Elements\\Garage_Unlock_Coins_Button.dcMenu", "button_unlock", "0.00"))
	end
	
	if (GearsPrice > 0) then
		ActiveMenu.ProceedButtonGear = ActiveMenu.Items:AddItem(InternalItemsPanel, MenuElements.LabelGarageUnlock(ActiveMenu.Items, "Elements\\Garage_Unlock_Coins_Button.dcMenu", "button_unlock", "0.00"))
	end
	
	if (ActiveMenu.ProceedButtonGold ~= nil) then
		ActiveMenu.ProceedButtonGold:SetText("", CoinsPrice, false)
		ActiveMenu.ProceedButtonGold:Format(ALIGN_WE, 20, 0, 0, 0, Left, Center, Center, 0.6)
	end
	
	if (ActiveMenu.ProceedButtonGear ~= nil) then
		ActiveMenu.ProceedButtonGear:SetText("", GearsPrice, true)
		ActiveMenu.ProceedButtonGear:Format(ALIGN_WE, -20, 0, 0, 0, Right, Center, Center, 0.6)
	end
	
	if (ActiveMenu.ProceedButtonGold ~= nil) then
		ActiveMenu.ProceedButtonGold.OnReleased = function()
			ActiveMenu.ProceedButtonGold.OnReleased = nil
			
			GameThread:Spawn(function()
				ActiveMenu:HideMenu()
				
				local genoegcredits = (VirtualCoins:GetCoinCount() >= CoinsPrice)
				if not genoegcredits then																--==== Player has not enough gold, go buy some gold
					local CreditsNeeded = (CoinsPrice - VirtualCoins:GetCoinCount())
					ShowConfirm_NoGold(CreditsNeeded, CarSelect)
				else
					BuyCar(Car, CoinsPrice, false, false)
				end
			end)
		end
	end
	
	if (ActiveMenu.ProceedButtonGear ~= nil) then
		ActiveMenu.ProceedButtonGear.OnReleased = function()
			ActiveMenu.ProceedButtonGear.OnReleased = nil
			
			GameThread:Spawn(function()
				ActiveMenu:HideMenu()
				
				local genoegcredits = (VirtualCoins:GetGoloCoinCount() >= GearsPrice)
				if not genoegcredits then																--==== Player has not enough gold, go buy some gold
					local CreditsNeeded = (GearsPrice - VirtualCoins:GetGoloCoinCount())
					ShowConfirm_NoGolo(CreditsNeeded, CarSelect)
				else
					BuyCar(Car, GearsPrice, true, true)
				end
			end)
		end
	end
end

function ShowConfirm_Mechanic(Price, Car)	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_mechanic_title"))
	--ActiveMenu:SetText(string.format(Translate("confirm_mechanic_text"), Price, 99999))
	ActiveMenu:SetText(string.format(Translate("confirm_mechanic_text"), Price))
	--ActiveMenu:SetText(Translate("confirm_mechanic_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function()
		ActiveMenu.ProceedAction = nil
		
		if (ActiveMenu.DisableCategoryButtons ~= nil) then
			ActiveMenu:DisableCategoryButtons()
		end
		
		HireMechanic(Price, Car, CarSelect)
	end
end

function ShowConfirm_WheelNotAvailable(Wheel)
	local CurrentTime = os.time()
	local TimeNeeded = Wheel.DeliveryTime - CurrentTime
	local HurryUp_GoloPrice = ClampMax(math.ceil(Wheel.ShopPriceCoins / 1000), 10)
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_WheelNotAvailable_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_WheelNotAvailable_text"), os.date("%M:%S", TimeNeeded), HurryUp_GoloPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_impatient"))
	ActiveMenu:SetCancelText(Translate("menu_wait"))
	
	local ClockThread = GameThread:Spawn(function()
		while(true)do
			local CurrentTime = os.time()
			local TimeNeeded = Wheel.DeliveryTime - CurrentTime
			
			if (TimeNeeded == nil or TimeNeeded <= 0) then
				Wheel.ShowPopup = false
				
				ActiveMenu.YesButtonPanel:Clear()
				local Text = "^3" .. Translate(Wheel.Title)
				ActiveMenu:SetTitle(Translate("confirm_delivery_car_title"))
				ActiveMenu:SetText(string.format(Translate("confirm_delivery_wheel_shorttext"), Text))
				ActiveMenu:SetCancelText(Translate("button_proceed"))
				
				GameThread:Stop(ClockThread)
				ClockThread = nil
			else
				ActiveMenu:SetText(string.format(Translate("confirm_WheelNotAvailable_text"), os.date("%M:%S", TimeNeeded), HurryUp_GoloPrice))
			end
			
			GameThread:Wait(1)
		end
	end)
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			GameThread:Stop(ClockThread)
			ClockThread = nil
			
			--instant delivery code here
			FastDeliverWheel(Wheel, HurryUp_GoloPrice, true)
			
			if (PreviousMenu.UpdateCarButtons ~= nil) then
				PreviousMenu:UpdateCarButtons()
			end
			
			if (PreviousMenu.UpdateWheelButtons ~= nil) then
				PreviousMenu:UpdateWheelButtons()
			end
		end)
	end
end

function ShowConfirm_BuyWheel(Wheel, Price)
	local Name = Translate(Wheel.Title)
	local HurryUp_GoldPrice = 100

	local EnableFastDelivery = false
	if (VirtualCoins:GetCoinCount() >= (Price + HurryUp_GoldPrice)) then
		--EnableFastDelivery = true
	end
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu, EnableFastDelivery)
	ActiveMenu:SetTitle(Translate("confirm_buywheel_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_buywheel_text"), Name, Price))
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	ActiveMenu:SetCheckBoxText(Translate("button_fastDelivery") .. " ^3(+ " .. tostring(HurryUp_GoldPrice) .. " " .. Translate("menu_gold") .. ")")
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		local TotalPrice = Price
		if (IsChecked) then
			TotalPrice = Price + HurryUp_GoldPrice
		end
		BuyWheel(Wheel, TotalPrice, IsChecked)
		
		if (PreviousMenu.UpdateCarButtons ~= nil) then
			PreviousMenu:UpdateCarButtons()
		end
		
		if (PreviousMenu.UpdateWheelButtons ~= nil) then
			PreviousMenu:UpdateWheelButtons()
		end
	end
end

function ShowConfirm_BuyUpgradeItem(UpgradeItem, Price, car, EnableFastDelivery, UseGolo)
	local Name = Translate(UpgradeItem.Name)
	local HurryUp_GoldPrice = 100
	--if (VirtualCoins:GetCoinCount() < (Price + HurryUp_GoldPrice)) then
		EnableFastDelivery = false
	--end
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu, EnableFastDelivery)
	ActiveMenu:SetTitle(Translate("confirm_buyupgradeitem_title"))
	
	local StandardText = string.format(Translate("confirm_buyupgradeitem_text"), Name)
	if (UseGolo) then
		ActiveMenu:SetText(StandardText .. " " .. string.format(Translate("confirm_buyupgradeitem_text_golo"), Price))
	else
		ActiveMenu:SetText(StandardText .. " " .. string.format(Translate("confirm_buyupgradeitem_text_gold"), Price))
	end
	
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("menu_button_order"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	ActiveMenu:SetCheckBoxText(Translate("button_fastDelivery") .. " ^3(+ " .. tostring(HurryUp_GoldPrice) .. " " .. Translate("menu_gold") .. ")")
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		ActiveMenu.ProceedAction = nil
			
		local TotalPrice = Price
		if (IsChecked) then
			TotalPrice = Price + HurryUp_GoldPrice
		end
		BuyUpgradeItem(UpgradeItem, TotalPrice, car, IsChecked, UseGolo)
		
		print(car.Upgrades[UpgradeItem.Name].BoughtLevel)
		--if (GetPlatform() == "Android") then
		
			if(UpgradeItem == UpgradeItem_Weight) then
				AnalyticsSendEvent("Upgrade", "Weight", "ItemLevel: " .. car.Upgrades[UpgradeItem.Name].BoughtLevel, "1")
			elseif(UpgradeItem == UpgradeItem_Suspension) then
				AnalyticsSendEvent("Upgrade", "Suspension", "ItemLevel: " .. car.Upgrades[UpgradeItem.Name].BoughtLevel, "1")
			elseif(UpgradeItem == UpgradeItem_AirShocks) then
				AnalyticsSendEvent("Upgrade", "AirShocks", "ItemLevel: " .. car.Upgrades[UpgradeItem.Name].BoughtLevel, "1")
			elseif(UpgradeItem == UpgradeItem_Engine) then
				AnalyticsSendEvent("Upgrade", "Engine", "ItemLevel: " .. car.Upgrades[UpgradeItem.Name].BoughtLevel, "1")
			elseif(UpgradeItem == UpgradeItem_Armor) then
				AnalyticsSendEvent("Upgrade", "Armor", "ItemLevel: " .. car.Upgrades[UpgradeItem.Name].BoughtLevel, "1")
			elseif(UpgradeItem == UpgradeItem_Tires) then
				AnalyticsSendEvent("Upgrade", "Tires", "ItemLevel: " .. car.Upgrades[UpgradeItem.Name].BoughtLevel, "1")
			end
			
			AnalyticsSendEvent("Car", "Upgraded", car.Name, "1")
		--end
		
		if (PreviousMenu.UpdateButtons ~= nil) then
			PreviousMenu:UpdateButtons()
		end
		
		if (PreviousMenu.CalcStats ~= nil) then
			PreviousMenu:CalcStats()
		end
	end
end

function ShowConfirm_BuyUpgradeItemPack(Car)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu, false)
	ActiveMenu:SetTitle(Translate("confirm_buyboost_title"))
	
	local LowestLevel = 9999
	for i, Upgrade in pairs(Car.Upgrades) do
		LowestLevel = Min(LowestLevel, Upgrade.CurrentLevel)
	end
	
	local TotalCoinsPrice = 0
	local TotalGearsPrice = 0
	local BoostPackage = {}
	for i, Upgrade in pairs(Car.Upgrades) do
		if (Upgrade.CurrentLevel <= LowestLevel) then
			
			for j, ShopUpgradeItem in ipairs(ShopUpgradeItems) do
				if (ShopUpgradeItem.Name == i) then
					print(ShopUpgradeItem.Name, i)
					local CoinsPrice = ShopUpgradeItem:GetPrice(Car)
					if (CoinsPrice ~= nil) then
						TotalCoinsPrice = TotalCoinsPrice + CoinsPrice
					end
					
					local GearsPrice = ShopUpgradeItem:GetGoloPrice(Car)
					if (GearsPrice ~= nil) then
						TotalGearsPrice = TotalGearsPrice + GearsPrice
					end
					
					table.insert(BoostPackage, ShopUpgradeItem)
				end
			end
		end
	end
	
	-- alles wat geen gears kost, 1 gear laten kosten (overige gears kosten zo laten)
	if (TotalGearsPrice == 0) then
		TotalGearsPrice = 1
	end
	
	local CoinsPrice = TotalCoinsPrice
	local GearsPrice = TotalGearsPrice
	
	if (CoinsPrice > 0 and GearsPrice > 0) then
		ActiveMenu:SetText(Translate("confirm_buyboost_text_valutachoice"))
	else
		--ActiveMenu:SetText(string.format(Translate("Would you like to buy %s?"), Translate(Car.Name.."_Name")))
		ActiveMenu:SetText(Translate("confirm_buyboost_text"))
	end
	
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	ActiveMenu.YesButtonPanel:Clear()
	
	local InternalItemsPanel = ActiveMenu.Menu:GetChild("Items")
	InternalItemsPanel:Clear()
	
	
	if (CoinsPrice > 0) then
		ActiveMenu.ProceedButtonGold = ActiveMenu.Items:AddItem(InternalItemsPanel, MenuElements.LabelGarageUnlock(ActiveMenu.Items, "Elements\\Garage_Unlock_Coins_Button.dcMenu", "button_unlock", "0.00"))
	end
	
	if (GearsPrice > 0) then
		ActiveMenu.ProceedButtonGear = ActiveMenu.Items:AddItem(InternalItemsPanel, MenuElements.LabelGarageUnlock(ActiveMenu.Items, "Elements\\Garage_Unlock_Coins_Button.dcMenu", "button_unlock", "0.00"))
	end
	
	if (ActiveMenu.ProceedButtonGold ~= nil) then
		ActiveMenu.ProceedButtonGold:SetText("", CoinsPrice, false)
		ActiveMenu.ProceedButtonGold:Format(ALIGN_WE, 20, 0, 0, 0, Left, Center, Center, 0.6)
	end
	
	if (ActiveMenu.ProceedButtonGear ~= nil) then
		ActiveMenu.ProceedButtonGear:SetText("", GearsPrice, true)
		ActiveMenu.ProceedButtonGear:Format(ALIGN_WE, -20, 0, 0, 0, Right, Center, Center, 0.6)
	end
	
	if (ActiveMenu.ProceedButtonGold ~= nil) then
		ActiveMenu.ProceedButtonGold.OnReleased = function()
			ActiveMenu.ProceedButtonGold.OnReleased = nil
			
			GameThread:Spawn(function()
				ActiveMenu:HideMenu()
				
				local genoegcredits = (VirtualCoins:GetCoinCount() >= CoinsPrice)
				if not genoegcredits then																--==== Player has not enough gold, go buy some gold
					local CreditsNeeded = (CoinsPrice - VirtualCoins:GetCoinCount())
					ShowConfirm_NoGold(CreditsNeeded, CarSelect)
				else
					for i, Upgrade in pairs(BoostPackage) do
						local SplitCoinPrice = CoinsPrice / #BoostPackage
						BuyUpgradeItem(Upgrade, SplitCoinPrice, Car, false, false)
					end
				end
			end)
		end
	end
	
	if (ActiveMenu.ProceedButtonGear ~= nil) then
		ActiveMenu.ProceedButtonGear.OnReleased = function()
			ActiveMenu.ProceedButtonGear.OnReleased = nil
			
			GameThread:Spawn(function()
				ActiveMenu:HideMenu()
				
				local genoegcredits = (VirtualCoins:GetGoloCoinCount() >= GearsPrice)
				if not genoegcredits then																--==== Player has not enough gold, go buy some gold
					local CreditsNeeded = (GearsPrice - VirtualCoins:GetGoloCoinCount())
					ShowConfirm_NoGolo(CreditsNeeded, CarSelect)
				else
					for i, Upgrade in pairs(BoostPackage) do
						BuyUpgradeItem(Upgrade, GearsPrice, Car, true, true)
						GearsPrice = 0
					end
				end
			end)
		end
	end
end

function ShowConfirm_UpgradeItemPackNotAvailable(Car)
	local LowestCurrentLevel = 9999
	local LowestBoughtLevel = 9999
	local LowestDeliveryLevel = 9999
	local LowestDeliveryTime = 999999999999
	for i, Upgrade in pairs(Car.Upgrades) do
		LowestCurrentLevel = Min(LowestCurrentLevel, Upgrade.CurrentLevel)
		LowestBoughtLevel = Min(LowestBoughtLevel, Upgrade.BoughtLevel)
		LowestDeliveryLevel = Min(LowestDeliveryLevel, Upgrade.DeliveryLevel)
		LowestDeliveryTime = Min(LowestDeliveryTime, Upgrade.DeliveryTime)
	end
	
	local TotalCoinsPrice = 0
	local TotalGearsPrice = 0
	local BoostPackage = {}
	for i, Upgrade in pairs(Car.Upgrades) do
		if (Upgrade.BoughtLevel <= LowestDeliveryLevel) then
			
			for j, ShopUpgradeItem in ipairs(ShopUpgradeItems) do
				if (ShopUpgradeItem.Name == i) then
					--print(ShopUpgradeItem.Name, i)
					--local CoinsPrice = ShopUpgradeItem:GetPrice(Car)
					--if (CoinsPrice ~= nil) then
						--TotalCoinsPrice = TotalCoinsPrice + CoinsPrice
					--end
					
					local GearsPrice = ShopUpgradeItem:GetDeliveryGoloPrice(Car)
					if (GearsPrice ~= nil) then
						TotalGearsPrice = TotalGearsPrice + GearsPrice
					end
					
					table.insert(BoostPackage, ShopUpgradeItem)
				end
			end
		end
	end
	
	--Delivery van boostpack altijd 1 gear
	TotalGearsPrice = 1
	
	local CurrentTime = os.time()
	local TimeNeeded = 0
	--[[if (GetMinDeliveryTime() ~= nil) then
		TimeNeeded = GetMinDeliveryTime() - CurrentTime --10 --self.SelectedWheel.DeliveryTime - CurrentTime
	end]]
	--TimeNeeded = ActiveGameMode:GetSelectedCar().Upgrades[UpgradeItem.Name].DeliveryTime - CurrentTime
	TimeNeeded = LowestDeliveryTime - CurrentTime
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_ItemNotAvailable_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_ItemNotAvailable_text"), os.date("%M:%S", TimeNeeded), TotalGearsPrice))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_impatient"))
	ActiveMenu:SetCancelText(Translate("menu_wait"))
	
	local ClockThread = GameThread:Spawn(function()
		while(true)do
			local CurrentTime = os.time()
			--TimeNeeded = ActiveGameMode:GetSelectedCar().Upgrades[UpgradeItem.Name].DeliveryTime - CurrentTime --GetMinDeliveryTime() - CurrentTime --10 --self.SelectedWheel.DeliveryTime - CurrentTime
			TimeNeeded = LowestDeliveryTime - CurrentTime
			
			if (TimeNeeded == nil or TimeNeeded <= 0) then
				--self.SelectedWheel.ShowPopup = false
				
				ActiveMenu.YesButtonPanel:Clear()
				local Text = "^3" --.. Translate(self.SelectedWheel.Title)
				ActiveMenu:SetTitle(Translate("confirm_delivery_item_title"))
				--ActiveMenu:SetText(string.format(Translate("confirm_delivery_item_shorttext"), Text))
				ActiveMenu:SetText(Translate("confirm_delivery_item_shorttext"))
				ActiveMenu:SetCancelText(Translate("button_proceed"))
				if (ClockThread ~= nil) then
					GameThread:Stop(ClockThread)
					ClockThread = nil
				end
			else		
				ActiveMenu:SetText(string.format(Translate("confirm_ItemNotAvailable_text"), os.date("%M:%S", TimeNeeded), TotalGearsPrice))
			end
			GameThread:Wait(1)
		end
	end)
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()
			if (ClockThread ~= nil) then
				GameThread:Stop(ClockThread)
				ClockThread = nil
			end
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function(Button, IsChecked)
			if (ClockThread ~= nil) then
				GameThread:Stop(ClockThread)
				ClockThread = nil
			end
			
			print('fast deliver UpgradeItem ') -- .. UpgradeItem.Name)

			local CoinsPrice = TotalGearsPrice
			if (VirtualCoins:BuyGoloProduct(CoinsPrice, function()
			
				--instant delivery code here
				--UpgradeItem:BuyDirect(ActiveGameMode:GetSelectedCar())
				--UpgradeItem:CheckDeliveries(ActiveGameMode:GetSelectedCar())
				
				--self:UpdateButtons()
				--self:CalcStats()
				
				for i, Upgrade in pairs(BoostPackage) do
					--local SplitCoinPrice = GearsPrice / #BoostPackage
					--BuyUpgradeItem(Upgrade, SplitCoinPrice, Car, true, false)
					
					Upgrade:BuyDirect(ActiveGameMode:GetSelectedCar())
					Upgrade:CheckDeliveries(ActiveGameMode:GetSelectedCar())
				end
				
				WriteGameSettings()
				VirtualCoins:UpdateCoincount()
				--self:UpdateMenuStats()
			end)) then
				--========
				local FailureFunction = function(Data)
					print("Error sending BoughtCarUpgradeDelivery stats")
				end
				
				local SuccessFunction = function(Data)
					print('AddBoughtCarUpgradeDeliveryStats')
				end
				
				local UserID = 0
				if (FBManager:IsLoginValid()) then
					UserID = FBManager.UserinfoTable['HighscoresID']
				else
					UserID = Scores.UserID
				end
				
				local Car = ActiveGameMode:GetSelectedCar()
				local CarID = IndexInTable(MonsterTruckCars.Cars, Car) + 1
				local CarUpgradeTypeID = 1
				
				--[[ if (UpgradeItem.Name == "garage_weight") then
					CarUpgradeTypeID = 1
				elseif (UpgradeItem.Name == "garage_suspension") then
					CarUpgradeTypeID = 2
				elseif (UpgradeItem.Name == "garage_airshocks") then
					CarUpgradeTypeID = 3
				elseif (UpgradeItem.Name == "garage_engine") then
					CarUpgradeTypeID = 4
				elseif (UpgradeItem.Name == "garage_armor") then
					CarUpgradeTypeID = 5
				elseif (UpgradeItem.Name == "garage_tires") then
					CarUpgradeTypeID = 6
				end
				--========
				
				local CarUpgradeLevel = Car.Upgrades[UpgradeItem.Name].BoughtLevel

				local CoinsUsed = 0
				local GearsUsed = CoinsPrice
				
				ScoresUser:AddBoughtCarUpgradeDeliveryStats(UserID, CarID, CarUpgradeTypeID, CarUpgradeLevel, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction) ]]
			else
				GameThread:Spawn(function()
					local CreditsNeeded = (TotalGearsPrice - VirtualCoins:GetGoloCoinCount())
					ShowConfirm_NoGolo(CreditsNeeded, CarSelect)
				end)
			end
		end)
	end
end

function ShowConfirm_BuyPowerupItem(UpgradeItem, Price, UseGolo)
	local Name = Translate(UpgradeItem.Name)
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_buypowerupitem_title"))
	
	local StandardText = string.format(Translate("confirm_buypowerupitem_text"), Name)
	if (UseGolo) then
		ActiveMenu:SetText(StandardText .. " " .. string.format(Translate("confirm_buyupgradeitem_text_golo"), Price))
	else
		ActiveMenu:SetText(StandardText .. " " .. string.format(Translate("confirm_buyupgradeitem_text_gold"), Price))
	end
	
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("menu_button_order"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function(Button)
	
		ActiveMenu.ProceedAction = nil

		AnalyticsSendEvent("PowerUp", "Bought", UpgradeItem.Name, "1")
		
		local TotalPrice = Price
		BuyPowerupItem(UpgradeItem, TotalPrice, UseGolo)
		
		if (PreviousMenu.UpdateButtons ~= nil) then
			PreviousMenu:UpdateButtons()
		end
	end
end

--function ShowConfirm_UpgradeItemNotAvailable(UpgradeItem)
--end

function ShowConfirm_NeedBatteryrackUpgrade()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_needbatteryrackupgrade_title"))
	ActiveMenu:SetText(Translate("confirm_needbatteryrackupgrade_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Daniela"], "DANIELA")
	ActiveMenu:SetCancelText(Translate("menu_ok"))
	
	ActiveMenu.YesButtonPanel:Clear()
	
	ActiveMenu.ProceedAction = function()
		--
	end
end


function ShowConfirm_GotoPowerUp()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_GotoPowerUp_title"))
	ActiveMenu:SetText(Translate("confirm_GotoPowerUp_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Daniela"], "DANIELA")
	ActiveMenu:SetCancelText(Translate("menu_NOTHANKS"))
	ActiveMenu:SetProceedText(translate("menu_OK"))
	
	ActiveMenu.ProceedAction = function()
		-- Eerst menu weg
		CutsceneBars = nil
		ActiveMenu.Menu:SetColor(Invisible)
		ClearMenuStack()
		ActiveMenu = nil
		CutsceneBars = nil
		HUD = nil

		-- Dan pauze uit
		var("IntPause", 0)

		-- Dan de prepare functie zetten
		PrepareScript = function()
			ActiveMission = MenuMission()
			
			ActiveMenu = UpgradesMenu(nil)
		end

		-- En dan pas laden
		loadscript("Menu")
	end
end

function ShowConfirm_NeedRequirement(Text)
	
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_needrequirements_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_needrequirements_text"), Text))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], translate("character_Johnny"))
	ActiveMenu:SetProceedText(translate("menu_OK"))
	ActiveMenu:SetCancelText(translate("menu_NOTHANKS"))
	
	ActiveMenu.ProceedAction = function()
		--self:Goto(Garage_Battery)
		
		GameThread:Spawn(function()
			--ActiveMenu = Target(Garage)
			ActiveMenu = Garage_Battery(Garage)
			
			PreviousMenu:HideMenu()
			
			if (PreviousMenu.DeleteButtons ~= nil) then
				PreviousMenu:DeleteButtons()
			end
		end)
	end
	
end

function ShowConfirm_BoringCar(StartFunction)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_boring_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_boring_text"), Text))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], Translate("Character_Johnny"))
	ActiveMenu:SetProceedText(Translate("menu_showme"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.CancelAction = function()
		GameThread:Spawn(function()			
			StartFunction()
		end)
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			ActiveMenu = Garage_Cars(Garage)
			PreviousMenu:HideMenu()
		end)
	end
end

function ShowConfirm_GoogleRateGame()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_googlerategame_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_googlerategame_text"), Text))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], Translate("Character_Johnny"))
	ActiveMenu:SetProceedText(string.upper(Translate("button_yes")))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.CancelAction = function()
		WriteGameSettings()
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			if (GetPlatform() == "Win32") then
				openwebsite("http://play.google.com/store/apps/details?id=com.gamemill.monsterjam")
			elseif (GetPlatform() == "Android") then
				openwebsite("market://details?id=com.gamemill.monsterjam")
			end
			
			GameThread:Sleep(1.5)
			
			if (FuelEnergy ~= nil) then
				FuelEnergy.LastRateDialog = 9999999
			end
			VirtualCoins.IsRated = true
			
			WriteGameSettings()
		end)
	end
end

function ShowConfirm_iOSRateGame()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_googlerategame_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_googlerategame_text"), Text))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], Translate("Character_Johnny"))
	ActiveMenu:SetProceedText(string.upper(Translate("button_yes")))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.CancelAction = function()
		WriteGameSettings()
	end
	
	ActiveMenu.ProceedAction = function()
		GameThread:Spawn(function()
			openwebsite("itms-apps://itunes.apple.com/WebObjects/MZStore.woa/wa/viewContentsUserReviews?id=781596959&onlyLatestVersion=true&pageNumber=0&sortOrdering=1&type=Purple+Software")
			
			GameThread:Sleep(1.5)
			
			if (FuelEnergy ~= nil) then
				FuelEnergy.LastRateDialog = 9999999
			end
			VirtualCoins.IsRated = true
			
			WriteGameSettings()
		end)
	end
end

function ShowConfirm_ChooseControls()
	PreviousMenu = nil
	ActiveMenu = ConfirmMenu(PreviousMenu, true, "Confirm_ChooseControls", "MenuButton_Small_ControlTilt", "MenuButton_Small_ControlScreen")
	ActiveMenu:SetTitle(Translate("confirm_choosecontrols_title"))
	--ActiveMenu:SetText(Translate("confirm_choosecontrols_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Grace"], Translate("character_Grace"))
	ActiveMenu:SetProceedText(Translate("button_tilt_control"))
	ActiveMenu:SetCancelText(Translate("button_screen_control"))
	ActiveMenu:SetCheckBoxText(Translate("button_donotshowagain"))
	ActiveMenu.YesButton:Activate()
	ActiveMenu.YesButton:Deactivate()
	
	--Tilt Control
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		ActiveMenu.ProceedAction = nil
		ActiveMenu.CancelAction = nil
		
		var("UseAccelerometer", 1)
		
		if (IsChecked) then
			var("SkipChooseControls", 1)
		end
		configsave()
		
		ActiveMenu = GameModeSelect()
	end
	
	--Screen Control
	ActiveMenu.CancelAction = function(Button, IsChecked)
		ActiveMenu.ProceedAction = nil
		ActiveMenu.CancelAction = nil
		
		var("UseAccelerometer", 0)
		
		if (IsChecked) then
			var("SkipChooseControls", 1)
		end
		configsave()
		
		ActiveMenu = GameModeSelect()
	end
end

-- ===========================
-- Expert tips
-- ===========================

function ShowConfirm_ExpertTip_Shows()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_experttip_shows_title"))
	ActiveMenu:SetText(Translate("confirm_experttip_shows_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], Translate("character_Johnny"))
	ActiveMenu:SetCancelText(Translate("menu_thanks"))
	
	ActiveMenu.YesButtonPanel:Clear()
end

function ShowConfirm_ExpertTip_Moves()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_experttip_moves_title"))
	ActiveMenu:SetText(Translate("confirm_experttip_moves_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], Translate("character_Johnny"))
	ActiveMenu:SetCancelText(Translate("menu_thanks"))
	
	ActiveMenu.YesButtonPanel:Clear()
end

function ShowConfirm_ThanksForPurchase()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_thanksforpurchase_title"))
	ActiveMenu:SetText(Translate("confirm_thanksforpurchase_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetCancelText(Translate("menu_ok"))
	
	ActiveMenu.YesButtonPanel:Clear()
end

function ShowConfirm_ExpertTip_Respect()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_experttip_respect_title"))
	ActiveMenu:SetText(Translate("confirm_experttip_respect_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Johnny"], Translate("character_Johnny"))
	ActiveMenu:SetCancelText(Translate("menu_thanks"))
	
	ActiveMenu.YesButtonPanel:Clear()
end

-- ===========================
-- Special offers
-- ===========================

function ShowConfirm_5DaysIdle(FreeCoins)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_5days_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_5days_text"), FreeCoins))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetCancelText(Translate("menu_okcool"))
	ActiveMenu.YesButtonPanel:Clear()
end

function ShowConfirm_SpecialOffer_Car(Car, Discount, TotalPrice)
	
	local Name = Translate(Car.Title)
	local Discount = tostring(Discount * 100) .. "%"
	local Price = NumberCommas(TotalPrice)

	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_specialoffer_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_specialoffer_text"), Name, Discount, Name, Price))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		BuyCar(Car, TotalPrice, true)
		
		--[[ if (PreviousMenu.UpdateCarButtons ~= nil) then
			PreviousMenu:UpdateCarButtons()
		end
		
		if (PreviousMenu.HideCarButtons ~= nil and PreviousMenu.ShowCategoryButtons ~= nil) then
			PreviousMenu:HideCarButtons()
			PreviousMenu:ShowCategoryButtons()
		end
		
		if (PreviousMenu.Items_Cars ~= nil) then
			PreviousMenu.Items_Cars = nil
		end ]]
	end
end

function ShowConfirm_SpecialOffer_Track(Track, Discount, TotalPrice)
	
	local Name = Translate(Track.Title)
	local Discount = tostring(Discount * 100) .. "%"
	local Price = NumberCommas(TotalPrice)

	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_specialoffer_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_specialoffer_text"), Name, Discount, Name, Price))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		FastDeliverRace(Track, TotalPrice, false)
		
		--[[ if (PreviousMenu.UpdateCarButtons ~= nil) then
			PreviousMenu:UpdateCarButtons()
		end
		
		if (PreviousMenu.HideCarButtons ~= nil and PreviousMenu.ShowCategoryButtons ~= nil) then
			PreviousMenu:HideCarButtons()
			PreviousMenu:ShowCategoryButtons()
		end
		
		if (PreviousMenu.Items_Cars ~= nil) then
			PreviousMenu.Items_Cars = nil
		end ]]
	end
end

function ShowConfirm_SpecialOffer_Wheel(Wheel, Discount, TotalPrice)
	
	local Name = Translate(Wheel.Title)
	local Discount = tostring(Discount * 100) .. "%"
	local Price = NumberCommas(TotalPrice)

	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_specialoffer_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_specialoffer_text"), Name, Discount, Name, Price))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		BuyWheel(Wheel, TotalPrice, true)
		
		--[[ if (PreviousMenu.UpdateCarButtons ~= nil) then
			PreviousMenu:UpdateCarButtons()
		end
		
		if (PreviousMenu.HideCarButtons ~= nil and PreviousMenu.ShowCategoryButtons ~= nil) then
			PreviousMenu:HideCarButtons()
			PreviousMenu:ShowCategoryButtons()
		end
		
		if (PreviousMenu.Items_Cars ~= nil) then
			PreviousMenu.Items_Cars = nil
		end ]]
	end
end

function ShowConfirm_SpecialOffer_UpgradeItem(UpgradeItem, Discount, TotalPrice)
	local Name = Translate(UpgradeItem.Name)
	local Discount = tostring(Discount * 100) .. "%"
	local Price = NumberCommas(TotalPrice)

	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_specialoffer_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_specialoffer_text"), Name, Discount, Name, Price))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_mine"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.ProceedAction = function(Button, IsChecked)
		BuyUpgradeItem(UpgradeItem, TotalPrice, ActiveGameMode:GetSelectedCar(), true, false)
		
		--[[ if (PreviousMenu.UpdateButtons ~= nil) then
			PreviousMenu:UpdateButtons()
		end
		
		if (PreviousMenu.CalcStats ~= nil) then
			PreviousMenu:CalcStats()
		end ]]
	end
end

-- ===========================
-- Daily Rewards
-- ===========================

function ShowConfirm_DailyReward_ReceiveReward(Text)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_DailyReward_ReceiveReward_title"))
	ActiveMenu:SetText(Text)
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	--ActiveMenu:SetProceedText(Translate("menu_thanks"))
	ActiveMenu:SetCancelText(Translate("menu_thanks"))
	ActiveMenu.YesButtonPanel:Clear()
	--ActiveMenu.NoButtonPanel:Clear()
	
	ActiveMenu.ProceedAction = function()
		--
	end
end

function ShowConfirm_DailyReward_Mulligan(ProceedAction, CancelAction, TotalGears)
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_DailyReward_Mulligan_title"))
	ActiveMenu:SetText(string.format(Translate("confirm_DailyReward_Mulligan_text"), TotalGears))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	ActiveMenu:SetProceedText(Translate("menu_okcool"))
	ActiveMenu:SetCancelText(Translate("menu_nothanks"))
	
	ActiveMenu.CancelAction = function()
		CancelAction()
	end
	
	ActiveMenu.ProceedAction = function()
		ProceedAction()
	end
end

function ShowConfirm_DailyReward_LostStreak()
	PreviousMenu = ActiveMenu
	ActiveMenu = ConfirmMenu(PreviousMenu)
	ActiveMenu:SetTitle(Translate("confirm_DailyReward_LostStreak_title"))
	ActiveMenu:SetText(Translate("confirm_DailyReward_LostStreak_text"))
	ActiveMenu:SetCharacter(CharacterTextures["Pete"], Translate("character_Pete"))
	--ActiveMenu:SetProceedText(Translate("menu_getsome"))
	ActiveMenu:SetCancelText(Translate("menu_ohno"))
	ActiveMenu.YesButtonPanel:Clear()
	
	ActiveMenu.ProceedAction = function()
		--
	end
end


-- dit moet ergens anders
function PlayerHasCar()
	for i, CarClass in ipairs(ActiveGameMode.CarClasses) do
		for j, Car in ipairs(CarClass.Cars) do
			if ((Car.Bought and Car.Available) or DebugUnlockAll) then
				return true
			end
		end
	end
	return false
end

-- Shop stuff
function BuyFuel(TotalRecharge_GoloPrice, PreviousMenu)
	if (VirtualCoins:BuyGoloProduct(TotalRecharge_GoloPrice, function()
		local AmountRefilled = FuelEnergy:GetTotalCapacity() - FuelEnergy.CurrentCapacity
		
		FuelEnergy:TotalRecharge()
		print("TotalRecharge!!!")
		
		FuelEnergy:SendFuelStats(AmountRefilled, 0, 0, TotalRecharge_GoloPrice)
		
		WriteGameSettings()
		
		VirtualCoins:UpdateCoincount()
	end)) then
		--okay
	else	
		--not enough coins or error
		local CreditsNeeded = (TotalRecharge_GoloPrice - VirtualCoins:GetGoloCoinCount())
		ShowConfirm_NoGolo(CreditsNeeded, PreviousMenu)
	end
end

function BuyRace(Race, Golo)
	print('buy track ' .. Race.Name)
	
	local BuyFunction = function()
		--Race.Available = true
		Race.Bought = true
		
		if (Golo) then
			Race.Available = true
		end
		
		Race.DeliveryTime = os.time() + CalcDeliveryTime(Race.ShopPriceCoins) --GlobalDeliveryTime_Race
		
		print('Bought "' .. Race.Name .. "'")
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	if (Golo) then
		local CoinsPrice = Race.ShopPriceGoloCoins
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			--
		end
	else
		local CoinsPrice = Race.ShopPriceCoins
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			--
		end
	end	
end

function FastDeliverRace(Race, TotalPrice, Golo)
	print('fast deliver track ' .. Race.Name)
	
	local BuyFunction = function()
	
		print('fast delivered "' .. Race.Name .. "'")
		
		Race.Bought = true
		Race.Available = true
		Race.DeliveryTime = os.time()
		
		--Apply Race selection
		--ActiveGameMode:SetSelectedCar(Car)
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	if (Golo) then
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			--
		end
	else
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			--
		end
	end	
end

--function HireMechanic(Price, Races, PreviousMenu)
function HireMechanic(Price, Car, PreviousMenu)
	local TotalPrice = Price
	--local TotalRaces = Races
	
	local BuyFunction = function()
	
		--print("^1Mechanic hired for " .. tostring(TotalRaces) .. " Races")
		
		--MechanicActiveRaces = TotalRaces
		
		Car.MechanicActiveRaces = 1
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	--========
	local FailureFunction = function(Data)
		print("Error sending BoughtConsumable stats")
	end
	
	local SuccessFunction = function(Data)
		print('AddBoughtConsumableStats')
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	local ConsumableTypeID = 1
	--========
	
	if (VirtualCoins:BuyGoloProduct(TotalPrice, BuyFunction)) then
		local CoinsUsed = 0
		local GearsUsed = TotalPrice
		
		ScoresUser:AddBoughtConsumableStats(UserID, ConsumableTypeID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
	else
		--print("^1not enough coins!")
		GameThread:Spawn(function()
			ShowConfirm_NoGolo(TotalPrice, PreviousMenu)
		end)
	end
end

function BuyCar(Car, TotalPrice, FastDelivery, Gears)
	print('buy car ' .. Car.Name)
	
	local BuyFunction = function()
	
		print('Bought "' .. Car.Name .. "'")
		
		local PlayerHasCar = PlayerHasCar()
		
		Car.Bought = true
		
		if (not PlayerHasCar or FastDelivery) then		--Geen wachttijd als men nog geen auto heeft of bij FastDelivery
			Car.Available = true
			Car.DeliveryTime = os.time()
			
			--Apply car selection
			ActiveMission.CarManager:SaveCarSelection(Car)
		else
			Car.Available = false
			--Car.DeliveryTime = os.time() + (60 * 20)	-- 20 min.
			--Car.DeliveryTime = os.time() + (5)			-- 5 sec.
			Car.DeliveryTime = os.time() + CalcDeliveryTime(TotalPrice) --GlobalDeliveryTime_Car
		end
		
		for i, Wheel in ipairs(MonsterTruckCars.Wheels) do
			if (Wheel.Model == Car.WheelConfig.Model) then
				
				print(Wheel.Model .. " deze kopen!!!")
				Wheel.Bought = true
				
				Wheel.ShowPopup = false
				
				if (not PlayerHasCar or FastDelivery) then		--Geen wachttijd als men nog geen auto heeft of bij FastDelivery
					Wheel.Available = true
					Wheel.DeliveryTime = os.time()
				else
					Wheel.Available = false
					Wheel.DeliveryTime = Car.DeliveryTime --Min(GlobalDeliveryTime_Wheel, Car.DeliveryTime)
				end
			end
		end
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	--========
	local FailureFunction = function(Data)
		print("Error sending BoughtCar stats")
	end
	
	local SuccessFunction = function(Data)
		print('AddBoughtCarStats')
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	local CarID = IndexInTable(MonsterTruckCars.Cars, Car) + 1
	--========
	
	if (Gears) then
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			local CoinsUsed = 0
			local GearsUsed = CoinsPrice
			
			ScoresUser:AddBoughtCarStats(UserID, CarID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	else
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			local CoinsUsed = CoinsPrice
			local GearsUsed = 0
			
			ScoresUser:AddBoughtCarStats(UserID, CarID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	end
end

function FastDeliverCar(Car, TotalPrice, Golo)
	print('fast deliver car ' .. Car.Name)
	
	local BuyFunction = function()
	
		print('fast delivered "' .. Car.Name .. "'")
		
		Car.Available = true
		Car.DeliveryTime = os.time()
		
		--Apply car selection
		ActiveMission.CarManager:SaveCarSelection(Car)
		PreviousMenu:ShowMenu() -- om het menu te updaten, mischien best lelijk, maar het werkt :D
		
		for i, Wheel in ipairs(MonsterTruckCars.Wheels) do
			if (Wheel.Model == Car.WheelConfig.Model) then
				Wheel.Available = true
				Wheel.DeliveryTime = os.time()
			end
		end
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	--========
	local FailureFunction = function(Data)
		print("Error sending BoughtCarDelivery stats")
	end
	
	local SuccessFunction = function(Data)
		print('AddBoughtCarDeliveryStats')
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	local CarID = IndexInTable(MonsterTruckCars.Cars, Car) + 1
	--========
	
	if (Golo) then
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			local CoinsUsed = 0
			local GearsUsed = CoinsPrice
			
			ScoresUser:AddBoughtCarDeliveryStats(UserID, CarID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		else
			GameThread:Spawn(function()
				ShowConfirm_NoGolo(TotalPrice, CarSelect)
			end)
		end
	else
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			local CoinsUsed = CoinsPrice
			local GearsUsed = 0
			
			ScoresUser:AddBoughtCarDeliveryStats(UserID, CarID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	end	
end


function BuyWheel(Wheel, TotalPrice, FastDelivery)
	print('buy wheel ' .. Wheel.Name)
	
	local CoinsPrice = TotalPrice --Wheel.ShopPriceCoins
	if (VirtualCoins:BuyProduct(CoinsPrice, function()
		
		print('Bought "' .. Wheel.Name .. "'")
		Wheel.Bought = true
		
		Wheel.Bought = true
		
		if (FastDelivery) then
			Wheel.Available = true
			Wheel.DeliveryTime = os.time()
			
			--Apply wheel selection
			--ActiveGameMode:SetSelectedCar(Car)
		else
			Wheel.Available = false
			Wheel.DeliveryTime = os.time() + CalcDeliveryTime(CoinsPrice) --GlobalDeliveryTime_Wheel
		end
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end)) then
		
	end
	
end

function FastDeliverWheel(Wheel, TotalPrice, Golo)
	print('fast deliver wheel ' .. Wheel.Name)
	
	local BuyFunction = function()
	
		print('fast delivered "' .. Wheel.Name .. "'")
		
		Wheel.Available = true
		Wheel.DeliveryTime = os.time()
		
		--Apply wheel selection
		--ActiveGameMode:SetSelectedCar(Car)
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	if (Golo) then
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			--
		end
	else
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			--
		end
	end	
end

function BuyUpgradeItem(UpgradeItem, TotalPrice, Car, FastDelivery, Golo)
	print('buy upgradeitem ')-- .. Wheel.Name)
	
	if (Golo) then
		FastDelivery = true
	end
	
	local BuyFunction = function()
		print("^2BUY FUNCTIE")
		UpgradeItem:Buy( (not FastDelivery), Car)
		
		if (FastDelivery) then
			UpgradeItem:BuyDirect(Car)
		end
		
		print('Upgraded "' .. UpgradeItem.Name .. '" to level: ' .. Car.Upgrades[UpgradeItem.Name].CurrentLevel)
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	--========
	local FailureFunction = function(Data)
		print("Error sending BoughtCarUpgrade stats")
	end
	
	local SuccessFunction = function(Data)
		print('AddBoughtCarUpgradeStats')
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	local CarID = IndexInTable(MonsterTruckCars.Cars, Car) + 1
	local CarUpgradeTypeID = 1
	
	if (UpgradeItem.Name == "garage_weight") then
		CarUpgradeTypeID = 1
	elseif (UpgradeItem.Name == "garage_suspension") then
		CarUpgradeTypeID = 2
	elseif (UpgradeItem.Name == "garage_airshocks") then
		CarUpgradeTypeID = 3
	elseif (UpgradeItem.Name == "garage_engine") then
		CarUpgradeTypeID = 4
	elseif (UpgradeItem.Name == "garage_armor") then
		CarUpgradeTypeID = 5
	elseif (UpgradeItem.Name == "garage_tires") then
		CarUpgradeTypeID = 6
	end
	--========
	
	if (Golo) then
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			local CarUpgradeLevel = Car.Upgrades[UpgradeItem.Name].BoughtLevel
			
			local CoinsUsed = 0
			local GearsUsed = CoinsPrice
			
			ScoresUser:AddBoughtCarUpgradeStats(UserID, CarID, CarUpgradeTypeID, CarUpgradeLevel, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	else
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			local CarUpgradeLevel = Car.Upgrades[UpgradeItem.Name].BoughtLevel
		
			local CoinsUsed = CoinsPrice
			local GearsUsed = 0
			
			ScoresUser:AddBoughtCarUpgradeStats(UserID, CarID, CarUpgradeTypeID, CarUpgradeLevel, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	end
end

function CalcDeliveryTime(GoldCoins)
	if (GoldCoins > 0) then
		local Time = (60 * 0.02) * GoldCoins
		return ClampMax(Time, (60 * 59))
	else
		return 0
	end
end

function BuyPowerupItem(PowerupItem, TotalPrice, Golo)
	print('buy powerupitem ')		
	
	local BuyFunction = function()
		print("^2BUY FUNCTIE")
		PowerupItem:Buy()
		
		print('Bought Powerup: ' .. PowerupItem.Name)
		
		WriteGameSettings()
		VirtualCoins:UpdateCoincount()
	end
	
	--========
	local FailureFunction = function(Data)
		print("Error sending BoughtPowerup stats")
	end
	
	local SuccessFunction = function(Data)
		print('AddBoughtPowerupStats')
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	local PowerupTypeID = IndexInTable(PowerUpItems, PowerupItem) + 1
	--========
	
	if (Golo) then
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyGoloProduct(CoinsPrice, BuyFunction)) then
			local CoinsUsed = 0
			local GearsUsed = CoinsPrice
			
			ScoresUser:AddBoughtPowerupStats(UserID, PowerupTypeID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	else
		local CoinsPrice = TotalPrice
		if (VirtualCoins:BuyProduct(CoinsPrice, BuyFunction)) then
			local CoinsUsed = CoinsPrice
			local GearsUsed = 0
			
			ScoresUser:AddBoughtPowerupStats(UserID, PowerupTypeID, CoinsUsed, GearsUsed, SuccessFunction, FailureFunction)
		end
	end
end