function cls()
	if (ActiveConsole ~= nil) then
		ActiveConsole.ItemsPanel:Clear()
		ActiveConsole.Items = ItemList()
		ActiveConsole:ResetScroll()
	end
end

function ActiveConsole_Output(...)
	ActiveConsole:Output(...)
end

Console = class(function(self)
	self.Stack = CreateConsole("Console.dcMenu")

	-- gooi menu er op
	self.Menu = self.Stack:GetRootPanel()

	-- Pak het hoofdpanel en overige panels
	self.Main = self.Menu:GetChild("Main")
	self.ItemsPanel = self.Main:GetChild("Items")

	self.ConsoleInput = self.Menu:GetChild("ConsoleInput")
	self.ConsoleInput:SetText("_");
		
	-- Panels leegmaken en lijst aanmaken
	self.ItemsPanel:Clear()
	self.Items = ItemList()
		
	-- Lijst configureren en opmaken
	self.Items.Active = 0

	-- Input lijst
	self.Input = { "" }
	self.CurrentInput = 0
	self.CursorInput = 0
	
	self.Template = MenuItem("Console_LineTemplate.dcMenu")
end)

function Console:Activate()
	-- Kaap de processor
	self.ProcessCharCopy = ProcessChar;

	-- Zet hem op die van ons
	ProcessChar = function(...)
		self:ProcessChar(...);
	end

	DisableKeyListeners()	--Disable currently active KeyListeners
	self.KeyListeners = {}

	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_LEFT", "TimedDown", function(Listener)
		if (self.CursorInput > 0) then
			self.CursorInput = Dec(self.CursorInput)
			self:UpdateInputString()
		end
	end))

	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_RIGHT", "TimedDown", function(Listener)
		local InputLine = self.Input[self.CurrentInput + 1]; 
		if (self.CursorInput < string.len(InputLine)) then
			self.CursorInput = Inc(self.CursorInput)
			self:UpdateInputString()
		end
	end))

	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_UP", "TimedDown", function(Listener)
		if (self.CurrentInput > 0) then
			self.CurrentInput = self.CurrentInput - 1;
			self.CursorInput = string.len(self.Input[self.CurrentInput + 1]);
			self:UpdateInputString()
		end
	end))

	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_DOWN", "TimedDown", function(Listener)
		if (self.CurrentInput < (#self.Input - 1)) then
			self.CurrentInput = self.CurrentInput + 1;
			self.CursorInput = string.len(self.Input[self.CurrentInput + 1]);
			self:UpdateInputString()
		end
	end))

	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_HOME", "Down", function(Listener)
		self.CursorInput = 0
		self:UpdateInputString()
	end))

	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_END", "Down", function(Listener)
		local InputLine = self.Input[self.CurrentInput + 1]; 
		self.CursorInput = string.len(InputLine)

		self:UpdateInputString()
	end))
	
	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_DELETE", "TimedDown", function(Listener)
		self.Input[self.CurrentInput + 1] = InputLine_Head .. string.sub(InputLine_Tail, 2, string.len(InputLine_Tail))
		
		self:UpdateInputString()
	end))
	
	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_PAGEUP", "TimedDown", function(Listener)
		self:ConsoleScroll(-120)
	end))
	
	table.inserttable(self.KeyListeners, AddKeyListeners("DCK_PAGEDOWN", "TimedDown", function(Listener)
		self:ConsoleScroll(120)
	end))
	
	self.Main:SetScrollFunc(function(Item, Delta)
		self:ConsoleScroll(Delta)
	end)

	self.CaretThread = GameThread:Spawn(function()
		while (true) do
			if (true) then
				local InputLine = self.Input[self.CurrentInput + 1]; 

				local InputLine_Head = string.sub(InputLine, 0, self.CursorInput)
				local InputLine_Tail = ""
				if (self.CursorInput < string.len(InputLine)) then
					InputLine_Tail = string.sub(InputLine, self.CursorInput + 1, string.len(InputLine))
				end

				local InputString0 = InputLine_Head .. '_' .. InputLine_Tail
				self.ConsoleInput:SetText(InputString0);
			end

			GameThread:Wait(0.5)

			if (true) then
				local InputLine = self.Input[self.CurrentInput + 1]; 
	
				local InputLine_Head = string.sub(InputLine, 0, self.CursorInput)
				local InputLine_Tail = ""
				if (self.CursorInput < string.len(InputLine)) then
					InputLine_Tail = string.sub(InputLine, self.CursorInput + 1, string.len(InputLine))
				end

				local InputString1 = InputLine_Head .. ' ' .. InputLine_Tail
				self.ConsoleInput:SetText(InputString1);
			end

			GameThread:Wait(0.5)
		end
	end)
end

function Console:Deactivate()
	-- Zet de boel terug
	ProcessChar = self.ProcessCharCopy;

	DelKeyListeners(self.KeyListeners)
	EnableKeyListeners()	--Enable previous disabled KeyListeners

	-- Ruim op
	self.ProcessCharCopy = nil;

	GameThread:Stop(self.CaretThread);
end

function Console:ProcessChar(Character)
	-- We mogen alleen wijzigen als we in de laatste row zitten. Zo niet, dan kopieer
	local InputCount = #self.Input;

	if (self.CurrentInput ~= (InputCount - 1)) then
		self.Input[InputCount] = self.Input[self.CurrentInput + 1];

		self.CurrentInput = InputCount - 1
	end

	-- Dit is een kopie, dus hier kunnen we niet naartoe schrijven.
	local InputLine = self.Input[self.CurrentInput + 1]; 
	
	local InputLine_Head = string.sub(InputLine, 0, self.CursorInput)
	local InputLine_Tail = ""
	if (self.CursorInput < string.len(InputLine)) then
		InputLine_Tail = string.sub(InputLine, self.CursorInput + 1, string.len(InputLine))
	end

	-- Logica
	if (Character == '\t') then -- Tab
		--Ignore tab
	elseif (Character == '\b') then -- Backspace
		self.Input[self.CurrentInput + 1] = string.sub(InputLine_Head, 0, self.CursorInput - 1) .. InputLine_Tail
		self.CursorInput = ClampMin(Dec(self.CursorInput), 0)
	elseif (Character == '\r') then
		-- Voeg alleen een regel toe aan de cache als ie anders is dan de vorige regel
		if (InputCount == 1 or self.Input[InputCount - 1] ~= InputLine) then
			table.insert(self.Input, "")
			self.CurrentInput = InputCount;
		else
			self.CurrentInput = InputCount - 1;
			self.Input[self.CurrentInput + 1] = ""
		end

		-- Print
		print(InputLine)

		-- Voer het commando uit
		ExecuteCommand(InputLine)
		
		self.CursorInput = 0
	else
		self.CursorInput = Inc(self.CursorInput)
		self.Input[self.CurrentInput + 1] = InputLine_Head .. Character .. InputLine_Tail
	end
	
	self:UpdateInputString()
end

function Console:UpdateInputString()
	InputLine = self.Input[self.CurrentInput + 1]; 
	InputLine_Head = string.sub(InputLine, 0, self.CursorInput)
	InputLine_Tail = ''
	if (self.CursorInput < string.len(InputLine)) then
		InputLine_Tail = string.sub(InputLine, self.CursorInput + 1, string.len(InputLine))
	end
	
	local InputString = InputLine_Head .. '_' .. InputLine_Tail
	
	self.ConsoleInput:SetText(InputString);
end

function Console:Output(String)
	if (self.Stack ~= nil) then
		if (#self.Items.Items > 50) then
			self.Items:DelItem(0)
		end
		
		self.Items:AddItem(MenuLabel(self.ItemsPanel, self.Template:Clone(), String), String)
		
		--self.Items:Format(0, 0, 0, 20)
		self.Items:Format()
	end
end

function Console:ResetScroll()
	self.ItemsPanel:SetPos(0, -20)
end

function Console:ConsoleScroll(Delta)
	local posx, posy = self.ItemsPanel:GetPos()
	
	posy = posy - Delta
	posy = ClampMin(posy, -20)
	
	self.ItemsPanel:SetPos(0, posy)
end

function Console:Refresh()
	ActiveConsole = Console()
end