AudioSwitch = class(function(self, ItemList, Template, Name)
	self.Panel = ItemList.Panel
	self.Enabled = true
	self.ItemList = ItemList

	self.Item = self.Panel:AddItem(Template)

	-- Init
	self.NameItem = self.Item:GetChild("Name")
	self.ItemFlash = self.Item:GetChild("Item_Flash")
	self.NameItemFlash = self.Item:GetChild("Name_Flash")
	self.NameItem:SetText(translate(Name))
	self.NameItemFlash:SetText(translate(Name))
	
	self.On = self.Item:GetChild("On")
	self.Off = self.Item:GetChild("Off")

	self.NameItem:SetColorData((color(0, 0, 0, 0.6)), 0, None, false)

	-- Functions
	self.Item:SetActivateFunc(function()
		if (self.Enabled) then
			--ItemList:SetSelection(self)

			if (self.OnActivate ~= nil) then
				self.OnActivate()
			end
		end
	end)

	self.Item:SetDeactivateFunc(function()
		if (self.Enabled) then
			--ItemList:UnsetSelection(self)

			if (self.OnDeactivate ~= nil) then
				self.OnDeactivate()
			end
		end
	end)

	self.Item:SetClickedFunc(function()
		if (self.Enabled) then
			ItemList:SetSelection(self)

			if (self.OnClicked ~= nil) then
				self.OnClicked()
			end
		end
	end)

	self.Item:SetReleasedFunc(function()
		if (self.Enabled) then
			ItemList:SetSelection(self)

			if (self.OnReleased ~= nil and not self.ItemList.Clicked) then				
				self.OnReleased()
			end
		end
	end)

	self.Item:SetScrollFunc(function(Value)
		if (self.Enabled) then
			if (self.OnScroll ~= nil) then
				self.OnScroll(Value)
			end
		end
	end)
end)

function AudioSwitch:DelItem(Index)
	self.Panel:DelItem(self.Item)
end

function AudioSwitch:TurnOn()
	var("FxVolume", 1.0)
	var("MusicVolume", 0.5)
	self.On:SetColorImmediate(White)
	self.Off:SetColorImmediate(Invisible)
	
	configsave()
end

function AudioSwitch:TurnOff()
	var("FxVolume", 0)
	var("MusicVolume", 0)
	self.On:SetColorImmediate(Invisible)
	self.Off:SetColorImmediate(White)
	
	configsave()
end

function AudioSwitch:Init()
	if (tonumber(getvar("MusicVolume")) == 0) then
		self.On:SetColorImmediate(Invisible)
		self.Off:SetColorImmediate(White)
	elseif (tonumber(getvar("MusicVolume")) == 1) then
		self.On:SetColorImmediate(White)
		self.Off:SetColorImmediate(Invisible)
	else
		self.On:SetColorImmediate(White)
		self.Off:SetColorImmediate(Invisible)
	end
end

function AudioSwitch:Activate()
	if (self.Enabled) then		
		self.PanelBack = self.Item:GetChild("PanelBack")
		self.PanelBack:SetColorFade(1, 1)
		self.PanelBack:SetColorImmediate(color(0.8, 1, 1, 1))
	end
end

function AudioSwitch:Deactivate()
	if (self.Enabled) then		
		self.PanelBack = self.Item:GetChild("PanelBack")
		self.PanelBack:SetColor(color(0, 0.5, 1, 0.75))
	end
end

function AudioSwitch:Enable()
	if (not self.Enabled) then
		self.Enabled = true
		
		self.PanelBack = self.Item:GetChild("PanelBack")
		self.PanelBack:SetColor(color(0, 0.5, 1, 0.75))
		
		self.NameItem:SetColor(color(1, 1, 1, 1))
		self.NameItemFlash:SetColor(color(0, 0, 0, 0))
	end
end

function AudioSwitch:Disable()
	if (self.Enabled) then
		self.Enabled = false
		
		self.PanelBack = self.Item:GetChild("PanelBack")
		self.PanelBack:SetColor(color(1, 1, 1, 0.5))
		
		self.NameItem:SetColor(color(1, 1, 1, 0.2))
		self.NameItemFlash:SetColor(color(1, 1, 1, 0.2))
	end
end