MenuElements.CircularList = class(function(self, PrevKey, NextKey)
	self.Menu = Menu
	self.Items = {}
	self.Enabled = true

	if (PrevKey ~= nil) then
		AddKeyListeners(PrevKey, "TimedDown", function(Listener)
			if (Menu:Active()) then
				self:Prev()
			else
				DelKeyListeners(Listener)
			end
		end)
	end
	
	if (NextKey ~= nil) then
		AddKeyListeners(NextKey, "TimedDown", function(Listener)
			if (Menu:Active()) then
				self:Next()
			else
				DelKeyListeners(Listener)
			end
		end)
	end
end)

function MenuElements.CircularList:AddItem(Panel, Item, FormatFunction)
	-- Format
	if (FormatFunction ~= nil) then
		FormatFunction(Panel, Item.Item, #self.Items, 0);
	end

	-- Add to the panel
	Panel:AddItem(Item.Item)
	
	-- Add to internal list
	table.insert(self.Items, Item)

	-- Store the format function
	Item.FormatFunction = FormatFunction;
	Item.Panel = Panel
	
	-- Return the item
	return Item
end

function MenuElements.CircularList:GetSelection()
	if (self.Active ~= nil) then
		return self.Items[(self.Active % #self.Items) + 1];
	else
		return nil
	end
end

function MenuElements.CircularList:DeactivateSelection()
	if (self.Active ~= nil) then
		local Active = self:GetSelection()

		if (Active.Deactivate ~= nil) then
			Active:Deactivate()
		end
	end
end

function MenuElements.CircularList:ActivateSelection()
	if (self.Active ~= nil) then
		local Active = self:GetSelection()

		if (Active.Activate ~= nil) then
			Active:Activate()
		end

		local Offset = self.Active

		-- Circular format
		for i = 1, #self.Items do
			local Item = self.Items[i]
			Item.FormatFunction(Item.Panel, Item.Item, i - 1, Offset);-- - Offset)
		end

		if (self.ActivateCallback ~= nil) then
			self.ActivateCallback()
		end
	end
end

function MenuElements.CircularList:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End)
	self:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, nil)
end

function MenuElements.CircularList:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End)
	self:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, EASE_OUT_ELASTIC, 0, 2)
end

function MenuElements.CircularList:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, TweenEasing, TweenDelay, TweenTime)
	if (Alignment == nil) then
		Alignment = ALIGN_NS
	end

	if (OffsetX == nil) then
		OffsetX = 0
	end
	
	if (OffsetY == nil) then
		OffsetY = 0
	end
	
	if (ItemOffsetX == nil) then
		ItemOffsetX = 0
	end
	
	if (ItemOffsetY == nil) then
		ItemOffsetY = 0
	end

	if (Start == nil) then
		Start = 0
	end
	
	if (End == nil) then
		End = #self.Items
	end
	
	local OffsetStartWidth = 0
	local OffsetStartHeight = 0
	local LastPanel = nil
	for i = Start, End - 1 do
		local Item = self.Items[i + 1]
		local PrevItem = self.Items[i]
		
		if (LastPanel ~= nil and LastPanel == Item.Panel) then
			if (PrevItem ~= nil) then
				local ItemWidth, ItemHeight = PrevItem.Item:getrect()
				
				if (Alignment == ALIGN_NE or Alignment == ALIGN_SE or Alignment == ALIGN_WE or Alignment == ALIGN_WN or Alignment == ALIGN_WS) then		--Align to east
					OffsetStartWidth = OffsetStartWidth + ItemWidth
				elseif (Alignment == ALIGN_NW or Alignment == ALIGN_SW or Alignment == ALIGN_EW or Alignment == ALIGN_ES or Alignment == ALIGN_EN) then	--Align to west
					OffsetStartWidth = OffsetStartWidth - ItemWidth
				end
				
				if (Alignment == ALIGN_NS or Alignment == ALIGN_ES or Alignment == ALIGN_WS or Alignment == ALIGN_NE or Alignment == ALIGN_SW) then		--Align to south
					OffsetStartHeight = OffsetStartHeight + ItemHeight
				elseif (Alignment == ALIGN_SN or Alignment == ALIGN_EN or Alignment == ALIGN_WN or Alignment == ALIGN_NW or Alignment == ALIGN_SE) then	--Align to north
					OffsetStartHeight = OffsetStartHeight - ItemHeight
				end
				
				if (i > Start) then
					OffsetStartWidth = OffsetStartWidth + ItemOffsetX
					OffsetStartHeight = OffsetStartHeight + ItemOffsetY
				end
			end
		else
			OffsetStartWidth = 0
			OffsetStartHeight = 0
		end

		if (TweenEasing == nil) then
			if (AlignX == nil and AlignY == nil and ScreenAlignX == nil and ScreenAlignY == nil) then
			--if (AlignX == nil or AlignY == nil or ScreenAlignX == nil or ScreenAlignY == nil) then
				Item.Item:setpos(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight)
			else
				Item.Item:SetPosData(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 0, None, false)
			end
		else
			Item.Item:SetPosDataTween(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 1, (i/5), false, TweenEasing, 1.5, 1)
		end
		
		if (i == self.Active and Item.Select ~= nil) then
			Item:Select()
		end
		
		LastPanel = Item.Panel
	end
end

function MenuElements.CircularList:Prev()
	local PrevActive = self.Active

	self:DeactivateSelection()
	
	repeat
		if (self.Active ~= nil) then
			self.Active = Dec(self.Active);
		else
			self.Active = 0;
		end
	until (self:GetSelection().Enabled)

	self:ActivateSelection()

	if (self.Active ~= PrevActive) then
		--playsound("Clicked.wav")
	end
end

function MenuElements.CircularList:Next()
	local PrevActive = self.Active

	self:DeactivateSelection()

	repeat
		if (self.Active ~= nil) then
			self.Active = Inc(self.Active);
		else
			self.Active = 0;
		end
	until (self:GetSelection().Enabled)

	self:ActivateSelection()

	if (self.Active ~= PrevActive) then
		--playsound("Clicked.wav")
	end
end

function MenuElements.CircularList:SetSelection(Item)
	local Index = table.findindex(self.Items, Item)

	if (Index ~= self.Active) then
		self:DeactivateSelection()

		self.Active = Index

		self:ActivateSelection()

		--playsound("Clicked.wav")
	end
end

function MenuElements.CircularList:UnsetSelection(Item)
	if (Item == self:GetSelection()) then
		self:DeactivateSelection()

		self.Active = nil
	end
end