GameModeSelect_Template = class(function(self, ItemList, Template, GameMode, Name, LocationName, Description, Image, Score)
	self.Item = MenuItem(Template)
	self.ItemList = ItemList
	self.Enabled = true
	self.Selectable = true
	self.GameMode = GameMode
	self.Name = Name
	self.LocationName = LocationName
	self.Description = Description
	self.Image = Image
	self.Score = Score
	self.InfoClicked = false
	
	self.NameItem = self.Item:GetChild("Name")
	self.NameItemFlash = self.Item:GetChild("NameFlash")
	self.LocationNameItem = self.Item:GetChild("LocationName")

	self.GamemodeImage = self.Item:GetChild("GamemodeImage")

	self.SelectionOutline = self.Item:GetChild("SelectionOutline")
	self.SelectionPijl = self.Item:GetChild("SelectionPijl")
	
	self.ScoreButtonPanel = self.Item:GetChild("ScoreButtonPanel")
	self.ScoreButton = MenuElements.InfoButton(self.ItemList, "elements\\ScoreButton.dcMenu", self.ScoreButtonPanel)
	
	self.ScoreButton.Item:SetClickedFunc(function()
		if (not self.ItemList:IsSliding() and not self.ItemList.Clicked) then
			self.InfoClicked = true
			if (self.ScoreButton.Enabled) then
				self:Disable()
			end
		end
	end)
	
	self.ScoreButton.Item:SetReleasedFunc(function()
		if (not self.ItemList:IsSliding() and not self.ItemList.Clicked) then
			if (self.ScoreButton.Enabled) then				
				ShowConfirm_ScoreButton(self.GameMode.ActiveChapter.ActiveRace)
				self:Enable()
			end
		end
	end)
	
	
	self.InfoButtonPanel = self.Item:GetChild("InfoButtonPanel")
	self.InfoButton = MenuElements.InfoButton(self.ItemList, "elements\\InfoButton.dcMenu", self.InfoButtonPanel)
	
	self.InfoButton.Item:SetClickedFunc(function()
		if (not self.ItemList:IsSliding() and not self.ItemList.Clicked) then
			self.InfoClicked = true
			if (self.InfoButton.Enabled) then
				self:Disable()
			end
		end
	end)
	
	self.InfoButton.Item:SetReleasedFunc(function()
		if (not self.ItemList:IsSliding() and not self.ItemList.Clicked) then
			if (self.InfoButton.Enabled) then				
				ShowConfirm_InfoButton(self.Name, self.Description)
				self:Enable()
			end
		end
	end)
	
	self.StarPanel = self.Item:GetChild("StarPanel")
	self.Stars = MenuElements.Stars(nil, "hud\\elements\\hud_Star.dcMenu", nil, self.StarPanel, 5, 205, 290, ALIGN_EW)
	
	-- Init
	self:SetText(Translate(Name), translate(Description), translate(LocationName))
	
	self.GamemodeImage:SetImage(self.Image)
	
	self.NameItem:SetColorData(White, 0, None, false)
	self.SelectionOutline:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)

	-- Functions
	if (self.ItemList ~= nil) then
		self.Item:SetActivateFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end
				
				if (self.OnActivate ~= nil) then
					self.OnActivate()
				end
			end
		end)

		self.Item:SetDeactivateFunc(function()
			--[[ if (self.Enabled) then
				self.ItemList:UnsetSelection(self)
					
				if (self.OnDeactivate ~= nil) then
					self.OnDeactivate()
				end
			end ]]--
		end)
		
		self.Item:SetClickedFunc(function()
			--print("GMClick")
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end

				if (self.OnClicked ~= nil and not self.ItemList.Clicked) then				
					self.OnClicked()
				end				
			end
		end)

		self.Item:SetReleasedFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end

				if (self.OnReleased ~= nil and not self.ItemList.Clicked) then					
					self.OnReleased()
				end
			end
		end)

		self.Item:SetScrollFunc(function(Value)
			if (self.Enabled) then
				if (self.OnScroll ~= nil) then
					self.OnScroll(Value)
				end
			end
		end)
	end
	--print("Score for gamemode", Name, " Score", self.Score)
	self.Stars:ShowStars(self.Score)
end)

function GameModeSelect_Template:SetText(Text, Description, LocationName)
	self.NameItem:SetText(Text)
	self.NameItemFlash:SetText(Text)
	
	self.LocationNameItem:SetText(LocationName)
end

function GameModeSelect_Template:SetStarScore(Score)
	self.Stars:ShowStars(Score)
end

function GameModeSelect_Template:Activate()
	self.SelectionOutline:SetColorData(color(0.75, 0.75, 0.75, 1), 0, None, false)
	self.SelectionPijl:SetColorData(White, 0, None, false)
	self:Flash(2)
end

function GameModeSelect_Template:Deactivate()
	self.SelectionOutline:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)
	self.SelectionPijl:SetColorData(color(0, 0, 0, 0), 0, None, false)
end

function GameModeSelect_Template:Flash(Speed)
	self.NameItemFlash:SetColorData(White, 0, None, false)
	self.NameItemFlash:SetColorData(color(1, 1, 1, 0), Speed, QuarterSine, false)
end

function GameModeSelect_Template:DelItem(Index)
	--self.Panel:DelItem(Index)
end

function GameModeSelect_Template:Enable()
	if (not self.Enabled) then
		self.Enabled = true
		self.ScoreButton:Enable()
		self.InfoButton:Enable()
		self.InfoClicked = false
		
		self.NameItem:SetColorData(White, 1, Sine, false)
		--self.DescriptionItem:SetColorData(White, 1, Sine, false)
		self.SelectionOutline:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)
		self:Deactivate()
	end
end

function GameModeSelect_Template:Disable()
	if (self.Enabled) then
		self.Enabled = false
		
		if (not self.InfoClicked) then
			self.NameItem:SetColorData(color(0.5, 0.5, 0.5, 0.5), 0, None, false)
			--self.DescriptionItem:SetColorData(color(0.5, 0.5, 0.5, 0.5), 0, None, false)
			self.SelectionOutline:SetColorData(color(0.2, 0.2, 0.2, 0.5), 0, None, false)
		end
	end
end

function GameModeSelect_Template:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex)
		end
	end
end

function GameModeSelect_Template:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ReverseDelay)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex, ReverseDelay)
		end
	end
end

function GameModeSelect_Template:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
		end
	end
end