MenuElements.Label = class(function(self, ItemList, Template, Name, Description, Blinking)
	if (type(Template) == "string") then
		self.Item = MenuItem(Template)
	else
		self.Item = Template
	end
	
	self.ItemList = ItemList
	self.Enabled = true
	self.Selectable = true
	self.Name = Name
	self.Description = Description
	self.Blinking = Blinking
	
	self.Speed = 2
	self.Loop = false
	self.FlashBackground = false
	
	self.NameItem = self.Item:GetChild("Name")
	self.NameItemFlash = self.Item:GetChild("NameFlash")
	self.Background = self.Item:GetChild("Background")
	self.SelectionPijl = self.Item:GetChild("SelectionPijl")

	-- Init
	self:SetText(Translate(Name))
	
	self.BackgroundColor_Normal = color(0.2, 0.2, 0.2, 1)
	self.BackgroundColor_Active = color(0.75, 0.75, 0.75, 1)
	
	self.NameItem:SetColorData(White, 0, None, false)
	self.Background:SetColorData(self.BackgroundColor_Normal, 0, None, false)
	
	self.FlashAfterHover = false
	self.FlashAfterHoverSpeed = nil
	self.FlashAfterHoverLoop = nil
	self.FlashAfterHoverFlashBackground = nil
	self.FlashAfterHoverStrength = nil

	if self.Blinking then
		self:StartBlinking()
	end
	
	-- Functions
	if (self.ItemList ~= nil) then
		self.Item:SetActivateFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end
				
				if (self.OnActivate ~= nil) then
					self.OnActivate()
				end
			end
		end)

		self.Item:SetDeactivateFunc(function()
			--[[ if (self.Enabled) then
				self.ItemList:UnsetSelection(self)
					
				if (self.OnDeactivate ~= nil) then
					self.OnDeactivate()
				end
			end ]]--
		end)
		
		self.Item:SetClickedFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end

				if (self.OnClicked ~= nil) then
					self.OnClicked()
				end
			end
		end)

		self.Item:SetReleasedFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end

				if (self.OnReleased ~= nil and not self.ItemList.Clicked) then
					self.ItemList.Clicked = true
					
					self.OnReleased()
				end
			end
		end)

		self.Item:SetScrollFunc(function(Value)
			if (self.Enabled) then
				if (self.OnScroll ~= nil) then
					self.OnScroll(Value)
				end
			end
		end)
	end
end)

function MenuElements.Label:SetText(Text)
	self.NameItem:SetText(Text)
	self.NameItemFlash:SetText(Text)
end

function MenuElements.Label:SetTextDirect(Text) 
	self.NameItem:SetText(Text)
	self.NameItemFlash:SetText(Text)
end

function MenuElements.Label:Activate()
	self.Background:SetColorData(self.BackgroundColor_Active, 0, None, false)
	self.SelectionPijl:SetColorData(White, 0, None, false)
	self:Flash(self.Speed, false, false, 1)
	if (self.ItemList.Menu.UpdateTheUpgrades ~= nil) then
		self.ItemList.Menu:UpdateTheUpgrades()
	end
end

function MenuElements.Label:Deactivate()
	if (not self.FlashBackground) then
		self.Background:SetColorData(self.BackgroundColor_Normal, 0, None, false)
	end
	self.SelectionPijl:SetColorData(color(0, 0, 0, 0), 0, None, false)
	
	if self.Blinking then
		self.Background:SetColorData(color(0.4, 0.4, 0.4, 1), 0, None, false)
		self.Background:SetColorData(self.BackgroundColor_Normal, 1, Sine, true)
	end
	
	if(self.FlashAfterHover == true) then
		self:Flash(self.FlashAfterHoverSpeed, self.FlashAfterHoverLoop, self.FlashAfterHoverFlashBackground, self.FlashAfterHoverStrength, true)
	end
end

function MenuElements.Label:Flash(Speed, Loop, FlashBackground, Strength, FAH)
	if(FAH == nil) then
		FAH = false
	end
	if(FAH == true) then
		print("verse appeltjes")
	end
	self.FlashAfterHover = FAH
	self.FlashAfterHoverSpeed = Speed
	self.FlashAfterHoverLoop = Loop
	self.FlashAfterHoverFlashBackground = FlashBackground
	self.FlashAfterHoverStrength = Strength
	
	if (Strength == nil) then
		Strength = 1
	end
	if (Loop == nil) then
		Loop = false
	end
	self.Loop = Loop
	self.Speed = Speed

	self.NameItemFlash:SetColorData(White, 0, None, false)
	self.NameItemFlash:SetColorData(color(Strength, Strength, Strength, 0), Speed, QuarterSine, Loop)
	
	if (FlashBackground) then
		self.Background:SetColorData(color(Strength, Strength, Strength, 1), 0, None, false)
		self.Background:SetColorData(self.BackgroundColor_Normal, Speed, Sine, self.Loop)
		
		self.FlashBackground = true
	else
		self.FlashBackground = false
	end
end

function MenuElements.Label:StartBlinking()
	self.Background:SetColorData(color(0.4, 0.4, 0.4, 1), 0, None, false)
	self.Background:SetColorData(self.BackgroundColor_Normal, 1, Sine, true)
end

function MenuElements.Label:DelItem(Index)
	--self.Panel:DelItem(Index)
end

function MenuElements.Label:Enable(Deactivate)
	if (not self.Enabled) then
		self.Enabled = true
		
		self.NameItem:SetColorData(White, 1, Sine, false)
		self.Background:SetColorData(self.BackgroundColor_Normal, 0, None, false)
		
		if (Deactivate == nil) then
			Deactivate = true
		end

		if (Deactivate) then
			self:Deactivate()
		end
	end
end

function MenuElements.Label:Disable()
	if (self.Enabled) then
		self.Enabled = false
		
		self.NameItem:SetColorData(color(0.5, 0.5, 0.5, 0.5), 0, None, false)
		self.Background:SetColorData(color(0.2, 0.2, 0.2, 0.5), 0, None, false)
	end
end

function MenuElements.Label:ChangeBackgroundColor(Color, ColorActive)
	self.BackgroundColor_Normal = Color
	self.BackgroundColor_Active = ColorActive
	
	self.Background:SetColorData(self.BackgroundColor_Normal, 0, None, false)
end

function MenuElements.Label:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex)
		end
	end
end

function MenuElements.Label:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ReverseDelay)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex, ReverseDelay)
		end
	end
end

function MenuElements.Label:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
		end
	end
end