MenuElements.DailyMission = class(function(self, ItemList, Template, Mission, DayValue)--, Name, DayValue, Reward, Achieved)
	self.Item = MenuItem(Template)
	self.ItemList = ItemList
	self.Enabled = true
	self.Selectable = true
	self.Name = Mission.Name
	self.DayValue = DayValue
	self.Reward = GetDailyMissionReward(self.DayValue)
	self.Achieved = Mission.Achieved
	self.NameItem = self.Item:GetChild("Name")
	self.NameItemFlash = self.Item:GetChild("NameFlash")
	self.Selection_Background = self.Item:GetChild("Selection_Background")
	self.DailyMissionImage = self.Item:GetChild("DailyMissionImage")
	self.DailyMissionText = self.Item:GetChild("DailyMissionText")
	self.DailyMissionTextValue = self.Item:GetChild("DailyMissionTextValue")
	
	self.DailyMissionPanelRand = self.Item:GetChild("DailyMissionPanelRand")
	self.DailyMissionCheck = self.Item:GetChild("DailyMissionCheck")
	
	-- Init
	self:SetText(self.DayValue)
	self:SetValue(self.Reward)
	self:SetDailyImage(self.DayValue)
	self:SetDefaultLook()
	self:Validate()
	self.NameItem:SetColorData(White, 0, None, false)
	
	-- Functions
	if (self.ItemList ~= nil) then
		self.Item:SetActivateFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end
				
				if (self.OnActivate ~= nil) then
					self.OnActivate()
				end
			end
		end)

		self.Item:SetDeactivateFunc(function()
			--[[ if (self.Enabled) then
				self.ItemList:UnsetSelection(self)
					
				if (self.OnDeactivate ~= nil) then
					self.OnDeactivate()
				end
			end ]]--
		end)
		
		self.Item:SetClickedFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end

				if (self.OnClicked ~= nil) then
					self.OnClicked()
				end
			end
		end)

		self.Item:SetReleasedFunc(function()
			if (self.Enabled) then
				if (self.Selectable) then
					self.ItemList:SetSelection(self)
				end

				if (self.OnReleased ~= nil) then
					self.OnReleased()
				end
			end
		end)

		self.Item:SetScrollFunc(function(Value)
			if (self.Enabled) then
				if (self.OnScroll ~= nil) then
					self.OnScroll(Value)
				end
			end
		end)
	end
	
end)

function MenuElements.DailyMission:Validate()
	if (DailyMissionsManager.CurrentMission == self.DayValue) then
		self:Activate()
	end
	if(self.Achieved and self.DayValue <= DailyMissionsManager.CurrentMission) then
		self:SetCompleted(true)
	end
end

function MenuElements.DailyMission:SetText(DayValue)
	self.DailyMissionText:SetText(translate("DailyMission_Day") .. " " .. DayValue)
end

function MenuElements.DailyMission:SetValue(Reward)
	self.DailyMissionTextValue:SetText(Reward .. " " .. (Translate("menu_gold")))
end

function MenuElements.DailyMission:SetDefaultLook()
	self.Selection_Background:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)
	self.DailyMissionCheck:SetColorData(Invisible, 0, None, false)
end

function MenuElements.DailyMission:SetCompleted(Achieved)
	if Achieved then
		self.DailyMissionCheck:SetColorData(White, 0, None, false)
		self.Selection_Background:SetColorData(color(0, 0.5, 0, 1), 0, None, false)
	else
		self.DailyMissionCheck:SetColorData(Invisible, 0, None, false)
		self.Selection_Background:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)
	end
end

function MenuElements.DailyMission:SetDailyImage(Image)
	local Image = Image + 0 
	self.DailyMissionImage:SetImage(DailyMissionsManager.Textures[Image])
end


function MenuElements.DailyMission:Activate()
	self.Selection_Background:SetColorData(color(0, 1, 0, 1), 0, None, false)
	self.Selection_Background:SetColorData(color(0, 0.5, 0, 1), 1, Sine, false)
	
	self.DailyMissionText:SetText(translate("DailyMission_Today"))
	
	-- self.Selection_Background:SetColorData(color(0.75, 0.75, 0.75, 1), 0, None, false)
	-- self:Flash(2)
end

function MenuElements.DailyMission:Deactivate() -- in dit geval is het even "Achieved"
	self.DailyMissionCheck:SetColorData(White, 0, None, false)
	self.Selection_Background:SetColorData(color(0, 0.3, 0, 1), 0, None, false)
	self:SetText(self.DayValue)
end

function MenuElements.DailyMission:Flash(Speed)
	self.NameItemFlash:SetColorData(White, 0, None, false)
	self.NameItemFlash:SetColorData(color(1, 1, 1, 0), Speed, QuarterSine, false)
end

function MenuElements.DailyMission:DelItem(Index)
	--self.Panel:DelItem(Index)
end

function MenuElements.DailyMission:Enable()
	if (not self.Enabled) then
		self.Enabled = true
		
		self.NameItem:SetColorData(White, 1, Sine, false)
		self.Selection_Background:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)
		self:Deactivate()
	end
end

function MenuElements.DailyMission:Disable()
	if (self.Enabled) then
		self.Enabled = false
		
		self.NameItem:SetColorData(color(0.5, 0.5, 0.5, 0.5), 0, None, false)
		self.Selection_Background:SetColorData(color(0.2, 0.2, 0.2, 0.5), 0, None, false)
	end
end

function MenuElements.DailyMission:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex)
		end
	end
end

function MenuElements.DailyMission:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ReverseDelay)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex, ReverseDelay)
		end
	end
end

function MenuElements.DailyMission:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
	if (self.ItemList ~= nil) then
		local ItemIndex = self.ItemList:GetItemIndex(self)
		if (ItemIndex ~= nil) then
			self.ItemList:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, ItemIndex - 1, ItemIndex, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
		end
	end
end