-- text item
MenuElements.Race = class(function(self, ItemList, Template, Race)
	self.Item = MenuItem(Template)
	self.Enabled = true
	self.Race = Race

	self.Item:SetColorFade(1, 8)

	self.ImageItem = self.Item:GetChild("Image")
	self.LockedText = self.Item:GetChild("LockedText")
	self.TimeText = self.Item:GetChild("TimeText")
	self.TimeValue = self.Item:GetChild("TimeValue")
	self.NameItem = self.Item:GetChild("Name")
	
	self.DirectionImage = self.Item:GetChild("DirectionImage")
	
	self.StarScore_Low = self.Item:GetChild("StarScore_Low")
	self.StarScore_Medium = self.Item:GetChild("StarScore_Medium")
	self.StarScore_High = self.Item:GetChild("StarScore_High")
	
	-- Init
	self:SetText(self.Race.Title)
	
	self.Item:SetColorImmediate(Halfblack)
	self:Deactivate()

	self:UpdateInfo()
	
	if (self.Race.Texture ~= nil) then
		self.ImageItem:SetImage(self.Race.Texture)
	end

	-- Functions
	self.Item:SetActivateFunc(function()
		if (self.Enabled) then
			--ItemList:SetSelection(self)

			if (self.OnActivate ~= nil) then
				self.OnActivate()
			end
		end
	end)

	self.Item:SetDeactivateFunc(function()
		if (self.Enabled) then
			--ItemList:UnsetSelection(self)

			if (self.OnDeactivate ~= nil) then
				self.OnDeactivate()
			end
		end
	end)

	self.Item:SetClickedFunc(function()
		if (self.Enabled) then
			--ItemList:SetSelection(self)

			if (self.OnClicked ~= nil) then
				self.OnClicked()
			end
		end
	end)

	self.Item:SetReleasedFunc(function()
		if (self.Enabled) then
			--ItemList:SetSelection(self)

			if (self.OnReleased ~= nil) then
				self.OnReleased()
			end
		end
	end)

	self.Item:SetScrollFunc(function(Value)
		if (self.Enabled) then
			if (self.OnScroll ~= nil) then
				self.OnScroll(Value)
			end
		end
	end)
end)

function MenuElements.Race:SetText(Text)
	self.NameItem:SetText(translate(Text))
	--self.NameItem2:SetText(translate(Text))
end

function MenuElements.Race:Activate() -- standaard multilist selectie kleuren 
	self.Item:SetColor(White)
	self.ImageItem:SetColor(White)
end

function MenuElements.Race:Deactivate()
	self.Item:SetColor(Halfblack)
	self.ImageItem:SetColor(color(0.5, 0.5, 0.5, 1))
end

function MenuElements.Race:DelItem(Index)
	--self.Panel:DelItem(Index)
end

function MenuElements.Race:UpdateInfo()
	self.StarScore_Low:SetColorFade(1, 10)
	self.StarScore_Medium:SetColorFade(1, 10)
	self.StarScore_High:SetColorFade(1, 10)
	
	self.StarScore_Low:SetColor(Invisible)
	self.StarScore_Medium:SetColor(Invisible)
	self.StarScore_High:SetColor(Invisible)

	if (self.Race.Bought or DebugUnlockAll) then
		if (self.Race.HighestScore ~= nil) then
			local TotalStars = 3
			local MaximumScore = self.Race.MaximumScore
			local ScoreProgress = self.Race.HighestScore / (MaximumScore / TotalStars)
			
			if (ScoreProgress > 1) then
				self.StarScore_Low:SetColor(White)
				if (ScoreProgress > 2) then
					self.StarScore_Medium:SetColor(White)
					if (ScoreProgress > 3) then
						self.StarScore_High:SetColor(White)
					end
				end
			end
		end
	
		if (self.Race.Available or DebugUnlockAll) then
			self.LockedText:SetColor(Invisible)
			self.TimeText:SetColor(Invisible)
		else
			self.LockedText:SetColor(Invisible)
			self.TimeText:SetColor(White)
			
			if (self.ClockThread ~= nil) then
				GameThread:Stop(self.ClockThread)
				self.ClockThread = nil
			end
			
			self.ClockThread = GameThread:Spawn(function()
				while (not self.Race.Available) do
					
					local CurrentTime = os.time()
					if (self.Race.DeliveryTime ~= nil) then
						local WaitTime = self.Race.DeliveryTime - CurrentTime
						
						if (WaitTime >= 0) then
							--self.DeliveryValueItem:SetText(os.date("%M:%S", WaitTime))
							--self.DeliveryValueFlashItem:SetText(os.date("%M:%S", WaitTime))
							self.TimeValue:SetText(os.date("%M:%S", WaitTime))
						end
					end
					
					GameThread:Wait(1)
				end
			end)
			
		end
	else
		self.LockedText:SetColor(White)
		self.TimeText:SetColor(Invisible)
	end
end