MenuElements.MenuSlider = class(function(self, ItemList, Template, Name, OnChangeFunc, InitValue, MinValue, MaxValue, Precision, MinPerc, MaxPerc, Vertical)
	self.Item = MenuItem(Template)
	self.Enabled = true
	self.Selected = false
	
	--self.Var = Var
	self.OnChangeFunc = OnChangeFunc
	
	self.Min = MinValue
	self.Max = MaxValue
	
	if (Precision ~= nil) then
		self.Precision = Precision
	else
		self.Precision = 0.1
	end
	
	if (MinPerc ~= nil) then
		self.MinPerc = MinPerc
	else
		self.MinPerc = 0
	end
	
	if (MaxPerc ~= nil) then
		self.MaxPerc = MaxPerc
	else
		self.MaxPerc = 1
	end
	
	if (Vertical == nil) then
		Vertical = false
	end
	self.Vertical = Vertical
	
	self.Perc = 1
	
	-- Init
	self.NameItem = self.Item:GetChild("Name")
	self.NameItemFlash = self.Item:GetChild("NameFlash")
	self.NameItem:SetText(translate(Name))
	self.NameItemFlash:SetText(Translate(Name))

	self.ClickableArea = self.Item:GetChild("ClickableArea")
	self.SliderPanel = self.Item:GetChild("Slider")
	self.Background = self.Item:GetChild("Background")
	self.SliderBlock = self.Item:GetChild("SliderBlock")
	--self.SliderValue = self.Item:GetChild("SliderValue")
	
	self.SliderBackground = self.Item:GetChild("SliderBackground")
	
	self.SliderMaxWidth, self.SliderMaxHeight = self.SliderPanel:GetSize()
	self.BlockWidth, self.BlockHeight = self.SliderBlock:GetSize()
	
	if (self.Vertical) then
		self.SliderMinWidth = self.SliderMaxWidth
		self.SliderMinHeight = 1
	else
		self.SliderMinWidth = 1
		self.SliderMinHeight = self.SliderMaxHeight
	end
	
	self.Background:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)	
	
	-- Functions
	self.Item:SetActivateFunc(function()
		if (self.Enabled) then
			ItemList:SetSelection(self)

			if (self.OnActivate ~= nil) then
				self.OnActivate()
			end
		end
	end)

	self.Item:SetDeactivateFunc(function()
		--[[ if (self.Enabled) then
			ItemList:UnsetSelection(self)

			if (self.OnDeactivate ~= nil) then
				self.OnDeactivate()
			end
		end ]]--
	end)

	self.ClickableArea:SetClickedFunc(function(Item, X, Y)
		if (self.Enabled) then
			self.OffsetPosX = MousePos(0).x - X
			self.OffsetPosY = MousePos(0).y - Y
			
			self.SliderThread = GameThread:Spawn(function()
				while (MouseKeyPressed(0)) do
					local Perc = self.Perc
					if (self.Vertical) then
						Perc = (MousePos(0).y - self.OffsetPosY) / self.SliderMaxHeight
					else
						Perc = (MousePos(0).x - self.OffsetPosX) / self.SliderMaxWidth
					end
					Perc = ClampMinMax(Perc, 0, 1)
					
					self.Perc = Perc
					self:UpdateSlider(true)
					
					GameThread:Wait(1/60)
				end
			end)
		end
	end)

	--self.Item:SetReleasedFunc(function()
	self.ClickableArea:SetReleasedFunc(function(Item, X, Y)
		if (self.Enabled) then
			--ItemList:SetSelection(self)
			
			if (self.OnReleased ~= nil) then
				self.OnReleased()
			end
		end
	end)

	self.Item:SetScrollFunc(function(Item, Delta)
		if (self.Enabled) then
			if (self.Precision ~= 0) then
				self.Perc = self.Perc + ((Delta / 120) * -self.Precision)
			else
				self.Perc = self.Perc + ((Delta / 120) * -0.05)
			end
			self:UpdateSlider()
		
			if (self.OnScroll ~= nil) then
				self.OnScroll(Item, Delta)
			end
		end
	end)
	
	if (InitValue ~= nil) then
		self:SetValue(InitValue, true)
	end
	self:UpdateSlider(true)
end)

function MenuElements.MenuSlider:DelItem(Index)
	self.Panel:DelItem(self.Item)
end

function MenuElements.MenuSlider:Activate()
	self.Background:SetColorData(color(0.35, 0.35, 0.35, 1), 0, None, false)
	self:Flash(2)
end

function MenuElements.MenuSlider:Deactivate()
	self.Background:SetColorData(color(0.2, 0.2, 0.2, 1), 0, None, false)
end

function MenuElements.MenuSlider:Flash(Speed)
	self.NameItemFlash:SetColorImmediate(White)
	self.NameItemFlash:SetColorData(color(1, 1, 1, 0), Speed, QuarterSine, false)
end

function MenuElements.MenuSlider:Select()
	
end

function MenuElements.MenuSlider:Deselect()
	
end

function MenuElements.MenuSlider:Enable()
	if (not self.Enabled) then
		self.Enabled = true
	end
end

function MenuElements.MenuSlider:Disable()
	if (self.Enabled) then
		self.Enabled = false
	end
end

function MenuElements.MenuSlider:SetColor(ItemColor)
	if (ItemColor ~= nil) then
		self.SliderPanel:SetColorImmediate(ItemColor)
		self.SliderBackground:SetColorImmediate(ItemColor)
	end
end

function MenuElements.MenuSlider:Left()
	if (self.Perc > 0) then
		if (self.Precision ~= 0) then
			self.Perc = self.Perc - self.Precision
		else
			self.Perc = self.Perc - 0.05
		end
		self:UpdateSlider(true)
	end
end

function MenuElements.MenuSlider:Right()
	if (self.Perc < 1) then
		if (self.Precision ~= 0) then
			self.Perc = self.Perc + self.Precision
		else
			self.Perc = self.Perc + 0.05
		end
		self:UpdateSlider(true)
	end
end

function MenuElements.MenuSlider:SetValue(Value, Force)
	self.Perc = self:GetPerc(Value)
	if (Force == nil) then
		Force = false
	end
	self:UpdateSlider(Force)
end

function MenuElements.MenuSlider:GetPerc(Value)
	return (Value - self.Min) / (self.Max - self.Min)
end

function MenuElements.MenuSlider:GetValue()
	return Lerp(self.Min, self.Max, self.Perc)
end

function MenuElements.MenuSlider:UpdateSlider(Force)
	self.Perc = ClampMinMax(self.Perc, self.MinPerc, self.MaxPerc)
	--print(self.Perc)
	
	--Precision chopping
	--if (self.Precision ~= 0) then
		--local Chopped = math.floor((self.Perc + (self.Precision / 2)) / self.Precision);
		--self.Perc = Chopped * self.Precision;
	--end
	
	local Value = self:GetValue()
	
	--Set SliderPanel Size
	local SliderSize = 0 --self.SliderMaxHeight
	if (self.Vertical) then
		SliderSize = self.SliderMaxHeight - (self.SliderMaxHeight * self.Perc)
	else
		SliderSize = self.SliderMaxWidth * self.Perc
	end
	
	local Alpha = 1
	if (self.Vertical) then
		if (SliderSize < self.SliderMinHeight) then
			Alpha = ClampMin(SliderSize / self.SliderMinHeight, 0)
		end
	else
		if (SliderSize < self.SliderMinWidth) then
			Alpha = ClampMin(SliderSize / self.SliderMinWidth, 0)
		end
	end
	
	--print(self.Perc)
	--print(SliderSize)
	
	if (self.Vertical) then
		SliderSize = ClampMinMax(SliderSize, self.SliderMinHeight, self.SliderMaxHeight)
	else
		SliderSize = ClampMinMax(SliderSize, self.SliderMinWidth, self.SliderMaxWidth)
	end
	
	--self.SliderPanel:SetSize(SliderSize, self.SliderMaxHeight)
	self:SetSliderSize(SliderSize, Force)
	
	self.SliderPanel:SetAlpha(Alpha)
	
	--Set SliderBlock Position
	if (self.Vertical) then
		self.SliderBlock:setpos( 0, (self.SliderMaxHeight * self.Perc) - (self.BlockWidth / 2))
	else
		self.SliderBlock:setpos( (self.SliderMaxWidth * self.Perc) - (self.BlockWidth / 2), 0)
	end
	
	--Set SliderValue Text
	--self.SliderValue:settext(Value)
	
	--if (self.Var ~= "") then
		--var(self.Var, Value)
	--end
	if (self.OnChangeFunc ~= nil) then
		if (self.Vertical) then
			self.OnChangeFunc(1 - Value)
		else
			self.OnChangeFunc(Value)
		end
	end
end

function MenuElements.MenuSlider:SetSliderSize(SliderSize, Force)
	--if (self.SliderThread ~= nil) then
		--GameThread:Stop(self.SliderThread);
	--end
	
	local SliderWidth, SliderHeight = self.SliderPanel:GetSize()
	local LerpVal = 0
	
	if (Force) then
		if (self.Vertical) then
			self.SliderPanel:SetSize(self.SliderMaxWidth, SliderSize)
		else
			self.SliderPanel:SetSize(SliderSize, self.SliderMaxHeight)
		end
	else
		self.SliderThread = GameThread:Run(function()
		
			while(LerpVal < 1.2) do
				if (self.Vertical) then
					local NewSliderHeight = Lerp(SliderHeight, SliderSize, LerpVal)
					self.SliderPanel:SetSize(self.SliderMaxWidth, NewSliderHeight)
				else
					local NewSliderWidth = Lerp(SliderWidth, SliderSize, LerpVal)
					self.SliderPanel:SetSize(NewSliderWidth, self.SliderMaxHeight)
				end
				
				LerpVal = LerpVal + 0.05
				GameThread:Wait(1/60)
			end
		end)
	end
end