MenuElements.FuelButton = class(function(self, ItemList, Template, Name)
	self.Item = MenuItem(Template)
	self.Enabled = true
	self.Selectable = true

	self.Background = self.Item:GetChild("FuelItemBack")
	self.BackgroundFlash = self.Item:GetChild("FuelItemFlash")
	
	self.BlocksPanel = self.Item:GetChild("FuelItem")
	self.BlocksVisible = FuelEnergy.GetCurrentCapacity()
	
	self.Debug = true
	
	-- Functions
	self.Item:SetActivateFunc(function()
		if (self.Enabled) then
			if (self.Selectable) then
				ItemList:SetSelection(self)
			end
			
			if (self.OnActivate ~= nil) then
				self.OnActivate()
			end
		end
	end)

	self.Item:SetDeactivateFunc(function()
		if (self.Enabled) then
			ItemList:UnsetSelection(self)
				
			if (self.OnDeactivate ~= nil) then
				self.OnDeactivate()
			end
		end
	
	end)
	
	self.Item:SetClickedFunc(function()
		if (self.Enabled) then
			if (self.Selectable) then
				ItemList:SetSelection(self)
			end

			if (self.OnClicked ~= nil) then
				self.OnClicked()
			end
		end
	end)

	self.Item:SetReleasedFunc(function()
		if (self.Enabled) then
			if (self.Selectable) then
				ItemList:SetSelection(self)
			end

			if (self.OnReleased ~= nil) then
				self.OnReleased()
			end
		end
	end)

	--[[ self.Item:SetScrollFunc(function(Value)
		if (self.Enabled) then
			if (self.OnScroll ~= nil) then
				self.OnScroll(Value)
			end
		end
	end) ]]
	
	self:CreateBlocks(FuelEnergy:GetTotalCapacity())
	self:ShowBlocks(self.BlocksVisible)
end)

function MenuElements.FuelButton:Activate()
	self.BackgroundFlash:SetColorData(color(1,1,1,0.5), 0, None, false)
	self.BackgroundFlash:SetColorData(Invisible, 1, Sine, false)
	self:Flash(2)
end

function MenuElements.FuelButton:Deactivate()
	--
end

function MenuElements.FuelButton:Flash(Speed)
		
end

function MenuElements.FuelButton:DelItem(Index)
	--self.Panel:DelItem(Index)
end

function MenuElements.FuelButton:Enable()
	if (not self.Enabled) then
		self.Enabled = true
		self.Background:SetColorFade(White, 1, Sine, false)
		self:Deactivate()
	end
end

function MenuElements.FuelButton:Disable()
	if (self.Enabled) then
		self.Enabled = false
		self.Background:SetColorData(color(0.5, 0.5, 0.5, 0.5), 0, None, false)
	end
end

--==================================

function MenuElements.FuelButton:CreateBlocks(TotalBlocks)
	if (self.Blocks ~= nil) then
		if ((#self.Blocks - 1) == TotalBlocks) then
			return false
		end
	end

	self.Blocks = {}
	self.BlocksPanel:Clear()
	
	local Width, Height = self.BlocksPanel:GetSize()
	local Margin = 1
	--local TotalBlocks = 6
	
	local BlockSize = ((Width - 1) - (Margin * TotalBlocks)) / TotalBlocks
	
	if (self.Debug) then
		print("Width: " ..tostring(Width))
		print("BlockSize: " ..tostring(BlockSize))
	end
	
	--BlockSize = math.floor(BlockSize)		--Aanzettten op alle blokjes gelijk te houden, maar kan lelijk worden bij veel items met ExtraSpaceLeft aan
	
	if (self.Debug) then
		print("BlockSize: " ..tostring(BlockSize))
		print("TotalSize: " ..tostring(BlockSize * TotalBlocks))
	end
	
	local BlockSpace = 1 + (Margin * (TotalBlocks + 1)) + (BlockSize * (TotalBlocks + 1))
	local LeftOver = Width - BlockSpace
	local ExtraSpaceLeft = 0 --(LeftOver / 2)
	
	if (self.Debug) then
		print("BlockSpace: " ..tostring(BlockSpace))
		print("LeftOver: " ..tostring(LeftOver))
	end
	
	for i = 0, TotalBlocks do
		self:CreateBlock(BlockSize, i, ExtraSpaceLeft)
	end
end

function MenuElements.FuelButton:CreateBlock(BlockSize, Index, ExtraSpaceLeft)
	local Template = "elements\\Fuel_Template.dcMenu"
	
	local Block = {}
	Block.Item = MenuItem(Template)
	table.insert(self.Blocks, Block)
	self.BlocksPanel:AddItem(Block.Item)
	
	local PosX = ExtraSpaceLeft + 1 + (1 * Index) + (BlockSize * Index)
	Block.Item:SetPos(PosX, 0)
	Block.Item:SetWidth(BlockSize)
	Block.Item:SetColorFade(1, 4)
end

function MenuElements.FuelButton:ShowBlocks(ShowBlocks)
	for i = 0, #self.Blocks do
		local Block = self.Blocks[i]
		if (Block ~= nil) then
			if (i <= ShowBlocks and ShowBlocks > 1) then
				Block.Item:SetColor(White)
			else
				Block.Item:SetColor(Invisible)
			end
		end
	end
	
	self.BlocksVisible = ShowBlocks
	
	if (ShowBlocks == 1) then
		self:StartLowFuel()
	else
		self:StopLowFuel()
	end
	
	if (ShowBlocks == 0) then
		--self.Background:SetColor(Red)
		self:StartEmptyFuel()
	else
		--self.Background:SetColor(White)
		self:StopEmptyFuel()
	end
	
end

function MenuElements.FuelButton:StartLowFuel()
	if (self.LowFuel == nil) then
		self.LowFuel = GameThread:Spawn(function()
			local Block = self.Blocks[1]
			while (self.BlocksVisible == 1) do
				if (Block ~= nil) then
					Block.Item:SetColorData(Red, 0 , None, false)
					PhysicsThread:Wait(1)
					Block.Item:SetColorData(Invisible, 0, None, false)
				end
				PhysicsThread:Wait(1)
			end
		end)
	end
end

function MenuElements.FuelButton:StopLowFuel()
	if (self.LowFuel ~= nil) then
		GameThread:Stop(self.LowFuel)
		self.LowFuel = nil
	end
	
	local Block = self.Blocks[1]
	if (Block ~= nil) then
		--Block.Item:SetColorData(White, 0 , None, false)
	end
end

function MenuElements.FuelButton:StartEmptyFuel()
	if (self.EmptyFuel == nil) then
		self.EmptyFuel = GameThread:Spawn(function()
			while (self.BlocksVisible == 0) do
				self.Background:SetColor(Red)
				PhysicsThread:Wait(1)
				self.Background:SetColor(Invisible)
				PhysicsThread:Wait(0.2)
			end
			self.Background:SetColor(White)
		end)
	end
end

function MenuElements.FuelButton:StopEmptyFuel()
	if (self.EmptyFuel ~= nil) then
		GameThread:Stop(self.EmptyFuel)
		self.EmptyFuel = nil
	end
	
	self.Background:SetColor(White)
end