NewItemList = class(function(self, Menu, PrevKey, NextKey, SelectionCallback)
	self.Menu = Menu
	self.Items = {}
	self.Wrapping = true
	self.EnableInput = true
	self.ListHeight = 99999
	self.ListWidth = 99999
	self.Clicked = false
	
	self.SelectionCallback = SelectionCallback

	if (PrevKey ~= nil) then
		AddKeyListeners(PrevKey, "TimedDown", function(Listener)
			if (Menu.Active ~= nil and Menu:Active()) then
				if (self.EnableInput) then
					self:Prev()
				end
			else
				DelKeyListeners(Listener)
			end
		end)
	end
	
	if (NextKey ~= nil) then
		AddKeyListeners(NextKey, "TimedDown", function(Listener)
			if (Menu.Active ~= nil and Menu:Active()) then
				if (self.EnableInput) then
					self:Next()
				end
			else
				DelKeyListeners(Listener)
			end
		end)
	end
end)

function NewItemList:AddItem(Panel, Item, FormatFunction)
	-- Format
	if (FormatFunction ~= nil) then
		FormatFunction(Panel, Item.Item, #self.Items, 0);
	end

	-- Add to the panel
	if ( Item.Item ~= nil ) then
		Panel:AddItem(Item.Item)
	end

	-- Add to internal list
	table.insert(self.Items, Item)

	-- Store the format function
	Item.FormatFunction = FormatFunction;
	Item.Panel = Panel
	if (Item.Selectable == nil) then
		Item.Selectable = true
	end
	
	-- Return the item
	return Item
end

function NewItemList:InsertItem(Index, Item)
	table.insert(self.Items, Index, Item)

	return Item;
end

function NewItemList:DelItem(Item)
	if (Item.Destroy ~= nil) then
		Item:Destroy()
	end

	table.removeObject(self.Items, Item)
end

function NewItemList:GetItem(Index)
	local Item = self.Items[Index + 1]
	return Item
end

function NewItemList:GetSelection()
	if (self.Active ~= nil) then
		return self.Items[self.Active + 1];
	else
		return nil
	end
end

function NewItemList:ReplaceItem(Item, NewItem)
	local Index = table.findindex(self.Items, Item)
	if (Index ~= nil) then
		if (Item.Destroy ~= nil) then
			Item:Destroy()
		end

		self.Items[Index + 1] = Item
	end
end

function NewItemList:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End)
	self:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, nil)
end

function NewItemList:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, ReverseDelay)
	self:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, EASE_OUT_ELASTIC, nil, 2, ReverseDelay)
end

function NewItemList:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, TweenEasing, TweenDelay, TweenTime, ReverseDelay)
	if (Alignment == nil) then
		Alignment = ALIGN_NS
	end

	if (OffsetX == nil) then
		OffsetX = 0
	end
	
	if (OffsetY == nil) then
		OffsetY = 0
	end
	
	if (ItemOffsetX == nil) then
		ItemOffsetX = 0
	end
	
	if (ItemOffsetY == nil) then
		ItemOffsetY = 0
	end

	if (Start == nil) then
		Start = 0
	else
		Start = ClampMin(Start, 0)
	end
	
	if (End == nil) then
		End = #self.Items
	else
		End = ClampMax(End, #self.Items)
	end
	
	local OffsetStartWidth = 0
	local OffsetStartHeight = 0
	self.LastPanel = nil
	for i = Start, End - 1 do
		local Item = self.Items[i + 1]
		local PrevItem = nil
		if (i > Start) then
			PrevItem = self.Items[i]
		end
		
		if (Item ~= nil) then
			--if (Item.FormatFunction ~= nil) then
				--Item.FormatFunction(Item.Panel, Item.Item, #self.Items, i)
			--else
				if (self.LastPanel ~= nil and self.LastPanel == Item.Panel) then
					if (PrevItem ~= nil) then
						local ItemWidth, ItemHeight = PrevItem.Item:GetFinalSize()
						
						if (Alignment == ALIGN_NE or Alignment == ALIGN_SE or Alignment == ALIGN_WE or Alignment == ALIGN_WN or Alignment == ALIGN_WS) then		--Align to east
							OffsetStartWidth = OffsetStartWidth + ItemWidth
						elseif (Alignment == ALIGN_NW or Alignment == ALIGN_SW or Alignment == ALIGN_EW or Alignment == ALIGN_ES or Alignment == ALIGN_EN) then	--Align to west
							OffsetStartWidth = OffsetStartWidth - ItemWidth
						end
						
						if (Alignment == ALIGN_NS or Alignment == ALIGN_ES or Alignment == ALIGN_WS or Alignment == ALIGN_NE or Alignment == ALIGN_SW) then		--Align to south
							OffsetStartHeight = OffsetStartHeight + ItemHeight
						elseif (Alignment == ALIGN_SN or Alignment == ALIGN_EN or Alignment == ALIGN_WN or Alignment == ALIGN_NW or Alignment == ALIGN_SE) then	--Align to north
							OffsetStartHeight = OffsetStartHeight - ItemHeight
						end
						
						if (i > Start) then
							OffsetStartWidth = OffsetStartWidth + ItemOffsetX
							OffsetStartHeight = OffsetStartHeight + ItemOffsetY
						end
					else
						OffsetStartWidth = 0
						OffsetStartHeight = 0
					end
				else
					OffsetStartWidth = 0
					OffsetStartHeight = 0
				end

				if (TweenEasing == nil) then
					if (AlignX == nil and AlignY == nil and ScreenAlignX == nil and ScreenAlignY == nil) then
					--if (AlignX == nil or AlignY == nil or ScreenAlignX == nil or ScreenAlignY == nil) then
						Item.Item:setpos(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight)
					else
						Item.Item:setpos(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight)	--Weird dat dit nog extra moet
						Item.Item:SetPosData(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 0, None, false)
					end
				else
					if (ReverseDelay) then
						local Delay = TweenDelay
						if (Delay == nil) then
							Delay = ((End - (i - Start))/10)
						end
						Item.Item:SetPosDataTween(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 1, Delay, false, TweenEasing, 2, 2)
					else
						local Delay = TweenDelay
						if (Delay == nil) then
							Delay = ((i - Start)/10)
						end
						Item.Item:SetPosDataTween(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 1, Delay, false, TweenEasing, 2, 2)
					end
				end
			--end
			
			if (i == self.Active and Item.Select ~= nil) then
				Item:Select()
			end
			
			self.LastPanel = Item.Panel
			
			local ItemPosX = OffsetX + OffsetStartWidth
			local ItemPosY = OffsetY + OffsetStartHeight
			local ItemWidth, ItemHeight = Item.Item:GetSize()
			self.ListWidth = ItemPosX + ItemWidth
			self.ListHeight = ItemPosY + ItemHeight
		end
	end
end

	
--[[function NewItemList:Format(OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, Start, End)
	if (Start == nil) then
		Start = 0
	end
	
	if (End == nil) then
		End = #self.Items - 1
	end
	
	for i = Start, End do
		local Item = self.Items[i + 1]

		local OffsetStart = i - Start

		--Item.Item:SetPos(OffsetX + (ItemOffsetX * (OffsetStart)), OffsetY + (ItemOffsetY * (OffsetStart)));
		--Item.Item:SetPosDataTween(OffsetX + (ItemOffsetX * (OffsetStart)), OffsetY + (ItemOffsetY * (OffsetStart)), 0, 0, 0, 0, 1, (i/5), false, 3, 1.5, 1)
	end
end--]]

function NewItemList:EnableSliding(ItemsPanel, ClickArea, ButtonSize, ListSpace, Horizontal, StartCallback, StopCallback, ChangedCallback, CardHack)
	self:DisableSliding()
	
	if (CardHack ~= nil) then
		self.CardHack = CardHack
	else
		self.CardHack = false
	end
	
	--ButtonSize kan evt. ook weg als de PositieOffset uit de ActiveItem kan worden gelezen
	
	--ListSpace kan dus eigenlijk weg uit de arguments
	local Width, Height = ItemsPanel:GetSize()
	if (Horizontal) then
		ListSpace = Width
	else
		ListSpace = Height
	end
	
	self.Sliding = false
	self.Sliding_Panel = ItemsPanel
	self.ClickArea = ClickArea
	self.Sliding_ButtonSize = ButtonSize
	self.Sliding_ListSpace = ListSpace
	self.Sliding_Horizontal = Horizontal
	self.Sliding_StartCallback = StartCallback
	self.Sliding_StopCallback = StopCallback
	self.Sliding_ChangedCallback = ChangedCallback
	
	if (self.Sliding_ChangedCallback ~= nil) then
		--self.Sliding_ChangedCallback(self.Active)
	end
	
	if (self.Sliding_Panel ~= nil) then
		self.Sliding_Panel:SetScrollFunc(function(Item, Delta)
			local Times = math.floor(Delta / 120)
			while (Times ~= 0) do
				if (Times > 0) then
					self:Next()
					Times = Dec(Times)
				end
				
				if (Times < 0) then
					self:Prev()
					Times = Inc(Times)
				end
			end
		end)
		
		ClickArea:SetClickedFunc(function(Item, X, Y)
			self.Sliding = false
			local MousePosX = MousePos(0).x
			local MousePosY = MousePos(0).y
			local PanelPosX, PanelPosY = self.Sliding_Panel:GetPos()
			local StartPanelPosX = tonumber(PanelPosX)
			local StartPanelPosY = tonumber(PanelPosY)

			if (self.SliderThread ~= nil) then
				GameThread:Stop(self.SliderThread)
				self.SliderThread = nil
			end
				
			if (self.EnableInput) then
				self.SliderThread = GameThread:Spawn(function()
					local MouseSpeed = 0
					
					if (self.SnapThread ~= nil) then
						GameThread:Stop(self.SnapThread);
						self.SnapThread = nil
					end

					-- Mouse scrolling
					while (MouseKeyPressed(0) and self.EnableInput) do
						local DeltaX = MousePos(0).x - MousePosX
						local DeltaY = MousePos(0).y - MousePosY
						MousePosX = MousePos(0).x
						MousePosY = MousePos(0).y
						--PanelPosX = PanelPosX + DeltaX
						--PanelPosY = PanelPosY + DeltaY
						
						if (Horizontal) then
							local MinPos = 0
							local MaxPos = ListSpace - self.ListWidth
							if (self.CardHack) then	--Tijdelijk
								--256 = buttonsize zonder de spacing
								MaxPos = ListSpace - self.ListWidth - ListSpace + 256
							end
							
							if ((PanelPosX + DeltaX) < MaxPos) then
								local Perc = (MaxPos / (PanelPosX + DeltaX))
								DeltaX = DeltaX * (Perc * 0.5)
							elseif ((PanelPosX + DeltaX) > MinPos) then
								local Perc = 1 - ((PanelPosX + DeltaX) / ListSpace)
								DeltaX = DeltaX * (Perc * 0.5)
							end
							PanelPosX = PanelPosX + DeltaX
							MouseSpeed = DeltaX
							
							if (self.Sliding) then
								--self.Sliding_Panel:SetPosData(PanelPosX, 0, Left, Top, left, Top, 3.5, 0, false)
								self.Sliding_Panel:SetPosData(PanelPosX, 0, Left, Center, Left, Center, 3.5, 0, false)
							end
						else
							local MinPos = 0
							local MaxPos = ListSpace - self.ListHeight
							if ((PanelPosY + DeltaY) < MaxPos) then
								local Perc = (MaxPos / (PanelPosY + DeltaY))
								DeltaY = DeltaY * (Perc * 0.5)
							elseif ((PanelPosY + DeltaY) > MinPos) then
								local Perc = 1 - ((PanelPosY + DeltaY) / ListSpace)
								DeltaY = DeltaY * (Perc * 0.5)
							end
							PanelPosY = PanelPosY + DeltaY
							MouseSpeed = DeltaY
							
							if (self.Sliding) then
								self.Sliding_Panel:SetPosData(0, PanelPosY, Left, Top, Left, Top, 3.5, 0, false)
							end
						end
						
						local DiffX = StartPanelPosX - PanelPosX
						local DiffY = StartPanelPosY - PanelPosY
						if (math.abs(DiffX) > 10 or math.abs(DiffY) > 10) then
							if (StartCallback ~= nil and not self.Sliding) then
								StartCallback()
							end
							self.Sliding = true
						end
						
						if (ChangedCallback ~= nil) then
							ChangedCallback(self.Active)
						end

						GameThread:Wait(1 / 60)
					end
					
					if (self.SnapThread ~= nil) then
						GameThread:Stop(self.SnapThread);
						self.SnapThread = nil
					end
					
					--Forward speed
					local Offset = (MouseSpeed * 5)
					
					--Snapping
					local Active = self.Active
					if (Horizontal) then
						Active = math.floor(((-PanelPosX + -Offset) / ButtonSize) + 0.5)
					else
						Active = math.floor(((-PanelPosY + -Offset) / ButtonSize) + 0.5)
					end

					Active = ClampMinMax(Active, 0, #self.Items - 1)
					local Changed = (self.Active ~= Active)
					
					if (Changed) then
						--self:DeactivateSelection()
						self.Active = Active
						--self:ActivateSelection()
						if (ChangedCallback ~= nil) then
							ChangedCallback(Active)
						end
					end
					
					if (StopCallback ~= nil and self.Sliding) then
						StopCallback(self.Active)
					end
					self.Sliding = false
					
					
					local StaticPos = -Active * ButtonSize
					
					if (Horizontal) then
						if (self.ListWidth > ListSpace) then
							local MaxPos = ListSpace - self.ListWidth
							if (self.CardHack) then	--Tijdelijk
								--256 = buttonsize zonder de spacing
								MaxPos = ListSpace - self.ListWidth - ListSpace + 256
							end
							
							local DistMaxPos = math.abs(PanelPosX - MaxPos)
							local DistSnapPos = math.abs(PanelPosX - StaticPos)
							
							if (DistMaxPos < DistSnapPos) then
								--print(StaticPos, MaxPos)
								--StaticPos = MaxPos
							end
							
							StaticPos = ClampMin(StaticPos, MaxPos)
						end
					else
						if (self.ListHeight > ListSpace) then
							local MaxPos = ListSpace - self.ListHeight
							
							local DistMaxPos = math.abs(PanelPosY - MaxPos)
							local DistSnapPos = math.abs(PanelPosY - StaticPos)
							
							if (DistMaxPos < DistSnapPos) then
								StaticPos = MaxPos
							end
							
							StaticPos = ClampMin(StaticPos, MaxPos)
						end
					end
					
					
					self.SnapThread = GameThread:Spawn(function()
						for i = 0, 30 do
							MouseSpeed = MouseSpeed * 0.8
							
							if (Horizontal) then
								PanelPosX = PanelPosX + MouseSpeed
								local FinalPos = Lerp(PanelPosX, StaticPos, math.sin(i / 30 * math.pi / 2))
								--self.Sliding_Panel:SetPosData(FinalPos, 0, Left, Top, left, Top, 3.5, 0, false)
								self.Sliding_Panel:SetPosData(FinalPos, 0, Right, Center, Right, Center, 3.5, 0, false)
								
								--print(FinalPos)
							else
								PanelPosY = PanelPosY + MouseSpeed
								local FinalPos = Lerp(PanelPosY, StaticPos, math.sin(i / 30 * math.pi / 2))
								self.Sliding_Panel:SetPosData(0, FinalPos, Left, Top, Left, Top, 3.5, 0, false)
							end
							
							if (ChangedCallback ~= nil) then
								ChangedCallback(self.Active)
							end

							GameThread:Wait(1 / 120)
						end
						self.SnapThread = nil
					end)
					self.SliderThread = nil
				end)
			end
		end)
		
		--self:SnapSliding()
		if (self.Sliding_ChangedCallback ~= nil) then
			--self.Sliding_ChangedCallback(self.Active)
		end
	else
		print("No valid ItemsPanel")
	end
end

function NewItemList:DisableSliding()
	if (self.SliderThread ~= nil) then
		GameThread:Stop(self.SliderThread);
		self.SliderThread = nil
	end
	
	if (self.SnapThread ~= nil) then
		GameThread:Stop(self.SnapThread);
		self.SnapThread = nil
	end

	if (self.Sliding_Panel ~= nil) then
		self.Sliding_Panel:SetClickedFunc(nil)
	end
	
	if (self.ClickArea ~= nil) then
		self.ClickArea:SetClickedFunc(nil)
	end
	
	self.Sliding = false
	--[[ self.Sliding_Panel = nil
	self.Sliding_ButtonSize = nil
	self.Sliding_ListSpace = nil
	self.Sliding_Horizontal = nil
	self.Sliding_StartCallback = nil
	self.Sliding_StopCallback = nil
	self.Sliding_ChangedCallback = nil ]]
end

function NewItemList:IsSliding()
	return self.Sliding
end

function NewItemList:OnTop()
	local PanelPosX, PanelPosY = self.Sliding_Panel:GetPos()
	PanelPosX = math.ceil(PanelPosX)
	PanelPosY = math.ceil(PanelPosY)
	
	local Horizontal = self.Sliding_Horizontal
	if (Horizontal) then
		return (PanelPosX >= 0)
	else
		return (PanelPosY >= 0)
	end
end

function NewItemList:OnBottom()
	local PanelPosX, PanelPosY = self.Sliding_Panel:GetPos()
	PanelPosX = math.floor(PanelPosX)
	PanelPosY = math.floor(PanelPosY)
	
	local Horizontal = self.Sliding_Horizontal
	if (Horizontal) then
		local ListSpace = self.Sliding_ListSpace
		local MaxPos = ListSpace - self.ListWidth
		
		return (PanelPosX <= MaxPos)
	else
		local ListSpace = self.Sliding_ListSpace
		local MaxPos = ListSpace - self.ListHeight
		
		return (PanelPosY <= MaxPos)
	end
end

function NewItemList:SnapSliding(Force)
	if (self.Sliding_Panel == nil or self.ListHeight == nil or self.Sliding_ListSpace == nil) then
		return false
	end

	--print("SnapSliding 1")

	if (self.SliderThread ~= nil) then
		GameThread:Stop(self.SliderThread);
		self.SliderThread = nil
	end
	
	--print("SnapSliding 2")
	
	--if (self.EnableInput) then
		--print("SnapSliding 3")
		
		GameThread:Spawn(function()
			--if (self.SnapThread ~= nil) then
				--GameThread:Stop(self.SnapThread)
				--self.SnapThread = nil
			--end
			
			self.SnapThread = GameThread:Spawn(function()
			
				--print("SnapSliding 4")
				
				if (self.Sliding_StartCallback ~= nil) then
					self.Sliding_StartCallback()
				end
				
				local Active = self.Active
				if (Active == nil) then
					Active = 0
				end
				local ButtonSize = self.Sliding_ButtonSize
				local ListSpace = self.Sliding_ListSpace
				local Horizontal = self.Sliding_Horizontal
				local MouseSpeed = 0
				local PanelPosX, PanelPosY = self.Sliding_Panel:GetPos()
				local Offset = (MouseSpeed * 5)
				
				if (Horizontal) then
					--Active = math.floor(((-PanelPosX + -Offset) / ButtonSize) + 0.5)
					--print( (Active - 0.5) * ButtonSize )
				else
					--Active = math.floor(((-PanelPosY + -Offset) / ButtonSize) + 0.5)
				end
				
				Active = ClampMinMax(Active, 0, #self.Items - 1)
				local StaticPos = -Active * ButtonSize
				--if (self.ListHeight > ListSpace) then
					--24 is voor de extra space aan de onderkant, dit moet dus anders.
					--StaticPos = ClampMinMax(StaticPos - ListSpace + ButtonSize, -self.ListHeight - 24, 0) + ListSpace - ButtonSize
				--end
				
				if (Horizontal) then
					if (self.ListWidth > ListSpace) then
						local MaxPos = ListSpace - self.ListWidth
						if (self.CardHack) then	--Tijdelijk
							--256 = buttonsize zonder de spacing
							MaxPos = ListSpace - self.ListWidth - ListSpace + 256
						end
						
						local DistMaxPos = math.abs(PanelPosX - MaxPos)
						local DistSnapPos = math.abs(PanelPosX - StaticPos)
						
						if (DistMaxPos < DistSnapPos) then
							--print(StaticPos, MaxPos)
							--StaticPos = MaxPos
						end
						
						StaticPos = ClampMin(StaticPos, MaxPos)
					end
				else
					if (self.ListHeight > ListSpace) then
						local MaxPos = ListSpace - self.ListHeight
						
						local PanelP_X, PanelP_Y = self.Sliding_Panel:GetPos()
						
						local DistMaxPos = math.abs(PanelP_Y - MaxPos)
						local DistSnapPos = math.abs(PanelP_Y - StaticPos)
						
						if (DistMaxPos < DistSnapPos) then
							StaticPos = MaxPos
						end
						
						StaticPos = ClampMin(StaticPos, MaxPos)
					end
				end
				
				
				
				if (Force) then
					if (Horizontal) then
						--self.Sliding_Panel:SetPosData(StaticPos, 0, Left, Center, Left, Center, 0, None, false)
						--self.Sliding_Panel:SetPosData(StaticPos, 0, Right, Center, Right, Center, 0, None, false)
						self.Sliding_Panel:SetPos(StaticPos, 0)
						
						--print("waaaah", StaticPos)
					else
						--self.Sliding_Panel:SetPosData(0, StaticPos, Left, Top, Left, Top, 0, None, false)
						self.Sliding_Panel:SetPos(0, StaticPos)
					end
					
					--GameThread:Wait(1 / 120)
					
					GameThread:Spawn(function()
						if (self.Sliding_ChangedCallback ~= nil) then
							self.Sliding_ChangedCallback(self.Active)
						end
						
						if (self.Sliding_StopCallback ~= nil and self.Sliding) then
							self.Sliding_StopCallback(self.Active)
						end
					end)
				else
				
					for i = 0, 30 do
						MouseSpeed = MouseSpeed * 0.8
						
						if (Horizontal) then
							PanelPosX = PanelPosX + MouseSpeed
							local FinalPos = Lerp(PanelPosX, StaticPos, math.sin(i / 30 * math.pi / 2))
							--self.Sliding_Panel:SetPosData(FinalPos, 0, Left, Top, left, Top, 3.5, 0, false)
							--self.Sliding_Panel:SetPosData(FinalPos, 0, Left, Center, Left, Center, 3.5, 0, false)
							self.Sliding_Panel:SetPosData(FinalPos, 0, Right, Center, Right, Center, 3.5, 0, false)
						else
							PanelPosY = PanelPosY + MouseSpeed
							local FinalPos = Lerp(PanelPosY, StaticPos, math.sin(i / 30 * math.pi / 2))
							self.Sliding_Panel:SetPosData(0, FinalPos, Left, Top, Left, Top, 3.5, 0, false)
						end
						
						if (self.Sliding_ChangedCallback ~= nil) then
							self.Sliding_ChangedCallback(self.Active)
						end

						GameThread:Wait(1 / 120)
					end
				end
				
				if (self.Sliding_StopCallback ~= nil) then
					self.Sliding_StopCallback(self.Active)
				end
				
				self.SnapThread = nil
			end)
		end)
	--end
end

function NewItemList:DeactivateSelection()
	if (self.Active ~= nil) then
		local Active = self:GetSelection()

		if (Active ~= nil and Active.Deactivate ~= nil) then
			Active:Deactivate()
		end
	end
end

function NewItemList:ActivateSelection()
	if (self.Active ~= nil) then
		local Active = self:GetSelection()
		
		if (Active ~= nil) then
			if (Active.Activate ~= nil) then
				Active:Activate()
			end

			if (self.ActivateCallback ~= nil) then
				self.ActivateCallback()
			end
		end
	end
end

function NewItemList:Prev()
	local PrevActive = self.Active
	local PrevSelection = self:GetSelection()
	local NewActive = self.Active

	if (not self.Wrapping and self.Active == 0) then
		return false
	end
	
	repeat
		if (NewActive ~= nil) then
			if (self.Wrapping) then
				NewActive = Mod(Dec(NewActive), #self.Items)
			elseif (NewActive > 0) then
				NewActive = Dec(NewActive)
			else
				break
			end
		else
			NewActive = 0
		end
	until (self:GetItem(NewActive).Enabled and self:GetItem(NewActive).Selectable) 
	
	
	if (self:GetItem(NewActive).Enabled and self:GetItem(NewActive).Selectable) then
		self.Active = NewActive
		
		if (self.SelectionCallback ~= nil) then
			self:SelectionCallback()
		end
	end

	local Changed = (self.Active ~= PrevActive)
	if (Changed) then
		if (PrevSelection ~= nil and PrevSelection.Deactivate ~= nil) then
			PrevSelection:Deactivate()
		end
		
		self:ActivateSelection()
		--playsound("Clicked.wav")
		
		self:SnapSliding()
	end
	
	return Changed
end

function NewItemList:Next()
	local PrevActive = self.Active
	local PrevSelection = self:GetSelection()
	local NewActive = self.Active
	
	if (not self.Wrapping and self.Active == #self.Items - 1) then
		return false
	end

	repeat
		if (NewActive ~= nil) then
			if (self.Wrapping) then
				NewActive = Mod(Inc(NewActive), #self.Items)
			elseif (NewActive < #self.Items - 1) then
				NewActive = Inc(NewActive)
			else
				break
			end
		else
			NewActive = 0
		end
	until (self:GetItem(NewActive).Enabled and self:GetItem(NewActive).Selectable) 
	
	if (self:GetItem(NewActive).Enabled and self:GetItem(NewActive).Selectable) then
		self.Active = NewActive
		
		if (self.SelectionCallback ~= nil) then
			self:SelectionCallback()
		end
	end
	

	local Changed = (self.Active ~= PrevActive)
	if (Changed) then
		if (PrevSelection ~= nil and PrevSelection.Deactivate ~= nil) then
			PrevSelection:Deactivate()
		end
		
		self:ActivateSelection()
		--playsound("Clicked.wav")
		
		self:SnapSliding()
	end
	
	return Changed
end

function NewItemList:GetItemIndex(Item)
	if (Item ~= nil) then
		for i, tmp_Item in ipairs(self.Items) do
			if (Item == tmp_Item) then
				return i
			end
		end
	end
	return nil
end

function NewItemList:Count()
	if (self.Items ~= nil) then
		return #self.Items
	else
		return 0
	end
end

function NewItemList:SetSelection(Item)
	local Index = table.findindex(self.Items, Item)

	if (Index ~= self.Active) then
		self:DeactivateSelection()

		self.Active = Index
		
		if (self.SelectionCallback ~= nil) then
			self:SelectionCallback()
		end

		self:ActivateSelection()

		--playsound("Clicked.wav")
	end
end

function NewItemList:UnsetSelection(Item)
	local Index = table.findindex(self.Items, Item)

	if (Index == self.Active) then
		self:DeactivateSelection()

		self.Active = nil
		if (self.SelectionCallback ~= nil) then
			self:SelectionCallback()
		end
	end
end