MenuSlider = class(function(self, ItemList, Template, Text, Var, MinValue, MaxValue, Precision, MinPerc, MaxPerc)
	--self.Panel = Panel
	--self.Item = Panel:AddItem(Template)
	self.Item = MenuItem(Template)
	self.Enabled = true
	self.Selected = false
	
	self.Var = Var
	
	self.Min = MinValue
	self.Max = MaxValue
	
	if (Precision ~= nil) then
		self.Precision = Precision
	else
		self.Precision = 0.1
	end
	
	if (MinPerc ~= nil) then
		self.MinPerc = MinPerc
	else
		self.MinPerc = 0
	end
	
	if (MaxPerc ~= nil) then
		self.MaxPerc = MaxPerc
	else
		self.MaxPerc = 1
	end
	
	self.Perc = 1
	self.MouseOffsetX = 0
	self.MouseOffsetY = 0
	self.MouseHold = false
	
	-- Init
	self.NameItem = self.Item:GetChild("Name")
	self.NameItem:SetText(translate(Text))
	
	self.SelectedPanel = self.Item:GetChild("Selected")
	self.HighlightPanel = self.Item:GetChild("Highlight")
	
	self.SliderPanel = self.Item:GetChild("Slider")
	self.SliderBar = self.Item:GetChild("SliderBar")
	self.SliderBlock = self.Item:GetChild("SliderBlock")
	self.SliderValue = self.Item:GetChild("SliderValue")
	
	self.BarPosX, self.BarPosY = self.SliderPanel:GetPos()
	self.BarMaxWidth, self.BarMaxHeight = self.SliderBar:GetSize()
	self.AreaP0x = self.BarPosX
	self.AreaP0y = self.BarPosY
	self.AreaP1x = self.BarPosX + self.BarMaxWidth
	self.AreaP1y = self.BarPosY + self.BarMaxHeight
	self.BlockWidth, selfBlockHeight = self.SliderBlock:GetSize()
	
	self:SetValue(getvar(self.Var))
	self:UpdateSlider()
	
	-- Functions
	self.Item:SetActivateFunc(function()
		if (self.Enabled) then
			ItemList:SetSelection(self)

			if (self.OnActivate ~= nil) then
				self.OnActivate()
			end
		end
	end)

	self.Item:SetDeactivateFunc(function()
		if (self.Enabled) then
			ItemList:UnsetSelection(self)

			if (self.OnDeactivate ~= nil) then
				self.OnDeactivate()
			end
		end
	end)

	self.Item:SetClickedFunc(function()
		if (self.Enabled) then
			ItemList:SetSelection(self)
			self:Left(self)
			if (self.OnClicked ~= nil) then
				self.OnClicked()
				
			end
		end
	end)

	self.Item:SetReleasedFunc(function()
		if (self.Enabled) then
			ItemList:SetSelection(self)

			if (self.OnReleased ~= nil) then
				self.OnReleased()
			end
		end
	end)

	self.Item:SetScrollFunc(function(Value)
		if (self.Enabled) then
			if (self.OnScroll ~= nil) then
				self.OnScroll(Value)
				self:MouseScroll()
			end
		end
	end)
end)

function MenuSlider:DelItem(Index)
	self.Panel:DelItem(self.Item)
end

function MenuSlider:Left()
	if (self.Perc > 0) then
		if (self.Precision ~= 0) then
			self.Perc = self.Perc - self.Precision
		else
			self.Perc = self.Perc - 0.05
		end
		self:UpdateSlider()
		--playsound("click.wav")
	end
end

function MenuSlider:Right()
	if (self.Perc < 1) then
		if (self.Precision ~= 0) then
			self.Perc = self.Perc + self.Precision
		else
			self.Perc = self.Perc + 0.05
		end
		self:UpdateSlider()
		--playsound("click.wav")
	end
end

function MenuSlider:MouseClick(MenuItem, MouseX, MouseY)
	print('^3Mouse Click (MenuSlider): ^7' .. self.Item:GetIntName())
	
	local MousePosX = MousePos(0):getx()
	local MousePosY = MousePos(0):gety()
	self.MouseOffsetX = MousePosX - MouseX
	self.MouseOffsetY = MousePosY - MouseY
	
	--if (self:InsideClickArea(MouseX, MouseY) and self.Selected) then
		--if (self.Selected) then
			self.Perc = self:CalcMousePerc(MouseX, MouseY)
			self:UpdateSlider()
			
			if (self.Parent:SetSelection(self)) then
				playsound("Clicked.wav")
			end
		--end
	--end
	
	self.MouseHold = true
	
	return false
end

function MenuSlider:MouseRelease(MenuItem, MouseX, MouseY)
	print('^3Mouse Release (MenuSlider): ^7' .. self.Item:GetIntName())
	
	self.MouseHold = false
	
	return true
end

function MenuSlider:MouseScroll(MenuItem, MouseZ)
	if (self.Selected) then
		if (self.Precision ~= 0) then
			self.Perc = self.Perc + ((MouseZ / 120) * -self.Precision)
		else
			self.Perc = self.Perc + ((MouseZ / 120) * -0.05)
		end
		self:UpdateSlider()
	end
end

function MenuSlider:MouseActivate()
	print('^3Mouse Activate (MenuSlider): ^7' .. self.Item:GetIntName())
	
	self.HighlightPanel:SetColorFade(1, 10)
	self.HighlightPanel:SetColor(White)
end

function MenuSlider:MouseDeactivate()
	print('^3Mouse Deactivate (MenuSlider): ^7' .. self.Item:GetIntName())
	
	self.HighlightPanel:SetColor(Invisible)
end

function MenuSlider:Select()
	print('^3Select (MenuSlider): ^7' .. self.Item:GetIntName())
	self.Selected = true
	
	self.SelectedPanel:SetColorFade(1, 10)
	self.SelectedPanel:SetColor(White)
end

function MenuSlider:Deselect()
	print('^3Deselect (MenuSlider): ^7' .. self.Item:GetIntName())
	self.Selected = false
	
	self.SelectedPanel:SetColor(Invisible)
end

function MenuSlider:CalcMousePerc(MouseX, MouseY)
	return (MouseX - self.BarPosX) / self.BarMaxWidth
end

function MenuSlider:SetPerc(Perc)
	self.Perc = Perc
end

function MenuSlider:SetValue(Value)
	self:SetPerc(self:GetPerc(Value))
end

function MenuSlider:GetPerc(Value)
	return (Value - self.Min) / (self.Max - self.Min)
end

function MenuSlider:GetValue()
	return Lerp(self.Min, self.Max, self.Perc)
end

function MenuSlider:UpdateSlider()
	self.Perc = ClampMinMax(self.Perc, self.MinPerc, self.MaxPerc)
	
	--Precision chopping
	if (self.Precision ~= 0) then
		local Chopped = math.floor((self.Perc + (self.Precision / 2)) / self.Precision);
		self.Perc = Chopped * self.Precision;
	end
	
	local Value = self:GetValue()
	
	--Set SliderBar Size
	self.SliderBar:SetSize(self.BarMaxWidth * self.Perc, self.BarMaxHeight)
	
	--Set SliderBlock Position
	self.SliderBlock:setpos( (self.BarMaxWidth * self.Perc) - (self.BlockWidth / 2), 0)
	
	--Set SliderValue Text
	self.SliderValue:settext(Value)
	
	var(self.Var, Value)
end

function MenuSlider:InsideClickArea(MouseX, MouseY)
	if (MouseX > self.AreaP0x and MouseX < self.AreaP1x and
		MouseY > self.AreaP0y and MouseY < self.AreaP1y) then
		return true
	end
	
	return false
end

function MenuSlider:Update(ElapsedTime)
	local MouseX = MousePos(0):getx() - self.MouseOffsetX
	local MouseY = MousePos(0):gety() - self.MouseOffsetY
	
	if (not MouseKeyPressed(0)) then
		self.MouseHold = false
	end

	if (self.MouseHold) then
		--if (self:InsideClickArea(MouseX, MouseY) and self.Selected) then
			--if (self.Selected) then
				self.Perc = self:CalcMousePerc(MouseX, MouseY)
				self:UpdateSlider()
			--end
		--end
	end
end