HUD = nil

local FSError = {
	NO_ISSUES = 0,
	AVAILABLE = 1,
	NOT_AVAILABLE = 2,
	FILE_CORRUPT = 3,
	MEMORY_CORRUPT = 4,
	NAND_DAMAGE = 5,
	NO_ACCESS = 6,
	BUSY = 7,
	INSUFFICIENT_SPACE = 8,
	INSUFFICIENT_INODES = 9,
	UNKNOWN = 10,
}

if NAND == nil then
	NAND = {}
	NAND.DeleteAllFiles = function() end
	NAND.CheckFS = function() return FSError.AVAILABLE end
	NAND.OpenFS = function() return FSError.NO_ISSUES end
	NAND.MemCheck = function() return FSError.NO_ISSUES end
	NAND.CreateFS = function() return FSError.NO_ISSUES end
	NAND.Disable = function() end
	NAND.LockSystem = function() print("Locking system") end
	NAND.ExitToDatamanager = function() print("Exit to datamanager") end
end

WiiNANDMenu = class(function(self, NextMenu)
	self.NextMenu = NextMenu
	self.FSName = "FS.dat"
	self.FSSize = 1024 * 64

	self.Menu = pushmenu("WiiNANDMenu", 0)
	self.MainPanel = self.Menu:GetChild("MainPanel")
	self.ItemsPanel = self.Menu:GetChild("ItemsPanel")	
	self.TextField = self.Menu:GetChild("TextField")
	
	local ErrorCode = NAND.CheckFS(self.FSName)
	
	if ErrorCode == FSError.AVAILABLE then
		local ErrorCode = NAND.OpenFS(self.FSName)
		
		if ErrorCode == FSError.NO_ISSUES then
			GameThread:Spawn(function()
				self.MainPanel:SetColorImmediate(color(0, 0, 0, 0))
				ActiveMenu = self.NextMenu()
			end)
		elseif ErrorCode == FSError.FILE_CORRUPT then
			self:SaveFile_Corrupt()
		elseif ErrorCode == FSError.NAND_DAMAGE then
			self:SaveFile_NANDDamage()
		elseif ErrorCode == FSError.NO_ACCESS then
			self:SaveFile_NoAccess()
		elseif ErrorCode == FSError.UNKNOWN then
			self:SaveFile_UnknowError()
		end
		
	elseif ErrorCode == FSError.NOT_AVAILABLE then
		NAND.DeleteAllFiles()
		local ErrorCode = NAND.MemCheck(self.FSName, self.FSSize)
		
		if ErrorCode == FSError.NO_ISSUES then
			self:SaveFile_Create()
		elseif ErrorCode == FSError.INSUFFICIENT_SPACE then
			self:SaveFile_Blocks()
		elseif ErrorCode == FSError.INSUFFICIENT_INODES then
			self:SaveFile_Inodes()
		elseif ErrorCode == FSError.UNKNOWN then
			self:SaveFile_UnknowError()
		end
	end
end)

function WiiNANDMenu:SaveFile_Create()
	self.TextField:SetText(Translate("WII_NAND_09"))
	self.ItemsPanel:clear()
	self.Items = nil
	
	local ErrorCode = NAND.CreateFS(self.FSName, self.FSSize)
	
	if ErrorCode ~= FSError.NO_ISSUES then
		if ErrorCode == FSError.FILE_CORRUPT then
			self:SaveFile_Corrupt()
		elseif ErrorCode == FSError.NO_ACCESS then
			self:SaveFile_NoAccess()
		elseif ErrorCode == FSError.NAND_DAMAGE then
			self:SaveFile_NANDDamage()
		elseif ErrorCode == FSError.UNKNOWN then
			self:SaveFile_UnknowError()
		end
	else
		GameThread:Spawn(function()
			GameThread:Sleep(2)	
			self.MainPanel:SetColorImmediate(color(0, 0, 0, 0))
			ActiveMenu = self.NextMenu()
		end)
	end
end

function WiiNANDMenu:SaveFile_Blocks()
	self.TextField:SetText(Translate("WII_NAND_02"))
	self.ItemsPanel:clear()
	self.Items = ItemList()
	self.Items:AddItem(MenuLabel(self.ItemsPanel, "elements\\MenuLabel_template.dcMenu", Translate("WII_NAND_02_M1")))
	self.Items:AddItem(MenuLabel(self.ItemsPanel, "elements\\MenuLabel_template.dcMenu", Translate("WII_NAND_02_M2")))
	self.Items.Active = 0
	self.Items:Format(0, 0, 0, 40, 0)
	
	self.Proceed = function()
		if self.Items.Active == 0 then
			NAND.Disable()
			self.MainPanel:SetColorImmediate(color(0, 0, 0, 0))
			ActiveMenu = self.NextMenu()
		end
		if self.Items.Active == 1 then
			NAND.ExitToDatamanager()
		end
	end
end

function WiiNANDMenu:SaveFile_Inodes()
	self.TextField:SetText(Translate("WII_NAND_03"))
	self.ItemsPanel:clear()
	self.Items = ItemList()
	self.Items:AddItem(MenuLabel(self.ItemsPanel, "elements\\MenuLabel_template.dcMenu", Translate("WII_NAND_03_M1")))
	self.Items:AddItem(MenuLabel(self.ItemsPanel, "elements\\MenuLabel_template.dcMenu", Translate("WII_NAND_03_M2")))
	self.Items.Active = 0
	self.Items:Format(0, 0, 0, 40, 0)
	
	self.Proceed = function()
		if self.Items.Active == 0 then
			NAND.Disable()
			self.MainPanel:SetColorImmediate(color(0, 0, 0, 0))
			ActiveMenu = self.NextMenu()
		end
		if self.Items.Active == 1 then
			NAND.ExitToDatamanager()
		end
	end
end

function WiiNANDMenu:SaveFile_Corrupt()
	self.TextField:SetText(Translate("WII_NAND_07"))
	self.ItemsPanel:clear()
	self.Items = ItemList()
	self.Items:AddItem(MenuLabel(self.ItemsPanel, "elements\\MenuLabel_template.dcMenu", Translate("WII_NAND_07_M1")))
	self.Items:AddItem(MenuLabel(self.ItemsPanel, "elements\\MenuLabel_template.dcMenu", Translate("WII_NAND_07_M2")))
	self.Items.Active = 0
	self.Items:Format(0, 0, 0, 40, 0)
	
	self.Proceed = function()
		if self.Items.Active == 0 then
			NAND.Disable()
			self.MainPanel:SetColorImmediate(color(0, 0, 0, 0))
			ActiveMenu = self.NextMenu()
		end
		if self.Items.Active == 1 then
			NAND.DeleteAllFiles()
			self:SaveFile_Create()
		end
	end
end

function WiiNANDMenu:SaveFile_NANDDamage()
	NAND.LockSystem()
	self.TextField:SetText(Translate("WII_NAND_08"))
	self.ItemsPanel:clear()
	self.Items = nil
end

function WiiNANDMenu:SaveFile_NoAccess()
	NAND.LockSystem()
	self.TextField:SetText(Translate("WII_NAND_11"))
	self.ItemsPanel:clear()
	self.Items = nil
end

function WiiNANDMenu:SaveFile_UnknowError()
	NAND.LockSystem()
	self.TextField:SetText(Translate("WII_NAND_12"))
	self.ItemsPanel:clear()
	self.Items = nil
end

function WiiNANDMenu:Up()
	self.Items:Prev()
end

function WiiNANDMenu:Down()
	self.Items:Next()
end

function WiiNANDMenu:Proceed()
end

function WiiNANDMenu:Refresh()
	ActiveMenu = StrapReminder()
end