ItemList = class(function(self)
	self.Items = {}
	self.Active = 0
end)

function ItemList:AddItem(Item, ID)
	table.insert(self.Items, Item)
	
	if (Item.Item ~= nil and ID ~= nil) then
		Item.Item:SetIntName(ID)
		Item.Parent = self
	end
	
	return Item
end

function ItemList:InsertItem(Index, Item)
	--
end

function ItemList:SetItem(Index, Item)
	self.Items[Index + 1]:DelItem(Index)
	
	self.Items[Index + 1] = Item
end

function ItemList:DelItem(Index)
	local Item = self.Items[Index + 1];
	
	table.remove(self.Items, Index + 1)
		
	if Item ~= nil then
		Item:DelItem(Index)
	end
end

--function ItemList:GetItem(Index)
	--return self.Items[Index + 1];
--end

function ItemList:GetItem(MenuItem)
	if (MenuItem ~= nil) then
		for i, Item in ipairs(self.Items) do
			if (Item.Item == MenuItem) then
				return Item
			end
		end
	end
	return nil
end

function ItemList:GetItemIndex(Item)
	if (Item ~= nil) then
		for i, tmp_Item in ipairs(self.Items) do
			if (Item == tmp_Item) then
				return i
			end
		end
	end
	return nil
end

function ItemList:GetMenuItemIndex(MenuItem)
	if (MenuItem ~= nil) then
		for i, Item in ipairs(self.Items) do
			if (Item.Item == MenuItem) then
				return i
			end
		end
	end
	return nil
end

function ItemList:GetSelectedItem()
	return self.Items[self.Active + 1];
end

function ItemList:GetSelectedID()
	return self.Items[self.Active + 1].Item:GetIntName();
end

function ItemList:Format(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End)
	self:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, nil)
end

function ItemList:FormatTween(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End)
	self:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, EASE_OUT_ELASTIC, 0, 2)
end

function ItemList:FormatList(Alignment, OffsetX, OffsetY, ItemOffsetX, ItemOffsetY, AlignX, AlignY, ScreenAlignX, ScreenAlignY, Start, End, TweenEasing, TweenDelay, TweenTime)
	if (Alignment == nil) then
		Alignment = ALIGN_NS
	end

	if (OffsetX == nil) then
		OffsetX = 0
	end
	
	if (OffsetY == nil) then
		OffsetY = 0
	end
	
	if (ItemOffsetX == nil) then
		ItemOffsetX = 0
	end
	
	if (ItemOffsetY == nil) then
		ItemOffsetY = 0
	end

	if (Start == nil) then
		Start = 0
	end
	
	if (End == nil) then
		End = #self.Items
	end
	
	local OffsetStartWidth = 0
	local OffsetStartHeight = 0
	local LastPanel = nil
	for i = Start, End - 1 do
		local Item = self.Items[i + 1]
		local PrevItem = self.Items[i]
		
		if (LastPanel ~= nil and LastPanel == Item.Panel) then
			if (PrevItem ~= nil) then
				local ItemWidth, ItemHeight = PrevItem.Item:getrect()
				
				if (Alignment == ALIGN_NE or Alignment == ALIGN_SE or Alignment == ALIGN_WE or Alignment == ALIGN_WN or Alignment == ALIGN_WS) then		--Align to east
					OffsetStartWidth = OffsetStartWidth + ItemWidth
				elseif (Alignment == ALIGN_NW or Alignment == ALIGN_SW or Alignment == ALIGN_EW or Alignment == ALIGN_ES or Alignment == ALIGN_EN) then	--Align to west
					OffsetStartWidth = OffsetStartWidth - ItemWidth
				end
				
				if (Alignment == ALIGN_NS or Alignment == ALIGN_ES or Alignment == ALIGN_WS or Alignment == ALIGN_NE or Alignment == ALIGN_SW) then		--Align to south
					OffsetStartHeight = OffsetStartHeight + ItemHeight
				elseif (Alignment == ALIGN_SN or Alignment == ALIGN_EN or Alignment == ALIGN_WN or Alignment == ALIGN_NW or Alignment == ALIGN_SE) then	--Align to north
					OffsetStartHeight = OffsetStartHeight - ItemHeight
				end
				
				if (i > Start) then
					OffsetStartWidth = OffsetStartWidth + ItemOffsetX
					OffsetStartHeight = OffsetStartHeight + ItemOffsetY
				end
			end
		else
			OffsetStartWidth = 0
			OffsetStartHeight = 0
		end

		if (TweenEasing == nil) then
			if (AlignX == nil and AlignY == nil and ScreenAlignX == nil and ScreenAlignY == nil) then
			--if (AlignX == nil or AlignY == nil or ScreenAlignX == nil or ScreenAlignY == nil) then
				Item.Item:setpos(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight)
			else
				Item.Item:SetPosData(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 0, None, false)
			end
		else
			Item.Item:SetPosDataTween(OffsetX + OffsetStartWidth, OffsetY + OffsetStartHeight, AlignX, AlignY, ScreenAlignX, ScreenAlignY, 1, (i/5), false, TweenEasing, 1.5, 1)
		end
		
		if (i == self.Active and Item.Select ~= nil) then
			Item:Select()
		end
		
		LastPanel = Item.Panel
	end
end



function ItemList:Prev()
	if (self.Items[self.Active + 1] ~= nil) then
		--self.Items[self.Active + 1]:Deactivate()
		self.Items[self.Active + 1]:Deselect()
	end
	
	local PrevActive = self.Active
	self.Active = Mod(Dec(self.Active), #self.Items)
	
	while (not self.Items[self.Active + 1].Enabled) do
		self.Active = Mod(Dec(self.Active), #self.Items)
		if (PrevActive == self.Active) then
			break
		end
	end
	
	--self.Items[self.Active + 1]:Activate(false)
	self.Items[self.Active + 1]:Select()
	
	if (self.Active ~= PrevActive) then
		--playsound("Clicked.wav")
	end
end

function ItemList:Next()
	if (self.Items[self.Active + 1] ~= nil) then
		--self.Items[self.Active + 1]:Deactivate()
		self.Items[self.Active + 1]:Deselect()
	end
	
	print(self.Active)
	print(#self.Items)
	
	local PrevActive = self.Active
	self.Active = Mod(Inc(self.Active), #self.Items)
	
	while (not self.Items[self.Active + 1].Enabled) do
		self.Active = Mod(Inc(self.Active), #self.Items)
		if (PrevActive == self.Active) then
			break
		end
	end
	
	print(self.Active)
	
	--self.Items[self.Active + 1]:Activate(false)
	self.Items[self.Active + 1]:Select()
	
	if (self.Active ~= PrevActive) then
		--playsound("Clicked.wav")
	end
end

--function ItemList:SetSelection(MenuItem, FirstFrame)


--function ItemList:SetSelection(MenuItem)
function ItemList:SetSelection(Item)

	--if (MenuItem == nil) then
		--if (self.Active ~= -1) then
			--self.Items[self.Active + 1]:Deactivate()
		--end
		
		--self.Active = -1	-- moet eigenlijk -1 zijn
		
		--return false
	--end
	
	--local ItemIndex = self:GetMenuItemIndex(MenuItem)
	local ItemIndex = self:GetItemIndex(Item)
	if (ItemIndex ~= nil and self.Items[ItemIndex].Select ~= nil) then
		if (self.Active ~= ItemIndex - 1) then
			if (self.Items[self.Active + 1] ~= nil) then
				--self.Items[self.Active + 1]:Deactivate()
				self.Items[self.Active + 1]:Deselect()
			end
		
			self.Active = ItemIndex - 1
			--self.Items[self.Active + 1]:Activate(FirstFrame)
			self.Items[self.Active + 1]:Select()
			return true
		end
	end
	return false

	--[[for i, Item in ipairs(self.Items) do
		if (Item.Item == Selected) then
			if (self.Items[self.Active + 1] ~= nil) then
				self.Items[self.Active + 1]:Deactivate()
			end
	
			if (self.Active ~= i - 1 and not FirstFrame) then
				playsound("Clicked.wav")
			end
			
			self.Active = i - 1
			
			self.Items[self.Active + 1]:Activate(FirstFrame)
			return true
		end
	end]]---
	
	--return false
end





-- Misc
White = color(1, 1, 1, 1)
Black = color(0, 0, 0, 1)
Halfblack = color(0, 0, 0, 0.5)
Invisibleblack = color(0, 0, 0, 0)
Grey = color(0.5, 0.5, 0.5, 1)

Invisible = color(1, 1, 1, 0)

Red = color(1, 0, 0, 1)
Green = color(0, 1, 0, 1)
Blue = color(0, 0, 1, 1)
Yellow = color(1, 1, 0, 1)

SelectedOrange = color(1, 0.8, 0, 1)
SelectedOrangeResults = color(1, 0.8, 0, 0.5)
Unselected  = color(0, 0, 0, 0.5)
Unselected2  = color(0, 0, 0, 0.2)

White10 = color(10, 10, 10, 10)


-- Kleurtjes
activecolor = color(1, 0.8, 0, 1)
defaultcolor = color(0, 0, 0, 0.25)
invisiblecolor = color(1, 1, 1, 0)
invisiblefailcolor = color(1, 0, 0, 0)
whitecolor = color(1, 1, 1, 1)
whitecolor2 = color(1, 1, 1, 6)
failcolor2 = color(1, 0, 0, 6)

--http://www.w3schools.com/tags/ref_color_tryit.asp
AliceBlue		= color(0.94, 0.97, 1.00, 1)
AntiqueWhite	= color(0.98, 0.92, 0.84, 1)
