BaseMission = class(function(self, RaceData)
	print("^3BaseMission created")
	
	table.insert(ResetFuncs, function()
		self:Destroy()
	end)
	
	-- Photoshoot vars
	self.NormalColor = color(1,1,1,1)
	self.NormalIntensity = 0
	self.NormalGrayValue = 0
	
	self.PhotoColor = color(0,0,0,1)
	self.PhotoIntensity = 0
	self.PhotoGrayValue = 0.75

	self.HighscoreClient = {}
	
	var("extrageometry", 1)
	var("ExtraFrontGeometry", 1)
	
	SetShaderVar("ColorOverlay", self.NormalColor * self.NormalIntensity)
	SetShaderVar("InvGreyValue", 1 - self.NormalGrayValue)
	SetShaderVar("GreyScale", Vector(0.222, 0.707, 0.071) * self.NormalGrayValue)
	
	self.RaceData = RaceData
	self.Score = 0
	self.StarScore = 0
	
	self.RaceTimerStart = 0
	
	self.Players = {}
	self.HumanPlayers = {}
	
	self.Cameras = InitializeCameras(self)
	self.Checkpoints = {}
	self.CurrentNextCheckpoint = nil
	self.RaceDataUpdated = false
	self.RaceLength = 0
	self.IngameMenuEnabled = false
	
	self.PrevCarVolume = 1
	
	self.ClickedPowerUp = nil
	
	self.PickUpManager = PickUpObjectsManager()
	
	self.Listeners = AddKeyListeners("DCK_SCREENBUTTON_2", "Down", function(Listener)
		GameThread:Spawn(function()
			for i, Player in ipairs(self.Players) do
				if (Player.PowerupManager ~= nil and Player.Human) then
					if (Player.PowerupManager.RepairPowerup.Enabled and not Player.PowerupManager.RepairPowerup.Used) then
						if (Player.HUD ~= nil and Player.HUD.RepairButton ~= nil and Player.HUD.RepairButton.Enabled) then
							print("RepairButton SHOOT")
							Player:Repair()
						
							Player.HUD.RepairButton:TurnOff()
						end
					elseif (not Player.PowerupManager.RepairPowerup.Enabled) then
						if (Player.HUD ~= nil) then
							local Power = Player.PowerupManager:GetPowerUp(Player.PowerupManager.RepairPowerup)
							self:ClickedPowerUps(Power)
						end
					end
				end
			end
		end)
	end)
	
	self.Listeners = AddKeyListeners("DCK_SCREENBUTTON_3", "Down", function(Listener)
		GameThread:Spawn(function()
			for i, Player in ipairs(self.Players) do
				if (Player.PowerupManager ~= nil and Player.Human) then
					if (Player.PowerupManager.JumpPowerup.Enabled and not Player.PowerupManager.JumpPowerup.Used) then
						if (Player.HUD ~= nil and Player.HUD.JumpButton ~= nil and Player.HUD.JumpButton.Enabled) then
							print("Jump SHOOT")
							Player:Jump()
							
							Player.HUD.JumpButton:TurnOff()
						end
					elseif (not Player.PowerupManager.JumpPowerup.Enabled) then
						if (Player.HUD ~= nil) then
							local Power = Player.PowerupManager:GetPowerUp(Player.PowerupManager.JumpPowerup)
							self:ClickedPowerUps(Power)
						end
					end
				end
			end
		end)
	end)
	
	self.Listeners = AddKeyListeners("DCK_SCREENBUTTON_4", "Down", function(Listener)
		GameThread:Spawn(function()
			for i, Player in ipairs(self.Players) do
				if (Player.PowerupManager ~= nil and Player.Human) then
					if (Player.PowerupManager.FirworksPowerup.Enabled and not Player.PowerupManager.FirworksPowerup.Used) then
						if (Player.HUD ~= nil and Player.HUD.FireWorksButton ~= nil and Player.HUD.FireWorksButton.Enabled) then
							print("FireWorks SHOOT")
							Player:ShootFireworks()
							
							Player.HUD.FireWorksButton:TurnOff()
						end
					elseif (not Player.PowerupManager.FireworksPowerup.Enabled) then
						if (Player.HUD ~= nil) then
							local Power = Player.PowerupManager:GetPowerUp(Player.PowerupManager.FirworksPowerup)
							self:ClickedPowerUps(Power)
						end
					end
				end
			end
		end)
	end)
	
	DailyMissionsManager:StartChecking(self, "Challenge_16")
end)

function BaseMission:PostConstruction()
	local Player = self.HumanPlayers[1]

	if (#self.Players > 0) then
		self.SuspensionManager = SuspensionManager(self)
		self.SuspensionManager:SetPlayer(Player)
		self.SuspensionManager:Enable()
		
		self.StuntManager = StuntManager(self)
	end
	
	AddKeyListeners("DCK_K", "Down", function(Listeners)
		local TempMatrix = Player.Car:GetMatrix():Translate(Vector(0,20,0))
		TempMatrix = TempMatrix:RotateLocal(Vector(1,0,0), math.rad(-90))
		Player.Car:SetMatrix(TempMatrix)
	end)
	
	--[[PhysicsThread:Spawn(function()
		local CarPos = Player.Car:GetMatrix():GetTranslation()
		local FirstCheckpointPos = self.Checkpoints[1]:GetMatrix():GetTranslation()
		self.StartProgressOffset = CarPos:DistanceTo(FirstCheckpointPos)
	end)]]
end

function BaseMission:Destroy()
	local Player = self.HumanPlayers[1]
	
	if (Player ~= nil and Player.HUD ~= nil) then
		Player.HUD:Destroy()
	end
	
	if (self.MusicThread ~= nil) then
		PhysicsThread:Spawn(function()
			PhysicsThread:Stop(self.MusicThread)
			self.MusicThread = nil
		end)
	end
	
	self:StopMusic()

	DelKeyListeners(self.Listeners)

	GetEntityScene():DelEntityCallback(self.CallbackWrapper)

	self.CallbackWrapper = nil
	
	if (self.PhotoshootCameraThread ~= nil) then
		PhysicsThread:Spawn(function()
			PhysicsThread:Stop(self.PhotoshootCameraThread)
			self.PhotoshootCameraThread = nil
		end)
	end
	
	for i, Camera in ipairs(self.Cameras) do
		if (Camera ~= nil) then
			Camera:Destroy()
			Camera = nil
		end
	end
	
	if (self:is_a(MenuMission)) then
		print("Destroy MenuMission")
		--UnloadMenuTextures()
	else
		print("Destroy GameMission")
		--UnloadGameTextures()
	end
end

function BaseMission:Refresh()
	for i, Player in ipairs(self.HumanPlayers) do
		if (Player.HUD ~= nil) then
			self.HumanPlayers[1].HUD = Player.HUD:Refresh(Player)
		end
	end
end

function BaseMission:AddPlayer(Player)
	table.insert(self.Players, Player)
	return Player
end

function BaseMission:CreateCarBuilder(Car, Skin)
	return LuaCarBuilder(Car, Skin)
end

function BaseMission:CreateHumanPlayer(Car, Skin, PlayerRef, RecordPlayer)
	-- Create a CarBuilder
	self.CarBuilder = self:CreateCarBuilder(Car, Skin)--LuaCarBuilder(Car, Skin)
	
	-- Create the player and the upgrader
	local Upgrader = Upgrader(Car)
	
	-- Build the car
	local Car = self.CarBuilder:Build()
	local Player = LUAPlayer(Car)
	
	-- Set the upgrader to the player, and apply all upgrades
	Player.Upgrader = Upgrader
	Player.Upgrader:ApplyUpgrades(self.CarBuilder)
	
	-- Add the player
	self:AddPlayer(Player)
	
	-- Add the car to the entity scene
	GetEntityScene():AddEntity(Player.Car)
	Player.Car:Disable()

	local Matrix = GetScriptMatrix("Player 1")
	if (RecordPlayer ~= nil) then
		Matrix = GetScriptMatrix("Player " .. RecordPlayer)
	end
	
	if (Matrix ~= nil) then
		Player.Car:SetMatrix(Matrix)
	else
		Player.Car:SetMatrix(MatrixIdentity)
	end
	
	Player.Human = true
	Player.Name = self.RaceData.Name
	Player.ID = self.RaceData.PlayerID
	Player.CarType = self.RaceData.CarType

	table.insert(self.HumanPlayers, Player)
	return Player
end

function BaseMission:CreateAIPlayer(SpawnIndex, AIPlayer, RecordPlayer)
	local UseReplayCar = true
	local Player_ = nil
	local CarMatrix = nil
	
	local Index = 2 --AIPlayer.ID + 1
	local CarType = AIPlayer.Car
	local GhostFileName = self:GetGhostFileName(tostring(Index) .. "_" .. tostring(CarType))
	
	--if (UseReplayCar and GetFS():Exists(GhostFileName .. "1")) then
	if (UseReplayCar and GetFS():Exists(GhostFileName)) then
		
		CarMatrix = GetScriptMatrix("Player " .. Index)
		if CarMatrix ~= nil then
			Player_ = self:AddPlayer(LUAPlayer(ReplayCar(AIPlayer.Car, AIPlayer.Skin)))
			--Player_.Car:LoadReplay(GhostFileName .. "1")
			--Player_.Car:LoadReplay(GhostFileName .. "2")
			Player_.Car:LoadReplay(GhostFileName)
			
			if (RecordPlayer == nil) then
				Player_.Car:EnableCollision()
			else
				Player_.Car:DisableCollision()
			end
		end
	else
		CarMatrix = GetScriptMatrix("Player " .. SpawnIndex)
		Player_ = self:AddPlayer(LUAPlayer(LuaCar(AIPlayer.Car, AIPlayer.Skin)))
	end
	
	--print("Player " .. Index)
	
	--print("CarMatrix")
	--print(CarMatrix)
	
	if Player_ ~= nil then
		GetEntityScene():AddEntity(Player_.Car)
		Player_.Car:Disable()
		
		if (CarMatrix ~= nil) then
			print("Using carmatrix")
			
			Player_.Car:SetMatrix(CarMatrix)
		else
			Player_.Car:SetMatrix(MatrixIdentity)
		end
		
		Player_.Human = false
		Player_.Name = AIPlayer.Name
		Player_.ID = AIPlayer.PlayerID
		Player_.CarType = AIPlayer.CarType
		
		--if (UseReplayCar and GetFS():Exists(GhostFileName .. "1")) then
		if (UseReplayCar and GetFS():Exists(GhostFileName)) then
			--Replay/Ghost AI
			--MessageBox("Replay/Ghost AI", "Filename: " .. tostring(GhostFileName))
		else
			if (true) then
				Player_.Driver = SmartRaceDriver(Player_.Car)
			else
				--Player_.Driver = AgentAI_RaceDriver(Player_.Car)
				Player_.Driver = AgentAI_SteeringDriver(Player_.Car)
				MessageBox("AgentAI_SteeringDriver")
			end
		end
		
		Player_.Car:DisableInput(1)
	end
	
	return Player_
end

function BaseMission:SetAxisAlignment(AxisVector, DirectionSign)
	-- If an alignment thread already exists, stop and remove it
	if(self.AlignmentThread ~= nil) then
		PhysicsThread:Spawn(function()
			PhysicsThread:Stop(self.AlignmentThread)
			self.AlignmentThread = nil
		end)
	end
	
	-- Axis vector failsafe
	if(AxisVector ~= VectorX and AxisVector ~= VectorZ) then
		print("^1ERROR: The provided vector for axis alignment is not a valid axis vector. "
			.."Aborting alignment.")
		return
	end
	
	-- Direction sign failsafe
	if(DirectionSign < -1 or DirectionSign > 1) then
		DirectionSign = math.sign(DirectionSign)
	elseif(Direction == 0) then
		print("^1ERROR: The direction sign provided for axis alignment can not be zero. "
			.."Aborting alignment.")
		return
	end
	
	-- Create a new alignment thread
	self.AlignmentThread = PhysicsThread:Spawn(function()
		-- We start fetching the Player inside this PhysicsThread, because we need for
		-- the mission to be initialized first. Trying to obtain the player above this
		-- scope is not possible, because the ActiveMission will not yet have been set.
		local Player = self.HumanPlayers[1]
		local Car = Player.Car
		
		-- Set the axis correction vector used for forcing X, Y or Z to 0. For instance,
		-- a direction of (1,0,1) with AxisCorrection (1,1,0) would make for a new
		-- direction of (1,0,0). This way we have forced the Z-field to zero. Keep in
		-- mind this only works for unsigned rounded vectors like (1,0,0) or (0,1,0) etc.
		local Forward = AxisVector
		local Right = VectorY:Cross(Forward)
		local AbsRight = Vector(math.abs(Right.X), math.abs(Right.Y), math.abs(Right.Z))
		local AxisCorrection = VectorOne - AbsRight -- Example outcome: Vector(1,1,1) - Vector(1,0,0) = Vector(0,1,1)
		
		-- Apply the direction sign
		Forward = Forward * DirectionSign
		Right = Right * DirectionSign
		
		-- Make sure the car is initially facing in the correct direction
		local CarMat = Car:GetPhysicsMatrix()
		CarMat = CarMat:SetAxii(Right, VectorY, Forward)
		Car:SetPhysicsMatrix(CarMat)
	
		-- Alignment processing
		while(true) do
			local CarMat = Car:GetPhysicsMatrix()
			local CarVel = Car:GetVelocity()
			local CarTrans = CarMat:GetTranslation()
			
			--[[ Alter translation ]]
			-- Correct the translation
			-- TODO: Can cause the slider to malfunction when in the starting position using VectorX. Investigate & fix.
			CarMat = CarMat:SetTranslation(CarTrans * AxisCorrection)
			
			--[[ Alter direction ]]
			-- Correct the up-vector
			local Up = CarMat:GetUp() * AxisCorrection
			Up = Up:Normalize()
			
			-- Correct the direction-vector
			local Dir = CarMat:GetDirection() * AxisCorrection
			Dir = Dir:Normalize()
			
			-- Set the new direction
			CarMat = CarMat:SetAxii(Right, Up, Dir)
			
			-- Set the new car matrix
			Car:SetPhysicsMatrix(CarMat)
			
			--[[ Alter velocity ]]
			Car:SetVelocity(CarVel * AxisCorrection)
			PhysicsThread:Wait(1)
		end
	end)
end

function BaseMission:AddCheckpoint(Checkpoint)
	table.insert(self.Checkpoints, Checkpoint)
	
	self.RaceLength = self.RaceLength + Checkpoint:GetLength()
end

function BaseMission:Checkpoint(Car, Checkpoint)
	local Player = self:GetPlayer(Car)
	
	if (Player == nil or Player.Dead or Player.Finished) then
		return false
	end
	
	if (Player.Human) then
		Player.NextCheckpoint = Checkpoint:GetNextCheckpoint(0)
	else
		Player.NextCheckpoint = Checkpoint:GetNextCheckpoint(1)
	end

	if (Player.Driver ~= nil and Player.Driver.Checkpoint ~= nil) then
		Player.Driver:Checkpoint(Checkpoint)
	end
	
	Player.DistanceTraveled = Player.DistanceTraveled + Player.DistanceToGo	
	
	Player.CheckCount = Inc(Player.CheckCount)
	Player.CheckDist = Checkpoint:GetLength()
	
	if (Player.Human) then
		self.CurrentNextCheckpoint = Checkpoint
	end
end

function BaseMission:NextCheckpoint(Car, Checkpoint)
	local Player = self:GetPlayer(Car)
	
	local ShowModel = false
	
	if (Player == nil) then
		return
	end
	
	if (Player.NextCheckpoint ~= nil) then
		if (Player.NextCheckpoint == Checkpoint) then
			--
		else
			return
		end
	end
	
	if (Player.Driver ~= nil and Player.Driver.NextCheckpoint ~= nil) then
		Player.Driver:NextCheckpoint(Checkpoint)
	end
	
	if (not Player.Human) then
		return
	end
	
	if (Player.HUD ~= nil) then
		Player.HUD:ShowCheckpoint(Checkpoint)
	end
	
	if (ShowModel) then
		PhysicsThread:Spawn(function()
			local CheckpointModel = CreateAdvancedMesh(nil, "Models\\CheckpointModel.dcm", "")
			GetEntityScene():AddEntity(CheckpointModel)
			
			local Matrix = Checkpoint:GetMatrix():TranslateLocal(Vector(0,0,0))
			CheckpointModel:Update(Matrix, VectorZero, VectorZero)
			
			PhysicsThread:Sleep(1)
		
			if (self.CheckpointModel ~= nil) then
				self.CheckpointModel:Destroy()
				self.CheckpointModel = nil
			end
			
			self.CheckpointModel = CheckpointModel
		end)
	end
end

function BaseMission:Lap(Player)
	return false
end

function BaseMission:Back()
	if (self.IngameMenuEnabled) then
		if (ActiveMenu == nil) then
			self:ShowIngameMenu()
		end
	end
end

function BaseMission:Start()
	if (CutsceneBars ~= nil and ActiveGameMode.Name ~= "Test")then
		CutsceneBars:Destroy()
		CutsceneBars = nil
	end
	
	self.IngameMenuEnabled = true
	
	self.Listeners = AddKeyListeners(ToggleMenuButtons, "Down", function(Listener)
		GameThread:Spawn(function()
			if (self.IngameMenuEnabled) then
				if (ActiveMenu == nil) then
					self:ShowIngameMenu()
				end
			end
		end)
	end)
	
	print("BaseMission:Start()")
	self.HumanPlayers[1].HUD = ActiveGameMode:HUD(self, self.HumanPlayers[1])
	self.HumanPlayers[1].HUD:Show(true)
	
	--if (TutorialsEnabled) then
		--WriteGameSettings()
		--self:StartTutorial()
	--else
		self:StartCountdown()
	--end
	
	if (FuelEnergy) then
		FuelEnergy:DecreaseFuel(1)
		
		if (FuelEnergy.Stats_FreeFuelCounter > 0) then
			FuelEnergy:SendFuelStats(0, FuelEnergy.Stats_FreeFuelCounter, 0, 0)
			FuelEnergy.Stats_FreeFuelCounter = 0
		end
	end
	
	if (MechanicActiveRaces > 0) then
		MechanicActiveRaces = Dec(MechanicActiveRaces)
	end
	
	for i, Player in ipairs(self.Players) do
		if (Player.PowerupManager ~= nil and Player.Human) then
			if (Player.PowerupManager.BoostPowerup.Enabled) then
				if (Player.HUD ~= nil and Player.HUD.AccelerateSlider ~= nil) then
					Player.HUD.AccelerateSlider:TurnBoostOn()
				end
			end
		end
	end
end

function BaseMission:StartCountdown()
	print("^1Basemission:StartCountdown(); Function not implemented!")
end

function BaseMission:Countdown(Event)
	for i, Player in ipairs(self.Players) do
		local HUD = Player.HUD

		if (HUD ~= nil) then
			if (Event == 3) then
				HUD:ShowCountdown("3")
				playsound("audio\\countdown_3.wav")
			elseif (Event == 2) then 
				HUD:ShowCountdown("2")
				playsound("audio\\countdown_2.wav")
			elseif (Event == 1) then
				HUD:ShowCountdown("1")
				playsound("audio\\countdown_1.wav")
			elseif (Event == 0) then
				HUD:ShowCountdown(Translate("countdown_go"))

				self.RaceTimer = self.RaceTimerStart
				
				playsound("audio\\RaceStart_" .. math.random(1,7) .. ".wav")
			end
		end
	end
end

function BaseMission:StartTutorial()
	--
end

function BaseMission:StartPhotoshoot(Player)
	StartPhotoshoot(self, Player, true, nil)
end

function BaseMission:CarClipped()
	--self:EndGame() 
end

function BaseMission:EndGame()
	self:SetScore()
	
	self.IngameMenuEnabled = false
	
	local HumanPlayer
	for i, Player in ipairs(ActiveMission.Players) do
		if (Player.InputManager ~= nil) then
			Player.InputManager:Reset()
		end
	
		if (Player.Human) then
			HumanPlayer = Player
		end
	end
	
	AnalyticsSendEvent("GameMode", "MostPlayed", ActiveGameMode.Name, "1")
	AnalyticsSendEvent("GameMode", "TimePlayed", ActiveGameMode.Name, tostring(self.RaceTimer))
	AnalyticsSendEvent("Car", "MostPlayed", ActiveGameMode:GetSelectedCar().Name, "1")
	
	local Player = self.HumanPlayers[1]
	
	if (Player.InputManager ~= nil) then
		Player.InputManager:Disable()
	end
	
	if (Player.HUD ~= nil) then
		Player.HUD:Hide()
	end
end

function BaseMission:StartTargetCam()
	local Camera = EntityCamera(0)
	local CamPos = Camera:GetMatrix():GetTranslation()
	local CamMatrix = Camera:GetMatrix()
	local CamLerpMatrix = CamMatrix
	local CamStartMatrix = Camera:GetMatrix()
	
	Camera:SetMode(13)
	
	if (self.HumanPlayers[1].Driver ~= nil) then
		self.HumanPlayers[1].Car:KillDriver()
	end

	PhysicsThread:Spawn(function()				
		local Length = 5*60
		local TotalForce = 300000
		
		for i = 1, Length do						
			local CamDir = Vector(0,0,0)
			
			if (self.TowTestObject ~= nil) then
				if (i < Length / 2) then
					CamDir = (Camera:GetMatrix():GetTranslation() - self.TowTestObject:GetMatrix():GetTranslation()):Normalize() * -1
				else
					CamDir = (Camera:GetMatrix():GetTranslation() - self.HumanPlayers[1].Car:GetMatrix():GetTranslation()):Normalize() * -1						
				end
			else
				CamDir = (Camera:GetMatrix():GetTranslation() - self.HumanPlayers[1].Car:GetMatrix():GetTranslation()):Normalize() * -1		
			end
			
			CamDir.x = CamDir.x * -1
			CamMatrix = CamMatrix:LookAt(Vector(0,0,0), CamDir, 0)
			
			CamMatrix = CamMatrix:Rotate(Vector(1,0,0), math.rad(180)):Inverse()
			CamMatrix = CamMatrix:Rotate(Vector(0,0,1), math.rad(180))
			
			CamMatrix = CamMatrix:Translate(CamPos)
			CamMatrix = Camera:GetMatrix():Lerp(CamMatrix, 0.7)

			CamMatrix = CamStartMatrix:Lerp(CamMatrix, 0.1)
			CamLerpMatrix = CamLerpMatrix:Lerp(CamMatrix, 0.1)
			
			Camera:SetMatrix(CamLerpMatrix)
			
			PhysicsThread:Wait(1)
		end
	end)
end

function BaseMission:PowerUpPopup()
	--[[ if (self.ClickedPowerUp ~= nil) then
		ShowConfirm_GotoPowerUp()
		self.ClickedPowerUp = nil
	end ]]
end

function BaseMission:ShowIngameMenu()
	print("Show Ingamemenu")
	
	var("IntPause", 1)
	
	self.PrevCarVolume = self:GetHumanCarVolume()
	
	self:SetCarsVolume(0)
	
	for i, Player in ipairs(self.Players) do
		if (Player.HUD) then
			Player.HUD:Hide()
		end
	end
	
	ActiveMenu = InGameMenu()
end

function BaseMission:HideIngameMenu()
	print("Hide Ingamemenu")
	
	var("IntPause", 0)
	
	self:SetCarsVolume(self.PrevCarVolume)
	
	for i, Player in ipairs(self.Players) do
		if (Player.HUD) then
			Player.HUD:Show()
		end
	end

	if (ActiveMenu ~= nil) then
		ActiveMenu:HideMenu()
	end
	ActiveMenu = nil
end

function BaseMission:UpdatePhysics(ElapsedTime)
	if (self.StuntManager ~= nil) then
		self.StuntManager:Update(ElapsedTime)
	end
	
	if (self.RaceTimer ~= nil) then
		self.RaceTimer = self.RaceTimer + ElapsedTime
	end

	for i, Player in ipairs(self.Players) do
		if (Player.InputManager ~= nil) then
			Player.InputManager:Update(ElapsedTime)
		end	
		
		if (Player.HUD ~= nil) then
			Player.HUD:UpdatePhysics(ElapsedTime)
		end
		
		if (not ActiveMission:is_a(FrameTwisterMission)) then
			if (Player.HUD ~= nil and self.RaceTimer ~= nil) then
				Player.HUD:SetScore(self.RaceTimer, 0, true)
			elseif (Player.HUD ~= nil) then
				Player.HUD:SetScore(0, 0, true)
			end
		end
	end
end

function BaseMission:Update(ElapsedTime)
	--collectgarbage("collect")
	
	if (self.PickUpManager ~= nil) then
		self.PickUpManager:Update(ElapsedTime)
	end

	if (self.StuntManager ~= nil) then
		self.StuntManager:UpdateScoreMessages(ElapsedTime)
	end
	
	for i = 1, #self.Players do
		local Player = self.Players[i]
		
		if (Player.Human) then
			Player.Progress = (Player.DistanceTraveled + ((Player.DistanceToGo) - Player.CheckDist)) / self.RaceLength
			if (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME4) then
				Player.Progress = Player.Progress * 2
			elseif (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME3) then
				Player.Progress = Player.Progress * 1.2
			end
		else
			if (Player.Car.GetReplayProgress ~= nil) then
				Player.Progress = Player.Car:GetReplayProgress()
			end
		end
		
		if (Player.Human and Player.HUD ~= nil) then
			if (not Player.Finished) then			
				local Matrix = Player.Car:GetMatrix()
				local Up = Matrix:GetUp();
				local Vel = Player.Car:GetVelocity()
				
				if (self.FlippedTimer == nil) then
					self.FlippedTimer = 0
				end
				
				if (Up.y < -0.6 and Vel:Magnitude() < 50) then
					if (Player.PowerupManager.RepairPowerup ~= nil and Player.PowerupManager.RepairPowerup.Enabled and not Player.PowerupManager.RepairPowerup.Used) then
						--Player.Finished = true
						--Player.HUD:CarFlipped(5)
						
						self.FlippedTimer = self.FlippedTimer + ElapsedTime
						
						if (self.FlippedTimer > 2) then
							Player.HUD.RepairButton:TurnOff()
							Player.PowerupManager:Repair()
							--Player.Car:Respawn()
							--teleport()
							
							-- ===============
							-- New teleport
							local CarPos = Player.Car:GetMatrix():GetTranslation()
							local CarDir = Player.Car:GetMatrix():GetDirection()
							local Matrix = Matrix34():Translate(CarPos)
							
							local Up = Player.Car:GetMatrix():GetUp()
							if (Up.y < 0) then
								CarDir = CarDir * -1
							end
							
							Matrix = Matrix:LookAt(CarPos, CarPos + CarDir, 0)
							
							Matrix = Matrix:Translate(Vector(0,30,0))
							Player.Car:SetMatrix(Matrix)
							
							Player.Car:SetVelocity(VectorZero)
							-- ===============
						end
					else
						self.FlippedTimer = 0
						
						if (Player.RespawnTimer == nil) then
							Player.RespawnTimer = 0	-- signal afgeven?
						else
							Player.RespawnTimer = Player.RespawnTimer + ElapsedTime

							if (Player.RespawnTimer > 1) then
								local EndGame = true
								if (EndGame) then
									Player.LastLapTime = self.RaceTimer
									Player.Failed = true
									self:EndGame(false)
								else
									Player.Car:Respawn()
								end
								
								Player.RespawnTimer = nil
							end
						end
					end
				else
					Player.RespawnTimer = nil
				end
			end
		end
	end
end

function BaseMission:UpdateRaceData(Player, CheckCount, CheckDist, TimeOffset, UpdateRaceDataTime)
	if (Player ~= nil) then			
		Player.CheckCount = CheckCount
		Player.CheckDist = CheckDist
		Player.TimeOffset = TimeOffset
		
		if (self.CurrentNextCheckpoint ~= nil) then
			Player.DistanceToGo = self.CurrentNextCheckpoint:GetLength()
		end
		
		if (UpdateRaceDataTime ~= nil and UpdateRaceDataTime < 0) then
			self.RaceDataUpdated = true
			
			local DebugRaceData = false
			if (DebugRaceData and Player.Human) then
				print("HUD: ^3" .. tostring(Player.DistanceToGo - Player.CheckDist) .. "^7 // ^3" .. tostring(Player.DistanceToGo))
			end
		end
	end
end

function BaseMission:CarDead(Car)
	local Player = self:GetPlayer(Car)
	
	print("^3CarDead")
	
	if (not Player.Human) then
		return
	end
	
	self:StartTargetCam()

	PhysicsThread:Spawn(function()
		PhysicsThread:Sleep(1)
		
		self:EndGame()
	end)
end

function BaseMission:Respawned(Car)
	self:EndGame()
end

function BaseMission:GetScore()
	return self.Score
end

function BaseMission:SetScore()
	self.Score = 0
end

function BaseMission:SetStuntScore(StuntName, Score)
	--
end

function BaseMission:GetGhostFileName(Index)
	return "Replays\\" .. GetScriptName() .. "_" .. Index .. ".ghost"
end

function BaseMission:EntityCallback(Name, ID)
	--
end

function BaseMission:MaySlip(Car, AccelForce, MaxWheelTorque, OverPower)
	return false
end

function BaseMission:SetCarsVolume(Value)
	for i, Player in ipairs(self.Players) do
		Player.Car:SetVolume(Value)
	end
end

function BaseMission:GetPlayerCount()
	return #self.Players
end

function BaseMission:GetPlayerCar(Index)
	if (self.Players[Index + 1] ~= nil) then
		return self.Players[Index + 1].Car
	else
		return nil
	end
end

function BaseMission:GetHumanCarVolume()
	local Player = self.HumanPlayers[1]
	
	if (Player ~= nil) then
		return Player.Car:GetVolume()
	else
		return 0
	end
end

function BaseMission:GetHumanPlayer(PlayerIndex)
	for i, Player in ipairs(self.Players) do
		if (not Player.AI) then
			return Player
		end
	end
	
	return nil
end

function BaseMission:GetPlayerRef(Index)
	return self.Players[Index + 1];
end

function BaseMission:GetPlayer(Car)
	for i, Player in ipairs(self.Players) do
		if (Player.Car == Car) then
			return Player
		end
	end
	
	return nil
end

function BaseMission:GetPlace(Car)
	local Player = self:GetPlayer(Car)
	
	table.sort(self.Players, RacePosition_PlayerSort)
				
	local Index = IndexInTable(self.Players, Player)
	
	return Index
end

function BaseMission:GetRealPlace(Table, Player)
	table.sort(Table, RacePosition_PlayerSort2)
	local Index = IndexInTable(Table, Player)
	
	return Index;
end

function BaseMission:PlayMusic(FileName)
	CreateSound(FileName, false, function(Sound)
		self:StopMusic()

		MusicGroup:Add(Sound)
		Sound:Play(true)

		Music = Sound
	end)
end

function BaseMission:StopMusic()
	if (Music ~= nil) then
		Music:Stop()
		Music:Destroy()
		Music = nil
	end
end

function RacePosition_PlayerSort(x, y)
	if (x.Finished and y.Finished) then
		return x.TrackTime < y.TrackTime
	elseif (x.Finished and not y.Finished) then
		return true
	elseif (not x.Finished and y.Finished) then
		return false
	end
			
	if (x.Dead and not y.Dead) then
		return false
	elseif (not x.Dead and y.Dead) then
		return true
	end
		
	if (x.CheckCount > y.CheckCount) then
		return true
	elseif (x.CheckCount == y.CheckCount) then
		if (x.CheckDist < y.CheckDist) then
			return true
		else
			return false
		end
	else
		return false
	end
end

function IDSortingFunction(x, y)
	if (x.ID and y.ID) then
		return x.ID < y.ID
	elseif (x.ID and not y.ID) then
		return true
	elseif (not x.ID and y.ID) then
		return false
	end
			
	if (x.Human and not y.Human) then
		return false
	elseif (not x.Human and y.Human) then
		return true
	end
		
	return false
end

function BaseMission:TargetSort_Direction(x, y)
	return x.Dot > y.Dot
end

function BaseMission:GetTargets(Attacker, Dir)
	local Pos = Attacker:GetPos()
	
	local Targets = {}

	for i, Player in ipairs(self.Players) do
		local Car = Player.Car;
		
		if (Car ~= Attacker) then
			local TargetPos = Car:GetPos()
			
			local TargetDir = Pos:DirectionTo(TargetPos)
			
			local Dot = TargetDir:Dot(Dir)
			
			local Target = {}
			Target.Dot = Dot;
			Target.Car = Car;

			table.insert(Targets, Target);
		end
	end
	
	table.sort(Targets, function(x, y) return self:TargetSort_Direction(x, y) end)

	return Targets
end

function BaseMission:ReturnSlipRotationValue()
	return 0
end

function BaseMission:ClickedPowerUps(Power)
	self.ClickedPowerUp = Power
end

-- Server functions
function BaseMission:SendScore(NewHighscore)
	print("BaseMission:SendScore")

	-- Send scores
	
	local FailureFunction = function(Data)
		print("Error sending score")
	end
	
	local SuccessFunction = function(Data)
		print('OnAddScore')
		
		if (Data ~= nil and Data['Scores'] ~= nil) then
			self.HighscoreClient.Results = Data
			
			--if (ActiveMenu ~= nil and ActiveMenu.OnReceiveResults ~= nil) then
				--ActiveMenu:OnReceiveResults()
			--end
		else
			FailureFunction()
		end
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	local Script = GetScriptName()
	local Score = NewHighscore
	local CarID = IndexInTable(MonsterTruckCars.Cars, ActiveGameMode:GetSelectedCar()) + 1
	
	ScoresUser:AddScore(UserID, Script, Score, CarID, SuccessFunction, FailureFunction)
end

function BaseMission:SendStats(NewHighscore)
	print("BaseMission:SendStats")

	-- Send stats
	
	local FailureFunction = function(Data)
		print("Error sending stats")
	end
	
	local SuccessFunction = function(Data)
		print('OnAddStats')
		
		--if (Data ~= nil and Data['Scores'] ~= nil) then
			--self.HighscoreClient.Results = Data
			
			--if (ActiveMenu ~= nil and ActiveMenu.OnReceiveResults ~= nil) then
				--ActiveMenu:OnReceiveResults()
			--end
		--else
			--FailureFunction()
		--end
	end
	
	local UserID = 0
	if (FBManager:IsLoginValid()) then
		UserID = FBManager.UserinfoTable['HighscoresID']
	else
		UserID = Scores.UserID
	end
	
	if (self.RaceTimer ~= nil) then
		local Script = GetScriptName()
		local Score = NewHighscore
		local CarID = IndexInTable(MonsterTruckCars.Cars, ActiveGameMode:GetSelectedCar()) + 1
		local PlayingTime = string.format("%f", self.RaceTimer)
		
		ScoresUser:AddScriptStats(UserID, Script, Score, CarID, PlayingTime, SuccessFunction, FailureFunction)
	end
end

dofile("Lua\\Missions\\MenuMission.lua")
dofile("Lua\\Missions\\RaceMission.lua")
dofile("Lua\\Missions\\TowTestMission.lua")
dofile("Lua\\Missions\\HillJumpMission.lua")
dofile("Lua\\Missions\\FreestyleFestMission.lua")
dofile("Lua\\Missions\\DestroyerMission.lua")
dofile("Lua\\Missions\\DeathMatchMission.lua")
dofile("Lua\\Missions\\FrameTwisterMission.lua")
dofile("Lua\\Missions\\HillClimbMission.lua")
dofile("Lua\\Missions\\TankTrapMission.lua")
dofile("Lua\\Missions\\TestMission.lua")