FrameTwisterMission = class(BaseMission, function(self, RaceData)
	print("^3FrameTwisterMission created")
	
	-- Set gravity 
	setgravity(750)
	
	-- Create player
	local Player = self:CreateHumanPlayer(self.RaceData.Car, self.RaceData.Skin, "Player 1")
	local Car = Player.Car
	Car:SetMatrix(Car:GetMatrix():SetRotation(MatrixIdentity:GetRotation():RotateLocal(VectorY, math.rad(90))))
	
	-- Define startpos
	self.StartPos = Car:GetMatrix():GetTranslation()
	
	-- Car axis-alignment
	if (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME2) then
		self:SetAxisAlignment(VectorX, 1)
	else
		self:SetAxisAlignment(VectorZ, 1)
	end
	
	-- Play frametwister music
	self:PlayMusic("Music\\FRAMETWISTER.ogg")
	
	-- Game related vars
	self.TotalDistance = 0
	self.DistanceReached = 0
	self.CheckpointDistance = 100
	self.CurrentCheckpointIndex = 1
	
	-- Store level end matrix
	local LevelEndMatrix = GetScriptMatrix("LevelEnd")
	
	-- Check for finish
	PhysicsThread:Spawn(function()				
		-- Define end position
		if (LevelEndMatrix ~= nil) then
			if (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME2) then
				self.EndPos = LevelEndMatrix:GetTranslation()
			else
				print("setting endpos NOT THEME2")
				self.EndPos = LevelEndMatrix:GetTranslation()
				self.EndPos.x = 0.0
				self.EndPos.y = 0.0
			end
		else	
			print("No legitemate ^1'LevelEnd'-trigger target^7 defined! Using ^1HARDCODED^7 endpos!")
		end
		
		-- Loop until finish
		while (true) do
			self.Pos = Car:GetMatrix():GetTranslation()
			
			-- Check distance to end pos. If within range, end the game
			if (not Player.Finished and self.Pos:DistanceTo(self.EndPos) < self.CheckpointDistance) then
				Player.LastLapTime = self.RaceTimer
				self:EndGame()
				break
			end
			
			PhysicsThread:Wait(1)
		end
	end)
	
	-- Init StartPoint, Checkpoints and LevelEnd
	self.Checkpoints = {}

	-- Add startposition as checkpoint
	table.insert(self.Checkpoints, MJ_Checkpoint(self.StartPos, 0, 0))
	
	-- Search for checkpoints and init
	for i = 1, 25 do
		local LevelCheckpointMatrix = GetScriptMatrix("Checkpoint" .. i)
		
		if (LevelCheckpointMatrix ~= nil) then
			local Pos = LevelCheckpointMatrix:GetTranslation()
			local Distance = Pos:DistanceTo(self.Checkpoints[i]:GetPos())
			
			self.TotalDistance = self.TotalDistance + Distance
			table.insert(self.Checkpoints, MJ_Checkpoint(Pos, Distance, self.TotalDistance))
		else
			local Pos = LevelEndMatrix:GetTranslation()
			local Distance = Pos:DistanceTo(self.Checkpoints[i]:GetPos())
			
			self.TotalDistance = self.TotalDistance + Distance
			table.insert(self.Checkpoints, MJ_Checkpoint(Pos, Distance, self.TotalDistance))
			break
		end
	end
	
	-- Render checkpoints
	local RenderCheckpoints = false
	if (RenderCheckpoints) then
		GameThread:Spawn(function()
			while (true) do
				for i = 1, #self.Checkpoints do
					local FromCheckpoint = self.Checkpoints[i]
					local ToCheckpoint = self.Checkpoints[i + 1]
					
					-- Progress
					if (FromCheckpoint ~= nil and ToCheckpoint ~= nil) then
						DrawLineASync(FromCheckpoint:GetPos(), ToCheckpoint:GetPos(), color(1, i / #self.Checkpoints, 0, 1))
					end
					
					-- Distance
					if (FromCheckpoint ~= nil) then
						local Color = White
						if (i == self.CurrentCheckpointIndex) then
							Color = color(0,1,0,1)
						end
						
						DrawLineASync(FromCheckpoint:GetPos(), FromCheckpoint:GetPos() + (VectorY * self.CheckpointDistance), Color)
					end
				end
				
				GameThread:Wait(0.001)
			end
		end)
	end
	
	-- Tutorial vars
	TutorialsEnabled = false
	
	-- Start the game
	self:Start()
	
	-- Call post constructor
	self:PostConstruction()
end)

function FrameTwisterMission:Start()
	self.Super.Start(self)

	for i, Player in ipairs(self.Players) do
		if (Player.Human) then
			print("Maak een human")
			Player.Driver = HumanDriver(Player.Car, 0)
			Player.InputManager = InputManager(Player, self)
			Player.InputManager:DisableSteering()
			Player.Car:Enable()
			
			if (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME2) then
				SetCamIndex(3)
			else
				SetCamIndex(1)
			end
		else
			print("Maak een AI")
		end

		if (Player.Driver ~= nil) then
			GetEntityScene():AddEntity(Player.Driver)

			Player.Driver:Init()
		end
	end
	
	self:SetCarsVolume(1)
end

function FrameTwisterMission:StartCountdown()
	PhysicsThread:Run(function()
		for i = 3, 1, -1 do
			self:Countdown(i)
			PhysicsThread:Wait(60)
		end 

		-- De GO!
		self:Countdown(0)
		self.Finished = false
		
		for i, Player in ipairs(self.Players) do
			Player.Car:EnableInput()
			
			if (Player.HUD ~= nil) then
				Player.HUD:EnablePowerUps()
			end

			if (Player.Driver ~= nil) then
				if (not Player.Human) then
					Player.Driver:Enable()
				elseif (Player.InputManager ~= nil) then
					Player.InputManager:Enable()
				end
			end
		end
	end)
end

function FrameTwisterMission:EndGame()
	self.Super.EndGame(self)
	
	--DailyMissionsManager:StartChecking(self, "Challenge_9")
	--DailyMissionsManager:StartChecking(self, "Challenge_10")
	
	print("Race finnished in", self.RaceTimer)
	
	GameThread:Spawn(function()
		local Player = self.HumanPlayers[1]
		if (not Player.Finished) then
			Player.Finished = true
			local Car = Player.Car

			if (Player.HUD ~= nil) then
				Player.HUD.SliderEnabled = false
				Player.HUD.CarAccel = 0
				Player.HUD.CarReverse = 0
				
				Player.HUD:Hide()
				Player.HUD = nil
			end
			
			local NewTrackRecord = nil
			local Result, Index = ActiveGameMode.ActiveChapter.ActiveRace:AddTrackScore(0, Player.Name, "carname", self.RaceTimer, FormatTime(self.RaceTimer), tostring(1))
			if (Result) then
				NewTrackRecord = FormatTime(self.RaceTimer)
			end
			
			ActiveMenu = RaceResultScreen()
			
			while (ActiveMenu ~= nil) do
				Car:Accelerate(0)
				Car:ChangeGear(0)
				Car:Brake(1)
				GameThread:Wait(1/60)
			end
		end
	end)
end

function FrameTwisterMission:Update(ElapsedTime)
	-- Call super
	self.Super.Update(self, ElapsedTime)
	
	-- Override play progress, base on checkpoints
	local Player = self.HumanPlayers[1]
	
	local Distance = 0
	for i = 1, #self.Checkpoints do
		if (i <= self.CurrentCheckpointIndex) then
			Distance = Distance + self.Checkpoints[i].Distance
		elseif (i - 1 == self.CurrentCheckpointIndex) then
			local TotalDistance = self.Checkpoints[i].Distance
			local CurrentDistance = Player.Car:GetMatrix():GetTranslation():DistanceTo(self.Checkpoints[i]:GetPos())
			Distance = Distance + (TotalDistance - CurrentDistance)
		end
	end
	
	-- Set distance, based on player progress
	self.DistanceReached = Distance
	local Score = ConvertToFeet(self.DistanceReached)
	Score = ClampMinMax(Score, 0.0, Score)
	Player.HUD:SetScore(Score, 0, false)
end

function FrameTwisterMission:UpdatePhysics(ElapsedTime)
	-- Call super
	self.Super.UpdatePhysics(self, ElapsedTime)
	
	-- Update vars
	local Car = self.HumanPlayers[1].Car
	self.Pos = Car:GetMatrix():GetTranslation()
	
	-- Handle checkpoints	
	for i = 1, #self.Checkpoints do
		local Checkpoint = self.Checkpoints[i]
	
		local CheckpointTriggered = Checkpoint:GetTriggered(self.Pos, self.CheckpointDistance)
		
		if (CheckpointTriggered) then
			-- Check if we enter the checkpoint in "front" or at the "back" of it, and handle accordingly
			if (self.DistanceReached < Checkpoint:GetTotalDistance()) then
				self.CurrentCheckpointIndex = i
			else
				self.CurrentCheckpointIndex = ClampMin(i - 1, 1)
			end
		end
	end
end

function ConvertToFeet(Decimeters)
	return Decimeters / 10 * 3.28084
end

function FrameTwisterMission:SetScore()
	local ConvertedPos = 0
	
	self.DistanceReached = Max(ClampMin(ConvertedPos, 0), self.DistanceReached)
	
	local Player = self.HumanPlayers[1]

	local MaxScoreDistance = 700000
	local MaxScoreHP = 100000
	
	self.Score = self.DistanceReached / (self.TotalDistance) * MaxScoreDistance
	self.Score = self.Score + ((Player.Car:GetHP() / Player.MaxHP) * (MaxScoreHP))
	self.Score = ClampMinMax(self.Score, 0, MaxScoreDistance + MaxScoreHP)
	
	print("^2" .. tostring(self.Score))
	
	self.Super.SendScore(self, self.Score)
	self.Super.SendStats(self, self.Score)
end

function FrameTwisterMission:CreateCarBuilder(Car, Skin)
	return LuaCarBuilder(Car, Skin, "PC_HillJump.XML")
end



MJ_Checkpoint = class(function(self, Pos, Distance, TotalDistance)
	self.Pos = Pos
	self.Distance = Distance
	self.TotalDistance = TotalDistance
	
	self.Triggered = false
end)

function MJ_Checkpoint:GetPos()
	return self.Pos
end

function MJ_Checkpoint:GetTotalDistance()
	return self.TotalDistance
end

function MJ_Checkpoint:GetTriggered(CarPos, Distance)
	if (not self.Triggered and CarPos:DistanceTo(self.Pos) < Distance) then
		self.Triggered = true
		return true
	elseif (self.Triggered and CarPos:DistanceTo(self.Pos) > Distance) then
		self.Triggered = false
	end

	return false
end