RaceMission = class(BaseMission, function(self, RaceData)
	print("^3RaceMission created")

	TotalDamage = 0
	
	LoadCharacterTextures()
	
	self.FinishRange = 200
	self.FinishLaps = 1
	
	-- Mission Related Variables
	self.MaxLapCount = self.RaceData.LapCount
	self.CurrentLap = 1
	self.Finished = false
	
	self:PlayMusic("Music\\Freestyle_INTRO_ONLY.ogg")
	
	-- ReplayCar
	local RecordPlayer = getvar("RecordPlayer")
	local PlayerRef = "Player 1"
	local Car = self.RaceData.Car
	local Skin = self.RaceData.Skin
	
	if (RecordPlayer ~= "" and tonumber(RecordPlayer) > 1) then
		PlayerRef = "Player ".. RecordPlayer
		Car = self.RaceData.AI[tonumber(RecordPlayer) - 1].Car
		Skin = self.RaceData.AI[tonumber(RecordPlayer) - 1].Skin
	else
		RecordPlayer = nil
	end 
	
	self.MinTime = 10
	self.MaxTime = 60
	
	self.CurrentIndex = 1

	-- Create player car
	--local Player = self:CreateHumanPlayer(self.RaceData.Car, self.RaceData.Skin, "Player 1")
	local Player = self:CreateHumanPlayer(Car, Skin, PlayerRef, RecordPlayer)
	if (RecordPlayer ~= nil) then
		local CarType = self.RaceData.AI[1].Car
		Player.RecordFileName = self:GetGhostFileName(tostring(RecordPlayer) .. "_" .. tostring(CarType))
		MessageBox("Recording player " .. RecordPlayer, "Filename: " .. tostring(Player.RecordFileName))
	end
	
	local AIPlayer = nil
	-- Create AI player, skip when recording a replay
	if (RecordPlayer == nil) then
		AIPlayer = self:CreateAIPlayer(2, self.RaceData.AI[1], RecordPlayer)
	end
	
	self:SetCarsVolume(0)
	
	PhysicsThread:Spawn(function()
		Player.HUD = nil

		CutsceneBars = nil
		
		local ImmediateStart = false
		
		if (ImmediateStart) then
			self:Start()
		else
			-- Show the Cutscene
			local HumanCarType = self.RaceData.Cartype
			local AICarType = self.RaceData.AI[1].Car
			
			CutsceneBars = HUD_CutsceneBars(HumanCarType, AICarType)
		end
		
		FinishStart()
		
		Player.Car:EnableInput()
		Player.Car:Enable()
		
		Player.Car:Brake(1)
		Player.Car:Handbrake(1)
		
		if (AIPlayer ~= nil) then
			AIPlayer.Car:EnableInput()
			AIPlayer.Car:Disable()
		end
		
		-- Init the bread crum trail
		PhysicsThread:Wait(10)
		
		if (self.TrailNodes ~= nil) then
			local TrailNodeSortinFunc = function(x, y)
				if (x.ID < y.ID) then
					return true
				else
					return false
				end
			end
			
			table.sort(self.TrailNodes, TrailNodeSortinFunc)
			
			for i, TrailNode in ipairs(self.TrailNodes) do
				TrailNode:Hide()
			end
		end
	end)
	
	self:PostConstruction()
	
	AddKeyListeners("DCK_MOUSE0", "Down", function(Listeners)
		local Collision, PosX, PosY, PosZ = WorldPos()			--Moet via ViewRay geom gebeuren
		if (Collision) then
			for i, Player in ipairs(self.Players) do
			
				--if (Player.Human and Player.Agent ~= nil) then
				if (Player.Driver ~= nil and Player.Driver.Agent ~= nil) then
					print(Player.Driver.Agent)
					
					Player.Driver.Agent:FindPath(PosX, PosY, PosZ)
				end
			end
		end
	end)
end)

function RaceMission:Destroy()
	self.Super.Destroy(self)
	
	self:RemoveTrail()
end

function RaceMission:CreateCarBuilder(Car, Skin)
	return LuaCarBuilder(Car, Skin, "PC_Race.XML")
end

function RaceMission:StartTutorial()
	local function TutorialEnd()
		print("TutorialEnd")
		
		PhysicsThread:Spawn(function()
			PhysicsThread:Wait(60 * 1)
			self:StartCountdown()
		end)
	end
	
	local function StartTutorialMapBar()
		print("TutorialMapBar")
		self.HumanPlayers[1].HUD.MapBar:ShowTutorial(TutorialEnd)
	end
	
	local function StartTutorialPowerups()
		print("TutorialPowerups")
		self.HumanPlayers[1].HUD.FireWorksButton:ShowTutorial(StartTutorialMapBar)
	end
	
	local function StartTutorialDiffTimer()
		print("StartTutorialDiffTimer")
		self.HumanPlayers[1].HUD.ScoreElement:ShowTutorial(StartTutorialPowerups)
	end
	
	local function StartTutorialTimer()
		print("StartTutorialTimer")
		self.HumanPlayers[1].HUD.Timer:ShowTutorial(StartTutorialDiffTimer)
	end
	
	local function StartTutorialHealthBar()
		print("TutorialHealthBar")
		self.HumanPlayers[1].HUD.HealthBar:ShowTutorial(StartTutorialTimer)
	end
	
	local function StartTutorialAccelerateSlider()
		print("TutorialAccelerateSlider")
		--self.HumanPlayers[1].HUD.AccelerateSlider:ShowTutorial(StartTutorialHealthBar)
	end
	
	--self.HumanPlayers[1].HUD:ShowSteerTutorial(StartTutorialAccelerateSlider)
	self.HumanPlayers[1].HUD:ShowSteerTutorial(StartTutorialHealthBar)
end

function RaceMission:Start()
	self.Super.Start(self)
	
	self:PlayMusic("Music\\CrowdArena.ogg")

	-- Creeer de drivers
	for i, Player in ipairs(self.Players) do
		if (Player.Human) then
			print("Maak een human")
			Player.Driver = HumanDriver(Player.Car, 0)
			Player.InputManager = InputManager(Player, self)
			
			SetCamIndex(0)
		else
			print("Maak een AI")
		end

		if (Player.Driver ~= nil) then
			if (Player.Driver.Agent ~= nil) then
				GetEntityScene():AddEntity(Player.Driver.Agent)
			else
				GetEntityScene():AddEntity(Player.Driver)
			end
			
			Player.Driver:Init()
		end
	end
	
	self:SetCarsVolume(1)
	
	TriggerTarget("SkipIntro")
	
	-- Zet de HUD aan
	for i, Player in ipairs(self.Players) do
		if (Player.HUD ~= nil) then
			Player.HUD:ShowLapCount()
			Player.HUD:SetLapCount(1, self.RaceData.LapCount)
			
			local Race = ActiveGameMode.ActiveChapter.ActiveRace
			local LapRecordTime = Translate('hud_nolaprecord')
			if Race.LapScores[#Race.LapScores] ~= nil then
				LapRecordTime = Race.LapScores[#Race.LapScores].VisualScore
			end
			Player.HUD:SetLapRecord(LapRecordTime)
			
			Player.HUD:ShowRank()
		end
	end
end

function RaceMission:StartCountdown()
	PhysicsThread:Run(function()
		for i = 3, 1, -1 do
			self:Countdown(i)
			PhysicsThread:Wait(60)
		end 

		-- De GO!
		self:Countdown(0)
		
		self.Finished = false

		for i, Player in ipairs(self.Players) do
			Player.Car:EnableInput()
			Player.Car:Enable()
			
			if (Player.HUD ~= nil) then
				Player.HUD:EnablePowerUps()
			end

			if (Player.Driver ~= nil) then
				if (not Player.Human) then
					Player.Driver:Enable()
				elseif (Player.InputManager ~= nil) then
					Player.InputManager:Enable()
				end
			end
			
			if (Player.RecordFileName ~= nil) then
				Player.Recorder = EntityRecorder(Player.Car)
				GetEntityScene():AddEntity(Player.Recorder);
			end
		end
	end)
end

function RaceMission:StartPhotoshoot(Player)
	StartPhotoshoot(self, Player, false, false, true)
end

function RaceMission:Lap(Player)
	local Car = Player.Car
	
	if (self.RaceTimer ~= nil) then
		Player.LastLapTime = self.RaceTimer
	else
		Player.LastLapTime = 0
	end
	
	-- Add recording
	if (Player.Recorder ~= nil) then		
		PhysicsThread:Spawn(function()
			local CheckSpeed = true
			while (CheckSpeed) do
				Car:Accelerate(0)
				Car:Brake(1)
				Car:Handbrake(1)
				
				if (Car:GetSpeed() < 1) then
					MessageBox("Saving player " .. Player.RecordFileName, "Saving player " .. Player.RecordFileName)
					Player.Recorder:Save(Player.RecordFileName)
					CheckSpeed = false
				end
				PhysicsThread:Wait(1)
			end
		end)
	end
	
	-- Disable car 
	if (Player.Driver) then
		Player.Driver:Disable()
	end
	
	if (Player.Human) then
		self.CurrentLap = Inc(self.CurrentLap)
	
		local HumanTime = Player.LastLapTime
		
		-- Manage points
		if (not self.Finished) then
			Player.Points = Inc(Player.Points)
			self.Finished = false
		end
		
		-- No PhotoShoot during recording
		if (Player.Recorder == nil) then
			self:StartPhotoshoot(Player)
		end
		
		if (self.CurrentLap > self.MaxLapCount) then	
			for i, PlayerInst in ipairs(self.Players) do
				if (not PlayerInst.Human) then
					if (PlayerInst.LastLapTime == 0) then
						PlayerInst.LastLapTime = HumanTime + (HumanTime - (HumanTime * ClampMinMax(PlayerInst.Progress, 0, 0.9)))
					end
				end
			end
		
			-- Calculate the winner
			table.sort(self.Players, PlayerSort_LapTime)
			
			self.IngameMenuEnabled = false
			
			InputThread:Spawn(function()
				InputThread:Sleep(4)
				
				-- Show endscreen
				local HumanWon = self.Players[1].Human
				local Winner = self.Players[1]
				
				self:EndGame(HumanWon, Winner)
			end)
			
			return true
		else
			-- Restart
			PhysicsThread:Spawn(function()
				local FadeDuration = 1.5
			
				PhysicsThread:Sleep(FadeDuration / 2)
				
				-- Screen Fade			
				if (Player.HUD ~= nil) then
					Player.HUD:FadeOut(FadeDuration)
				end
				
				PhysicsThread:Sleep(FadeDuration)
				
				for i, Player in ipairs(self.Players) do
					if (Player.Human) then
						local Matrix = GetScriptMatrix("Player 1")
						if (Matrix ~= nil) then
							Matrix = Matrix:TranslateLocal(Vector(0,10,0))
							Player.Car:SetMatrix(Matrix)
						else
							Player.Car:SetMatrix(MatrixIdentity)
						end
					else
						local Matrix = GetScriptMatrix("Player " .. i)
						if (Matrix ~= nil) then
							Player.Car:SetMatrix(Matrix)
						else
							Player.Car:SetMatrix(MatrixIdentity)
						end
					end
					
					Player.Car:SetVelocity(VectorZero)
					Player.Car:Disable()
					Player.Car:KillDriver()
					
					self:SetCarsVolume(0)
				end
				
				if (Player.HUD ~= nil) then
					Player.HUD:FadeIn(FadeDuration)
				end
				
				PhysicsThread:Sleep(FadeDuration)
				
				self:StartCountdown()
			end)
		end
	else
		if (Player.Car.KillDriver ~= nil) then
			Player.Car:KillDriver()
		end
		
		-- Manage points
		if (not self.Finished) then
			Player.Points = Inc(Player.Points)
			self.Finished = false
		end
	end
	
	return false
end

function RaceMission:EndGame(HumanWon, Winner)
	DailyMissionsManager:StartChecking(self, "Challenge_13")
	DailyMissionsManager:StartChecking(self, "Challenge_14")
	
	local Player = self.HumanPlayers[1]
	local HumanTime = Player.LastLapTime
	for i, PlayerInst in ipairs(self.Players) do
		if (not PlayerInst.Human) then
			if (PlayerInst.LastLapTime == 0) then
				PlayerInst.LastLapTime = ClampMinMax(PlayerInst.Progress, 0, 0.9)
			end
		end
	end
	
	self.Super.EndGame(self)
	
	self:RemoveTrail()
	
	--local Player = self.HumanPlayers[1]
	
	if (Player.HUD ~= nil) then
		Player.HUD:Hide()
		Player.HUD = nil
	end
	
	CutsceneBars = HUD_CutsceneFinished()
	self.SliderEnabled = false
		
	local Text = "--"
	local SubText = "--"
		
	--[[ if (HumanWon) then
		Text = Winner.Name .. " has won the race!"
		SubText = "Great job!"
	else
		Text = Winner.Name .. " has won the race."
		SubText = "You can do better than this!"
	end ]]
	
	--CutsceneBars:SetTitle(Text)
	--CutsceneBars:SetSubTitle(SubText)
end

function RaceMission:UpdatePhysics(ElapsedTime)
	self.Super.UpdatePhysics(self, ElapsedTime)
	
	local HumanPlayer = nil
	local AIPlayer = nil
	
	for i, Player in ipairs(self.Players) do
		if (Player.Driver ~= nil and Player.Driver.Update ~= nil) then
			if (Player.Driver:is_a(AgentAI_RaceDriver) or Player.Driver:is_a(AgentAI_SteeringDriver)) then
				Player.Driver:Update(ElapsedTime)
			end
		end
		
		if (Player.Human) then
			HumanPlayer = Player
		else
			AIPlayer = Player
			
			for i = 1, 10 do
				local Matrix = GetScriptMatrix("Finish_" .. i)
				if (Matrix == nil) then
					break
				end
				
				local Distance = AIPlayer.Car:GetMatrix():GetTranslation():DistanceTo(Matrix:GetTranslation())
				if (Distance < self.FinishRange and self.AICanFinish) then
					if (self.FinishLaps > 0) then
						self.FinishLaps = Dec(self.FinishLaps)
					else
						self:Lap(Player)
					end
					self.AICanFinish = false
				elseif (Distance > self.FinishRange) then
					if (not self.AICanFinish) then
						self.AICanFinish = true
					end
				end
			end
		end
	end
	
	if (HumanPlayer ~= nil and AIPlayer ~= nil) then
		if (HumanPlayer.HUD ~= nil and self.RaceTimer ~= nil) then							
			local TotalScore = 0
			
			local Mult = 0.04
			TotalScore = (HumanPlayer.Progress - AIPlayer.Progress) / Mult
			
			if (self:GetPlace(HumanPlayer.Car) == 1) then
				TotalScore = TotalScore * -1
			end
			
			HumanPlayer.HUD:SetScore(TotalScore, true, true)
		elseif (HumanPlayer.HUD ~= nil) then
			HumanPlayer.HUD:SetScore(0, true, true)
		end
	end
	
	if (self.TrailNodes ~= nil) then
		if (self.Perc == nil) then
			self.Perc = 0
		end
		
		self.CurrentIndex = Lerp(self.CurrentIndex, Round(self.Perc * #self.TrailNodes), 0.05)
				
		local Area = 25
		local LoopStart = ClampMinMax(Round(self.CurrentIndex - (Area / 2)), 1, #self.TrailNodes)
		local LoopEnd = ClampMinMax(Round(self.CurrentIndex + (Area / 2)), 1, #self.TrailNodes)
		local Offset = 5
		
		for i = 1, #self.TrailNodes do
			if (i > LoopStart and i < LoopEnd) then
				if (i < ClampMinMax(self.CurrentIndex - Offset, 1, #self.TrailNodes)) then
					self.TrailNodes[i]:Hide()
				else
					self.TrailNodes[i]:Show()
				end
			end
			
			self.TrailNodes[i]:Update()
		end
	end
end

function RaceMission:UpdateRaceData(Player, CheckCount, CheckDist, TimeOffset, UpdateRaceDataTime)
	self.Super.UpdateRaceData(self, Player, CheckCount, CheckDist, TimeOffset, UpdateRaceDataTime)
	
	-- Updating the bread crum path
	local Player = self.HumanPlayers[1]
	if (Player ~= nil) then
		self.Perc = Player.Progress
		--print(self.Perc)
	end
end

function RaceMission:Update(ElapsedTime)
	self.Super.Update(self, ElapsedTime)

	-- Rubber banding (dit hoeft op zich niet iedere frame als het veel zou kosten)
	local HumanRank = self:GetPlace(self.HumanPlayers[1].Car)
	--[[ if (HumanRank ~= nil) then
		for i, Player in ipairs(self.Players) do
			if (not Player.Human) then
				if (Player.Car.SetSpeedMult ~= nil) then
					if (Player.SpeedMult == nil) then
						Player.SpeedMult = 1
					end
					
					local OveralSpeedMultiplier = 1
					local HumanInFront = 1.25
					local HumanBehindMult = 0.75
					
					if (i <= HumanRank) then
						Player.SpeedMult = Lerp(Player.SpeedMult, OveralSpeedMultiplier * HumanInFront, ElapsedTime)
					else
						Player.SpeedMult = Lerp(Player.SpeedMult, OveralSpeedMultiplier * HumanBehindMult, ElapsedTime)
					end

					Player.Car:SetSpeedMult(Player.SpeedMult)
				end
			end
		end
	end ]]
end

function RaceMission:SetScore()
	self.Score = Round(ClampMinMax(ClampMin((self.MinTime + self.MaxTime) - self.RaceTimer, 0) * (700000 / 2 / self.MaxTime), 0, 700000))
	
	table.sort(self.Players, PlayerSort_LapTime)
	local HumanWon = self.Players[1].Human
	
	if (HumanWon) then
		self.Score = self.Score + 350000
	elseif (self.Players[2].Failed) then
		self.Score = 0
	end
	
	self.Super.SendScore(self, self.Score)
	self.Super.SendStats(self, self.Score)
end

function PlayerSort_LapTime(x, y)
	if (x.LastLapTime == 0) then
		return false
	elseif (y.LastLapTime == 0) then
		return true
	end
	
	if (x.LastLapTime < y.LastLapTime) then
		return true
	else
		return false
	end
end

function InitTrail(WorldObject)	
	local Node = TrailNode(WorldObject)
	
	return Node
end

function RaceMission:RemoveTrail()
	if (self.TrailNodes == nil) then
		return
	end

	for i, TrailNode in ipairs(self.TrailNodes) do
		TrailNode:Hide()
	end
	
	if (self.TrailThread ~= nil) then
		PhysicsThread:Spawn(function()
			PhysicsThread:Stop(self.TrailThread)
			self.TrailThread = nil
		end)
	end
	
	for i, TrailNode in ipairs(self.TrailNodes) do
		if (TrailNode ~= nil) then
			TrailNode:Destroy()
			TrailNode = nil
		end
	end
	self.TrailNodes = {}
end

TrailNode = class(function(self, WorldObject)
	if (ActiveGameMode.Name ~= "SingleRace") then
		return
	end
	
	GameThread:Spawn(function()	
		if (ActiveMission.TrailNodes == nil) then
			ActiveMission.TrailNodes = {}
		end
	
		self.WorldObject = WorldObject
		self.Name = WorldObject:GetName()
		self.Matrix = WorldObject:GetMatrix()
		self.ID = tonumber(string.sub(self.Name, string.len(self.Name) - 1, string.len(self.Name)))
		self.Shown = true
		self.MinHeight = self.Matrix:GetTranslation().y
		self.MaxHeight = 500
		
		table.insert(ActiveMission.TrailNodes, self)
		
		-- Add Mesh here
		self.Mesh = CreateAdvancedMesh(nil, "Models\\TrailNode.dcm", "")
		GetEntityScene():AddEntity(self.Mesh)
		self.Mesh:Update(self.Matrix, VectorZero, VectorZero)
	end)
	
	self.Thread = nil
end)

function TrailNode:Update()
	local Pos = self.Matrix:GetTranslation()
	local Y = 0
	
	local Animate = true
	if (Animate) then
		if (self.Shown) then
			Y = Lerp(Pos.y, self.MinHeight, 0.15) --0.15
		else
			Y = Lerp(Pos.y, self.MaxHeight, 0.05) --0.05
		end
	else
		Y = self.MinHeight;
	end
	
	self.Matrix = self.Matrix:SetTranslation(Vector(Pos.x, Y, Pos.z))
	self.Mesh:Update(self.Matrix, VectorZero, VectorZero)
end

function TrailNode:Destroy()
	--
end

function TrailNode:Show()
	if (not self.Shown) then
		self.Mesh:ShowObject("TrailNode")
		self.Shown = true
	end
end

function TrailNode:Hide()
	if (self.Shown) then
		self.Mesh:HideObject("TrailNode")
		self.Shown = false
	end
end