TankTrapMission = class(BaseMission, function(self, RaceData)
	print("^3TankTrapMission created")
	
	setgravity(750)
	
	if (MONSTERJAM_THEME_SETTING == MONSTERJAM_THEME3) then
		self.MinTime = 28
		self.MaxTime = 200
	else
		self.MinTime = 30
		self.MaxTime = 190
	end
	
	
	local Player = self:CreateHumanPlayer(self.RaceData.Car, self.RaceData.Skin, "Player 1")
	
	local StartMatrix = GetScriptMatrix("Player 1")
	local StartPos = StartMatrix:GetTranslation()
	
	-- Car axis-alignment
	--self:SetAxisAlignment(VectorZ, 1)
	
	self:PlayMusic("Music\\TankTrap.ogg")
	
	-- Check for finish
	PhysicsThread:Spawn(function()
		local Player = self.HumanPlayers[1]
		local Car = Player.Car
		
		self.StartPos = Car:GetMatrix():GetTranslation()
		
		self.EndPos = -8600
		
		local LevelEndMatrix = GetScriptMatrix("LevelEnd")
		if (LevelEndMatrix ~= nil) then
			self.EndPos = LevelEndMatrix:GetTranslation().z
		else	
			print("No legitemate ^1'LevelEnd'-trigger target^7 defined! Using ^1HARDCODED^7 endpos!")
		end
		
		while(true) do
			self.Pos = Car:GetMatrix():GetTranslation()
			local Dist = self.Pos:DistanceTo(LevelEndMatrix:GetTranslation())
			
			--if (not Player.Finished and self.Pos.z < self.EndPos) then
			if (not Player.Finished and Dist < 50) then
				Player.LastLapTime = self.RaceTimer
				self:EndGame()
			end
			
			PhysicsThread:Wait(1)
		end
	end)
	
	PhysicsThread:Spawn(function()
		Player.HUD = nil

		CutsceneBars = nil
		
		Player.Car:Enable()
		
		local ImmediateStart = false
		
		if (ImmediateStart) then
			self:Start()
		else
			-- Show the Cutscene
			CutsceneBars = HUD_CutsceneBars()
		end
		
		FinishStart()
	end)
	
	-- AddKeyListeners("DCK_U", "Down", function(Listeners)
		-- self.HumanPlayers[1].Car:SetHP(0)
	-- end)
	
	self:PostConstruction()
end)

function TankTrapMission:CreateCarBuilder(Car, Skin)
	return LuaCarBuilder(Car, Skin, "PC_Tanktrap.XML")
end

function TankTrapMission:StartTutorial()
	local function TutorialEnd()
		print("TutorialEnd")
		
		PhysicsThread:Spawn(function()
			PhysicsThread:Wait(60 * 1)
			self:StartCountdown()
		end)
	end
	
	local function StartTutorialPowerups()
		print("TutorialPowerups")
		self.HumanPlayers[1].HUD.FireWorksButton:ShowTutorial(TutorialEnd)
	end

	local function StartTutorialTimer()
		print("StartTutorialTimer")
		self.HumanPlayers[1].HUD.Timer:ShowTutorial(StartTutorialPowerups)
	end
	
	local function StartTutorialHealthBar()
		print("TutorialHealthBar")
		self.HumanPlayers[1].HUD.HealthBar:ShowTutorial(StartTutorialTimer)
	end
	
	self.HumanPlayers[1].HUD.AccelerateSlider:ShowTutorial(StartTutorialHealthBar)
end

function TankTrapMission:Start()
	self.Super.Start(self)

	for i, Player in ipairs(self.Players) do
		if (Player.Human) then
			print("Maak een human")
			Player.Driver = HumanDriver(Player.Car, 0)
			Player.InputManager = InputManager(Player, self)
			--Player.InputManager:DisableSteering()
			SetCamIndex(0)
		else
			print("Maak een AI")
		end

		if (Player.Driver ~= nil) then
			GetEntityScene():AddEntity(Player.Driver)

			Player.Driver:Init()
		end
	end
	
	self:SetCarsVolume(1)
end

function TankTrapMission:StartCountdown()
	PhysicsThread:Run(function()
		for i = 3, 1, -1 do
			self:Countdown(i)
			PhysicsThread:Wait(60)
		end 

		-- De GO!
		self:Countdown(0)
		self.Finished = false
		
		for i, Player in ipairs(self.Players) do
			Player.Car:EnableInput()
			Player.Car:Enable()
			
			if (Player.HUD ~= nil) then
				Player.HUD:EnablePowerUps()
			end

			if (Player.Driver ~= nil) then
				if (not Player.Human) then
					Player.Driver:Enable()
				elseif (Player.InputManager ~= nil) then
					Player.InputManager:Enable()
				end
			end
		end
	end)
end

function TankTrapMission:EndGame()
	self.Super.EndGame(self)
	
	DailyMissionsManager:StartChecking(self, "Challenge_9")
	DailyMissionsManager:StartChecking(self, "Challenge_10")
	
	print("Race finnished in", self.RaceTimer)
	
	GameThread:Spawn(function()
		local Player = self.HumanPlayers[1]
		if (not Player.Finished) then
			Player.Finished = true
			local Car = Player.Car

			if (Player.HUD ~= nil) then
				Player.HUD.SliderEnabled = false
				Player.HUD.CarAccel = 0
				Player.HUD.CarReverse = 0
				
				Player.HUD:Hide()
				Player.HUD = nil
			end
			
			local NewTrackRecord = nil
			local Result, Index = ActiveGameMode.ActiveChapter.ActiveRace:AddTrackScore(0, Player.Name, "carname", self.RaceTimer, FormatTime(self.RaceTimer), tostring(1))
			if (Result) then
				NewTrackRecord = FormatTime(self.RaceTimer)
			end
			
			ActiveMenu = RaceResultScreen()
			
			while (ActiveMenu ~= nil) do
				Car:Accelerate(0)
				Car:ChangeGear(0)
				Car:Brake(1)
				GameThread:Wait(1/60)
			end
		end
	end)
end

function TankTrapMission:CarDead(Car)
	local Player = self:GetPlayer(Car)

	self.Super.CarDead(self, Car)
	
	if (Player ~= nil) then
		Player.LastLapTime = self.RaceTimer
		Player.Failed = true
	end
end

function TankTrapMission:SetScore()
	local Player = self.HumanPlayers[1]
	if (Player.Failed) then
		--local Cal = (self.Pos.z - self.StartPos.z) / (self.EndPos - self.StartPos.z) * 10000
		self.Score = 0 --ClampMinMax(Round(ClampMin(Cal, 0) * (self.MaxTime / self.RaceTimer)), 0, 700000)
	else
		self.Score = ClampMinMax(Round(ClampMin((self.MinTime + self.MaxTime) - self.RaceTimer, 0) * (700000 / self.MaxTime)), 0, 700000)
	end
	
	self.Super.SendScore(self, self.Score)
	self.Super.SendStats(self, self.Score)
end