dofile("lua\\MonsterTruckCarWheel.lua")
dofile("lua\\MonsterTruckCarElement.lua")
dofile("lua\\MonsterTruckCarSlidingElement.lua")
dofile("lua\\MonsterTruckCarAxis.lua")

MonsterTruckCar = class(function(self, Car)
	self.Car = Car
	
	self.UpdateWheelConfig = false
	
	self.WheelConfig = {}
	self.WheelConfig.Name = "Oud"
	self.WheelConfig.Bounciness = 0
	self.WheelConfig.Force = 0
	self.WheelConfig.LimitCFM = 0.00001
	self.WheelConfig.LimitERP = 0.99
	self.WheelConfig.Limits = {}
	self.WheelConfig.Limits[1] = {1, 1} --{5, 6}		--FrontLeft Wheel
	self.WheelConfig.Limits[2] = {1, 1} --{5, 8}		--FrontRight Wheel
	self.WheelConfig.Limits[3] = {1, 1} --{5, 8}		--Back Wheels
	self.WheelConfig.SliderCFM = 0.001
	self.WheelConfig.Velocity = 0
	self.WheelConfig.FudgeFactor = 0
	
	self.WheelConfig.InertiaScale = VectorOne

	self.WheelConfig.Contact = {}
	self.WheelConfig.Contact.CFM = 0.00001;
	self.WheelConfig.Contact.ERP = 0.9999;
	self.WheelConfig.Contact.Friction1 = 500000
	self.WheelConfig.Contact.Friction2 = 500000
	self.WheelConfig.Contact.FrictionDir = VectorZ
	self.WheelConfig.MotorForce = {1000000 * 0.6, 1000000 * 0.6}
	self.WheelConfig.MotorAttack = 0.5
	self.WheelConfig.MotorRelease = 0.5

	self.WheelConfig.AccelForce = 20000
	self.WheelConfig.BrakeFace = 100000

	self.WheelConfig.AngularFriction = Vector(0,0,0)

	self.SoundEnabled = true
	self.RotCorrection = true
	
	self.Mesh = CreateAdvancedMesh(nil, "Vehicles\\" .. Car.Name .. "\\Chassis1.DCM", "")
		
	if (Car.ColorConfig == nil) then
		self:SetCarColorConfig(Car.ColorConfig)
	else
		local ColorConfig = {}
		ColorConfig.DiffuseColor = color(64, 0, 0, 255)
		ColorConfig.SpecularColor = color(255, 0, 0, 255)
		ColorConfig.SpecularIntensity = 10000000
		ColorConfig.SpecularPower = 40.0 / 255.0
		self:SetCarColorConfig(ColorConfig)
	end
	
	GetEntityScene():AddEntity(self.Mesh);
	
	self.Mesh:SetDestroyCallback(function(...)
		self:DestroyInternal(...)
	end)

	
	if (win32()) then
		--self.Mesh:SetRenderCallback(function(...)
			--self:Render(...)
		--end)
	end

	-- Get car object
	self.ChassisObject = self.Mesh:GetObject("Car")
	self.OffsetMatrix = self.ChassisObject:GetOffsetMatrix():Inverse()
	
	-- Offset matrix
	self.OffsetMatrix = MatrixIdentity--Matrix34(self.ChassisObject:GetCenter())

	-- Physics
	self.Body = Body(GetPhysicsScene())
	self.Body:SetMass(2500);
	self.Body:SetInertiaTensor(self.ChassisObject:GetExtents());
	--self.Body:SetCenterOfMass(Vector(0, -20, 0))
	
	--print("Center is " .. tostring(self.ChassisObject:GetCenter()))
	
	local COM = -4
	
	self.OffsetMatrix = MatrixIdentity:Translate(Vector(0, -COM, 0));

	-- Collision 
	self.Chassis = BoxGeometry()
	self.Chassis:SetEntity(self.Mesh)
	self.Chassis:SetBody(self.Body)
	self.Chassis:SetOffsetMatrix(self.OffsetMatrix:Translate(self.ChassisObject:GetCenter()))
	self.Chassis:SetExtents(self.ChassisObject:GetExtents())
	self.Chassis:SetType(bit32.bor(DCGF_REAL))
	self.Chassis:SetCollision(bit32.bor(DCGF_WORLD))
	self.Chassis:SetContactCallback(function(...)
		return self:OnContact(...)
	end)
	self.Chassis:Enable()

	GetPhysicsScene():AddGeometry(self.Chassis)

	-- Attach
	self.Chassis:SetBody(self.Body)

	-- Wheels
	self.Wheels = {}
		
	for i = 1, 4 do
		local Wheel = MonsterTruckCarWheel(self, i)
		table.insert(self.Wheels, Wheel)
	end
	
	-- Elements
	self.Elements = {}
	if (#self.Wheels > 0) then
		-- Voor
		local VoorAs_L = MonsterTruckCarElement(self, "VoorAs_L", self.Wheels[2], self.OffsetMatrix)
		table.insert(self.Elements, VoorAs_L);
		
		local VoorAs_R = MonsterTruckCarElement(self, "VoorAs_R", self.Wheels[1], self.OffsetMatrix)
		table.insert(self.Elements, VoorAs_R);
		
		local VoorPomp_Groot_L = MonsterTruckCarElement(self, "VoorPomp_Groot_L", VoorAs_L)
		table.insert(self.Elements, VoorPomp_Groot_L);
		
		local VoorPomp_Klein_L = MonsterTruckCarSlidingElement(self, "VoorPomp_Klein_L", VoorAs_L, self)
		table.insert(self.Elements, VoorPomp_Klein_L);
		
		local VoorPomp_Groot_R = MonsterTruckCarElement(self, "VoorPomp_Groot_R", VoorAs_R)
		table.insert(self.Elements, VoorPomp_Groot_R);
		
		local VoorPomp_Klein_R = MonsterTruckCarSlidingElement(self, "VoorPomp_Klein_R", VoorAs_R, self)
		table.insert(self.Elements, VoorPomp_Klein_R);
		
		-- Achter
		local AchterAs = MonsterTruckCarAxis(self, "AchterAs", self.Wheels[3], self.Wheels[4])
		table.insert(self.Elements, AchterAs);
		
		local AchterPomp_Groot_L = MonsterTruckCarElement(self, "AchterPomp_Groot_L", AchterAs)
		table.insert(self.Elements, AchterPomp_Groot_L);
		
		local AchterPomp_Klein_L = MonsterTruckCarSlidingElement(self, "AchterPomp_Klein_L", AchterAs, self)
		table.insert(self.Elements, AchterPomp_Klein_L);
		
		local AchterPomp_Groot_R = MonsterTruckCarElement(self, "AchterPomp_Groot_R", AchterAs)
		table.insert(self.Elements, AchterPomp_Groot_R);
		
		local AchterPomp_Klein_R = MonsterTruckCarSlidingElement(self, "AchterPomp_Klein_R", AchterAs, self)
		table.insert(self.Elements, AchterPomp_Klein_R);
	end
	
	-- Stuff
	local FinalMatrix = self.OffsetMatrix * self:GetMatrix()
	self.Mesh:Update(FinalMatrix, self:GetVelocity(), self:GetAngularVelocity())	--TODO
	
	self.WheelTimers = {0, 0, 0, 0}

	self.Contact = {}
end)

function MonsterTruckCar:Destroy()
	print("MonsterTruckCar:Destroy")
	--PhysicsThread:Spawn(function()
		self:Stop()
	
		if (self.Thread ~= nil) then
			PhysicsThread:Stop(self.Thread)
			self.Thread = nil
		end
		
		if (self.MoveThread ~= nil) then
			PhysicsThread:Stop(self.MoveThread)
			self.MoveThread = nil
		end
		
		if (self.Chassis ~= nil) then
			self.Chassis:Destroy()
			self.Chassis = nil
		end
		
		if (self.Body ~= nil) then
			self.Body:Destroy()
			self.Body = nil
		end
		
		for i = 1, #self.Wheels do
			local Wheel = self.Wheels[i]
			Wheel:Destroy()
		end
		
		if (self.Mesh ~= nil) then
			self.Mesh:Destroy()
			self.Mesh = nil
		end
	--end)
end

function MonsterTruckCar:DestroyInternal()
	
end

function MonsterTruckCar:Stop()
	--print("MonsterTruckCar:Stop() 1")
	self.Body:Disable()
	
	for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]
		Wheel.Body:Disable()
		Wheel.Geometry:Disable()
		Wheel.Joint:Disable()
	end
	--print("MonsterTruckCar:Stop() 2")
end

function MonsterTruckCar:Start()
	--print("MonsterTruckCar:Start() 1")
	self.Body:Enable()
	
	for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]
		Wheel.Body:Enable()
		Wheel.Geometry:Enable()
		Wheel.Joint:Enable()
		
		Wheel:Update()
	end
	--print("MonsterTruckCar:Start() 2")
end

function MonsterTruckCar:Reset()
	self:Stop()
	
	self.InitMatrix = nil
	self:SetMatrix(self.StartMatrix)
	
	self:Start()
end

function MonsterTruckCar:ChangeWheels(ModelName)
	self:Stop()
	
	for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]
		Wheel:ChangeMesh(ModelName)
	end
	
	self:Start()
end

function MonsterTruckCar:SetCarColorConfig(ColorConfig)
	if (self.Mesh ~= nil and ColorConfig ~= nil) then
		self.ColorConfig = ColorConfig
		for Key, Value in pairs(ColorConfig) do
			if (Key == "DiffuseColor" or Key == "SpecularColor") then
				self.Mesh:SetVar("Car" .. Key, Value / 511);
			else
				self.Mesh:SetVar("Car" .. Key, Value);
			end
		end
	end
end

function MonsterTruckCar:SetMatrix(Matrix)
	local Wheel = self.Wheels[1];

	if (Wheel ~= nil) then
		local Offset = -Wheel.OffsetMatrix:GetTranslation().y + Wheel.Object:GetExtents().y;	--TODO
		
		Offset = Offset + self.WheelConfig.Limits[3][1];
		
		
		if (self.InitMatrix == nil) then
			Matrix = Matrix:Translate(Vector(0, Offset, 0))	--TODO
			
			self.InitMatrix = Matrix
			self.InitMatrix_Position = self.InitMatrix:GetTranslation()
			self.InitMatrix_Rotation = self.InitMatrix:GetRotation()
			self.InitMatrix_RotationInv = self.InitMatrix:GetRotation():Inverse()
		end
	end

	self.Body:SetMatrix(Matrix)
	self.Body:SetVelocity(VectorZero)
	self.Body:SetAngularVelocity(VectorZero)
	self.Body:Enable()

	for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]
		
		Wheel:SetMatrix(Matrix)
	end
end

function MonsterTruckCar:SetPhysicsMatrix(Matrix)
	local CarMatrix = self:GetMatrix() 
	local WheelMatrices = {} 

	for k = 1, #self.Wheels do
		table.insert(WheelMatrices, self.Wheels[k].Body:GetMatrix() / CarMatrix)
	end

	self.Body:SetMatrix(CarMatrix)

	for k = 1, #self.Wheels do
		self.Wheels[k].Body:SetMatrix(WheelMatrices[k] * CarMatrix) 
	end
end

function MonsterTruckCar:GetMesh()
	return self.Mesh
end

function MonsterTruckCar:GetMatrix()
	--TODO (dit wordt mogelijk heel vaak aangeroepen)
	if (self.Body ~= nil) then
		return self.Body:GetMatrix()
	end
	return MatrixIdentity
end

function MonsterTruckCar:IsWheelLoose(Index)
	local Wheel = self.Wheels[Index]
	if (Wheel ~= nil) then
		return Wheel:IsWheelLoose()
	end
	
	return false
end

--TODO, Pitch, roll en yaw mooier/beter maken
-- GetAngle functie in C++?
function MonsterTruckCar:GetPitch()
	local Matrix = self:GetMatrix()
	local Dir = Matrix:GetDirection()
	local Right = Matrix:GetRight()
	local UpVector = VectorY
	
	local PitchAngle = math.acos(Dir:Dot(UpVector))
	local RefRight = UpVector:Cross(Right)
	local PitchSign = Dir:Dot(RefRight)
	if (PitchSign < 0) then
		PitchAngle = -PitchAngle
	end
	local Pitch = math.deg(PitchAngle)
	Pitch = -(((Pitch + 270) % 360) - 180)
	
	return Pitch
end

function MonsterTruckCar:GetRoll()
	local Matrix = self:GetMatrix()
	local Dir = Matrix:GetDirection()
	local Right = Matrix:GetRight()
	local UpVector = VectorY
	
	local RollAngle = math.acos(Right:Dot(UpVector))
	local RefUp = UpVector:Cross(Dir)
	local RollSign = Right:Dot(RefUp)
	if (RollSign < 0) then
		RollAngle = -RollAngle
	end
	local Roll = math.deg(RollAngle)
	Roll = (((Roll + 90) % 360) - 180)
	
	return Roll
end

function MonsterTruckCar:GetYaw()
	local Matrix = self:GetMatrix()
	local Dir = Matrix:GetDirection()
	local Right = Matrix:GetRight()
	local Up = Matrix:GetUp()
	--local UpVector = VectorY
	local ForwardVector = VectorZ
	
	local YawAngle = math.acos(Dir:Dot(ForwardVector))
	local RefForward = ForwardVector:Cross(Up)
	local YawSign = Dir:Dot(RefForward)
	if (YawSign < 0) then
		YawAngle = -YawAngle
	end
	local Yaw = math.deg(YawAngle)
	Yaw = Yaw + 180
	--Yaw = ((Yaw - 128) % 360)
	--Yaw = (Yaw % 360)
	
	return Yaw
end

--TODO
function MonsterTruckCar:GetHeight()
	local Matrix = self:GetMatrix()
	local Pos = Matrix:GetTranslation() - self.InitMatrix:GetTranslation()
	local Height = Pos.y
	
	return Height
end

function MonsterTruckCar:SetWheelMotorForce(Index, Value)
	local Wheel = self.Wheels[Index]
	Wheel.MotorForce = Value
end

function MonsterTruckCar:SetVolume(Value)
	--
end
function MonsterTruckCar:EnableInput(Value)
	--
end
function MonsterTruckCar:Enable(Value)
	--
end

function MonsterTruckCar:SetVelocity(Velocity)
	self.Body:SetVelocity(Velocity)

	for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]

		Wheel:SetVelocity(Velocity)
	end
end

function MonsterTruckCar:GetVelocity()
	return self.Body:GetVelocity()
end

function MonsterTruckCar:SetAngularVelocity(Velocity)
	self.Body:SetAngularVelocity(Velocity)
end

function MonsterTruckCar:GetAngularVelocity()
	return self.Body:GetAngularVelocity()
end

function MonsterTruckCar:SetAngularFriction(AngularFriction)
	self.AngularFriction = AngularFriction
end

function MonsterTruckCar:ChangeWheelConfig(WheelConfig, Steps)
	if (WheelConfig.Name ~= nil) then
		print("ChangeWheelConfig = " .. WheelConfig.Name)
	end
	
	if (WheelConfig.Scale ~= nil) then
		WheelConfig.ScaleVector = VectorOne * self.Car.Car.WheelConfig.Scale
	end
	
	self:ChangeWheelConfigInternal(self.WheelConfig, WheelConfig, Steps);
end
	
function MonsterTruckCar:ChangeWheelConfigInternal(CurValue, NewValue, Steps)
	if (Steps == nil) then
		Steps = 90
	end

	for Key, Value in pairs(NewValue) do
		if (type(Value) == "number" or type(Value) == "userdata") then
			PhysicsThread:Spawn(function()
				self.UpdateWheelConfig = true
				local t0 = CurValue[Key]
				local t1 = Value
				
				for i = 0, Steps do
					CurValue[Key] = Lerp(t0, t1, i / Steps)
					
					if (Key == "InertiaScale" and self.Body ~= nil) then
						self.Body:SetInertiaTensor(self.ChassisObject:GetExtents() * CurValue[Key]);
					end
					PhysicsThread:Wait(1)
				end
				CurValue[Key] = t1
				self.UpdateWheelConfig = false
			end)
		elseif (type(Value) == "table") then
			self:ChangeWheelConfigInternal(CurValue[Key], Value)
		end
	end
end

local DCK_E = GetKeyCode("DCK_E");

function MonsterTruckCar:Update()
	local Body_Matrix = self.Body:GetMatrix()
	local Body_Position = Body_Matrix:GetTranslation()
	local Body_Rotation = self.Body:GetRotation()
	local Body_RotationInv = self.Body:GetInverseRotation()
	
	--self.InitMatrix_Position
	--self.InitMatrix_Rotation
	--self.InitMatrix_RotationInv

	local PosOffset = Body_Position - self.InitMatrix_Position
	PosOffset.y = 0
	self.Body:AddForce(PosOffset * -10000)
	
	if win32() then
		if (KeyPressed(DCK_E)) then
			--local Bla = deepcopy(self.InitMatrix_Position)
			--Bla.y = -246.46
			--local PosOffset = Body_Position - self.InitMatrix_Position
			local PosOffset = Body_Position - self.InitMatrix_Position
			--print(PosOffset.y)
			--PosOffset.y = 0
			PosOffset.y = PosOffset.y / 2
		
			print(self.Wheels[1].OffsetMatrix:GetTranslation())
		
			--self.Body:SetVelocity(VectorZero)
			--self.Body:AddForce(PosOffset * -1000000)
		end
	
		if (KeyDown(DCK_E)) then
			self:Stop()
		elseif (KeyPressed(DCK_E)) then
			local Matrix = Body_Matrix:Lerp(self.InitMatrix, 0.01)
			self:SetMatrix(Matrix)
		elseif (KeyUp(DCK_E)) then
			self:Start()
		end
	end
	
	-- Ben er nog niet helemaal zeker van dat dit echt doet wat het moet doen
	if (self.RotCorrection) then
		local RotOffset = Body_Rotation * self.InitMatrix_RotationInv
		local RotAxis, RotAngle = RotOffset:GetAxisAngle()
	
		self.Body:AddTorque(self.InitMatrix_Rotation * (RotAxis * RotAngle * -1000000))
	end
	
	--AngularFriction
	if (self.WheelConfig.AngularFriction ~= nil) then
		local AVel = Body_RotationInv * self.Body:GetAngularVelocity()
		self.Body:AddTorque(Body_Rotation * (AVel * -self.WheelConfig.AngularFriction))
	end
	
	-- Pushen naar render
	local FinalMatrix = self.OffsetMatrix * Body_Matrix
	
	-- Update mesh
	self.Mesh:Update(FinalMatrix, self:GetVelocity(), self:GetAngularVelocity())	--TODO
	
	if (#self.Wheels > 0) then
		-- Hier input afhandelen!!
		if (ActiveMission ~= nil and ActiveMission.State == MISSION_STATE_PLAYING and self.Human) then
			if (win32()) then
				if (KeyDown(GetKeyCode("DCK_R"))) then
					self:Reset()
				end
			
				if (false) then
					if (KeyPressed(GetKeyCode("DCK_UP"))) then
						self.Body:AddForce(self:GetMatrix():GetDirection() * -2000000)
					end
					
					if (KeyPressed(GetKeyCode("DCK_DOWN"))) then
						self.Body:AddForce(self:GetMatrix():GetDirection() * 100000)
					end
				end
			end
			
			local WheelKeys = { Input_FrontLeftWheelUp, Input_FrontRightWheelUp, Input_RearLeftWheelUp, Input_RearRightWheelUp } 
			--local WheelKeysPressed = {false, false, false, false}
			if (self.WheelKeysPressed == nil) then
				self.WheelKeysPressed = {0, 0, 0, 0}
			end
			
			for i = 1, 4 do
				if (WheelKeys[i]:KeyPressed()) then
					if (self.WheelKeysPressed[i] < 2) then
						self.WheelKeysPressed[i] = 1
					end
				else
					local Released = true
					
					if (i == 1) then
						if (Input_FrontWheelsUp:KeyPressed() or Input_LeftWheelsUp:KeyPressed()) then
							Released = false
						end
					end
					
					if (i == 2) then
						if (Input_FrontWheelsUp:KeyPressed() or Input_RightWheelsUp:KeyPressed()) then
							Released = false
						end
					end
					
					if (i == 3) then
						if (Input_RearWheelsUp:KeyPressed() or Input_LeftWheelsUp:KeyPressed()) then
							Released = false
						end
					end
					
					if (i == 4) then
						if (Input_RearWheelsUp:KeyPressed() or Input_RightWheelsUp:KeyPressed()) then
							Released = false
						end
					end
					
					if (Released) then
						self.WheelKeysPressed[i] = 0
					end
				end
			end
			
			if (Input_FrontWheelsUp:KeyPressed()) then
				if (self.WheelKeysPressed[1] < 2) then
					self.WheelKeysPressed[1] = 1
				end
				
				if (self.WheelKeysPressed[2] < 2) then
					self.WheelKeysPressed[2] = 1
				end
			end
			
			if (Input_RearWheelsUp:KeyPressed()) then
				if (self.WheelKeysPressed[3] < 2) then
					self.WheelKeysPressed[3] = 1
				end
				
				if (self.WheelKeysPressed[4] < 2) then
					self.WheelKeysPressed[4] = 1
				end
			end
			
			if (Input_LeftWheelsUp:KeyPressed()) then
				if (self.WheelKeysPressed[1] < 2) then
					self.WheelKeysPressed[1] = 1
				end
				
				if (self.WheelKeysPressed[3] < 2) then
					self.WheelKeysPressed[3] = 1
				end
			end
			
			if (Input_RightWheelsUp:KeyPressed()) then
				if (self.WheelKeysPressed[2] < 2) then
					self.WheelKeysPressed[2] = 1
				end
				
				if (self.WheelKeysPressed[4] < 2) then
					self.WheelKeysPressed[4] = 1
				end
			end
			
			--Sync front
			if (self.WheelKeysPressed[1] == 1 and self.WheelKeysPressed[2] == 1) then
				self.WheelTimers[1] = Max(self.WheelTimers[1], self.WheelTimers[2])
				self.WheelTimers[2] = Max(self.WheelTimers[2], self.WheelTimers[1])
			end
			
			--Sync rear
			if (self.WheelKeysPressed[3] == 1 and self.WheelKeysPressed[4] == 1) then
				self.WheelTimers[3] = Max(self.WheelTimers[3], self.WheelTimers[4])
				self.WheelTimers[4] = Max(self.WheelTimers[4], self.WheelTimers[3])
			end
			
			--Sync left
			if (self.WheelKeysPressed[1] == 1 and self.WheelKeysPressed[3] == 1) then
				self.WheelTimers[1] = Max(self.WheelTimers[1], self.WheelTimers[3])
				self.WheelTimers[3] = Max(self.WheelTimers[3], self.WheelTimers[1])
			end
			
			--Sync right
			if (self.WheelKeysPressed[2] == 1 and self.WheelKeysPressed[4] == 1) then
				self.WheelTimers[2] = Max(self.WheelTimers[2], self.WheelTimers[4])
				self.WheelTimers[4] = Max(self.WheelTimers[4], self.WheelTimers[2])
			end
			
			--if (Input_AllWheelsUp:KeyPressed()) then
				--self.WheelKeysPressed[1] = 1
				--self.WheelKeysPressed[2] = 1
				--self.WheelKeysPressed[3] = 1
				--self.WheelKeysPressed[4] = 1
			--end
			
			local Delay = 4
			for i = 1, 4 do
				if (self.WheelKeysPressed[i] == 1) then
					self.WheelTimers[i] = self.WheelTimers[i] + 1
					
					if (self.WheelTimers[i] >= Delay) then
						self:SetWheelMotorForce(i, 1)
						
						--if (win32()) then -- Moet ook op iOs werken
							local tmp = Mod(self.WheelTimers[i], 32);
							local Matrix = self.Wheels[i]:GetMatrix()
							if (tmp == 0) then
								SpawnEffect("FX\\WheelEffect.xml", Matrix:GetTranslation().x, Matrix:GetTranslation().y, Matrix:GetTranslation().z)
							end
						--end
						
						if (self.Wheels[i].ContactTimer > 8 and self.Wheels[i].ContactTimer < 16) then
							self.WheelKeysPressed[i] = 2
						end
					end
					
				else
					self:SetWheelMotorForce(i, nil)
					self.WheelTimers[i] = 0;
				end
			end
		elseif (ActiveMission ~= nil and ActiveMission.State ~= MISSION_STATE_PLAYING) then
			for i = 1, 4 do
				self.WheelTimers[i] = 0
				--self:SetWheelMotorForce(i, nil)
			end
		end
	end
	
	-- Update the wheels
	for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]
		
		Wheel:Update()
	end
	
	-- Update the elements
	for i = 1, #self.Elements do
		local Element = self.Elements[i]

		Element:Update()
	end
end

function MonsterTruckCar:Render()
	--self.Chassis:Render(color(1, 0, 0, 1), self.Body:GetMatrix():GetTranslation())

	--[[ for i = 1, #self.Wheels do
		local Wheel = self.Wheels[i]

		Wheel:Render()
	end]]
end

function MonsterTruckCar:OnContact(Entity, Pos, Normal, Depth)
	--TODO: tabel ook apart opslaan?
	local Contact = self.Contact
	Contact.CFM = 0.000001
	Contact.ERP = 0.999
	Contact.Friction1 = 500000
	Contact.Friction2 = 500000

	local FrictionRight = Normal:Cross(self:GetMatrix():GetDirection() * -1):Normalize()
	Contact.FrictionDir = FrictionRight:Cross(Normal):Normalize();

	return Contact;
end

function MonsterTruckCar:PlaySuspensionSound()
	if (self.SuspensionTimer == nil and self.SoundEnabled) then
		playsound("Audio\\LR_Vering*.WAV");
		
		self.SuspensionTimer = PhysicsThread:Spawn(function()
			PhysicsThread:Wait(60);
			self.SuspensionTimer = nil
		end);
	end
end
