function InitializeCameras(Mission)
	local Cameras = {}
	
	local i = 1
	while (true) do
		local CamMatrix = GetScriptMatrix("PhotoshootCamera" .. i)
		
		if (CamMatrix == nil) then
			break
		end
		
		local Camera = PhotoshootCamera(CamMatrix, Mission, true)
		table.insert(Cameras, Camera)
		
		i = Inc(i)
	end
	
	local RealCamera = EntityCamera(0)
	
	-- Thread aanmaken voor ALLE cameras (betere performance dan met 1 thread per camera)
	local Rotate = false
	if (Rotate) then
		Mission.PhotoshootCameraThread = PhysicsThread:Spawn(function()
			while (true) do 
				for i, Camera in ipairs(Cameras) do
					-- Manipulate Camera-meshes here
					if (Camera.Mesh ~= nil) then	
						Camera.Matrix = Camera.Matrix:RotateLocal(Vector(0,1,0), math.rad(3))
						Camera.Mesh:Update(Camera.Matrix, VectorZero, VectorZero)
						
						-- Placing the effect behind the model, based on the camera's position
						local RealCameraPos = RealCamera:GetMatrix():GetTranslation()
						local RealCameraDirection = Camera.Matrix:GetTranslation() - RealCameraPos
						
						RealCameraDirection = RealCameraDirection:Normalize()
						
						if (not Camera.Grabbed) then
							Camera.FlashEffect:SetMatrix(Camera.Matrix:Translate(RealCameraDirection * 6))
						end
					end
				end
				
				PhysicsThread:Wait(1)
			end
		end)
	end
	
	-- Init colors
	Mission.NormalColor = color(1,1,1,1)
	Mission.NormalIntensity = 0
	Mission.NormalGrayValue = 0
	
	Mission.PhotoColor = color(0,0,0,1)
	Mission.PhotoIntensity = 0
	Mission.PhotoGrayValue = 0.75
	
	return Cameras
end

function StartPhotoshoot(Mission, Player, PauseInMiddle, ShowAtEnd, MayRotate)	
	local Camera = EntityCamera(0)
	
	if (MayRotate == nil) then
		MayRotate = false
	end
	
	local Rotate = MayRotate
	if (Rotate) then
		if (not PauseInMiddle) then
			Camera:StartRotatingCam(0, 0.8, 15, 0.14, 90, 450, false)
		else
			Camera:StartRotatingCam(0, 0.8, 15, 0.1, 90, 600, true)
		end
	end
	
	Mission.SlowMotionTime = 0.05
	
	if (Player.HUD ~= nil) then
		Player.HUD:Hide(false, true)
	end
	
	local StartCarVolume = Player.Car:GetVolume()
	local StartFOV = Camera:GetFOV()
	
	Camera:SetFOV(StartFOV)
	
	InputThread:Run(function()
		Camera:SetFOV(StartFOV)
	
		local Length = 60
		for i = 1, Length do
			local Color = Lerp(Mission.NormalColor, Mission.PhotoColor, i/Length)
			local Intensity = Lerp(Mission.NormalIntensity, Mission.PhotoIntensity, i/Length)
			local GrayValue = Lerp(Mission.NormalGrayValue, Mission.PhotoGrayValue, i/Length)
		
			SetShaderVar("ColorOverlay", Color * Intensity)
			SetShaderVar("InvGreyValue", 1 - GrayValue)
			SetShaderVar("GreyScale", Vector(0.222, 0.707, 0.071) * GrayValue)
			
			if (i < Length/2) then
				local Time = Lerp(1, Mission.SlowMotionTime, i/(Length/2))
				SetGameTimeScalar(Time)
			end
			
			local CarVolume = Lerp(StartCarVolume, 0, i/Length)
			Player.Car:SetVolume(CarVolume)
			
			Camera:SetFOV(Lerp(Camera:GetFOV(), math.rad(45), i/Length))
			
			InputThread:Wait(1)
		end
		
		local ShowFlash = false
		if (ShowFlash) then
			if (Player.HUD ~= nil) then
				Player.HUD:ShowScreenFlash(1)
				playsound("Flash.wav")
			end
		end
		
		local PlaySound = false
		if (PlaySound) then
			playsound("Applause.wav")
		end
		
		local ShowMessage = true
		if (ShowMessage) then
			if (Player.HUD ~= nil) then
				Player.HUD:ShowPhotoMessage()
			end
		end
		
		if (not PauseInMiddle) then
			EndPhotoshoot(Mission, Player, StartFOV, StartCarVolume, ShowAtEnd)
		else
			var("intpause", 1)
		end
	end)
end

function EndPhotoshoot(Mission, Player, StartFOV, StartCarVolume, ShowAtEnd)
	InputThread:Run(function()
		InputThread:Sleep(2)
		
		local Camera = EntityCamera(0)
		
		local CurrentFOV = Camera:GetFOV()
	
		local Length = 60
		for i = 1, Length do
			local Color = Lerp(Mission.PhotoColor, Mission.NormalColor, i/Length)
			local Intensity = Lerp(Mission.PhotoIntensity, Mission.NormalIntensity, i/Length)
			local GrayValue = Lerp(Mission.PhotoGrayValue, Mission.NormalGrayValue, i/Length)
		
			SetShaderVar("ColorOverlay", Color * Intensity)
			SetShaderVar("InvGreyValue", 1 - GrayValue)
			SetShaderVar("GreyScale", Vector(0.222, 0.707, 0.071) * GrayValue)
			
			if (i > Length/2) then
				local Time = Lerp(Mission.SlowMotionTime, 1, (i-(Length/2))/(Length/2))
				SetGameTimeScalar(Time)
			end
			
			local CarVolume = Lerp(0, StartCarVolume, i/Length)
			Player.Car:SetVolume(CarVolume)
			
			Camera:SetFOV(Lerp(CurrentFOV, StartFOV, i/Length))
			
			InputThread:Wait(1)
		end
		
		if (Player.HUD ~= nil and ShowAtEnd) then
			Player.HUD:Show(true)
		elseif (Player.HUD ~= nil) then
			Player.HUD:Hide()
		end
		
		Camera:UnlockFOV()
	end)
end

PhotoshootCamera = class(function(self, Matrix, Mission, Scale)
	self.Matrix = Matrix
	self.Mission = Mission
	self.Grabbed = false
	self.UseEffect = false
	
	-- Setting the scale
	if (Scale) then
		local Scale = 10
		local ScaleX = 300
		local ScaleY = 300
		local ScaleZ = nil
		
		local VectorScale = Vector(Scale, Scale, Scale)
		
		if (ScaleX ~= nil) then
			VectorScale.x = ScaleX
		end
		if (ScaleY ~= nil) then
			VectorScale.y = ScaleY
		end
		if (ScaleZ ~= nil) then
			VectorScale.z = ScaleZ
		end
		
		self.Matrix = self.Matrix:ScaleLocal(VectorScale)
	end

	-- Add Mesh here
	self.Mesh = CreateAdvancedMesh(nil, "Models\\Collider.dcm", "")
	GetEntityScene():AddEntity(self.Mesh)
	print(self.Matrix)
	self.Mesh:Update(self.Matrix, VectorZero, VectorZero)
	
	-- Add Flash Effect here
	if (self.UseEffect) then
		self.FlashEffect = CreateEffectSystem("FX\\DR_PhotoShootFlash.xml", true)
		GetEntityScene():AddEntity(self.FlashEffect)
		self.FlashEffect:SetMatrix(self.Matrix)
	end
	
	-- Add Collision here
	self.Collision = BoxGeometry()
	self.Collision:SetEntity(self.Mesh)
	self.Collision:SetOffsetMatrix(Matrix34(self.Mesh:GetCenter()))	-- Hier een SetCenter van maken?
	self.Collision:SetExtents(self.Mesh:GetExtents() * 1.4)
	self.Collision:SetType(bit32.bor(DCGF_DETECTION))
	self.Collision:SetCollision(bit32.bor(DCGF_CAR))
	self.Collision:Enable()
	self.Collision:SetMatrix(self.Matrix)
	
	self.Collision:SetContactCallback(function(...)		
		return self:OnContact(...)
	end)
	
	self.Collision:SetCollisionCallback(function(Object)		
		return self:OnCollision(Object)
	end)
	
	self.Mesh:SetRenderCallback(function(...)
		return self:OnRender(...)
	end)

	GetPhysicsScene():AddGeometry(self.Collision)
end)

function PhotoshootCamera:OnRender()
	--self.Collision:Render(color(1, 0, 0, 1), self.Collision:GetMatrix():GetTranslation())
end

function PhotoshootCamera:OnContact()
	return {}
end

function PhotoshootCamera:OnCollision(Object)
	print("COLLISION")
	self.Collision:Disable()
	self.Grabbed = true
	
	PhysicsThread:Spawn(function()
		if (self.UseEffect) then
			self.FlashEffect:SetMatrix(self.Matrix:Translate(Vector(0,10000,0)))
		end
	
		PhysicsThread:Sleep(3)

		self.Collision:Enable()
		
		self.Grabbed = false
	end)
	
	local Player = nil
	for i, PlayerInstance in ipairs(self.Mission.Players) do
		if (PlayerInstance.Car == Object) then
			Player = PlayerInstance
			break
		end
	end
	
	if (Player ~= nil) then
		if (not Player.Dead and not Player.Invulnerable) then
			StartPhotoshoot(self.Mission, Player, false, true, false)
		end
	end
	
	return false
end

function PhotoshootCamera:Destroy()	
	if (self.Body ~= nil) then
		self.Body:Destroy()
		self.Body = nil
	end
	
	if (self.Collision ~= nil) then
		self.Collision:Destroy()
		self.Collision = nil
	end
end