dofile("Lua\\Testing.lua")
dofile("Lua\\PowerupManager.lua")

LUAPlayer = class(function(self, Car)
	-- Misc
	self.Car = Car
	self.MaxHP = Car ~= nil and Car.GetMaxHP ~= nil and Car:GetMaxHP() or 0
	self.LapTimes = {}
	self.LapCount = 0
	self.Coins = 0
	self.Score = 0
	self.Scores = {}
	self.TotalScore = 0
	self.StarScores = {}
	self.TotalStarScores = 0
	self.MaxScorePerStunt = 0
	self.Points = 0	--Tournament
	self.Rank = 0
	self.LastLapTime = 0
	self.Finished = false
	self.Dead = false
	self.Delay = 0
	self.SuperMoveCount = 0
	self.CheckCount = 0
	self.CheckDist = 0
	self.TimeOffset = 0
	self.DistanceTraveled = 0
	self.DistanceToGo = 0
	self.Lives = 0
	self.Failed = false
	
	self.Progress = 0.5
	
	-- Powerups
	self.PowerupManager = PowerupManager(self)
end)

function LUAPlayer:AddScore(Score)
	self.Score = self.Score + Score
	if(self.HUD ~= nil) then
		self.HUD:UpdateScore(self)
	end
end

function LUAPlayer:SetCar(Car)
	self.Car = Car
	self.MaxHP = self.Car:GetMaxHP()
end

function LUAPlayer:OnCarDamaged(Damage, HP)
	if (self.HUD ~= nil and self.HUD.HealthBar ~= nil) then
		self.HUD.HealthBar:FlashHealthBar()
	end
end

function LUAPlayer:Revive()
	-- Make sure the player is eligible for a revive
	print("^3REVIVE")
	print(self.Lives)
	if(self.Lives <= 0) then
		return
	end
	self.Lives = self.Lives - 1
	
	-- Prevent further damage while the respawn message below is being processed
	self.Car:EnableGod()
	
	-- Reset the player
	self.Car:ResetHP()
	self.Car:Respawn()
	
	-- Display revived notification
	if(self.Human) then
		--self.HUD:ShowRevived("Revived!") -- Extra lives left: " ..self.Lives)
	end
	
	GameThread:Spawn(function()
		GameThread:Wait(0.1)
		
		-- Create the revive effect
		local Emitter = CreateEffectSystem("FX\\MJ_Revive.xml", true)
		local Matrix = self.Car:GetMatrix():Translate(Vector(0,20,0))
		Emitter:SetMatrix(Matrix) -- Translate for vehicle height
		self.Car:GetMesh():AddPlugin(Emitter, Car)
		
		 -- Allow the respawn message to process, then disable god mode
		GameThread:Wait(1)
		self.Car:DisableGod()
	end)
	
	--print("^2Player revived with " ..self.Car:GetHP() .." HP.")
end

function LUAPlayer:CarAddRelativeForce(Force, Position)
	local Car = self.Car
	local CarMat = Car:GetMatrix()
	local RotMatrix = CarMat:GetRotation()
	
	local AdjustedForce = Force * RotMatrix
	local VecPos = (Position * RotMatrix) + CarMat:GetTranslation()
	
	Car:AddForce(AdjustedForce, VecPos)
end

function LUAPlayer:CarAddPitchForce(Magnitude)
	local Position = Vector(0, 0, 120) * math.sign(Magnitude)
	local Force = VectorY * math.abs(Magnitude)
	self:CarAddRelativeForce(Force, Position)
end

function LUAPlayer:CarAddRollForce(Magnitude)
	local Position = Vector(120, 0, 0) * math.sign(Magnitude)
	local Force = VectorY * math.abs(Magnitude)
	self:CarAddRelativeForce(Force, Position)
end

function LUAPlayer:CarAddYawForce(Magnitude)
	local Position = Vector(0, 0, 120) * math.sign(Magnitude)
	local Force = VectorX * math.abs(Magnitude)
	self:CarAddRelativeForce(Force, Position)
end

----------------------------------------------------------
------------- Pitch, roll & yaw calculations -------------
----------------------------------------------------------

function LUAPlayer:GetCarPitch()
	local Matrix = self.Car:GetMatrix()
	local Dir = Matrix:GetDirection()
	local Right = Matrix:GetRight()
	local UpVector = VectorY
	
	local PitchAngle = math.acos(Dir:Dot(UpVector))
	local RefRight = UpVector:Cross(Right)
	local PitchSign = Dir:Dot(RefRight)
	if (PitchSign < 0) then
		PitchAngle = -PitchAngle
	end
	local Pitch = math.deg(PitchAngle)
	Pitch = -(((Pitch + 270) % 360) - 180)
	
	return Pitch
end

function LUAPlayer:GetCarRoll()
	local Matrix = self.Car:GetMatrix()
	local Dir = Matrix:GetDirection()
	local Right = Matrix:GetRight()
	local UpVector = VectorY
	
	local RollAngle = math.acos(Right:Dot(UpVector))
	local RefUp = UpVector:Cross(Dir)
	local RollSign = Right:Dot(RefUp)
	if (RollSign < 0) then
		RollAngle = -RollAngle
	end
	local Roll = math.deg(RollAngle)
	Roll = (((Roll + 90) % 360) - 180)
	
	return Roll
end

function LUAPlayer:GetCarYaw()
	local Matrix = self.Car:GetMatrix()
	local Dir = Matrix:GetDirection()
	local Right = Matrix:GetRight()
	local Up = Matrix:GetUp()
	--local UpVector = VectorY
	local ForwardVector = VectorZ
	
	local YawAngle = math.acos(Dir:Dot(ForwardVector))
	local RefForward = ForwardVector:Cross(Up)
	local YawSign = Dir:Dot(RefForward)
	if (YawSign < 0) then
		YawAngle = -YawAngle
	end
	local Yaw = math.deg(YawAngle)
	Yaw = Yaw + 180
	--Yaw = ((Yaw - 128) % 360)
	--Yaw = (Yaw % 360)
	
	return Yaw
end

function LUAPlayer:Jump()
	self.PowerupManager:Jump()
end

function LUAPlayer:Repair()
	self.PowerupManager:Repair()
end

function LUAPlayer:ShootFireworks()
	self.PowerupManager:ShootFireworks()
end

function GetTerrainDamageReduction(Car, Damage)
	-- Make sure the car belongs to the human player
	-- Terrain damage reduction is exclusive to the human player due to upgrades
	local Player = GetPlayer(Car)
	if(Player ~= ActiveMission.HumanPlayers[1]) then
		return 0
	end
	
	-- If there's no damage, don't process the damage reduction
	if(Damage == 0) then
		return 0
	end
	
	local DamageReduction = Player.Upgrader:GetTerrainDamageReduction(Damage)
	local NewDamage = (Damage - DamageReduction)
	
	--print("Reducing ^2" ..Damage .." ^7world damage to ^2" .. NewDamage .." ^7(" ..((DamageReduction/Damage) * 100) .."% reduction)")
	return DamageReduction
end

function GetDamageReduction(Car, Damage)
	local Player = GetPlayer(Car)
	if(Player == nil) then
		return Damage
	end
	
	local DamageReduction = Player.PowerupManager:GetDamageReduction(Damage)
	local NewDamage = (Damage - DamageReduction)
	
	-- If there's no damage, don't process the damage reduction
	if(Damage == 0) then
		return 0
	end
	
	-- Revive check
	if(Car:GetHP() - NewDamage <= 0) then
		print("Fatal blow detected! Damage: " ..Damage ..", player HP: " ..Car:GetHP())
		if(Player.Lives > 0) then
			PhysicsThread:Spawn(function()
				PhysicsThread:Sleep(2)
				
				if (ActiveMission ~= nil) then
					ActiveMission:EndGame()
				end
			end)
			
			return Damage -- Remove 100% damage
		end
	end
	
	--print("Reducing ^2" ..Damage .." ^7generic damage to ^2" .. NewDamage .." ^7(" ..((DamageReduction/Damage) * 100) .."% reduction)")
	
	return DamageReduction
end

-- Dit kan wat praktischer denk ik  -Steven
function GetPlayer(Car)
	for i, Player in ipairs(ActiveMission.Players) do
		if (Player.Car == Car) then
			return Player
		end
	end
	
	return nil
end

function LUAPlayer:CalculateAllScores()
	self.TotalScore = 0

	-- The following for-loop is to calculate the total score, the star-scores
	-- and to save them separately in the "Player" class
	for i = 1, (#self.Scores) do
		self.TotalScore = self.TotalScore + self.Scores[i]
		self.StarScores[i] = self:ConvertToStarScore(self.Scores[i])
	end
	
	-- This is to calculate the total 3 and 5 star score
	self.TotalStarScore = self:ConvertToStarScore(self.TotalScore / #self.Scores)
	self.Total5StarScore = self:ConvertTo5StarScore(self.TotalScore / #self.Scores)
	
	-- print("^3self.TotalStarScore = "..(self.TotalStarScore))
	-- print("^3self.Total5StarScore = "..(self.Total5StarScore))
end

function LUAPlayer:ConvertToStarScore(TempScore)
	if (TempScore < (self.MaxScorePerStunt / 3)) then -- Gelijk aan of lager dan een kwart
		TempScore = 0
	elseif (TempScore <= (self.MaxScorePerStunt / 3)) then -- Gelijk aan of lager dan de helft
		TempScore = 1
	elseif (TempScore <= (self.MaxScorePerStunt / 3)*2) then -- Gelijk aan of lager dan 3/4
		TempScore = 2
	else
		TempScore = 3
	end
	
	return TempScore
end

function LUAPlayer:ConvertTo5StarScore(TempScore)
	if (TempScore < (self.MaxScorePerStunt / 5)) then
		TempScore = 0
	elseif (TempScore <= (self.MaxScorePerStunt / 5)) then
		TempScore = 1
	elseif (TempScore <= ((self.MaxScorePerStunt / 5) * 2)) then
		TempScore = 2
	elseif (TempScore <= ((self.MaxScorePerStunt / 5) * 3))then
		TempScore = 3
	elseif (TempScore <= ((self.MaxScorePerStunt / 5) * 4)) then
		TempScore = 4
	else
		TempScore = 5
	end
	
	return TempScore
end

function LUAPlayer:InitializeScores(Manager)
	-- This function is only to set the max amount of
	-- score per stunt, the scores array size and to set
	-- its values to 0.
	
	self.MaxScorePerStunt = 10 / (#Manager.Stunts)

	for i = 1, (#Manager.Stunts) do
		self.Scores[i] = 0
		self.StarScores[i] = 0
		
		-- print("self.Scores["..(i).."] = "..(self.Scores[i]))
	end
	
	---- TEST CODE:
	--[[ (Dit is dus alleen maar om uit te proberen wat er gebeurt bij bepaalde waardes)
	for i = 1, (#Manager.Stunts) do
		if(i <= ((#Manager.Stunts)/2))then -- Om alleen de eerste helft score te geven
			-- self.Scores[i] = (self.MaxScorePerStunt/3) * 1 -- Voor 1 ster
			-- self.Scores[i] = (self.MaxScorePerStunt/3) * 2 -- Voor 2 sterren
			self.Scores[i] = self.MaxScorePerStunt -- Voor 3 sterren
			self.StarScores[i] = 0
		else -- De overige scores instellen op 0
			self.Scores[i] = 0
			self.StarScores[i] = 0
		end
		-- print("self.Scores["..(i).."] = "..(self.Scores[i]))
	end]]
end