PowerupManager = class(function(self, Player)
	-- Misc
	self.Player = Player
	
	self.Powerups = {}
	
	self.JumpPowerup = self:AddPowerup(Powerup(self.Player, "powerup_jump"))
	self.RepairPowerup = self:AddPowerup(Powerup(self.Player, "powerup_repair"))
	self.FirworksPowerup = self:AddPowerup(Powerup(self.Player, "powerup_fireworks"))
	self.UltraBumbersPowerup = self:AddPowerup(Powerup(self.Player, "powerup_bumpers"))
	self.BoostPowerup = self:AddPowerup(Powerup(self.Player, "powerup_nitro"))
end)

function PowerupManager:GetDamageReduction(Damage)
	-- Calculate adjusted damage based on the Ultra Destroyer Bumpers
	local AdjDamage = Damage
	if(self.UltraBumbersPowerup.Enabled) then
		AdjDamage = Damage * 0.3
	end
	
	-- Calculate the damage reduction
	local DamageRed = Damage - (AdjDamage * ActiveGameMode.DamageMult)
	
	--print("^1Damage reduction: " .. tostring(DamageRed))
	return DamageRed
end

function PowerupManager:AddPowerup(Powerup)
	table.insert(self.Powerups, Powerup)
	
	return Powerup
end

function PowerupManager:GetPowerUp(Powerup)
	for i, Key in ipairs(PowerUpItems) do
		if(Key.Name == Powerup.Name) then
			return Key
		end
	end
end

Powerup = class(function(self, Player, Name)
	self.Name = Name	
	self.Player = Player

	for i, Item in ipairs(PowerUpItems) do
		if (Item.Name == self.Name) then
			self.PowerUpItem = Item
		end
	end

	self.Enabled = self.PowerUpItem.Bought
	self.Used = false
end)

function PowerupManager:Jump()
	if (self.HasPlayerJumped or ActiveMenu ~= nil) then
		return
	end
	
	local Car = self.Player.Car
	
	-- Create an explosion underneath the car
	local Effect = CreateEffectSystem("FX\\MJ_Jump.xml", true)
	Effect:SetMatrix(Car:GetMatrix():Translate(Vector(0, 10, 0)))
	GetEntityScene():AddEntity(Effect)
	
	-- Determine the upward force for the jump
	local ForceMultiplier = 5
	local LocalForce = false
	local Direction = Vector(0,1,0) + (Car:GetMatrix():GetDirection() * -0.5)
	Direction = Direction * (ForceMultiplier * 10000000)
	if (LocalForce) then
		Direction = Car:GetMatrix():GetUp() * (ForceMultiplier * 10000000)
	end
	
	-- Gradually add more force to the car
	PhysicsThread:Spawn(function()
		for i = 1, 10 do
			Car:AddForce(Direction)
			PhysicsThread:Wait(1)
		end
	end)
	
	self.JumpPowerup.Used = true
end

function PowerupManager:Repair()
	if(ActiveMenu ~= nil) then
		return
	end
	
	local Car = ActiveMission.HumanPlayers[1].Car --self.Player.Car
	local HUD = ActiveMission.HumanPlayers[1].HUD --self.Player.HUD
	
	-- Reset car health
	Car:ResetHP()
	if (HUD.HealthBar ~= nil) then
		HUD.HealthBar:FlashHealthBar()
	end
	
	--Reset car pos
	local Matrix = Car:GetMatrix()
	local Up = Matrix:GetUp();
	if (Up.y < -0.6) then
		--teleport()
		
		-- ===============
		-- New teleport
		local CarPos = Car:GetMatrix():GetTranslation()
		local CarDir = Car:GetMatrix():GetDirection()
		local Matrix = Matrix34():Translate(CarPos)
		
		local Up = Car:GetMatrix():GetUp()
		if (Up.y < 0) then
			CarDir = CarDir * -1
		end
		
		Matrix = Matrix:LookAt(CarPos, CarPos + CarDir, 0)
		
		Matrix = Matrix:Translate(Vector(0,30,0))
		Car:SetMatrix(Matrix)
		
		Car:SetVelocity(VectorZero)
		-- ===============
	end
	
	-- Create a visual effect
	local Effect = CreateEffectSystem("FX\\MJ_Repair.xml", true)
	Effect:SetMatrix(Car:GetMatrix():Translate(Vector(0, 20, 0)))
	Car:GetMesh():AddPlugin(Effect)
	
	self.RepairPowerup.Used = true
end

function PowerupManager:ShootFireworks()
	if(ActiveMenu ~= nil) then
		return
	end
	
	local Car = self.Player.Car
	
	-- Create the fireworks effect and add it to the car's mesh as a plugin
	local FireworksEmitter = CreateEffectSystem("FX\\MJ_Fireworks.xml", true)
	local WheelsFireworksEmitter = CreateEffectSystem("FX\\MJ_FireworksWheels.xml", true)
	Car:GetMesh():AddPlugin(FireworksEmitter, Car)
	Car:GetMesh():AddPlugin(WheelsFireworksEmitter, Car)
	
	-- Damage nearby vehicles after a short delay
	GameThread:Spawn(function()
		GameThread:Wait(1)
		local CarPos = Car:GetMatrix():GetTranslation() 
		
		for i = 1, #ActiveMission.Players do
			local AdjCar = ActiveMission.Players[i].Car;
			if(AdjCar ~= Car) then
				-- Calculate position difference
				local AdjCarPos = AdjCar:GetMatrix():GetTranslation()
				local Distance = ((CarPos.x-AdjCarPos.x) * (CarPos.x-AdjCarPos.x) + (CarPos.z-AdjCarPos.z) * (CarPos.z-AdjCarPos.z))
				
				-- If the car is close, apply damage
				if(Distance < 80000) then -- Roughly 20 metres
					if (AdjCar.SetHP ~= nil) then
						AdjCar:SetHP(AdjCar:GetHP() - 2000)
					end
				end
			end
		end
	end)
	
	self.FirworksPowerup.Used = true
end