
-- Misc includes
dofile("Lua\\Utils\\Queue.lua")
dofile("Lua\\Stunting\\ScoreDisplayMessage.lua")
dofile("Lua\\Stunting\\StuntResult.lua")
dofile("Lua\\Stunting\\WheelsState.lua")

-- Base stunt includes
dofile("Lua\\Stunting\\Stunts\\BaseStunt.lua")
dofile("Lua\\Stunting\\Stunts\\BaseBounceStunt.lua")
dofile("Lua\\Stunting\\Stunts\\BaseRotationStunt.lua")
dofile("Lua\\Stunting\\Stunts\\BaseDonutStunt.lua")

-- Stunt includes
dofile("Lua\\Stunting\\Stunts\\AirStunt.lua")
dofile("Lua\\Stunting\\Stunts\\StoppieStunt.lua")
dofile("Lua\\Stunting\\Stunts\\WheelieStunt.lua")
dofile("Lua\\Stunting\\Stunts\\SkyWheelieStunt.lua")
dofile("Lua\\Stunting\\Stunts\\BounceLeftRightStunt.lua")
dofile("Lua\\Stunting\\Stunts\\BounceBackForthStunt.lua")
dofile("Lua\\Stunting\\Stunts\\SidewaysWheelieStunt.lua")
dofile("Lua\\Stunting\\Stunts\\BackflipStunt.lua")
dofile("Lua\\Stunting\\Stunts\\FrontflipStunt.lua")
dofile("Lua\\Stunting\\Stunts\\DonutStunt.lua")
dofile("Lua\\Stunting\\Stunts\\CycloneDonutStunt.lua")
dofile("Lua\\Stunting\\Stunts\\OneTireWheelieStunt.lua")
dofile("Lua\\Stunting\\Stunts\\OneTireStoppieStunt.lua")

--------------------------
------ StuntManager ------
--------------------------

StuntManager = class(function(self, Mission)
	-- Misc
	self.Enabled = false
	self.ScoreMessages = Queue()
	self.WheelsHistory = Queue()
	self.MaxHistory = 10
	self.Mission = Mission
	
	-- Combo tracking
	self.ComboTrackingActive = false
	self.ComboStuntIntervalMax = 2.0
	self.ComboStuntIntervalCurrent = 0
	self.ComboStuntResultBuffer = {}
	
	-- Results
	self.StuntResults = {}
	self.ComboStuntResults = {}
	
	-- Stunts
	self.Stunts = {}
	self:AddStunt(AirStunt(self))
	self:AddStunt(WheelieStunt(self))
	self:AddStunt(SkyWheelieStunt(self))
	self:AddStunt(StoppieStunt(self))
--	self:AddStunt(BounceLeftRightStunt(self))	
	self:AddStunt(BounceBackForthStunt(self))	
--	self:AddStunt(SidewaysWheelieStunt(self))
	self:AddStunt(BackflipStunt(self))
	self:AddStunt(FrontflipStunt(self))
	self:AddStunt(DonutStunt(self))
--	self:AddStunt(CycloneDonutStunt(self))
--	self:AddStunt(OneTireWheelieStunt(self))
--	self:AddStunt(OneTireStoppieStunt(self))

	-- Add an inital WheelsState
	local Car = Mission.HumanPlayers[1].Car
	self:AddWheelsState(WheelsState(Car))
	
	-- Set the collision callback
	if (Car.SetCollisionCallback ~= nil) then
		Car:SetCollisionCallback(function(Entity)
			-- If the entity is not identifyable via tostring, ignore it
			if (Entity.__tostring == nil) then
				return true
			end
			
			local EntName = Entity:__tostring()
			if (EntName ~= "dcDestroyableObject") then
				return true
			end
			
			-- Increment ObjectsDestroyed on all stunts
			for i = 0, #self.Stunts do
				local Stunt = self.Stunts[i]
				if (Stunt ~= nil) then		-- waarom is dit nodig?
					Stunt:OnCollision(Entity)
				end
			end
			return true
		end)
	end
	
	-- For the ChangeCamera() function
	self.ChangeCamPos = false
	self.OffsetDistance = 110
	self.HeightOffset = 80
	self.MaxCamDistance = 400
	
	-- Score initialization
	local Player = Mission.HumanPlayers[1]
	Player:InitializeScores(self)
end)

function StuntManager:Enable()	
	if(self.Enabled == false) then
		self.Enabled = true
	end
end

function StuntManager:Disable()
	self.Enabled = false
end

function StuntManager:Update(ElapsedTime)
	if(not self.Enabled) then
		return
	end

	-- Combo tracking
	self:UpdateComboTracking(ElapsedTime)
	
	-- WheelsState updating
	local Car = ActiveMission.HumanPlayers[1].Car
	local CurrentWheelsState = WheelsState(Car)
	local LastStateIndex = self.WheelsHistory.Last
	local LastWheelsState = self.WheelsHistory[LastStateIndex]
	
	-- If the current state differs from the last state, push the current state
	if(CurrentWheelsState ~= LastWheelsState) then
		self:AddWheelsState(CurrentWheelsState)
	end
	
	self:UpdateStunts(ElapsedTime)
end

function StuntManager:UpdateStunts(ElapsedTime)
	for Key, Value in pairs(self.Stunts) do
		local Stunt = self.Stunts[Key]
		Stunt:Update(ElapsedTime)
	end
end

function StuntManager:UpdateComboTracking(ElapsedTime)
	local ActiveStunts = self:GetActiveStunts()
	
	-- Combo tracking
	if(self.ComboTrackingActive and #ActiveStunts == 0) then
		self.ComboStuntIntervalCurrent = self.ComboStuntIntervalCurrent + ElapsedTime
		if(self.ComboStuntIntervalCurrent >= self.ComboStuntIntervalMax) then
			-- Combo timeframe limit reached, dispatch the StuntResult collection
			table.insert(self.ComboStuntResults, self.ComboStuntResultBuffer)
			
			-- Clear reference to previous buffer table
			self.ComboStuntResultBuffer = {}
			
			-- Turn combo tracking off
			self.ComboTrackingActive = false
		end
	else
		self.ComboStuntIntervalCurrent = 0
	end
end

function StuntManager:UpdateScoreMessages(ElapsedTime)
	local Player = ActiveMission.HumanPlayers[1]
	if(Player.HUD == nil) then
		return
	end
	
	local Text = ""
	for i = 0, self.ScoreMessages.Count - 1 do
		local Message = self.ScoreMessages[self.ScoreMessages.First + i]
		Message:Process(ElapsedTime)
		if(Message.Enabled) then
			Text = Text .. Message.Text .. " +" .. Message.Score
		end
		if(i < self.ScoreMessages.Count -1) then
			Text = Text .. "…"
		end
	end
	
	-- Remove outdated messages
	local First = self.ScoreMessages.First;
	while(self.ScoreMessages[First] ~= nil and self.ScoreMessages[First].Enabled == false) do
		self.ScoreMessages:PopLeft()
	end
	
	-- Update the HUD
	if(self.ScoreMessages.Count > 0) then
		--Player.HUD:UpdateScoreAddition(Text)
	end
end

function StuntManager:AddStunt(Stunt)
	table.insert(self.Stunts, Stunt)
	Stunt.ID = #self.Stunts
end

function StuntManager:AddWheelsState(WheelsState)
	self.WheelsHistory:PushRight(WheelsState)
	--print("^2Added: " ..WheelsState:__tostring())
	
	if(self.WheelsHistory.Count > self.MaxHistory) then
		local LastIndex = self.WheelsHistory.Last
		--print("^3Popping: " ..self.WheelsHistory[LastIndex]:__tostring())
		self.WheelsHistory:PopLeft()
	end
	
	--print("Count: " ..self.WheelsHistory.Count)
end

function StuntManager:PostStuntResult(StuntResult)
	local Player = ActiveMission.HumanPlayers[1]
	--local Message = ScoreDisplayMessage(Text, Score)
	--self.ScoreMessages:PushRight(Message)
	--Player:AddScore(Score)
	
	-- Play "saved"-sound
	if(StuntResult.Saved) then
		--playsound("Moves\\Save_" .. math.random(1,2) .. ".wav")
	end
	
	-- (Combo-)Stunt registration
	table.insert(self.ComboStuntResultBuffer, StuntResult)
	self.ComboStuntIntervalCurrent = 0
	self.ComboTrackingActive = true

	print(StuntResult.Message, StuntResult:__tostring())
	
	local AssocStunt = self.Stunts[StuntResult:GetStuntID()]
	local ScreenName = Translate(StuntResult:GetStuntScreenName())
	local StuntStarScore =  Player:ConvertToStarScore(Player.Scores[StuntResult:GetStuntID()])
	
	self.Mission:SetStuntScore(ScreenName, StuntResult:GetStuntID(), StuntStarScore)
end

function StuntManager:GetTotalDuration()
	local TotalTime = 0
	
	for Key, Value in pairs(self.Stunts) do
		local Stunt = self.Stunts[Key]
		local GrandResult = Stunt:GetGrandResult()
		TotalTime = TotalTime + GrandResult.Duration
	end
	
	return TotalTime
end

function StuntManager:GetActiveStunts()
	local Stunts = {}
	
	for Key, Value in pairs(self.Stunts) do
		local Stunt = self.Stunts[Key]
		if(Stunt.Active) then
			table.insert(Stunts, Stunt)
		end
	end
	
	return Stunts
end

function StuntManager:ChangeCamera()
	if(self.ChangeCamPos)then
		local Camera = EntityCamera(0)
		local PlayerCar = ActiveMission.HumanPlayers[1].Car
		local PlayerPos = PlayerCar:GetMatrix():GetTranslation()

		if(KeyPressed(GetKeyCode("DCK_RIGHT")))then	
			self.DirectionOffset = self.OffsetDistance
		else
			self.DirectionOffset = -(self.OffsetDistance)
		end
		
		if(self.NewCamPos == nil) then		
			self.NewCamMatrix = MatrixIdentity
			self.NewCamPos = Vector(PlayerPos.x + self.OffsetDistance, PlayerPos.y + self.HeightOffset, PlayerPos.z + self.DirectionOffset )
			self.NewCamMatrix = self.NewCamMatrix:Translate(self.NewCamPos)
			self.NewTempMatrix = Camera:GetMatrix()
		end
		
		self.NewTempMatrix = self.NewTempMatrix:Lerp(self.NewCamMatrix, 0.05)
		
		local CamDir = (self.NewTempMatrix:GetTranslation() - PlayerPos):Normalize() * -1
		CamDir.x = CamDir.x * -1 -- Fix voor LookAt() bug
		
		-- LookAt() is stuk dus er zijn rare fixes.
		-- Eerste waarde moet Vector(0,0,0) zijn en de tweede de Direction
		self.CamMatrix = Camera:GetMatrix():LookAt(Vector(0, 0, 0), CamDir, 0) 
		self.CamMatrix = self.CamMatrix:Rotate(Vector(1, 0, 0), math.rad(180)):Inverse() -- Fix voor LookAt() bug
		self.CamMatrix = self.CamMatrix:Rotate(Vector(0, 0, 1), math.rad(180)) -- Fix voor LookAt() bug
		self.CamMatrix = self.CamMatrix:Translate(self.NewTempMatrix:GetTranslation())
		--self.CamMatrix = self.CamMatrix:Translate(self.NewCamPos) -- Voor als je geen lerp wil
		
		Camera:SetMatrix(self.CamMatrix)
		if( math.abs(PlayerPos.x - self.CamMatrix:GetTranslation().x) > self.MaxCamDistance ) or 
		( math.abs(PlayerPos.z - self.CamMatrix:GetTranslation().z) > self.MaxCamDistance )then
			self.ChangeCamPos = false
		end
	end
end
