BaseDonutStunt = class(BaseRotationStunt, function(self)
	-- Misc
	self.ActivationDelay = 50
	self.DeactivationDelay = 20
	
	-- Determine how quickly the player should rotate before the stunt is activated
	-- A higher value means quicker yaw-rotation
	self.YawDiffMin = 1.15
	self.YawDiffMax = 1.50
	
	-- WheelsState for verifying the car has any wheel-contact with the ground
	self.AirState = WheelsState()
	self.AirState.Wheels[0] = false
	self.AirState.Wheels[1] = false
	self.AirState.Wheels[2] = false
	self.AirState.Wheels[3] = false
	
	-- Tracking
	self.InitialYaw = 0
	self.Rotations = 0
	self.LastYaw = 0
	self.RotationSign = 0 -- -1 = clockwise, 1 = counter-clockwise
	self.InitialYawSet = false
	
	
	self.Progress = 0
end)

function BaseDonutStunt:IsActivationConditionMet(IsActivationPending)
	local Player = ActiveMission.HumanPlayers[1]
	local Yaw = Player:GetCarYaw()
	
	-- Determine the initial yaw and setup the rotation sign
	if(IsActivationPending and not self.InitialYawSet and not self.Active) then
		--print("^3Pending stunt activation")
		self.InitialYaw = Yaw
		self.InitialYawSet = true
		
		-- Determine the rotation direction sign. This will be used later when
		-- the stunt is eventually activated. We will determine the rotation
		-- sign here, because this is where we process self.LastYaw too.
		-- 1 = counter-clockwise, -1 = clockwise
		self.RotationSign = math.sign(Yaw - self.LastYaw)
	elseif(not IsActivationPending and self.InitialYawSet and not self.Active) then
		--print("^3Cancelling stunt activation")
		self.InitialYaw = 0
		self.InitialYawSet = false
	end
	
	-- Determine the yaw rotation relative to the last yaw rotation
	self.YawDiff = 0
	if(self.LastYaw > 0) then
		self.YawDiff = math.abs(Yaw - self.LastYaw)
	end
	
	local CurrentWheelsState = self:GetCurrentWheelsState()
	local Activate = self.YawDiff >= self.YawDiffMin and self.YawDiff <=self.YawDiffMax and CurrentWheelsState ~= self.AirState
	
	self.LastYaw = Yaw
	return Activate
end

function BaseDonutStunt:GetPerc()
	return math.abs(self.Progress) / 360
end
local PastFirstRotationAngle = 0
function BaseDonutStunt:Process(TimeElapsed)
	local Player = ActiveMission.HumanPlayers[1]
	local Yaw = Player:GetCarYaw()
	
	if (self.InitialYaw == nil) then
		self.InitialYaw = Yaw
	end
	
	local AdjustedYaw = self:CalcAdjustedYaw(Yaw)
	
	--[[if(self.LastAngle > self:GetAngle())then
		PastFirstRotationAngle = self.LastAngle
		print(PastFirstRotationAngle)
	else
		self.Progress = math.abs(self:GetAngle()) - self.InitialYaw	
	end]]
	
	--print(math.abs(self:GetAngle()) .. " - " .. tostring(self.InitialYaw) .. " = " .. tostring(self.Progress))
	
	--print("Adj: " .. AdjustedYaw, "Yaw: " ..Yaw, "Init: " ..self.InitialYaw, "Sign: " ..self.RotationSign)
	if(self.LastAdjustedYaw >= 180 and AdjustedYaw < 180) then
		self.Rotations = self.Rotations + 1
		--print("Rotated "..(self.Rotations).." circle at "..tostring(self:GetAngle())..(" degrees."))
	end
	
	self.LastAdjustedYaw = AdjustedYaw
end

function BaseDonutStunt:EnableHelperTuning()
	local Player = ActiveMission.HumanPlayers[1]
	local Car = Player.Car
	local CarType = Car:GetTypeObject()
	CarType:SetWheelAccelForce(0,0)
	CarType:SetWheelAccelForce(1,0)
	CarType:SetWheelAccelForce(2,1)
	CarType:SetWheelAccelForce(3,1)
	
	--CarType:SetWHeelFrictionRoll(0, CarType:GetWheelFrictionRoll(0) - 1000)
	--CarType:SetWHeelFrictionRoll(1, CarType:GetWheelFrictionRoll(1) * 10)
	--CarType:SetWheelFrictionSide(0, CarType:GetWheelFrictionSide(0) * 3)
	--CarType:SetWheelFrictionSide(1, CarType:GetWheelFrictionSide(1) * 3)
	
	--CarType:SetWheelFrictionSide(2, CarType:GetWheelFrictionSide(2) / 8)
	--CarType:SetWheelFrictionSide(3, CarType:GetWheelFrictionSide(3) / 8)
	
	--CarType:SetWheelSlipSide(0, CarType:GetWheelSlipSide(0) - 0.00000010)
	--CarType:SetWheelSlipSide(1, CarType:GetWheelSlipSide(1) - 0.00000010)
end

function BaseDonutStunt:DisableHelperTuning()
	local Player = ActiveMission.HumanPlayers[1]
	local Car = Player.Car
	local CarType = Car:GetTypeObject()
	
	CarType:SetWheelAccelForce(0,1)
	CarType:SetWheelAccelForce(1,1)
	CarType:SetWheelAccelForce(2,1)
	CarType:SetWheelAccelForce(3,1)
end

function BaseDonutStunt:Activate()
	self.Super.Super.Super.Activate(self)
	
	self.LastAdjustedYaw = 0
	self.Rotations = 0
	
	--self:EnableHelperTuning()
end

function BaseDonutStunt:Deactivate()
	self.Super.Super.Super.Deactivate(self)
	
	--self:DisableHelperTuning()
end

-- Calculates the yaw relative to the starting angle of the donut, keeping the rotation
-- direction in mind.
function BaseDonutStunt:CalcAdjustedYaw(Yaw)
	local AdjYaw = (360 - self.InitialYaw + Yaw) % 360
	
	if(self.RotationSign == -1) then
		AdjYaw = 360 - AdjYaw
	end
	
	return AdjYaw
end