DonutStunt = class(BaseDonutStunt, function(self)
	-- Misc
	self.ScreenName = "stunt_title_donut"
	self.ActivationDelay = 1.0
	self.DeactivationDelay = 1.0
	
	-- Determine how quickly the player should rotate before the stunt is activated
	-- A higher value means quicker yaw-rotation
	self.YawDiffMin = 1.00
	self.YawDiffMax = 3.00
	
	self.DonutActivatedAmount = 0
	
	self.WheelFrictionSide = {}
end) 

function DonutStunt:OnActivated()
	print("^2Donut activated") 
	self.InitialYaw = nil
	
	self.StuntManager.NewCamPos = nil
	self.StuntManager.ChangeCamPos = true
	
	for i = 0, 3 do
		local Wheel = ActiveMission.CarBuilder:GetWheel(i)
		self.WheelFrictionSide[i] = Wheel:GetFrictionSide()
		Wheel:SetFrictionSide(8000000)
	end
end 
 
function DonutStunt:Process(ElapsedTime)
	self.Super.Process(self, ElapsedTime)
	
	-- Check if the player is accelerating
	local Player = ActiveMission.HumanPlayers[1]
	local InputManager = Player.InputManager
	
	--print(self.YawDiff)
	local Accelerating = InputManager.AccelSpeed > 0
	if(Accelerating) then
		self:AidTruckRotation()
	end
end
  
function DonutStunt:AidTruckRotation()
	local Player = ActiveMission.HumanPlayers[1]
	Player:CarAddYawForce(8e+5 * -self.RotationSign)
end
 
function DonutStunt:OnDeactivated()
	print("^2Donut deactivated")

	if (self.Rotations > 0) then
		self.DonutActivatedAmount = self.DonutActivatedAmount + 1

		local Result = self:CreateResult()
		Result.Rotations = self.Rotations
		Result.Message = "Donut x" ..self.Rotations
		
		self:CalculateScore()
		self.Player.Scores[self.ID] = Max(self.Score, self.Player.Scores[self.ID])
		
		self:AddResult(Result)
		playsound("Moves\\Donut.wav")
	end
	
	self.StuntManager.ChangeCamPos = false
	
	for i = 0, 3 do
		local Wheel = ActiveMission.CarBuilder:GetWheel(i)
		Wheel:SetFrictionSide(self.WheelFrictionSide[i])
	end
end  
 
function DonutStunt:CalculateScore()

	-- Divided by 4 because after 4 rotations the player has reached the
    -- max score for this stunt. If this is set to lower than 4 (by example 3),
	-- the player gets 2 stars for only 1 rotation.
	self.Score = (self.Player.MaxScorePerStunt/3) * self.Rotations
	--print(self.Player.Scores[self.ID])
end