TowCarObject = class(function(self, FileName, StartFriction, MaxFriction, StartPos, EffectOffset)
	print("^3INIT TowCarObject")
	
	self.Mesh = CreateAdvancedMesh(nil, FileName, "")
	GetEntityScene():AddEntity(self.Mesh)
	
	self.EffectOffset = EffectOffset
	self.EffectTimer = 0
	self.EffectTimerMax = math.random(60,120)
	
	self.StartPos = StartPos
	
	if (win32()) then
		self.Mesh:SetRenderCallback(function(...)
			self:Render(...)
		end)
	end
	
	self.Mesh:SetDestroyCallback(function(...)
		self:Destroy(...)
	end)
	
	self.StartMass = 5000 --StartMass
	
	self.Friction = Friction
	self.StartFriction = StartFriction
	self.MaxFriction = MaxFriction
	
	self.ChassisObject = self.Mesh:GetObject("Shuttle")
	self.Matrix = MatrixIdentity
	self.Matrix = self.Matrix:Translate(self.StartPos)
	self.Matrix = self.Matrix:SetRotation(self.Mesh:GetMatrix():GetRotation())
	self.MeshMatrix = self.Mesh:GetMatrix()
	
	self.OffsetPos = Vector(0,0,0)
	
	-- Physics
	self.Body = Body(GetPhysicsScene())
	self.Body:SetMass(self.StartMass)
	self.Body:SetInertiaTensor(self.ChassisObject:GetExtents())
	self.Body:SetMatrix(self.Matrix)
	self.Body:Enable()
	
	-- Collision
	self.Geometry = BoxGeometry()
	self.Geometry:SetEntity(self.Mesh)
	self.Geometry:SetOffsetMatrix(MatrixIdentity:Translate(self.ChassisObject:GetCenter()))
	self.Geometry:SetBody(self.Body)
	self.Geometry:SetExtents(self.ChassisObject:GetExtents())
	self.Geometry:SetType(bit32.bor(DCGF_CAR, DCGF_REAL))
	self.Geometry:SetCollision(bit32.bor(DCGF_WORLD))
	self.Geometry:SetContactCallback(function(...)
		return self:OnContact(...)
	end)

	self.Effect = self:CreateEffect("MJ_TowTestObject_Light")
	self.Effect:SetAmountVar("ShowDamage", 0)
	
	self.Geometry:Enable()
	self.Geometry:SetMatrix(self.Matrix)

	GetPhysicsScene():AddGeometry(self.Geometry)
	
	self.Contact = {}
	
	PhysicsThread:Spawn(function()
		while (true) do
			self:Update()
			PhysicsThread:Wait(1)
		end
	end)
end)

function TowCarObject:Destroy()	
	if (self.Thread ~= nil) then
		PhysicsThread:Stop(self.Thread)
		self.Thread = nil
	end
	
	if (self.Geometry ~= nil) then
		self.Geometry:Destroy()
		self.Geometry = nil
	end
	
	if (self.Body ~= nil) then
		self.Body:Destroy()
		self.Body = nil
	end
end

function TowCarObject:SetMatrix(Matrix)
	local FinalMatrix = self.Matrix:Translate(Vector(0, 5, 0)) * Matrix
	
	self.Body:SetMatrix(FinalMatrix)
end

function TowCarObject:SetOffset(NewPos)
	self.OffsetPos = NewPos
end

function TowCarObject:SetEffectOffset(NewPos)
	self.EffectOffset = NewPos
end

function TowCarObject:GetMatrix()
	return self.Body:GetMatrix()
end

function TowCarObject:Update()
	local FinalMatrix = self:GetMatrix()
	
	self.Mesh:Update(FinalMatrix:Translate(self.OffsetPos), self.Body:GetVelocity(), self.Body:GetAngularVelocity())
	
	local Magnitude = self.Body:GetVelocity():Magnitude()
	
	if (not ActiveMission.Finished) then
		if (Magnitude > 40) then	
			-- Change amountvar to 1
			self.Effect:SetAmountVar("ShowDamage", 1)
		else
			-- Change amountvar to 0
			self.Effect:SetAmountVar("ShowDamage", 0)
		end
		
		if (self.Effect ~= nil) then
			self.Effect:SetOffsetPos(self.EffectOffset)
		end
	else
		self.Effect:SetAmountVar("ShowDamage", 0)
	end
end

function TowCarObject:CreateEffect(Name)
	local Effect = CreateEffectSystem("FX\\" .. Name .. ".xml", true)
	GetEntityScene():AddEntity(Effect)
	Effect:SetOffsetPos(self.EffectOffset)
	
	self.Mesh:AddPlugin(Effect, "", 1)
	
	return Effect
end

function TowCarObject:Render()
	-- self.Geometry:Render(color(0, 0, 1, 1), self.Geometry:GetMatrix():GetTranslation())
end

function TowCarObject:OnContact(Entity, Pos, Normal, Depth)
	local Contact = self.Contact
	Contact.CFM = 0.000001
	Contact.ERP = 0.999
	Contact.Friction1 = self.Friction
	Contact.Friction2 = self.Friction
	
	local FrictionRight = Normal:Cross(self:GetMatrix():GetDirection() * -1):Normalize()
	Contact.FrictionDir = FrictionRight:Cross(Normal):Normalize()

	return Contact
end
