-- Misc includes
dofile("Lua\\Upgrading\\UpgradeTypes.lua")

-- Upgrade includes
dofile("Lua\\Upgrading\\Upgrades\\BaseUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\ArmorUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\EngineUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\FrameUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\MechanicBoostUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\MechanicGearboxUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\ShocksUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\SuspensionUpgrade.lua")
dofile("Lua\\Upgrading\\Upgrades\\WheelsUpgrade.lua")

Upgrader = class(function(self, CarTypeName)
	self.CarTypeName = CarTypeName
	self.Upgrades = {}
	
	-- Menu assets
	self.MenuCar = CarClasses[1]:GetCarByName(CarTypeName)
	
	-- Add misc upgrades
	self:AddUpgrade(UpgradeTypes.Armor, ArmorUpgrade(self))
	self:AddUpgrade(UpgradeTypes.Engine, EngineUpgrade(self))
	self:AddUpgrade(UpgradeTypes.Frame, FrameUpgrade(self))
	self:AddUpgrade(UpgradeTypes.Shocks, ShocksUpgrade(self))
	self:AddUpgrade(UpgradeTypes.Suspension, SuspensionUpgrade(self))
	self:AddUpgrade(UpgradeTypes.Wheels, WheelsUpgrade(self))
	
	-- Add mechanic upgrades
	self:AddUpgrade(UpgradeTypes.MechanicBoost, MechanicBoostUpgrade(self))
	self:AddUpgrade(UpgradeTypes.MechanicGearbox, MechanicGearboxUpgrade(self))
end)

function Upgrader:PrintCarSpecs(CarBuilder)
	for i = 0, 3 do
		print("-- Wheel " ..i .." --")
		local Wheel = CarBuilder:GetWheel(i)
		print("ERP: " ..string.format("%0.6f", Wheel:GetERP()))
		print("CFM: " ..string.format("%0.6f", Wheel:GetCFM()))
		print("FrictionRoll: " ..Wheel:GetFrictionRoll())
		print("FrictionSide: " ..Wheel:GetFrictionSide())
		print("SlipRoll: " ..Wheel:GetSlipRoll())
		print("SlipSide: " ..string.format("%0.8f", Wheel:GetSlipSide()))
		print("SteeringVel0: " ..Wheel:GetSteeringVel(0))
		print("SteeringVel0: " ..Wheel:GetSteeringVel(1))
		print("SteeringAngle0: " ..math.deg(Wheel:GetSteeringAngle(0)))
		print("SteeringAngle1: " ..math.deg(Wheel:GetSteeringAngle(1)))
	end
	
	print("-- Misc --")
	print("MaxHP: " ..CarBuilder:GetMaxHP())
	print("TorqueScale: " ..CarBuilder:GetEngine():GetTorqueScale())
end

function Upgrader:AddUpgrade(UpgradeType, Upgrade)
	self.Upgrades[UpgradeType] = Upgrade
end

function Upgrader:GetUpgrade(UpgradeType)
	return self.Upgrades[UpgradeType]
end

function Upgrader:GetMenuCarUpgrade(UpgradeName)
	return self.MenuCar.Upgrades[UpgradeName]
end

function Upgrader:ApplyUpgrades(CarBuilder)
	print("^2--- Upgrading truck ---")
	for Key, Value in pairs(self.Upgrades) do
		local Upgrade = self.Upgrades[Key]
		Upgrade:Apply(CarBuilder)
		Upgrade:PrintDebugMessages()
	end
	print("^2------------------------")
	
	--self:PrintCarSpecs(CarBuilder)
end

function Upgrader:GetTerrainDamageReduction(Damage)
	local DamageReduction = 0
	local FrameUpgrade = self:GetUpgrade(UpgradeTypes.Frame)
	local ShocksUpgrade = self:GetUpgrade(UpgradeTypes.Shocks)
	
	-- Apply FrameUpgrade terrain damage reduction
	DamageReduction = DamageReduction + FrameUpgrade:GetTerrainDamageReduction(Damage)
	DamageReduction = DamageReduction + ShocksUpgrade:GetTerrainDamageReduction(Damage)
	
	return DamageReduction
end