WheelsUpgrade = class(BaseUpgrade, function(self)
	self.Type = UpgradeTypes.Wheels
	
	self.MenuUpgrade = self.Upgrader:GetMenuCarUpgrade("garage_tires")
	self:SetLevel(self.MenuUpgrade.CurrentLevel)
end)

function WheelsUpgrade:Apply(CarBuilder)
	local FrictionRollIncreaseFactor = 0.17 -- 5000 / 75000
	local FrictionSideIncreaseFactor = 0.01 -- 200000 / 3100000
	local SlipRollIncreaseFactor = 0
	local SlipSideDecreaseFactor = 0.17 -- -0.00000004 / 0.00000045
	
	if (ActiveGameMode == SingleRace) or (ActiveGameMode == Destroyer) then
		print("^3Using Race WheelSettings")
		FrictionRollIncreaseFactor = 0.07 -- 5000 / 75000
		FrictionSideIncreaseFactor = 0.10 -- 200000 / 3100000
		SlipRollIncreaseFactor = 0
		SlipSideDecreaseFactor = 0.09 -- -0.00000004 / 0.00000045
	end
	
	if (ActiveGameMode == FreestyleFest) then
		print("^3Using FreeStyle WheelSettings")
		FrictionRollIncreaseFactor = 0.07 -- 5000 / 75000
		FrictionSideIncreaseFactor = 0.07 -- 200000 / 3100000
		SlipRollIncreaseFactor = 0
		SlipSideDecreaseFactor = 0.09 -- -0.00000004 / 0.00000045
	end
	
	if (ActiveGameMode == TankTrap) then
		print("^3Using TankTrap WheelSettings")
		FrictionRollIncreaseFactor = 0.07 -- 5000 / 75000
		FrictionSideIncreaseFactor = 0.20 -- 200000 / 3100000
		SlipRollIncreaseFactor = 0
		SlipSideDecreaseFactor = 0.09 -- -0.00000004 / 0.00000045
	end
	--local FrictionRollIncrease = 5000 -- Default = 75000 -- Grip van rollende wielen
	--local FrictionSideIncrease = 200000 -- Default = 3100000 -- Zijwaartse grip
	--local SlipRollIncrease = 0 -- Default = 0
	--local SlipSideIncrease = -0.00000004 -- Default = 0.00000045 -- Lager = makkelijker draaien terwijl je beweegt
	
	local SteeringVelIncrease = 0
	local SteeringAngleIncrease = math.rad(3)
	
	local MultFactor = self:GetMultiplicationFactor()
	
	-- Set the wheel friction
	self:SetWheelFriction(CarBuilder, FrictionRollIncreaseFactor * MultFactor, FrictionSideIncreaseFactor * MultFactor)
	
	-- Set the wheel slip
	self:SetWheelSlip(CarBuilder, SlipRollIncreaseFactor * MultFactor, SlipSideDecreaseFactor * MultFactor)
	
	-- Increase steering for the two front wheels
	for i = 0, 3 do
		local Wheel = CarBuilder:GetWheel(i)
		
		-- Steering velocity adjustment
		local CurrentSteeringVel0 = Wheel:GetSteeringVel(0)
		local CurrentSteeringVel1 = Wheel:GetSteeringVel(1)
		local NewSteeringVel0 = CurrentSteeringVel0 + (SteeringVelIncrease * MultFactor)
		local NewSteeringVel1 = CurrentSteeringVel1 + (SteeringVelIncrease * MultFactor)
		if(self.Debug) then
			self:AddDebugMessage("Upgraded steering vel0 of wheel #" ..i .." from "
				..Round(CurrentSteeringVel0,2) .. " to " ..Round(NewSteeringVel0,2) ..".")
			self:AddDebugMessage("Upgraded steering vel1 of wheel #" ..i .." from "
				..Round(CurrentSteeringVel1,2) .. " to " ..Round(NewSteeringVel1,2) ..".")
		end
		Wheel:SetSteeringVel(0, NewSteeringVel0)
		Wheel:SetSteeringVel(1, NewSteeringVel1)
		
		-- Steering angle adjustment
		local SteeringAngle0 = Wheel:GetSteeringAngle(0)
		local SteeringAngle1 = Wheel:GetSteeringAngle(1)
		local SteeringSign0 = math.sign(SteeringAngle0)
		local SteeringSign1 = math.sign(SteeringAngle1)
		local NewSteeringAngle0 = SteeringAngle0 + (SteeringSign0 * (SteeringAngleIncrease * MultFactor))
		local NewSteeringAngle1 = SteeringAngle1 + (SteeringSign1 * (SteeringAngleIncrease * MultFactor))
		Wheel:SetSteeringAngle(0, NewSteeringAngle0)
		
		-- Only adjust SteeringAngle1 for the front two tires
		if(i == 0 or i == 1) then
			if(self.Debug) then
				self:AddDebugMessage("Upgraded steering angle0 of wheel #" ..i
					.." from " ..Round(math.deg(SteeringAngle0),2)
					.." to " ..Round(math.deg(NewSteeringAngle0),2) ..".")
				self:AddDebugMessage("Upgraded steering angle1 of wheel #" ..i
					.." from " ..Round(math.deg(SteeringAngle1),2)
					.." to " ..Round(math.deg(NewSteeringAngle1),2) ..".")
			end
			Wheel:SetSteeringAngle(1, NewSteeringAngle1)
		end
	end
	
	print("^2WheelsUpgrade applied (level " ..self.Level ..").")
end