function ListFunctions(Object)
	print("Listing functions for " .. tostring(Object));

	local Type = type(Object)

	if (Type == "userdata") then
		Object = Object.__index;
	end

	local Type = type(Object)
	print(Type)

	for Key, Value in pairs(Object) do
		local Type = type(Value);

		if (Type == "function") then
			print(Type .. ": " .. tostring(Key));
		end
	end

	print("Done");
end

function ListMembers(Object)
	print("Listing members for " .. tostring(Object));

	local Type = type(Object)

	if (Type == "userdata") then
		Object = Object.__index;
	end

	for Key, Value in pairs(Object) do
		local Type = type(Value);

		if (Type ~= "function") then
			print(Type .. ": " .. tostring(Key));
		end
	end

	print("Done");
end

function ListMembersValues(Object)
	print("Listing members with values for " .. tostring(Object));

	local Type = type(Object)

	if (Type == "userdata") then
		Object = Object.__index;
	end

	for Key, Value in pairs(Object) do
		local Type = type(Value);

		if (Type ~= "function") then
			print(Type .. ": " .. tostring(Key) .. " = " .. tostring(Value));
		end
	end

	print("Done");
end

function GetRegistryTable(Key)
	local regis = debug.getregistry()
	
	local Alt1Key = string.gsub(Key, "Create", "")
	local Alt2Key = string.gsub(Key, "Get", "")
	local Alt3Key = string.gsub(Key, "Lua", "New")
	
	if (regis["Team6.".. Key]) then
		return regis["Team6.".. Key], Key
	elseif (regis["Team6.".. Alt1Key]) then
		return regis["Team6.".. Alt1Key], Alt1Key
	elseif (regis["Team6.".. Alt2Key]) then
		return regis["Team6.".. Alt2Key], Alt2Key
	elseif (regis["Team6.".. Alt3Key]) then
		return regis["Team6.".. Alt3Key], Alt3Key
	end
	
	return nil
end

function ListAllFunctions()
	--[[ local regis = debug.getregistry()

	for Key, Value in pairs(regis) do
		local Type = type(Value)
		print(Key)
		print(Value)
		print(Type)
		
		if (Type == "table") then
			for Key2, Value2 in pairs(Value) do
				print(Key2)
				print(Value2)
			end
		end
		
		print('--------')
	end
	print('================') ]]--
	
	local CFunctions = {}
	local Team6Classes = {}
	local LuaFunctions = {}
	local GlobalUserdata = {}
	local GlobalTables = {}
	local GlobalVars = {}
	
	local globals = {}; 
	for k,v in pairs(_G) do 
		globals[k] = "global"
		
		local Type = type(v)
		
		if (Type == "function") then
			--local info = debug.getinfo(v, "nfS")
			--local info = debug.getinfo(v, "n")
			local info = debug.getinfo(v, "nS")
			local FunctionType = info.what
			if (FunctionType == "C") then
				--print("C function")
				
				local RegTable, RegName = GetRegistryTable(k)
				
				if (RegTable ~= nil) then
					local Team6Class = {RegName, k, RegTable}
					table.insert(Team6Classes, Team6Class)
					--Team6Classes[RegName] = k
				else
					table.insert(CFunctions, k)
				end
				
			elseif (FunctionType == "Lua") then
				--print("Lua function")
				--local Source = info.source
				--print(Source)
				
				local LuaFunction = {k, info.source}
				table.insert(LuaFunctions, LuaFunction)
			end
		elseif (Type == "userdata") then
			--table.insert(GlobalUserdata, k)
			
			local Userdata = {k, v}
			table.insert(GlobalUserdata, Userdata)
		elseif (Type == "table") then
			table.insert(GlobalTables, k)
		else
			local Var = {k, v}
			table.insert(GlobalVars, Var)
		end
	end
	
	local function InternTableSort(x, y)
		return x[1] < y[1]
	end
	
	print("^6 -=Team6 Classes=-")
	table.sort(Team6Classes, InternTableSort)
	for k,v in pairs(Team6Classes) do 
		print("Name: ^2" .. v[1])
		print("Constructor: " .. v[2])
		
		print("Functions:")
		local Functions = {}
		for Key, Value in pairs(v[3]) do
			if (Key ~= "__index" and Key ~= "__gc") then
				table.insert(Functions, Key)
			end
		end
		
		table.sort(Functions)
		for Key, Value in pairs(Functions) do
			print(" - " .. Value)
		end
	end
	
	print("")
	print("^6 -=C functions=-")
	table.sort(CFunctions)
	for k,v in pairs(CFunctions) do 
		print("^1" .. v)
	end
	
	print("")
	print("^6 -=Lua functions=-")
	table.sort(LuaFunctions, InternTableSort)
	for k,v in pairs(LuaFunctions) do 
		print("^1" .. v[1] .. "^7 (" .. v[2] .. ")")
	end
	
	print("")
	print("^6 -=Global userdata=-")
	table.sort(GlobalUserdata, InternTableSort)
	for k,v in pairs(GlobalUserdata) do 
		print("^1" .. v[1] .. "^7 (" .. type(v[2]) .. ")")
	end
	
	print("")
	print("^6 -=Global tables=-")
	table.sort(GlobalTables)
	for k,v in pairs(GlobalTables) do 
		print("^1" .. v)
	end
	
	print("")
	print("^6 -=Global vars=-")
	table.sort(GlobalVars, InternTableSort)
	for k,v in pairs(GlobalVars) do 
		print("^1" .. v[1] .. ":^2" .. tostring(v[2]) .. "^7 (" .. type(v[2]) .. ")")
	end
end

function ListMembersRecursive(Object, Spaces)
	if (Spaces == nil) then
		Spaces = ""
	end

	print(Spaces .. "Listing recursive members for " .. tostring(Object));
	
	local PrevSpaces = Spaces
	Spaces = Spaces .. "\t"

	local Type = type(Object)

	if (Type == "userdata") then
		Object = Object.__index;
	end

	for Key, Value in pairs(Object) do
		local Type = type(Value);

		if (Type ~= "function") then
			print(Spaces .. Type .. ": " .. tostring(Key) .. " = " .. Value);
		end
		
		if (Type == "table") then
			ListMembersRecursive(Value, Spaces)
		end
	end
	
	print(PrevSpaces .. "Done");
end