-- Thread structuur is als volgt:
-- Thread = {}
-- [1] = Time
-- [2] = Coroutine
-- Reden: sneller

Scheduler = class(function(self, SecondMultiplier)
	self.Time = 0;
	self.SecondMultiplier = SecondMultiplier	-- Moet nog ff niet hardcoded zijn

	self.Threads = {}
end)

function Scheduler:Destroy()
	self.Threads = nil
end

function Scheduler:Select(ElapsedTime)
	self.Time = self.Time + ElapsedTime;

	for i = #self.Threads, 1, -1 do
		if (self.Threads == nil) then
			break
		end
		
		local Thread = self.Threads[i]

		if (Thread ~= nil and self.Time >= Thread[1]) then
			table.remove(self.Threads, i)

			self:RunThread(Thread)
		end
	end
end

-- Api
function Scheduler:Spawn(Func)
	-- Create the thread
	local Thread = {
		self.Time,
		coroutine.create(Func)
	}

	self:Schedule(Thread);
	
	return Thread
end

function Scheduler:Stop(Thread)
	if (self.ContinueThread == Thread) then
		self.ContinueThread = nil
		--print("^2Thread removed")
	else
		if (not table.removeobject(self.Threads, Thread)) then
			--print("^1ERROR, Thread not removed!!!")
			--StackTrace()
		end
	end	
end

function Scheduler:Kill(Thread)
	self:Stop(Thread)
end

function Scheduler:Run(Func)
	-- Create the thread
	local Thread = {
		0,
		coroutine.create(Func)
	}

	-- Resume immediately
	self:RunThread(Thread);

	return Thread
end

function Scheduler:Wait(Time)
	return coroutine.yield(Time)
end

function Scheduler:Sleep(Time)
	return coroutine.yield(Time * self.SecondMultiplier)
end

function Scheduler:Resume(Thread)
	table.removeobject(self.Threads, Thread)

	self:RunThread(Thread)
end

-- Interne stuff
function Scheduler:RunThread(Thread)
	self.ContinueThread = Thread
	local Success, Wait = coroutine.resume(Thread[2], Thread);

	if (Success) then
		if (Wait ~= nil and self.ContinueThread) then
			self.ContinueThread = nil
			
			Thread[1] = self.Time + Wait;

			self:Schedule(Thread)
		end
	else
		print("Scheduler returned error: " .. Wait)
	end
end

function Scheduler:Schedule(Thread)
	table.insert(self.Threads, Thread);
end
