function IndexInTable(Table, Key)
	for i, Item in ipairs(Table) do
		if (Item == Key) then
			return i - 1;
		end
	end
	
	return -1
end

function table.inserttable(a, b) 
	for i, Item in ipairs(b) do
		table.insert(a, Item)
	end
end


table.findindex = function(Table, Object)
	for i, Item in ipairs(Table) do
		if (Item == Object) then
			return i - 1
		end
	end

	return nil
end


table.removeobject = function(Table, Object)
	for i, Item in ipairs(Table) do
		if (Item == Object) then
			table.remove(Table, i)
			return true
		end
	end
	
	return false
end

table.copy = function(Table)
	--StackTrace() 
	local u = {} 

	for k, v in pairs(Table) do
		u[k] = v
	end

	return u;
end

table.getsortedindex = function(Table, Object, Sort)
	for i, Item in ipairs(Table) do
		if (Sort(Object, Item)) then
			return i
		end
	end

	return #Table + 1	-- i?
end

function table.mergeinplace(t1, t2)
    for k,v in pairs(t2) do
    	if type(v) == "table" then
    		if type(t1[k] or false) == "table" then
    			table.mergeinplace(t1[k] or {}, t2[k] or {})
    		else
    			t1[k] = v
    		end
    	else
    		t1[k] = v
    	end
    end
end

function table.merge(t1, t2, MergeFunction)
	if (MergeFunction == nil) then
		MergeFunction = function(x, y)
			return y
		end
	end	

	local Result = deepcopy(t1)
	
    for k,v in pairs(t2) do
    	if type(v) == "table" then
    		if type(t1[k] or false) == "table" then
    			table.merge(t1[k] or {}, t2[k] or {})
    		else
    			Result[k] = MergeFunction(t1[k], t2[k])
    		end
    	else
    		Result[k] = MergeFunction(t1[k], t2[k])
    	end
    end
	
	return Result;
end

function table.empty(self) 
    for _, _ in pairs(self) do 
        return false 
    end

    return true 
end


function TableContains(Table, Key)
	for i, Item in ipairs(Table) do
		if (Item == Key) then
			return true;
		end
	end
	
	return false
end

function TableLength(Table, Key)
	local Count = 0
	
	for Item in pairs(Table) do
		Count = Count + 1
	end
	
	return Count
end

function FormatTime(Time)
	local Minutes = math.floor(Time / 60)
	local Seconds = Time % 60
	
	return string.format("%.0f:%06.3f", Minutes, Seconds)
end

function FormatTimeLong(Time)
	local Hours = math.floor(Time / 3600)
	local Minutes = math.floor(Time / 60) % 60
	local Seconds = Time % 60
	
	return string.format("%.0f:%02.0f:%02.0f", Hours, Minutes, Seconds)
end

function FormatTimeShort(Time)
	local Minutes = math.floor(Time / 60)
	local Seconds = math.floor(Time % 60)
	
	return string.format("%.0f:%02.0f", Minutes, Seconds)
end

function NumberCommas(amount)
	local formatted = amount
	while true do
		formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
		if (k==0) then
			break
		end
	end
	return formatted
end

function Inc(x)
	return x + 1
end

function Dec(x)
	return x - 1
end

function Add(x, y)
	return x + y
end

function Sub(x, y)
	return x - y
end

function Mul(x, y)
	return x * y
end

function Div(x, y)
	return x / y
end

function Mod(x, y)	-- werkt ook op negatieve waarden
	return x % y
end

function IncMod(x, y, z)
	return Mod(Inc(x, y), z)
end

function DecMod(x, y, z)
	return Mod(Dec(x, y), z)
end

function Min(x, y)
	if (x < y) then
		return x
	else 
		return y
	end
end

function Max(x, y)
	if (x > y) then
		return x
	else 
		return y
	end
end

function ClampMin(x, y)
	if (x < y) then
		return y
	else
		return x
	end
end

function ClampMax(x, y)
	if (x > y) then
		return y
	else
		return x
	end
end

function ClampMinMax(x, _min, _max)
	 if (x < _min ) then 
		return _min
	elseif (x > _max ) then
		return _max
	else
		return x
	end
end

function Lerp(x, y, z)
	return x + (y - x) * z
end

function math.sign(x)
   if x < 0 then
     return -1
   elseif x > 0 then
     return 1
   else
     return 0
   end
end

function Round(number, decimals)
	if (decimals == nil) then
		decimals = 0
	end
	return tonumber(string.format("%." .. (decimals or 0) .. "f", number))
end

function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

function split(pString, pPattern)
   local Table = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pPattern
   local last_end = 1
   local s, e, cap = pString:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
     table.insert(Table,cap)
      end
      last_end = e+1
      s, e, cap = pString:find(fpat, last_end)
   end
   if last_end <= #pString then
      cap = pString:sub(last_end)
      table.insert(Table, cap)
   end
   return Table
end

function explode(div,str)
	if (div=='') then return false end
	local pos,arr = 0,{}
	-- for each divider found
	for st,sp in function() return string.find(str,div,pos,true) end do
		table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
		pos = sp + 1 -- Jump past current divider
	end
	table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
	
	return arr
end

function copy(t) 
	local u = { } 
	for k, v in pairs(t)
		do u[k] = v
	end 

	return setmetatable(u, getmetatable(t)) 
end 

function Serialize(Object)
	local Data = ""

	if type(Object) == "number" then
		Data = Data .. Object
	elseif type(o) == "boolean" then
		Data = Data .. tostring(Object)
	elseif type(Object) == "string" then
		Data = Data .. string.format("%q", Object)
	elseif type(Object) == "nil" then
		Data = Data .. "nil"
	elseif type(Object) == "table" then
		Data = Data .. "{"

		for Key, Value in pairs(Object) do
			if type(Key) == "number" then
				Data = Data .. " [" .. Key .. "] = "
			else
				Data = Data .. " " .. Key .. " = "
			end

			Data = Data .. Serialize(Value)
			Data = Data .. ","
		end

		Data = Data .. " }"
	else
		Data = Data .. "nil"
	end
		
	return Data
end

function Serialize(Object)
	local Data = ""

	if type(Object) == "number" then
		Data = Data .. Object
	elseif type(o) == "boolean" then
		Data = Data .. tostring(Object)
	elseif type(Object) == "string" then
		Data = Data .. string.format("%q", Object)
	elseif type(Object) == "nil" then
		Data = Data .. "nil"
	elseif type(Object) == "table" then
		Data = Data .. "{ "

		for Key, Value in pairs(Object) do
			if type(Key) == "number" then
				Data = Data .. "[" .. Key .. "] = "
			else
				Data = Data .. Key .. " = "
			end

			Data = Data .. Serialize(Value)
			Data = Data .. ", "
		end

		Data = Data .. "}"
	else
		Data = Data .. "nil"
	end
		
	return Data
end

function SerializeKVTable(Data, Object, ExtraData)
	for Key, Value in pairs(Object) do
		Data = Data .. " " .. Key .. " = "

		Data = Data .. Serialize(Value)
		Data = Data .. ","
	end

	return Data
end

function UnSerialize(String)
	return loadstring("return " .. String)();
end

function Shuffle(t)
	local n = #t
	
	while n >= 2 do
		local k = math.random(n)
		t[n], t[k] = t[k], t[n]
		n = n -1
	end
	
	return t
end

function string.starts(String, Start)
	return string.sub(String, 1, string.len(Start)) == Start
end

function string.ends(String, End)
   return End == '' or string.sub(String, -string.len(End)) == End
end