uniform vec4 BentView[3];
uniform vec4 Proj[4];

uniform vec2 BendFactors;

uniform vec4 EmitterData[12];

attribute vec3 Position;    // Pos
attribute vec4 TexCoord0;   // Tex
attribute vec4 TexCoord1;   // Gravity
attribute vec4 TexCoord2;   // StartVel
attribute vec4 TexCoord3;   // Timing
attribute vec4 TexCoord4;   // Sizing

varying lowp vec4 fColor;
varying mediump vec2 fTex;

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec3 mul4x3(vec4 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);

	return ret;
}

vec3 Bend(vec3 P){
	float Angle = (P.z * BendFactors.y) - 1.5707963267948966192313216916398;
	
	float SinAngle = sin(Angle);
	float CosAngle = cos(Angle);
		
	vec3 Pos = vec3(SinAngle + 1.0, 0.0, CosAngle) * BendFactors.x;
	
	vec4 BendMatrix[3];
	BendMatrix[0].xyz = vec3(-SinAngle, 0.0, -CosAngle);	// Dit is loodrecht op de afgeleide
	BendMatrix[1].xyz = vec3(0.0, 1.0, 0.0);
	BendMatrix[2].xyz = vec3(-CosAngle, 0.0, SinAngle);	// Dit is de afgeleide dus
	
	BendMatrix[0].w = Pos.x;
	BendMatrix[1].w = Pos.y;
	BendMatrix[2].w = Pos.z;
	
	P = mul4x3(vec4(P.xy, 0, 1), BendMatrix);
	
	return P;
}


void main(){
	// Constant unpacking
	vec4 Matrix[3];
	Matrix[0] = EmitterData[0];
	Matrix[1] = EmitterData[1];
	Matrix[2] = EmitterData[2];
	
	float Time = EmitterData[5].w;
	
	float LocalStart = EmitterData[11].x;
	float LocalEnd = EmitterData[11].y;

	vec4 Color0 = EmitterData[7];
	vec4 Color1 = EmitterData[8];
	
	vec3 A = EmitterData[9].xyz;
	vec2 Size = EmitterData[10].xy;
	
	// Attribute unpacking
	vec3 WorldPos = Position;
	vec3 WorldVel = TexCoord0.xyz;// / 4;
	
	vec3 LocalPos = TexCoord1.xyz;
	vec3 LocalVel = TexCoord2.xyz;
	
	float StartRoll = TexCoord0.w;
	float RollSpeed = TexCoord2.w;
	
	vec2 StartSize = (TexCoord3.xy - 0.5) * Size;// * 1.141;
	float GrowSpeed = TexCoord3.z;
	float TTL = TexCoord3.w;

	float StartTime = TexCoord4.x;
	
	// Using data
	float T = (Time - StartTime);// * 2.0;
	float TimePercentage = clamp(T / TTL, 0.0, 1.0);

	vec3 HalfA = A * 0.5;

	// S(T) = S(0) + V(0)T + 1/2AT^2
	vec3 WPos = WorldPos + (WorldVel * T);
	vec3 LPos = LocalPos + (LocalVel * T);

	// Transform LPos
	LPos = mul4x3(vec4(LPos, 1), Matrix);

	// Lerp
	float Local = mix(LocalStart, LocalEnd, TimePercentage);
	vec3 Pos = mix(WPos, LPos, Local) + (HalfA * T * T);
	
	//Pos.xyz = Bend(Pos.xyz);

	// Transform to viewspace
	Pos = mul4x3(vec4(Pos, 1), BentView);
	
	// Scale and rotate
	vec2 FinalSize = StartSize + (StartSize * (GrowSpeed * T));

	float Roll = StartRoll + (RollSpeed * T);
	
	vec2 RotatedSize;
	RotatedSize.x = dot(FinalSize, vec2(cos(Roll), -sin(Roll)));
	RotatedSize.y = dot(FinalSize, vec2(sin(Roll), cos(Roll)));

	Pos.xy += RotatedSize;

	// Output pos
	gl_Position = mul4x4(vec4(Pos, 1), Proj);
	
	// Output color
#ifdef LINEAR
	fColor = mix(Color0, Color1, TimePercentage);
#elif defined(HALFSINE)
	fColor = mix(Color0, Color1, sin(TimePercentage * 3.141592654));
#elif defined(SINE)
	if (TimePercentage < 0.09){
		fColor = mix(Color0, Color1, TimePercentage / 0.09);
	}
	else if (TimePercentage < 0.36){
		fColor = Color1;
	}
	else{
		fColor = mix(Color1, Color0, (TimePercentage - 0.36) / 0.64);
	}
	//fColor = mix(Color0, Color1, sin(TimePercentage * 6.283185308));
#elif defined(ABSSINE)
	fColor = mix(Color0, Color1, abs(sin(TimePercentage * 6.283185308)));
#else
	fColor = mix(Color0, Color1, TimePercentage);
#endif
	//fColor = vec4(sin(TimePercentage * 3.141592654));
	//fColor = vec4(TimePercentage, 0.0, 0.0, 1.0);

	// Output tex
	fTex = TexCoord3.xy * vec2(1, -1);
	
#ifdef ANIM
	float Width = WIDTH;
	float Height = HEIGHT;
	
	float TotalFrames = FRAMECOUNT;
	float FrameIndex = clamp(TimePercentage, 0.0, 1.0) * TotalFrames;
	
	vec2 TileIndex;
	TileIndex.x = floor(FrameIndex);//(FrameIndex / Width) - mod(FrameIndex, Width);
	TileIndex.y = floor(FrameIndex / Width);
	
	TileIndex.x /= Width;
	TileIndex.y /= Height;
	
	fTex.x /= Width;
	fTex.y /= Height;
	
	fTex += TileIndex;
#endif
}