uniform vec4 View[3];
uniform vec4 Proj[4];
uniform float Timer;

attribute vec3 Position;    // Pos
attribute vec4 Color0;      // Color0
attribute vec4 Color1;      // Color1
attribute vec2 TexCoord0;   // Tex
attribute vec4 TexCoord1;   // Gravity
attribute vec4 TexCoord2;   // StartVel
attribute vec4 TexCoord3;   // Timing
attribute vec4 TexCoord4;   // Sizing

varying vec4 fColor0;
varying vec4 fColor1;
varying vec2 fTex;
varying vec2 fFade;

vec4 mul4x4(vec4 V, vec4 M[4]) {
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

void main() {
    // Data unpacking
    float StartTime = TexCoord3.x;
    float Fade = TexCoord3.y;
    float InvTTL = TexCoord3.z;
    float TTL = TexCoord3.w;
    
    vec2 Size = TexCoord4.xy;
    float GrowStart = TexCoord4.z;
    float GrowSpeed = TexCoord4.w;
    
    float StartRoll = TexCoord1.w;
    float RollSpeed = TexCoord2.w;
    
    // Data
    float T = Timer - StartTime;
    vec3 A = TexCoord1.xyz;
    vec3 V0 = TexCoord2.xyz;
    
    // S(T) = S(0) + V(0)T + 1/2AT^2
    vec4 WorldPos;
    WorldPos.xyz = Position + (V0 * T) + (0.5 * A * T * T);
    WorldPos.w = 1.0;
    
    vec4 ViewPos;
    ViewPos.x = dot(WorldPos, View[0]);
    ViewPos.y = dot(WorldPos, View[1]);
    ViewPos.z = dot(WorldPos, View[2]);
    ViewPos.w = 1.0;
    
    //if (T < TTL){
        if (T >= GrowStart){
            Size += Size * (T - GrowStart) * GrowSpeed;
        }
        
        float Roll = StartRoll + (RollSpeed * T);
        float SinRoll = sin(Roll);
        float CosRoll = cos(Roll);
        
        vec2 NewSize;
        NewSize.x = dot(Size, vec2(CosRoll, -SinRoll));
        NewSize.y = dot(Size, vec2(SinRoll, CosRoll));
        
        ViewPos.xy += NewSize;
    //}
	
	fColor0 = Color0;
    fColor1 = Color1;
    
    if (T >= TTL){
		fColor0.a = 0.0;
		fColor1.a = 0.0;
	}
	
    fTex = TexCoord0;
    fFade = vec2(T * InvTTL, Fade);
    gl_Position = mul4x4(ViewPos, Proj);
}
