uniform vec4 View[3];
uniform vec4 BentView[3];
uniform vec4 Proj[4];

uniform vec2 BendFactors;
uniform vec4 BentViewProj[4];

uniform float Timer;

attribute vec3 Position;    // Pos
attribute vec4 Color0;      // Color0
attribute vec4 Color1;      // Color1
attribute vec2 TexCoord0;   // Tex
attribute vec4 TexCoord1;   // Gravity
attribute vec4 TexCoord2;   // StartVel
attribute vec4 TexCoord3;   // Timing
attribute vec4 TexCoord4;   // Sizing

varying vec4 fColor;
varying vec2 fTex;
varying highp vec4 Vertex;

/*#define ANIM
#define WIDTH 8.0
#define HEIGHT 8.0*/

vec4 mul4x4(vec4 V, vec4 M[4]) {
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec3 mul4x3(vec4 V, vec4 M[3]) {
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);

	return ret;
}

vec3 Bend(vec3 P){
	float Angle = (P.z * BendFactors.y) - 1.5707963267948966192313216916398;
	
	float SinAngle = sin(Angle);
	float CosAngle = cos(Angle);
		
	vec3 Pos = vec3(SinAngle + 1.0, 0.0, CosAngle) * BendFactors.x;
	
	vec4 BendMatrix[3];
	BendMatrix[0].xyz = vec3(-SinAngle, 0.0, -CosAngle);	// Dit is loodrecht op de afgeleide
	BendMatrix[1].xyz = vec3(0.0, 1.0, 0.0);
	BendMatrix[2].xyz = vec3(-CosAngle, 0.0, SinAngle);	// Dit is de afgeleide dus
	
	BendMatrix[0].w = Pos.x;
	BendMatrix[1].w = Pos.y;
	BendMatrix[2].w = Pos.z;
	
	P = mul4x3(vec4(P.xy, 0, 1), BendMatrix);
	
	return P;
}

void main() {
    // Data unpacking
    float StartTime = TexCoord3.x;
    float Fade = TexCoord3.y;
    float InvTTL = TexCoord3.z;
    float TTL = TexCoord3.w;
    
    vec2 Size = TexCoord4.xy;
    float GrowStart = TexCoord4.z;
    float GrowSpeed = TexCoord4.w;
    
    float StartRoll = TexCoord1.w;
    float RollSpeed = TexCoord2.w;
    
    // Data
    float T = Timer - StartTime;
    vec3 A = TexCoord1.xyz;
    vec3 V0 = TexCoord2.xyz;
    
    // S(T) = S(0) + V(0)T + 1/2AT^2
    vec4 WorldPos;
    WorldPos.xyz = Position + (V0 * T) + (0.5 * A * T * T);
    WorldPos.w = 1.0;
	
	//WorldPos.xyz = Bend(WorldPos.xyz);
    
    vec4 ViewPos;
    ViewPos.x = dot(WorldPos, BentView[0]);
    ViewPos.y = dot(WorldPos, BentView[1]);
    ViewPos.z = dot(WorldPos, BentView[2]);
    ViewPos.w = 1.0;
    
    if (T < TTL){
        if (T >= GrowStart){
            Size += Size * (T - GrowStart) * GrowSpeed;
        }
        
        float Roll = StartRoll + (RollSpeed * T);
        float SinRoll = sin(Roll);
        float CosRoll = cos(Roll);
        
        vec2 NewSize;
        NewSize.x = dot(Size, vec2(CosRoll, -SinRoll));
        NewSize.y = dot(Size, vec2(SinRoll, CosRoll));
        
        ViewPos.xy += NewSize;
    }

    fTex = TexCoord0;
#ifdef ANIM
	float Width = WIDTH;
	float Height = HEIGHT;
	
	float TotalFrames = FRAMECOUNT;
	float FrameIndex = clamp(T * InvTTL, 0.0, 1.0) * TotalFrames;
	
	vec2 TileIndex;
	TileIndex.x = floor(FrameIndex);//(FrameIndex / Width) - mod(FrameIndex, Width);
	TileIndex.y = floor(FrameIndex / Width);
	
	TileIndex.x /= Width;
	TileIndex.y /= Height;
	
	fTex.x /= Width;
	fTex.y /= Height;
	
	fTex += TileIndex;
#endif
	if (true){//Fade <= 0.5 / 4.0){
		fColor = mix(Color0, Color1, clamp(T * InvTTL, 0.0, 1.0));
	}
	else fColor = mix(Color0, Color1, sin(T * InvTTL * 3.141592654));
	
    gl_Position = mul4x4(ViewPos, Proj);
	
	vertex = gl_Position;
	vertex.xy = 0.5 * gl_Position.xy + 0.5 * gl_Position.w; //from [-1, 1] to [0, 1]
}
