uniform vec4 WorldArray[96];
uniform vec4 ViewProj[4];

uniform vec2 BendFactors;
uniform vec4 BentViewProj[4];

uniform float FogNear;
uniform float InvFogRange;
uniform float LerpFogAlpha;

uniform lowp vec4 LightDirection[3];
uniform lowp vec4 LightColor[4];

uniform float InvDetailSize;

attribute vec4 Position;
attribute lowp vec3 Normal;
attribute vec2 TexCoord0;
attribute highp float BlendIndices;

varying lowp vec4 fColor;
varying vec2 fTexCoord;
varying lowp float fFog;

vec3 mul4x3(vec4 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	
	return ret;
}

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec3 Bend(vec3 P){
	float Angle = (P.z * BendFactors.y) - 1.5707963267948966192313216916398;
	
	float SinAngle = sin(Angle);
	float CosAngle = cos(Angle);
		
	vec3 Pos = vec3(SinAngle + 1.0, 0.0, CosAngle) * BendFactors.x;
	
	vec4 BendMatrix[3];
	BendMatrix[0].xyz = vec3(-SinAngle, 0.0, -CosAngle);	// Dit is loodrecht op de afgeleide
	BendMatrix[1].xyz = vec3(0.0, 1.0, 0.0);
	BendMatrix[2].xyz = vec3(-CosAngle, 0.0, SinAngle);	// Dit is de afgeleide dus
	
	BendMatrix[0].w = Pos.x;
	BendMatrix[1].w = Pos.y;
	BendMatrix[2].w = Pos.z;
	
	P = mul4x3(vec4(P.xy, 0, 1), BendMatrix);
	
	return P;
}

void main(){
	highp int MatIndex = int(BlendIndices) * 3;
	
	highp vec3 WorldNormal;	// mediump is ook genoeg wellicht?
	WorldNormal.x = dot(WorldArray[MatIndex + 0].xyz, (Normal - 0.5) * 2.0);	// dit nog een keer fixen
	WorldNormal.y = dot(WorldArray[MatIndex + 1].xyz, (Normal - 0.5) * 2.0);
	WorldNormal.z = dot(WorldArray[MatIndex + 2].xyz, (Normal - 0.5) * 2.0);
	WorldNormal = normalize(WorldNormal);
	
	// Diffuse lighting
	lowp vec4 Color = LightColor[0];
    Color += clamp(dot(WorldNormal, LightDirection[0].xyz), 0.0, 1.0) * LightColor[1];
    Color += clamp(dot(WorldNormal, LightDirection[1].xyz), 0.0, 1.0) * LightColor[2];
    Color += clamp(dot(WorldNormal, LightDirection[2].xyz), 0.0, 1.0) * LightColor[3];
	
	vec4 WorldPos;
	WorldPos.x = dot(Position, WorldArray[MatIndex + 0]);
	WorldPos.y = dot(Position, WorldArray[MatIndex + 1]);
	WorldPos.z = dot(Position, WorldArray[MatIndex + 2]);
	WorldPos.w = 1.0;
	
	//WorldPos.xyz = Bend(WorldPos.xyz);

	gl_Position = mul4x4(WorldPos, ViewProj);
	fTexCoord = TexCoord0;
	fColor = Color;
	
	fFog = clamp((gl_Position.z * InvFogRange) - FogNear, 0.0, 1.0) * LerpFogAlpha;
}
