uniform vec4 WorldArray[96];
uniform vec4 ViewProj[4];

uniform float FogNear;
uniform float InvFogRange;
uniform float LerpFogAlpha;

uniform lowp vec4 LightDirection[3];
uniform lowp vec4 LightColor[4];

uniform float InvDetailSize;

attribute vec4 Position;
attribute lowp vec3 Normal;
attribute vec2 TexCoord0;
attribute vec4 BlendIndices;
attribute vec4 BlendWeights;

varying lowp vec4 fColor;
varying vec2 fTexCoord;
varying lowp float fFog;

vec4 mul4x3(vec4 V, vec4 M[3]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = 1.0;
	
	return ret;
}

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

void main(){
	vec4 WorldPos = vec4(0, 0, 0, 1);
	vec3 WorldNormal = vec3(0, 0, 0);

	//for (int i = 0; i < 4; i++){
		//x
		highp int MatIndex = int(BlendIndices.x) * 3;
		
		vec3 LocalPos;
		LocalPos.x = dot(Position, WorldArray[MatIndex + 0]);
		LocalPos.y = dot(Position, WorldArray[MatIndex + 1]);
		LocalPos.z = dot(Position, WorldArray[MatIndex + 2]);

		WorldPos.xyz += LocalPos * BlendWeights.x;

		vec3 LocalNormal;
		LocalNormal.x = dot(Normal, WorldArray[MatIndex + 0].xyz);
		LocalNormal.y = dot(Normal, WorldArray[MatIndex + 1].xyz);
		LocalNormal.z = dot(Normal, WorldArray[MatIndex + 2].xyz);

		WorldNormal += LocalNormal * BlendWeights.x;
		
		//y
		MatIndex = int(BlendIndices.y) * 3;
		
		LocalPos.x = dot(Position, WorldArray[MatIndex + 0]);
		LocalPos.y = dot(Position, WorldArray[MatIndex + 1]);
		LocalPos.z = dot(Position, WorldArray[MatIndex + 2]);

		WorldPos.xyz += LocalPos * BlendWeights.y;

		LocalNormal.x = dot(Normal, WorldArray[MatIndex + 0].xyz);
		LocalNormal.y = dot(Normal, WorldArray[MatIndex + 1].xyz);
		LocalNormal.z = dot(Normal, WorldArray[MatIndex + 2].xyz);

		WorldNormal += LocalNormal * BlendWeights.y;
		
		//z
		MatIndex = int(BlendIndices.z) * 3;
		
		LocalPos.x = dot(Position, WorldArray[MatIndex + 0]);
		LocalPos.y = dot(Position, WorldArray[MatIndex + 1]);
		LocalPos.z = dot(Position, WorldArray[MatIndex + 2]);

		WorldPos.xyz += LocalPos * BlendWeights.z;

		LocalNormal.x = dot(Normal, WorldArray[MatIndex + 0].xyz);
		LocalNormal.y = dot(Normal, WorldArray[MatIndex + 1].xyz);
		LocalNormal.z = dot(Normal, WorldArray[MatIndex + 2].xyz);

		WorldNormal += LocalNormal * BlendWeights.z;
		
		//w
		MatIndex = int(BlendIndices.w) * 3;
		
		LocalPos.x = dot(Position, WorldArray[MatIndex + 0]);
		LocalPos.y = dot(Position, WorldArray[MatIndex + 1]);
		LocalPos.z = dot(Position, WorldArray[MatIndex + 2]);

		WorldPos.xyz += LocalPos * BlendWeights.w;

		LocalNormal.x = dot(Normal, WorldArray[MatIndex + 0].xyz);
		LocalNormal.y = dot(Normal, WorldArray[MatIndex + 1].xyz);
		LocalNormal.z = dot(Normal, WorldArray[MatIndex + 2].xyz);

		WorldNormal += LocalNormal * BlendWeights.w;
	//}

	WorldNormal = normalize(WorldNormal);

	// Diffuse lighting
	lowp vec4 Color = LightColor[0];
    Color += clamp(dot(WorldNormal, LightDirection[0].xyz), 0.0, 1.0) * LightColor[1];
    Color += clamp(dot(WorldNormal, LightDirection[1].xyz), 0.0, 1.0) * LightColor[2];
    Color += clamp(dot(WorldNormal, LightDirection[2].xyz), 0.0, 1.0) * LightColor[3];
	
	gl_Position = mul4x4(WorldPos, ViewProj);
	fTexCoord = TexCoord0;
	fColor = Color;
	
	fFog = clamp((gl_Position.z * InvFogRange) - FogNear, 0.0, 1.0) * LerpFogAlpha;
}
