// Defines
//#undef SPECULAR
//#undef REFLECTION

//#define SPECULAR
//#define REFLECTION

#define LODBIAS -1.5

// Textures
uniform sampler2D Tex0;
uniform sampler2D Tex1;
uniform sampler2D Tex7;
uniform sampler2D Tex10;
uniform samplerCube ReflectionTex;

uniform lowp vec4 LightDirection[3];
uniform lowp vec4 LightColor[4];

// Varyings
varying lowp vec3 fTangentMatrix0;
varying lowp vec3 fTangentMatrix1;
varying lowp vec3 fTangentMatrix2;

varying mediump vec2 fTexCoord;

#ifdef DIRTMAP
varying mediump vec2 fDirtTex;
#endif

#ifdef SEPERATENORMALMAP
varying mediump vec2 fNormalTex;
#endif

varying lowp vec4 fFog;
#if defined(SPECULAR) || defined(REFLECTION)
varying mediump vec3 fView;
#endif

// Code 
void main(){
	lowp vec4 Diffuse = texture2D(Tex0, fTexCoord, LODBIAS);
	
#ifdef DIRTMAP
	lowp vec4 DirtMap = texture2D(Tex10, fDirtTex, LODBIAS);
	Diffuse = Diffuse + DirtMap - 0.5;
#endif
	// Hier carpaint shader inserten
	
#ifdef ALPHATEST
	if (Diffuse.a < 0.5){
		discard;
	}
#endif

#ifndef SEPERATENORMALMAP
	lowp vec3 Normal = (texture2D(Tex7, fTexCoord, LODBIAS).xyz - 0.5) * 2.0;
#else
	lowp vec3 Normal = (texture2D(Tex7, fNormalTex, LODBIAS).xyz - 0.5) * 2.0;
#endif

	lowp mat3 TangentMatrix;
	TangentMatrix[0] = fTangentMatrix0;
	TangentMatrix[1] = fTangentMatrix1;
	TangentMatrix[2] = fTangentMatrix2;

	lowp vec3 N = TangentMatrix * Normal;

	lowp vec4 Lighting = LightColor[0];
	Lighting += clamp(dot(N, LightDirection[0].xyz), 0.0, 1.0) * LightColor[1];
    Lighting += clamp(dot(N, LightDirection[1].xyz), 0.0, 1.0) * LightColor[2];
    Lighting += clamp(dot(N, LightDirection[2].xyz), 0.0, 1.0) * LightColor[3];

	lowp vec4 Output = Diffuse * Lighting;
	
#ifdef PREMULTIPLY_DIFFUSE
	Output.rgb *= Output.a;
#endif

#if defined(SPECULAR) || defined(REFLECTION)
#ifndef TEX0SPECULAR
	lowp vec4 Specular = texture2D(Tex1, fTexCoord, LODBIAS);
#else
	lowp vec4 Specular = texture2D(Tex0, fTexCoord, LODBIAS).a;
#endif

	mediump vec3 V = fView;
	mediump vec3 ReflectionVector = 2.0 * dot(N, V) * N - V;
	
	// Hier carpaint shader inserten

#ifdef SPECULAR
	lowp vec3 R = normalize(ReflectionVector);
	mediump float Shininess = clamp(dot(R, LightDirection[0].xyz), 0.0, 1.0);
	Output.rgb += Specular.xyz * pow(Shininess, 16.0) * 2.0;	// hier specular sterkte instellen
#endif

#ifdef REFLECTION
	lowp vec4 Reflection = textureCube(ReflectionTex, ReflectionVector);
	// hier reflection sterkte instellen
	
//#ifdef FRESNEL
	lowp float Fresnel = (1.0 - clamp(dot(normalize(V), N), 0.0, 1.0)) * 1.0;
	Reflection *= Fresnel;
//#endif
	
	Output.rgb += Specular.xyz * Reflection.xyz * 2.0;
#endif
#endif

	// Fog
	Output.rgb = (Output.rgb * fFog.a) + fFog.rgb;
#ifndef ALPHA
	Output.a = fFog.a;
#endif
	gl_FragColor = Output;
}

