// Defines
//#undef SPECULAR
//#undef REFLECTION

//#define SPECULAR
//#define REFLECTION

// Uniforms
uniform vec4 World[3];
uniform vec4 View[3];
uniform vec4 Projection[4];
uniform vec4 ViewProj[4];
uniform vec4 WVP[4];

uniform vec2 BendFactors;
uniform vec4 BentViewProj[4];

uniform vec3 CamPos;

uniform float FogNear;
uniform float InvFogRange;
uniform vec3 LerpFogColor;
uniform float LerpFogAlpha;

uniform float InvDetailSize;
uniform float InvNormalMapSize;

// Attributes
attribute highp vec4 Position;
attribute lowp vec3 Normal;
attribute lowp vec3 Binormal;
attribute lowp vec3 Tangent;
attribute mediump vec2 TexCoord0;

// Varyings
#if defined(SPECULAR) || defined(REFLECTION)
varying mediump vec3 fView;
#endif

varying lowp vec3 fTangentMatrix0;
varying lowp vec3 fTangentMatrix1;
varying lowp vec3 fTangentMatrix2;

varying mediump vec2 fTexCoord;

#ifdef DIRTMAP
varying mediump vec2 fDirtTex;
#endif

#ifdef SEPERATENORMALMAP
varying mediump vec2 fNormalTex;
#endif

varying lowp vec4 fFog;

// Helpers
vec3 mul3x3(vec3 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0].xyz);
	ret.y = dot(V, M[1].xyz);
	ret.z = dot(V, M[2].xyz);
	
	return ret;
}

vec3 mul4x3(vec4 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);

	return ret;
}

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]); 
	ret.w = dot(V, M[3]);
	
	return ret;
}


vec3 Bend(vec3 P){
	float Angle = (P.z * BendFactors.y) - 1.5707963267948966192313216916398;
	
	float SinAngle = sin(Angle);
	float CosAngle = cos(Angle);
		
	vec3 Pos = vec3(SinAngle + 1.0, 0.0, CosAngle) * BendFactors.x;
	
	vec4 BendMatrix[3];
	BendMatrix[0].xyz = vec3(-SinAngle, 0.0, -CosAngle);
	BendMatrix[1].xyz = vec3(0.0, 1.0, 0.0);
	BendMatrix[2].xyz = vec3(-CosAngle, 0.0, SinAngle);
	
	BendMatrix[0].w = Pos.x;
	BendMatrix[1].w = Pos.y;
	BendMatrix[2].w = Pos.z;
	
	P = mul4x3(vec4(P.xy, 0, 1), BendMatrix);
	
	return P;
}


// Code
void main(){
	// Output data
	vec3 P = mul4x3(Position, World);
	
	gl_Position = mul4x4(vec4(P, 1), ViewProj);
	
	//mediump vec3 P = mul4x3(Position, World);
	
#ifdef UNSIGNEDNORMALS
	lowp vec3 N = mul3x3((Normal - 0.5) * 2.0, World);
	lowp vec3 T = mul3x3((Tangent - 0.5) * 2.0, World);
	lowp vec3 B = mul3x3((Binormal - 0.5) * 2.0, World);
#else
	lowp vec3 N = mul3x3(Normal, World);
	lowp vec3 T = mul3x3(Tangent, World);
	lowp vec3 B = mul3x3(Binormal, World);
#endif
	
#if defined(SPECULAR) || defined(REFLECTION)
	mediump vec3 V = CamPos - P;

	fView = V;
#endif

	fTangentMatrix0 = B;
	fTangentMatrix1 = T;
	fTangentMatrix2 = N;

	fTexCoord.xy = TexCoord0;

#ifdef DIRTMAP
	fDirtTex = P.xz * InvDetailSize;
#endif

#ifdef SEPERATENORMALMAP
	fNormalTex = P.xz * InvNormalMapSize;
#endif

	lowp float Fog = clamp((gl_Position.z * InvFogRange) - FogNear, 0.0, 1.0) * LerpFogAlpha;

	fFog.rgb = LerpFogColor * Fog;
	fFog.a = 1.0 - Fog;
}
