// Defines
/*#ifndef SPECULAR
#define SPECULAR
#endif

#ifndef REFLECTION
#define REFLECTION
#endif*/

#define LODBIAS -1.5

// Textures
uniform sampler2D Tex0;
uniform sampler2D Tex1;
uniform sampler2D Tex10;
uniform samplerCube ReflectionTex;

uniform lowp vec3 LightDirection2;
uniform mediump vec3 CamPos;

uniform lowp vec4 CarDiffuseColor;
uniform lowp vec4 CarSpecularColor;
uniform lowp float CarSpecularIntensity;
uniform lowp float CarSpecularPower;

// Varyings
varying lowp vec4 fColor;
varying mediump vec2 fTexCoord;

#ifdef DIRTMAP
varying mediump vec2 fDirtTex;
#endif

varying lowp vec4 fFog;
#if defined(SPECULAR) || defined(REFLECTION)
varying mediump vec4 fReflection;
#endif

// Code 
void main(){
#ifdef CAR
	lowp vec4 BasicPaint = texture2D(Tex0, fTexCoord, LODBIAS);

	lowp vec4 Diffuse;
	Diffuse.rgb = mix(BasicPaint.rgb, BasicPaint.rgb * CarDiffuseColor.rgb, BasicPaint.a);	// let op. lerp vervangen door een a + b * c
	Diffuse.a = 1.0;
#else
	lowp vec4 Diffuse = texture2D(Tex0, fTexCoord, LODBIAS);
#endif
	
#ifdef DIRTMAP
	lowp vec4 DirtMap = texture2D(Tex10, fDirtTex, LODBIAS);
	Diffuse = Diffuse + DirtMap - 0.5;
#endif
	
#ifdef ALPHATEST
	if (Diffuse.a < 0.5){
		discard;
	}
#endif

	lowp vec4 Output = Diffuse * fColor;
	
#ifdef PREMULTIPLY_DIFFUSE
	Output.rgb *= Output.a;
#endif
	
#if defined(SPECULAR) || defined(REFLECTION)
#ifdef CAR
	lowp vec3 SpecularPaint = texture2D(Tex1, fTexCoord, LODBIAS).xyz;

	lowp vec3 Specular = mix(SpecularPaint, SpecularPaint * CarSpecularColor.rgb/* * 2.0*/, BasicPaint.a);
#else
#ifndef TEX0SPECULAR
	lowp vec4 Specular = texture2D(Tex1, fTexCoord, LODBIAS);
#else
	lowp vec4 Specular = vec4(texture2D(Tex0, fTexCoord, LODBIAS).a);
#endif
#endif

#ifdef SPECULAR
	lowp vec3 R = normalize(fReflection.xyz);
	mediump float Shininess = clamp(dot(R, LightDirection2.xyz), 0.0, 1.0);
	Output.rgb += Specular.xyz * pow(Shininess, 16.0) * fFog.a;	// hier specular sterkte instellen
#endif

#ifdef REFLECTION
	lowp vec4 Reflection = textureCube(ReflectionTex, fReflection.xyz);
	// hier reflection sterkte instellen
#ifdef FRESNEL
	Reflection *= fReflection.w;
#endif
	
	Output.rgb += Specular.xyz * Reflection.xyz * fFog.a;
#endif
#endif

	// Fog
	Output.rgb = Output.rgb + fFog.rgb;
	Output.a = fFog.a;
	
	gl_FragColor = Output;
}

