// Defines
/*#ifndef SPECULAR
#define SPECULAR
#endif

#ifndef REFLECTION
#define REFLECTION
#endif*/

// Uniforms
uniform vec4 World[3];
uniform vec4 View[3];
uniform vec4 Projection[4];
uniform vec4 ViewProj[4];
uniform vec4 WVP[4];
uniform vec4 ShadowMatrix[4];

uniform vec2 BendFactors;
uniform vec4 BentViewProj[4];

uniform mediump vec3 CamPos;

uniform float FogNear;
uniform float InvFogRange;
uniform vec3 LerpFogColor;
uniform float LerpFogAlpha;

uniform vec4 LightDirection[3];
uniform vec4 LightColor[4];

uniform float InvDetailSize;
uniform float InvNormalSize;

// Attributes
attribute highp vec4 Position;
attribute lowp vec3 Normal;
attribute mediump vec2 TexCoord0;

// Varyings
varying lowp vec4 fColor[2];
#if defined(SPECULAR) || defined(REFLECTION)
varying mediump vec4 fReflection;
#endif

varying mediump vec2 fTexCoord;

#ifdef DIRTMAP
varying mediump vec2 fDirtTex;
#endif

varying highp vec3 fShadowPos;

varying lowp vec4 fFog;

// Helpers
vec3 mul3x3(vec3 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0].xyz);
	ret.y = dot(V, M[1].xyz);
	ret.z = dot(V, M[2].xyz);
	
	return ret;
}

vec3 mul4x3(vec4 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);

	return ret;
}

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]); 
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec3 Bend(vec3 P){
	float Angle = (P.z * BendFactors.y) - 1.5707963267948966192313216916398;
	
	float SinAngle = sin(Angle);
	float CosAngle = cos(Angle);
		
	vec3 Pos = vec3(SinAngle + 1.0, 0.0, CosAngle) * BendFactors.x;
	
	vec4 BendMatrix[3];
	BendMatrix[0].xyz = vec3(-SinAngle, 0.0, -CosAngle);	// Dit is loodrecht op de afgeleide
	BendMatrix[1].xyz = vec3(0.0, 1.0, 0.0);
	BendMatrix[2].xyz = vec3(-CosAngle, 0.0, SinAngle);	// Dit is de afgeleide dus
	
	BendMatrix[0].w = Pos.x;
	BendMatrix[1].w = Pos.y;
	BendMatrix[2].w = Pos.z;
	
	P = mul4x3(vec4(P.xy, 0, 1), BendMatrix);
	
	return P;
}

// Code
void main(){
	// Output data
	vec3 P = mul4x3(Position, World);
	
	gl_Position = mul4x4(vec4(P, 1), ViewProj);
	
#ifdef UNSIGNEDNORMALS
	lowp vec3 N = mul3x3((Normal - 0.5) * 2.0, World);
#else
	lowp vec3 N = mul3x3(Normal, World);
#endif

	// Diffuse lighting
	lowp vec4 Lighting = LightColor[0];
	//Lighting += clamp(dot(N, LightDirection[0].xyz), 0.0, 1.0) * LightColor[1];
    Lighting += clamp(dot(N, LightDirection[1].xyz), 0.0, 1.0) * LightColor[2];
    Lighting += clamp(dot(N, LightDirection[2].xyz), 0.0, 1.0) * LightColor[3];
	
	fColor[0] = clamp(dot(N, LightDirection[0].xyz), 0.0, 1.0) * LightColor[1];
	fColor[1] = Lighting;
	
	fColor[0].a = 0.0;
	fColor[1].a = 1.0;
	
#if defined(SPECULAR) || defined(REFLECTION)
	mediump vec3 V = CamPos - P;

	fReflection.xyz = 2.0 * dot(N, V) * N - V;
	fReflection.w = (1.5 - clamp(dot(normalize(V), N), 0.0, 1.0)) * 1.0;
#endif

	fTexCoord.xy = TexCoord0;
	
#ifdef DIRTMAP
	fDirtTex = P.xz * InvDetailSize;
#endif

#ifdef NORMALMAP
	fNormalMap = P.xz * InvNormalSize;
#endif

	highp vec4 ShadowPos = mul4x4(vec4(P, 1.0), ShadowMatrix);
	fShadowPos = ShadowPos.xyz;
	
	lowp float Fog = clamp((gl_Position.z * InvFogRange) - FogNear, 0.0, 1.0) * LerpFogAlpha;

	fFog.rgb = LerpFogColor * Fog;
	fFog.a = 1.0 - Fog;

	fColor[0].rgb *= 1.0 - Fog;
	fColor[1].rgb *= 1.0 - Fog;
}
