uniform vec4 World[3];

attribute vec4 Position;
attribute vec2 TexCoord0;

varying vec2 fTexCoord;

uniform vec2 BendFactors;
uniform vec4 ViewProj[4];


vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec3 mul4x3(vec4 V, vec4 M[3]) {
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);

	return ret;
}

vec3 Bend(vec3 P){
	float Angle = (P.z * BendFactors.y) - 1.5707963267948966192313216916398;
	
	float SinAngle = sin(Angle);
	float CosAngle = cos(Angle);
		
	vec3 Pos = vec3(SinAngle + 1.0, 0.0, CosAngle) * BendFactors.x;
	
	vec4 BendMatrix[3];
	BendMatrix[0].xyz = vec3(-SinAngle, 0.0, -CosAngle);	// Dit is loodrecht op de afgeleide
	BendMatrix[1].xyz = vec3(0.0, 1.0, 0.0);
	BendMatrix[2].xyz = vec3(-CosAngle, 0.0, SinAngle);	// Dit is de afgeleide dus
	
	BendMatrix[0].w = Pos.x;
	BendMatrix[1].w = Pos.y;
	BendMatrix[2].w = Pos.z;
	
	P = mul4x3(vec4(P.xy, 0.0, 1.0), BendMatrix);
	
	return P;
}

void main(){
	vec3 Pos = mul4x3(Position, World);
	
	//Pos = Bend(Pos);
	
	gl_Position = mul4x4(vec4(Pos, 1.0), ViewProj);
	fTexCoord = TexCoord0;
}
