// Textures
uniform sampler2D Tex7;
uniform samplerCube ReflectionTex;

// Uniforms
uniform lowp vec4 LerpFogColor;

// Varyings
varying highp vec2 fTexCoord0;
varying highp vec2 fTexCoord1;
varying lowp vec3 fTangentMatrix0;
varying lowp vec3 fTangentMatrix1;
varying lowp vec3 fTangentMatrix2;
varying highp vec3 fView;
varying lowp vec4 fFog;
varying lowp float fColor;

// Code
void main(){
	lowp vec3 N0 = texture2D(Tex7, fTexCoord0, -2.5).xyz - 0.5;
	lowp vec3 N1 = texture2D(Tex7, fTexCoord1, -2.5).xyz - 0.5;
	
	lowp vec3 InputNormal = N0 + N1;
	
	lowp mat3 TangentMatrix;
	TangentMatrix[0] = fTangentMatrix0;
	TangentMatrix[1] = fTangentMatrix1;
	TangentMatrix[2] = fTangentMatrix2;

	// Transform normal
	lowp vec3 Normal = TangentMatrix * InputNormal;
	
	lowp vec3 N = Normal;
	lowp vec3 V = normalize(fView);

	lowp vec3 R = 2.0 * dot(N, V) * N - V;

	lowp vec4 Output = textureCube(ReflectionTex, R * vec3(1.0, -1.0, 1.0));

	// Fog
	Output = (Output * fColor) + fFog;

	gl_FragColor = Output;
}