uniform vec4 ViewProj[4];

uniform vec3 LocalCamPos;
uniform vec4 World[3];

uniform mediump float FogNear;
uniform mediump float InvFogRange;
uniform lowp float LerpFogAlpha;
uniform lowp vec4 LerpFogColor;

uniform float Timer;

attribute vec4 Position;
attribute lowp vec3 Normal;
attribute lowp vec3 Binormal;
attribute lowp vec3 Tangent;
attribute vec2 TexCoord0;

varying vec2 fTexCoord0;
varying vec2 fTexCoord1;

varying lowp vec3 fTangentMatrix0;
varying lowp vec3 fTangentMatrix1;
varying lowp vec3 fTangentMatrix2;

varying vec3 fView;
varying vec4 fFog;
varying float fColor;

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec3 mul4x3(vec4 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);

	return ret;
}

void main(){
	mat3 WorldRot;
    WorldRot[0] = World[0].xyz;
    WorldRot[1] = World[1].xyz;
    WorldRot[2] = World[2].xyz;
    
	lowp mat3 TangentMatrix;
	TangentMatrix[0] = Binormal;
	TangentMatrix[1] = Tangent;
	TangentMatrix[2] = Normal;
	
	vec4 P = vec4(mul4x3(Position, World), 1.0);
	gl_Position = mul4x4(P, ViewProj);
	fTexCoord0 = TexCoord0 + vec2(Timer / 15.0, 0.0);
	fTexCoord1 = TexCoord0 + vec2(0.0, Timer / 17.4);
	
	fTangentMatrix0 = Binormal;
	fTangentMatrix1 = Tangent;
	fTangentMatrix2 = Normal;
	
	fView = (LocalCamPos - Position.xyz) * WorldRot;

	lowp float Fog = clamp((gl_Position.z * InvFogRange) - FogNear, 0.0, 1.0) * LerpFogAlpha;

	fFog = LerpFogColor * Fog;
	
	fColor = 1.0 - Fog;
}
