uniform sampler2D Tex0;
uniform sampler2D Tex1;
uniform sampler2D Tex7;
uniform sampler2D Tex10;

uniform sampler2D ShadowTex;
uniform samplerCube RealtimeReflectionTex;

uniform lowp vec4 LerpFogColor;

uniform lowp vec4 LightColor[4];

varying highp vec2 fTexCoord;
varying lowp vec3 fLightDir0;
varying lowp vec3 fLightDir1;
varying lowp vec3 fLightDir2;
varying highp vec2 fShadowPos[4];
varying highp float fShadowDepth;
varying mediump vec3 fPos;
varying lowp float fFog;

#define SMOOTH
#define SOFTSMOOTH

void main(){
	// Diffuse
	lowp vec4 Diffuse = texture2D(Tex0, fTexCoord);

	// Specular
	lowp float Specular = Diffuse.a;
	
	// Shadow
#ifdef SMOOTH
#ifdef SOFTSMOOTH
	int ShadowIndex = int(fShadowDepth / 2000.0);

	highp vec4 ShadowDepth = texture2D(ShadowTex, fShadowPos[ShadowIndex]).x;
#else
	highp vec4 ShadowDepth = texture2D(ShadowTex, fShadowPos.xy);
#endif
	
	highp vec4 Depth = vec4(fShadowDepth, fShadowDepth, fShadowDepth, fShadowDepth);
	lowp vec4 ShadowSamples = step(Depth, ShadowDepth);
	lowp float Shadow = dot(ShadowSamples, ShadowSamples) * 0.25;
#else
	lowp float Shadow;
	if (ShadowDepth.x < fShadowDepth){
		Shadow = 0.0;
	}
	else Shadow = 1.0;
#endif
	
	// Normal
	lowp vec3 InputNormal = (texture2D(Tex7, fTexCoord).xyw - 0.5) * 2.0;

	// Transform normal
	lowp vec3 Normal = InputNormal;//vec3(0.0, 0.0, 1.0);
	
	// Do diffuse lighting
	lowp vec4 Color = LightColor[0];
    Color += clamp(dot(Normal, fLightDir0), 0.0, 1.0) * LightColor[1] * Shadow;
    Color += clamp(dot(Normal, fLightDir1), 0.0, 1.0) * LightColor[2];
    Color += clamp(dot(Normal, fLightDir2), 0.0, 1.0) * LightColor[3];
	
	// Compute diffuse
	lowp vec4 Output = Diffuse * Color;
	
	// Do specular
	lowp vec3 N = Normal * 1.01;	// deze normalize is onnodig, maar zonder deze vernaggelt hij wel de shader
	lowp vec3 L = fLightDir0;
	lowp vec3 V = normalize(-fPos);
			
	lowp vec3 R = 2.0 * dot(N, L) * N - L;
	lowp float Shininess = clamp(dot(R, V), 0.0, 1.0);
			
	Output.rgb += (vec3(4.0, 4.0, 4.0) * clamp(pow(Shininess, 128.0), 0.0, 1.0)) * Specular * Shadow;
	
	// Fog
	Output.rgb = mix(Output.rgb, LerpFogColor.rgb, fFog);

	gl_FragColor = Output;
	
	//gl_FragColor.xyz = (InputNormal / 2.0) + 0.5;
}

