// Defines
#undef REFLECTION
#undef SPECULAR
#undef SHADOW
#undef ALPHATEST

#define DOUBLESIDED

// Defaults
#ifndef SPECULARPOWER
#define SPECULARPOWER 4.0
#endif

#ifndef SPECULARINTENSITY
#define SPECULARINTENSITY 2.0
#endif

#if !defined(REFLECTION) && !defined(SPECULAR)
#define FASTFOGGING
#endif

// Textures
uniform sampler2D Tex0;
uniform sampler2D Tex1;
uniform sampler2D Tex10;
uniform sampler2D ShadowTex;
uniform samplerCube ReflectionTex;

// Uniforms
uniform lowp vec3 CarReflectionPower;
uniform lowp vec3 CarReflectionMult;

uniform lowp vec3 WorldReflectionPower;
uniform lowp vec3 WorldReflectionMult;

uniform lowp vec4 LerpFogColor;

uniform lowp vec4 BloomColor;

// Varyings
#ifndef SHADOW
varying lowp vec4 fColor;
#ifdef DOUBLESIDED
varying lowp vec4 fColor2;
#endif
#else
varying lowp vec4 fMainLight;
varying lowp vec4 fLighting;
#ifdef DOUBLESIDED
varying lowp vec4 fMainLight2;
varying lowp vec4 fLighting2;
#endif
#endif

#ifndef DIRTMAP
varying mediump vec2 fTexCoord;
#else
varying mediump vec4 fTexCoord;
#endif

#if defined(REFLECTION) || defined(SPECULAR)
varying highp vec3 fView;

#ifdef REFLECTION
varying lowp vec3 fNormal;
#endif

#ifdef SPECULAR
varying lowp vec3 fReflection;
#endif
#endif

#ifdef SHADOW
varying mediump vec3 fShadowPos;
#endif

#ifdef FASTFOGGING
varying lowp vec3 fFog;
#else
varying lowp vec4 fFog;
#endif

// Code
void main(){
#ifdef SHADOW
	mediump vec4 ShadowDepth = texture2D(ShadowTex, fShadowPos.xy);
	
	mediump vec4 Depth = vec4(fShadowPos.z, fShadowPos.z, fShadowPos.z, fShadowPos.z);
	lowp vec4 ShadowSamples = step(Depth, ShadowDepth);
	lowp float Shadow = dot(ShadowSamples, ShadowSamples);// * 0.25;
#else
	lowp float Shadow = 1.0;
#endif
	
	// Diffuse
	lowp vec4 Diffuse = texture2D(Tex0, fTexCoord.xy, -1.5);
#ifdef ALPHATEST
	if (Diffuse.a < 0.5){
		discard;
	}
#endif
	
#ifdef DIRTMAP
	lowp vec4 DirtMap = texture2D(Tex10, fTexCoord.zw, -1.5);
	Diffuse.rgb = Diffuse.rgb + DirtMap.rgb - 0.5;
#endif

	// Specular
#if defined(REFLECTION) || defined(SPECULAR)
#ifdef SEPERATESPECULARTEXTURE
	lowp vec3 Specular = texture2D(Tex1, fTexCoord.xy, -1.5).xyz;
#else
	lowp float Specular = Diffuse.a;
#endif
#endif

	// Compute diffuse
#ifndef SHADOW
#ifndef DOUBLESIDED
	lowp vec4 Output = Diffuse * fColor;
#else
	lowp vec4 Output;
	if (gl_FrontFacing){
		Output = Diffuse * fColor2;
	}
	else Output = Diffuse * fColor;
#endif
#else
#ifndef DOUBLESIDED
	lowp vec4 Output = Diffuse * ((fMainLight * Shadow) + fLighting);
#else
	lowp vec4 Output;
	if (gl_FrontFacing){
		Output = Diffuse * ((fMainLight2 * Shadow) + fLighting2);
	}
	else Output = Diffuse * ((fMainLight * Shadow) + fLighting);
#endif
#endif

#ifdef PREMULTIPLY_DIFFUSE
	Output.rgb *= Diffuse.a;
#endif
	Output.a = Diffuse.a;
	
	// Do specular
#if defined(REFLECTION) || defined(SPECULAR)
	lowp vec3 V = normalize(fView);
#ifdef SPECULAR
	lowp vec3 S = fReflection;
	
	lowp float Shininess = clamp(dot(S, V), 0.0, 1.0);	
	Output.rgb += BloomColor.rgb * SPECULARINTENSITY * pow(Shininess, SPECULARPOWER) * Specular * Shadow;
#endif

	// Do reflection
#ifdef REFLECTION
	lowp vec3 N = fNormal;
	
	lowp vec3 R = 2.0 * dot(N, V) * N - V;
	
	lowp vec3 Reflection = textureCube(ReflectionTex, R * vec3(1.0, -1.0, 1.0)).xyz;
	
	Output.rgb += BloomColor.rgb * pow(Reflection, ReflectionPower) * Specular * ReflectionMult;
#endif
#endif
	// Fog
#ifdef FASTFOGGING
	Output.rgb = Output.rgb + fFog.rgb;
#else
	Output.rgb = (Output.rgb * fFog.a) + fFog.rgb;
#endif

	gl_FragColor = Output;
}

