uniform vec4 World[3];
uniform vec4 View[3];
uniform vec4 ViewProj[4];

uniform vec4 AmbientColor;
uniform vec4 LightData[6];

attribute vec3 Position;
attribute vec3 Normal;
attribute vec2 TexCoord0;

varying vec4 fColor;
varying vec2 fTexCoord;

vec3 mul3x3(vec3 V, vec4 M[3]){
	vec3 ret;
	
	ret.x = dot(V, M[0].xyz);
	ret.y = dot(V, M[1].xyz);
	ret.z = dot(V, M[2].xyz);
	
	return ret;
}

vec4 mul4x3(vec4 V, vec4 M[3]){
	vec4 ret;
	
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = 1.0;
	
	return ret;
}

vec4 mul4x4(vec4 V, vec4 M[4]){
	vec4 ret;
	 
	ret.x = dot(V, M[0]);
	ret.y = dot(V, M[1]);
	ret.z = dot(V, M[2]);
	ret.w = dot(V, M[3]);
	
	return ret;
}

vec4 ComputeProjPosition(vec3 LocalPosition, vec4 WorldMatrix[3], vec4 ViewProj[4]){
	vec4 WorldPosition = mul4x3(vec4(Position, 1), World);
	ProjPosition = mul4x4(WorldPosition, ViewProj);
    
    return ProjPosition;
}

vec3 ComputeWorldNormal(vec3 LocalNormal, vec4 WorldMatrix[3]){
	vec3 WorldNormal;
    
	WorldNormal.x = dot(LocalNormal, WorldMatrix[0].xyz);
	WorldNormal.y = dot(LocalNormal, WorldMatrix[1].xyz);
	WorldNormal.z = dot(LocalNormal, WorldMatrix[2].xyz);
    WorldNormal = normalize(WorldNormal);
    
    return WorldNormal;
}

vec4 ComputeLights(vec3 WorldNormal, vec4 LightData[6], vec4 AmbientColor){
    vec3 LightIntensity = AmbientColor.rgb;
    
    LightIntensity += clamp(dot(WorldNormal, LightData[0].xyz), 0.0, 1.0) * LightData[1].rgb;
    LightIntensity += clamp(dot(WorldNormal, LightData[2].xyz), 0.0, 1.0) * LightData[3].rgb;
    LightIntensity += clamp(dot(WorldNormal, LightData[4].xyz), 0.0, 1.0) * LightData[5].rgb;
    
    return vec4(LightIntensity, 1.0);
}

/*
void main() {
    vec4 ProjPosition = ComputeProjPosition(Position, World, ViewProj);
    vec3 WorldNormal = ComputeWorldNormal((Normal - 0.5) * 2.0, World);
    vec4 LightIntensity = ComputeLights(WorldNormal, LightData, AmbientColor);
    
	fColor = vec4(LightIntensity, 1.0);
	fTexCoord = TexCoord0;
	gl_Position = ProjPosition;
}
*/